@@ -1,5 +1,5 @@
 // Aqsis
-// Copyright  1997 - 2001, Paul C. Gregory
+// Copyright (C) 1997 - 2001, Paul C. Gregory
 //
 // Contact: pgregory@aqsis.org
 //
@@ -23,10 +23,13 @@
 		\author Paul C. Gregory (pgregory@aqsis.org)
 */
 
-#include	<math.h>
-#include	"matrix.h"
+#include "matrix.h"
 
-START_NAMESPACE( Aqsis )
+#include <iomanip>
+
+#include "aqsismath.h"
+
+namespace Aqsis {
 
 
 //---------------------------------------------------------------------
@@ -57,7 +60,7 @@ CqMatrix::CqMatrix( const TqFloat xs, const TqFloat ys, const TqFloat zs )
 		m_aaElement[ 2 ][ 2 ] = zs;
 		m_aaElement[ 3 ][ 3 ] = 1.0;
 
-		m_fIdentity = TqFalse;
+		m_fIdentity = false;
 	}
 }
 
@@ -73,7 +76,7 @@ CqMatrix::CqMatrix( const CqVector3D& Trans )
 
 	if ( Trans.x() != 0.0f || Trans.y() != 0.0f || Trans.z() != 0.0f )
   {
-		m_fIdentity = TqFalse;
+		m_fIdentity = false;
 
 		m_aaElement[ 3 ][ 0 ] = Trans.x();
 		m_aaElement[ 3 ][ 1 ] = Trans.y();
@@ -86,6 +89,8 @@ CqMatrix::CqMatrix( const CqVector3D& Trans )
 /** Rotate matrix constructor
  * \param Angle	The angle to rotate by.
  * \param Axis The axis about which to rotate.
+ *
+ * \todo code review: is m_fIdentity needed here  
  */
 
 CqMatrix::CqMatrix( const TqFloat Angle, const CqVector3D Axis )
@@ -106,7 +111,6 @@ CqMatrix::CqMatrix( const TqFloat Angle, const CqVector3D Axis )
  * For now base this on what Larry Gritz posted a while back.
  * There are some more optimizations that can be done.
  */
-#define	PI		3.14159265359f
 CqMatrix::CqMatrix( const TqFloat angle,
                     const TqFloat dx1, const TqFloat dy1, const TqFloat dz1,
                     const TqFloat dx2, const TqFloat dy2, const TqFloat dz2 )
@@ -120,7 +124,7 @@ CqMatrix::CqMatrix( const TqFloat angle,
 
 	TqFloat d1d2dot = d1 * d2;
 	TqFloat axisangle = static_cast<TqFloat>(acos( d1d2dot ));
-	if ( angle >= axisangle || angle <= ( axisangle - PI ) )
+	if ( angle >= axisangle || angle <= ( axisangle - M_PI ) )
 	{
 		// Skewed past the axes -- issue error, then just use identity matrix.
 		// No access to CqBasicError from here, so this will have to be down
@@ -202,7 +206,7 @@ CqMatrix::CqMatrix( TqFloat f )
     m_aaElement[ 1 ][ 1 ] = f;
     m_aaElement[ 2 ][ 2 ] = f;
     m_aaElement[ 3 ][ 3 ] = f;
-    m_fIdentity = TqFalse;
+    m_fIdentity = false;
   }
 }
 
@@ -230,7 +234,7 @@ void CqMatrix::Identity()
 	        m_aaElement[ 2 ][ 2 ] =
 	            m_aaElement[ 3 ][ 3 ] = 1.0f;
 
-	m_fIdentity = TqTrue;
+	m_fIdentity = true;
 }
 
 
@@ -273,7 +277,7 @@ void CqMatrix::Rotate( const TqFloat Angle, const CqVector3D Axis )
 		CqMatrix	R;
 		R.Identity();
 		CqVector3D	RotAxis = Axis;
-		R.m_fIdentity = TqFalse;
+		R.m_fIdentity = false;
 
 		RotAxis.Unit();
 
@@ -344,7 +348,7 @@ void CqMatrix::Translate( const TqFloat xt, const TqFloat yt, const TqFloat zt )
 void CqMatrix::ShearX( const TqFloat yh, const TqFloat zh )
 {
 	CqMatrix Shear;
-	Shear.m_fIdentity = TqFalse;
+	Shear.m_fIdentity = false;
 
 	Shear.m_aaElement[ 0 ][ 1 ] = yh;
 	Shear.m_aaElement[ 0 ][ 2 ] = zh;
@@ -362,7 +366,7 @@ void CqMatrix::ShearX( const TqFloat yh, const TqFloat zh )
 void CqMatrix::ShearY( const TqFloat xh, const TqFloat zh )
 {
 	CqMatrix Shear;
-	Shear.m_fIdentity = TqFalse;
+	Shear.m_fIdentity = false;
 
 	Shear.m_aaElement[ 1 ][ 0 ] = xh;
 	Shear.m_aaElement[ 1 ][ 2 ] = zh;
@@ -379,7 +383,7 @@ void CqMatrix::ShearY( const TqFloat xh, const TqFloat zh )
 void CqMatrix::ShearZ( const TqFloat xh, const TqFloat yh )
 {
 	CqMatrix Shear;
-	Shear.m_fIdentity = TqFalse;
+	Shear.m_fIdentity = false;
 
 	Shear.m_aaElement[ 2 ][ 0 ] = xh;
 	Shear.m_aaElement[ 2 ][ 1 ] = yh;
@@ -409,10 +413,12 @@ void CqMatrix::Skew( const TqFloat angle,
 
 //---------------------------------------------------------------------
 /** Normalise the matrix, returning the homogenous part of the matrix to 1.
+ * \todo code review might be removed since not used in codebase
  */
 
 void CqMatrix::Normalise()
 {
+	assert(m_aaElement[ 3 ][ 3 ] != 0);
 	for ( TqInt i = 0; i < 4; i++ )
 	{
 		for ( TqInt j = 0; j < 4; j++ )
@@ -519,7 +525,7 @@ CqMatrix &CqMatrix::operator*=( const CqMatrix &From )
 	                        + From.m_aaElement[ 3 ][ 2 ] * A.m_aaElement[ 2 ][ 3 ]
 	                        + From.m_aaElement[ 3 ][ 3 ] * A.m_aaElement[ 3 ][ 3 ];
 
-	m_fIdentity = TqFalse;
+	m_fIdentity = false;
 	return ( *this );
 }
 
@@ -605,7 +611,7 @@ CqMatrix& CqMatrix::PreMultiply( const CqMatrix &From )
 	                        + A.m_aaElement[ 3 ][ 2 ] * From.m_aaElement[ 2 ][ 3 ]
 	                        + A.m_aaElement[ 3 ][ 3 ] * From.m_aaElement[ 3 ][ 3 ];
 
-	m_fIdentity = TqFalse;
+	m_fIdentity = false;
 	return ( *this );
 }
 
@@ -855,7 +861,7 @@ CqMatrix &CqMatrix::operator+=( const CqMatrix &From )
 	m_aaElement[ 2 ][ 3 ] += From.m_aaElement[ 2 ][ 3 ];
 	m_aaElement[ 3 ][ 3 ] += From.m_aaElement[ 3 ][ 3 ];
 
-	m_fIdentity = TqFalse;
+	m_fIdentity = false;
 
 	return ( *this );
 }
@@ -899,7 +905,7 @@ CqMatrix &CqMatrix::operator-=( const CqMatrix &From )
 	m_aaElement[ 2 ][ 3 ] -= From.m_aaElement[ 2 ][ 3 ];
 	m_aaElement[ 3 ][ 3 ] -= From.m_aaElement[ 3 ][ 3 ];
 
-	m_fIdentity = TqFalse;
+	m_fIdentity = false;
 
 	return ( *this );
 }
@@ -958,7 +964,7 @@ CqMatrix &CqMatrix::operator=( TqFloat From[ 4 ][ 4 ] )
 	m_aaElement[ 2 ][ 3 ] = From[ 2 ][ 3 ];
 	m_aaElement[ 3 ][ 3 ] = From[ 3 ][ 3 ];
 
-	m_fIdentity = TqFalse;
+	m_fIdentity = false;
 
 	return ( *this );
 }
@@ -987,7 +993,7 @@ CqMatrix &CqMatrix::operator=( TqFloat From[ 16 ] )
 	m_aaElement[ 3 ][ 2 ] = From[ 14 ];
 	m_aaElement[ 3 ][ 3 ] = From[ 15 ];
 
-	m_fIdentity = TqFalse;
+	m_fIdentity = false;
 
 	return ( *this );
 }
@@ -1010,7 +1016,7 @@ CqMatrix CqMatrix::Inverse() const
 	else
 	{
 		b.Identity();
-		b.m_fIdentity = TqFalse;
+		b.m_fIdentity = false;
 
 		TqInt i;
 		TqInt j;
@@ -1135,7 +1141,7 @@ CqMatrix CqMatrix::Transpose() const
 		Temp.m_aaElement[ 3 ][ 2 ] = m_aaElement[ 2 ][ 3 ];
 		Temp.m_aaElement[ 3 ][ 3 ] = m_aaElement[ 3 ][ 3 ];
 
-		Temp.m_fIdentity = TqFalse;
+		Temp.m_fIdentity = false;
 	}
 
 	return ( Temp );
@@ -1208,78 +1214,28 @@ TqFloat CqMatrix::Determinant() const
  * \return The new state of Stream.
  */
 
-std::ostream &operator<<( std::ostream &Stream, const CqMatrix &Matrix )
+std::ostream &operator<<( std::ostream &outStream, const CqMatrix &matrix )
 {
-	if ( !Matrix.fIdentity() )
+	if ( !matrix.fIdentity() )
 	{
-		Stream << "|" << Matrix.m_aaElement[ 0 ][ 0 ] <<
-		"," << Matrix.m_aaElement[ 0 ][ 1 ] <<
-		"," << Matrix.m_aaElement[ 0 ][ 2 ] <<
-		"," << Matrix.m_aaElement[ 0 ][ 3 ] << "|" << std::endl <<
-		"|" << Matrix.m_aaElement[ 1 ][ 0 ] <<
-		"," << Matrix.m_aaElement[ 1 ][ 1 ] <<
-		"," << Matrix.m_aaElement[ 1 ][ 2 ] <<
-		"," << Matrix.m_aaElement[ 1 ][ 3 ] << "|" << std::endl <<
-		"|" << Matrix.m_aaElement[ 2 ][ 0 ] <<
-		"," << Matrix.m_aaElement[ 2 ][ 1 ] <<
-		"," << Matrix.m_aaElement[ 2 ][ 2 ] <<
-		"," << Matrix.m_aaElement[ 2 ][ 3 ] << "|" << std::endl <<
-		"|" << Matrix.m_aaElement[ 3 ][ 0 ] <<
-		"," << Matrix.m_aaElement[ 3 ][ 1 ] <<
-		"," << Matrix.m_aaElement[ 3 ][ 2 ] <<
-		"," << Matrix.m_aaElement[ 3 ][ 3 ] << "|" << std::ends;
+		for(TqInt i = 0; i < 4; ++i)
+		{
+			outStream << "[";
+			for(TqInt j = 0; j < 3; ++j)
+				outStream << matrix.m_aaElement[i][j] << ",";
+			outStream << matrix.m_aaElement[i][3] << "]\n";
+		}
 	}
 	else
 	{
-		Stream <<
-		"|" << 1.0f << "," << 0.0f << "," << 0.0f << "," << 0.0f << "|" << std::endl <<
-		"|" << 0.0f << "," << 1.0f << "," << 0.0f << "," << 0.0f << "|" << std::endl <<
-		"|" << 0.0f << "," << 0.0f << "," << 1.0f << "," << 0.0f << "|" << std::endl <<
-		"|" << 0.0f << "," << 0.0f << "," << 0.0f << "," << 1.0f << "|" << std::ends;
+		outStream <<
+		"[" << 1.0f << "," << 0.0f << "," << 0.0f << "," << 0.0f << "]\n" <<
+		"[" << 0.0f << "," << 1.0f << "," << 0.0f << "," << 0.0f << "]\n" <<
+		"[" << 0.0f << "," << 0.0f << "," << 1.0f << "," << 0.0f << "]\n" <<
+		"[" << 0.0f << "," << 0.0f << "," << 0.0f << "," << 1.0f << "]\n";
 	}
 
-	return ( Stream );
-}
-
-
-//----------------------------------------------------------------------
-/** Outputs a matrix to an output stream.
- * \param Stream Stream to output the matrix to.
- * \param Matrix The matrix to output.
- * \return The new state of Stream.
- */
-
-std::ostream &operator<<( std::ostream &Stream, CqMatrix &Matrix )
-{
-	if ( !Matrix.fIdentity() )
-	{
-		Stream << "|" << Matrix.m_aaElement[ 0 ][ 0 ] <<
-		"," << Matrix.m_aaElement[ 0 ][ 1 ] <<
-		"," << Matrix.m_aaElement[ 0 ][ 2 ] <<
-		"," << Matrix.m_aaElement[ 0 ][ 3 ] << "|" << std::endl <<
-		"|" << Matrix.m_aaElement[ 1 ][ 0 ] <<
-		"," << Matrix.m_aaElement[ 1 ][ 1 ] <<
-		"," << Matrix.m_aaElement[ 1 ][ 2 ] <<
-		"," << Matrix.m_aaElement[ 1 ][ 3 ] << "|" << std::endl <<
-		"|" << Matrix.m_aaElement[ 2 ][ 0 ] <<
-		"," << Matrix.m_aaElement[ 2 ][ 1 ] <<
-		"," << Matrix.m_aaElement[ 2 ][ 2 ] <<
-		"," << Matrix.m_aaElement[ 2 ][ 3 ] << "|" << std::endl <<
-		"|" << Matrix.m_aaElement[ 3 ][ 0 ] <<
-		"," << Matrix.m_aaElement[ 3 ][ 1 ] <<
-		"," << Matrix.m_aaElement[ 3 ][ 2 ] <<
-		"," << Matrix.m_aaElement[ 3 ][ 3 ] << "|" << std::ends;
-	}
-	else
-	{
-		Stream <<
-		"|" << 1.0f << "," << 0.0f << "," << 0.0f << "," << 0.0f << "|" << std::endl <<
-		"|" << 0.0f << "," << 1.0f << "," << 0.0f << "," << 0.0f << "|" << std::endl <<
-		"|" << 0.0f << "," << 0.0f << "," << 1.0f << "," << 0.0f << "|" << std::endl <<
-		"|" << 0.0f << "," << 0.0f << "," << 0.0f << "," << 1.0f << "|" << std::ends;
-	}
-
-	return ( Stream );
+	return outStream;
 }
 
 
@@ -1325,6 +1281,14 @@ CqVector4D operator*( const CqVector4D &Vector, const CqMatrix& Matrix )
 	return ( Matrix.PreMultiply( Vector ) );
 }
 
+//---------------------------------------------------------------------
+/** Compare two matrices.
+ * \param A One Matrix to be compared.
+ * \param B Second Matrix to be compared with.
+ * \return Result if matrices are equal or not.
+ *
+ * \todo code review float comparison might need some kind of tollerance.
+ */
 
 bool  operator==(const CqMatrix& A, const CqMatrix& B)
 {
@@ -1358,6 +1322,43 @@ bool  operator!=(const CqMatrix& A, const CqMatrix& B)
 	return(!(A==B));
 }
 
+bool isClose(const CqMatrix& m1, const CqMatrix& m2, TqFloat tol)
+{
+	// if the matrices are at the same address, or both the identity, then
+	// they're equal.
+	if(&m1 == &m2 || (m1.fIdentity() && m2.fIdentity()))
+		return true;
+	// Check whether one matrix (but not the other) is marked as the identity.
+	// If so, create an identity matrix which is not marked as the identity to
+	// compare it with (in principle, the identity flag and the matrix elements
+	// may be out of sync I guess)
+	if(m1.fIdentity())
+	{
+		CqMatrix ident;
+		ident.SetfIdentity(false);
+		return isClose(m2, ident);
+	}
+	else if(m2.fIdentity())
+	{
+		CqMatrix ident;
+		ident.SetfIdentity(false);
+		return isClose(m1, ident);
+	}
+	TqFloat norm1 = 0;
+	TqFloat norm2 = 0;
+	TqFloat diffNorm = 0;
+	const TqFloat* m1Elts = m1.pElements();
+	const TqFloat* m2Elts = m2.pElements();
+	for(int i = 0; i < 16; ++i)
+	{
+		norm1 += m1Elts[i]*m1Elts[i];
+		norm2 += m2Elts[i]*m2Elts[i];
+		diffNorm += (m1Elts[i] - m2Elts[i])*(m1Elts[i] - m2Elts[i]);
+	}
+	TqFloat tol2 = tol*tol;
+	return diffNorm <= tol2*norm1 || diffNorm <= tol2*norm2;
+}
+
 //---------------------------------------------------------------------
 
-END_NAMESPACE( Aqsis )
+} // namespace Aqsis
