@@ -1,5 +1,5 @@
 // CqNoise1234
-// Copyright  2003-2005, Stefan Gustavson
+// Copyright (C) 2003-2005, Stefan Gustavson
 //
 // Contact: stegu@itn.liu.se
 //
@@ -38,7 +38,7 @@
 
 #include	"noise1234.h"
 
-START_NAMESPACE( Aqsis )
+namespace Aqsis {
 
 // This is the new and improved, C(2) continuous interpolant
 #define FADE(t) ( t * t * t * ( t * ( t * 6 - 15 ) + 10 ) )
@@ -551,4 +551,4 @@ TqFloat CqNoise1234::pnoise( TqFloat x, TqFloat y, TqFloat z, TqFloat w,
 
 //-----------------------------------------------------------------------
 
-END_NAMESPACE( Aqsis )
+} // namespace Aqsis
