@@ -1,5 +1,5 @@
 // Aqsis
-// Copyright  1997 - 2001, Paul C. Gregory
+// Copyright (C) 1997 - 2001, Paul C. Gregory
 //
 // Contact: pgregory@aqsis.org
 //
@@ -28,12 +28,9 @@
 #include <iostream>
 #include <logging.h>
 #include <logging_streambufs.h>
-#include "sstring.h"
 
 #include <tiffio.h>
 
-using namespace Aqsis;
-
 #include <iomanip>
 #include <ios>
 #include <iostream>
@@ -44,9 +41,12 @@ using namespace Aqsis;
 #include <algorithm>
 #include <float.h>
 #include <time.h>
+#include <cstring>
 
 #include "ndspy.h"
+#include "aqsismath.h"
 
+using namespace Aqsis;
 
 #define	ZFILE_HEADER		"Aqsis ZFile" VERSION_STR
 #define	SHADOWMAP_HEADER	"Shadow"
@@ -70,15 +70,12 @@ extern "C"
 }
 #endif
 
-#define INT_MULT(a,b,t) ( (t) = (a) * (b) + 0x80, ( ( ( (t)>>8 ) + (t) )>>8 ) )
-#define INT_PRELERP(p, q, a, t) ( (p) + (q) - INT_MULT( a, p, t) )
-
-START_NAMESPACE( Aqsis )
+namespace Aqsis {
 
 
 static char datetime[21];
 static time_t start;
-static char* description = NULL;
+static CqString description;
 
 //----------------------------------------------------------------------
 /** SaveAsShadowMap() Save as a tiff an shadowmap
@@ -116,7 +113,7 @@ void SaveAsShadowMap(const std::string& filename, SqDisplayInstance* image, char
 			// Set common tags
 			TIFFCreateDirectory( pshadow );
 
-			sprintf( version, "%s %s (%s %s)", STRNAME, VERSION_STR, __DATE__, __TIME__);
+			sprintf( version, "%s %s (%s %s)", STRNAME_PRINT, VERSION_STR, __DATE__, __TIME__);
 
 			TIFFSetField( pshadow, TIFFTAG_SOFTWARE, ( char* ) version );
 			TIFFSetField( pshadow, TIFFTAG_PIXAR_MATRIX_WORLDTOCAMERA, image->m_matWorldToCamera );
@@ -124,8 +121,8 @@ void SaveAsShadowMap(const std::string& filename, SqDisplayInstance* image, char
 			TIFFSetField( pshadow, TIFFTAG_PIXAR_TEXTUREFORMAT, SHADOWMAP_HEADER );
 			TIFFSetField( pshadow, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_MINISBLACK );
 
-			if (image->m_hostname)
-				TIFFSetField( pshadow, TIFFTAG_HOSTCOMPUTER, image->m_hostname );
+			if (!image->m_hostname.empty())
+				TIFFSetField( pshadow, TIFFTAG_HOSTCOMPUTER, image->m_hostname.c_str() );
 			TIFFSetField( pshadow, TIFFTAG_IMAGEDESCRIPTION, mydescription);
 			// Write the floating point image to the directory.
 			TIFFSetField( pshadow, TIFFTAG_IMAGEWIDTH, image->m_width );
@@ -217,15 +214,15 @@ void WriteTIFF(const std::string& filename, SqDisplayInstance* image)
 	sprintf(datetime, "%04d:%02d:%02d %02d:%02d:%02d", year, ct->tm_mon + 1,
 	        ct->tm_mday, ct->tm_hour, ct->tm_min, ct->tm_sec);
 
-	if (description == NULL)
+	if (description.empty())
 	{
 		double nSecs = difftime(long_time, start);
-		sprintf(mydescription,"%d secs", static_cast<TqInt>(nSecs));
+		sprintf(mydescription,"Aqsis Renderer, %d secs rendertime", static_cast<TqInt>(nSecs));
 		start = long_time;
 	}
 	else
 	{
-		strcpy(mydescription, description);
+		strcpy(mydescription, description.c_str());
 	}
 
 
@@ -275,7 +272,7 @@ void WriteTIFF(const std::string& filename, SqDisplayInstance* image)
 
 		short ExtraSamplesTypes[ 1 ] = {EXTRASAMPLE_ASSOCALPHA};
 
-		sprintf( version, "%s %s (%s %s)", STRNAME, VERSION_STR, __DATE__, __TIME__);
+		sprintf( version, "%s %s (%s %s)", STRNAME_PRINT, VERSION_STR, __DATE__, __TIME__);
 		bool use_logluv = false;
 
 		TIFFSetField( pOut, TIFFTAG_SOFTWARE, ( char* ) version );
@@ -289,8 +286,8 @@ void WriteTIFF(const std::string& filename, SqDisplayInstance* image)
 		TIFFSetField( pOut, TIFFTAG_ORIENTATION, ORIENTATION_TOPLEFT );
 		TIFFSetField( pOut, TIFFTAG_SAMPLESPERPIXEL, image->m_iFormatCount );
 		TIFFSetField( pOut, TIFFTAG_DATETIME, datetime);
-		if (image->m_hostname)
-			TIFFSetField( pOut, TIFFTAG_HOSTCOMPUTER, image->m_hostname );
+		if (!image->m_hostname.empty())
+			TIFFSetField( pOut, TIFFTAG_HOSTCOMPUTER, image->m_hostname.c_str() );
 		TIFFSetField( pOut, TIFFTAG_IMAGEDESCRIPTION, mydescription);
 
 
@@ -347,7 +344,7 @@ void WriteTIFF(const std::string& filename, SqDisplayInstance* image)
 			if ( use_logluv )
 			{
 				/* use SGI LogLuv compression */
-				TIFFSetField( pOut, TIFFTAG_SAMPLEFORMAT, SAMPLEFORMAT_INT );
+				TIFFSetField( pOut, TIFFTAG_SAMPLEFORMAT, SAMPLEFORMAT_UINT );
 				TIFFSetField( pOut, TIFFTAG_BITSPERSAMPLE, 16 );
 				TIFFSetField( pOut, TIFFTAG_COMPRESSION, COMPRESSION_SGILOG );
 				TIFFSetField( pOut, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_LOGLUV );
@@ -363,7 +360,7 @@ void WriteTIFF(const std::string& filename, SqDisplayInstance* image)
 			}
 			if (image->m_format == PkDspyUnsigned16)
 			{
-				TIFFSetField( pOut, TIFFTAG_SAMPLEFORMAT, SAMPLEFORMAT_INT );
+				TIFFSetField( pOut, TIFFTAG_SAMPLEFORMAT, SAMPLEFORMAT_UINT );
 				TIFFSetField( pOut, TIFFTAG_BITSPERSAMPLE, 16 );
 			}
 
@@ -396,30 +393,34 @@ void WriteTIFF(const std::string& filename, SqDisplayInstance* image)
 void CompositeAlpha(TqInt r, TqInt g, TqInt b, TqUchar &R, TqUchar &G, TqUchar &B, 
 		    TqUchar alpha )
 { 
+#	define INT_MULT(a,b,t) ( (t) = (a) * (b) + 0x80, ( ( ( (t)>>8 ) + (t) )>>8 ) )
+#	define INT_PRELERP(p, q, a, t) ( (p) + (q) - INT_MULT( a, p, t) )
 	TqInt t;
 	// C = INT_PRELERP( A, B, b, t )
 	TqInt R1 = static_cast<TqInt>(INT_PRELERP( R, r, alpha, t ));
 	TqInt G1 = static_cast<TqInt>(INT_PRELERP( G, g, alpha, t ));
 	TqInt B1 = static_cast<TqInt>(INT_PRELERP( B, b, alpha, t ));
-	R = CLAMP( R1, 0, 255 );
-	G = CLAMP( G1, 0, 255 );
-	B = CLAMP( B1, 0, 255 );
+	R = clamp<TqUchar>(R1, 0, 255);
+	G = clamp<TqUchar>(G1, 0, 255);
+	B = clamp<TqUchar>(B1, 0, 255);
+#	undef INT_MULT
+#	undef INT_PRELERP
 }
 
 
-END_NAMESPACE( Aqsis )
+} // namespace Aqsis
 
 
-PtDspyError DspyImageOpen(PtDspyImageHandle * image,
-                          const char *drivername,
-                          const char *filename,
-                          int width,
-                          int height,
-                          int paramCount,
-                          const UserParameter *parameters,
-                          int iFormatCount,
-                          PtDspyDevFormat *format,
-                          PtFlagStuff *flagstuff)
+extern "C" PtDspyError DspyImageOpen(PtDspyImageHandle * image,
+                                     const char *drivername,
+                                     const char *filename,
+                                     int width,
+                                     int height,
+                                     int paramCount,
+                                     const UserParameter *parameters,
+                                     int iFormatCount,
+                                     PtDspyDevFormat *format,
+                                     PtFlagStuff *flagstuff)
 {
 	SqDisplayInstance* pImage;
 
@@ -437,7 +438,7 @@ PtDspyError DspyImageOpen(PtDspyImageHandle * image,
 			pImage->m_imageType = Type_File;
 #ifndef	AQSIS_NO_FLTK
 
-		else if(strcmp(drivername, "framebuffer")==0)
+		else if((strcmp(drivername, "framebuffer")==0) || (strcmp(drivername, "legacyframebuffer")==0))
 			pImage->m_imageType = Type_Framebuffer;
 #endif // AQSIS_NO_FLTK
 
@@ -457,8 +458,7 @@ PtDspyError DspyImageOpen(PtDspyImageHandle * image,
 
 		*image = pImage;
 
-		pImage->m_filename = new char[strlen(filename)+1];
-		strcpy(pImage->m_filename, filename);
+		pImage->m_filename = filename;
 
 		// Scan the formats table to see what the widest channel format specified is.
 		TqUint widestFormat = PkDspySigned8;
@@ -474,6 +474,12 @@ PtDspyError DspyImageOpen(PtDspyImageHandle * image,
 		else if(widestFormat == PkDspySigned32)
 			widestFormat = PkDspyUnsigned32;
 
+		int scanorder;
+		if( DspyFindIntInParamList("scanlineorder", &scanorder, paramCount, parameters ) == PkDspyErrorNone )
+		{
+			flagstuff->flags = PkDspyFlagsWantsScanLineOrder;
+		}
+
 		// If we are recieving "rgba" data, ensure that it is in the correct order.
 		if(pImage->m_imageType == Type_File || pImage->m_imageType == Type_Framebuffer )
 		{
@@ -484,12 +490,19 @@ PtDspyError DspyImageOpen(PtDspyImageHandle * image,
 			        {"b", widestFormat},
 			        {"a", widestFormat},
 			    };
-			PtDspyError err = DspyReorderFormatting(iFormatCount, format, MIN(iFormatCount,4), outFormat);
+			PtDspyError err = DspyReorderFormatting(iFormatCount, format, min(iFormatCount,4), outFormat);
 			if( err != PkDspyErrorNone )
 			{
 				return(err);
 			}
 		}
+		else if(pImage->m_imageType == Type_ZFramebuffer)
+		{
+			// For the zframebuffer, ensure that we're actually receiving a
+			// single channel of z data in floating point format.
+			if(iFormatCount != 1 || strcmp(format->name, "z") != 0 || format->type != PkDspyFloat32)
+				return PkDspyErrorBadParams;
+		}
 
 		// Create and initialise a byte array if rendering 8bit image, or we are in framebuffer mode
 		if(pImage->m_imageType == Type_Framebuffer)
@@ -497,7 +510,7 @@ PtDspyError DspyImageOpen(PtDspyImageHandle * image,
 #ifndef	AQSIS_NO_FLTK
 			// Allocate the buffer, even if the formatcount <3, always allocated 3, as that is what's needed for the
 			// display.
-			pImage->m_data = new TqUchar[ pImage->m_width * pImage->m_height * pImage->m_iFormatCount ];
+			pImage->m_data = malloc( pImage->m_width * pImage->m_height * pImage->m_iFormatCount * sizeof(TqUchar));
 			pImage->m_entrySize = pImage->m_iFormatCount * sizeof(TqChar);
 
 			// Initialise the display to a checkerboard to show alpha
@@ -528,7 +541,7 @@ PtDspyError DspyImageOpen(PtDspyImageHandle * image,
 			pImage->m_theWindow = new Fl_Window(pImage->m_width, pImage->m_height);
 			pImage->m_uiImageWidget = new Fl_FrameBuffer_Widget(0,0, pImage->m_width, pImage->m_height, pImage->m_iFormatCount, reinterpret_cast<TqUchar*>(pImage->m_data));
 			pImage->m_theWindow->resizable(pImage->m_uiImageWidget);
-			pImage->m_theWindow->label(pImage->m_filename);
+			pImage->m_theWindow->label(pImage->m_filename.c_str());
 			pImage->m_theWindow->end();
 			Fl::visual(FL_RGB);
 			pImage->m_theWindow->show();
@@ -540,23 +553,23 @@ PtDspyError DspyImageOpen(PtDspyImageHandle * image,
 			// Determine the appropriate format to save into.
 			if(widestFormat == PkDspyUnsigned8)
 			{
-				pImage->m_data = malloc( pImage->m_width * pImage->m_height * pImage->m_iFormatCount * sizeof(TqUchar));
-				pImage->m_entrySize = pImage->m_iFormatCount * sizeof(TqUchar);
+				pImage->m_data = malloc( pImage->m_width * pImage->m_height * pImage->m_iFormatCount * sizeof(PtDspyUnsigned8));
+				pImage->m_entrySize = pImage->m_iFormatCount * sizeof(PtDspyUnsigned8);
 			}
 			else if(widestFormat == PkDspyUnsigned16)
 			{
-				pImage->m_data = malloc( pImage->m_width * pImage->m_height * pImage->m_iFormatCount * sizeof(TqUshort));
-				pImage->m_entrySize = pImage->m_iFormatCount * sizeof(TqUshort);
+				pImage->m_data = malloc( pImage->m_width * pImage->m_height * pImage->m_iFormatCount * sizeof(PtDspyUnsigned16));
+				pImage->m_entrySize = pImage->m_iFormatCount * sizeof(PtDspyUnsigned16);
 			}
 			else if(widestFormat == PkDspyUnsigned32)
 			{
-				pImage->m_data = malloc( pImage->m_width * pImage->m_height * pImage->m_iFormatCount * sizeof(TqUlong));
-				pImage->m_entrySize = pImage->m_iFormatCount * sizeof(TqUlong);
+				pImage->m_data = malloc( pImage->m_width * pImage->m_height * pImage->m_iFormatCount * sizeof(PtDspyUnsigned32));
+				pImage->m_entrySize = pImage->m_iFormatCount * sizeof(PtDspyUnsigned32);
 			}
 			else if(widestFormat == PkDspyFloat32)
 			{
-				pImage->m_data = malloc( pImage->m_width * pImage->m_height * pImage->m_iFormatCount * sizeof(TqFloat));
-				pImage->m_entrySize = pImage->m_iFormatCount * sizeof(TqFloat);
+				pImage->m_data = malloc( pImage->m_width * pImage->m_height * pImage->m_iFormatCount * sizeof(PtDspyFloat32));
+				pImage->m_entrySize = pImage->m_iFormatCount * sizeof(PtDspyFloat32);
 			}
 		}
 		pImage->m_lineLength = pImage->m_entrySize * pImage->m_width;
@@ -571,7 +584,7 @@ PtDspyError DspyImageOpen(PtDspyImageHandle * image,
 			pImage->m_theWindow = new Fl_Window(pImage->m_width, pImage->m_height);
 			pImage->m_uiImageWidget = new Fl_FrameBuffer_Widget(0,0, pImage->m_width, pImage->m_height, 3, reinterpret_cast<TqUchar*>(pImage->m_zfbdata));
 			pImage->m_theWindow->resizable(pImage->m_uiImageWidget);
-			pImage->m_theWindow->label(pImage->m_filename);
+			pImage->m_theWindow->label(pImage->m_filename.c_str());
 			pImage->m_theWindow->end();
 			Fl::visual(FL_RGB);
 			pImage->m_theWindow->show();
@@ -585,7 +598,7 @@ PtDspyError DspyImageOpen(PtDspyImageHandle * image,
 		char* hostname;
 		if( DspyFindStringInParamList("HostComputer", &hostname, paramCount, parameters ) == PkDspyErrorNone )
 		{
-			pImage->m_hostname= strdup(hostname);
+			pImage->m_hostname = hostname;
 		}
 		if( DspyFindStringInParamList("compression", &compression, paramCount, parameters ) == PkDspyErrorNone )
 		{
@@ -635,7 +648,7 @@ PtDspyError DspyImageOpen(PtDspyImageHandle * image,
 		{
 			// Do something about it; the user will want to add its copyright notice.
 			if (ydesc && *ydesc)
-				description = strdup(ydesc);
+				description = ydesc;
 		}
 	}
 	else
@@ -645,29 +658,37 @@ PtDspyError DspyImageOpen(PtDspyImageHandle * image,
 }
 
 
-PtDspyError DspyImageData(PtDspyImageHandle image,
-                          int xmin,
-                          int xmaxplus1,
-                          int ymin,
-                          int ymaxplus1,
-                          int entrysize,
-                          const TqUchar *data)
+extern "C" PtDspyError DspyImageData(PtDspyImageHandle image,
+                                     int xmin,
+                                     int xmaxplus1,
+                                     int ymin,
+                                     int ymaxplus1,
+                                     int entrysize,
+                                     const TqUchar *data)
 {
 	SqDisplayInstance* pImage;
 	pImage = reinterpret_cast<SqDisplayInstance*>(image);
 
-	TqInt xmin__ = MAX((xmin-pImage->m_origin[0]), 0);
-	TqInt ymin__ = MAX((ymin-pImage->m_origin[1]), 0);
-	TqInt xmaxplus1__ = MIN((xmaxplus1-pImage->m_origin[0]), pImage->m_width);
-	TqInt ymaxplus1__ = MIN((ymaxplus1-pImage->m_origin[1]), pImage->m_height);
+	// If the image is not cropped, then the origin shouldn't be used.
+	if(pImage->m_OriginalSize[0] == pImage->m_width && pImage->m_OriginalSize[1] == pImage->m_height)
+	{
+		pImage->m_origin[0] = 0;
+		pImage->m_origin[1] = 0;
+	}
+		
+
+	TqInt xmin__ = max((xmin-pImage->m_origin[0]), 0);
+	TqInt ymin__ = max((ymin-pImage->m_origin[1]), 0);
+	TqInt xmaxplus1__ = min((xmaxplus1-pImage->m_origin[0]), pImage->m_width);
+	TqInt ymaxplus1__ = min((ymaxplus1-pImage->m_origin[1]), pImage->m_height);
 	TqInt bucketlinelen = entrysize * (xmaxplus1 - xmin);
 	TqInt copylinelen = entrysize * (xmaxplus1__ - xmin__);
 
 	pImage->m_pixelsReceived += (xmaxplus1__-xmin__)*(ymaxplus1__-ymin__);
 
 	// Calculate where in the bucket we are starting from if the window is cropped.
-	TqInt row = MAX(pImage->m_origin[1] - ymin, 0);
-	TqInt col = MAX(pImage->m_origin[0] - xmin, 0);
+	TqInt row = max(pImage->m_origin[1] - ymin, 0);
+	TqInt col = max(pImage->m_origin[0] - xmin, 0);
 	const TqUchar* pdatarow = data;
 	pdatarow += (row * bucketlinelen) + (col * entrysize);
 
@@ -821,7 +842,7 @@ PtDspyError DspyImageData(PtDspyImageHandle image,
 		Fl::check();
 		TqFloat percent = pImage->m_pixelsReceived / (TqFloat) (pImage->m_width * pImage->m_height);
 		percent *= 100.0f;
-		percent = CLAMP(percent, 0.0f, 100.0f);
+		percent = clamp(percent, 0.0f, 100.0f);
 		std::stringstream strTitle;
 		if (percent < 99.9f)
 			strTitle << pImage->m_filename << ": " << std::fixed << std::setprecision(1) << std::setw(5) << percent << "% complete" << std::ends;
@@ -835,7 +856,7 @@ PtDspyError DspyImageData(PtDspyImageHandle image,
 }
 
 
-PtDspyError DspyImageClose(PtDspyImageHandle image)
+extern "C" PtDspyError DspyImageClose(PtDspyImageHandle image)
 {
 	SqDisplayInstance* pImage;
 	pImage = reinterpret_cast<SqDisplayInstance*>(image);
@@ -849,16 +870,9 @@ PtDspyError DspyImageClose(PtDspyImageHandle image)
 	// Delete the image structure.
 	if (pImage->m_data)
 		free(pImage->m_data);
-	if (pImage->m_hostname)
-		//free(pImage->m_hostname);
 	if(pImage->m_imageType == Type_ZFramebuffer)
 		free(pImage->m_zfbdata);
-	if (description)
-	{
-		free(description);
-		description = NULL;
-	}
-	delete[](pImage->m_filename);
+	description = "";
 	delete(pImage);
 
 
@@ -866,7 +880,7 @@ PtDspyError DspyImageClose(PtDspyImageHandle image)
 }
 
 
-PtDspyError DspyImageDelayClose(PtDspyImageHandle image)
+extern "C" PtDspyError DspyImageDelayClose(PtDspyImageHandle image)
 {
 	SqDisplayInstance* pImage;
 	pImage = reinterpret_cast<SqDisplayInstance*>(image);
@@ -893,8 +907,8 @@ PtDspyError DspyImageDelayClose(PtDspyImageHandle image)
 					        [i] >= FLT_MAX )
 						continue;
 
-					mindepth = MIN( mindepth, reinterpret_cast<const TqFloat*>(pImage->m_data)[ i ] );
-					maxdepth = MAX( maxdepth, reinterpret_cast<const TqFloat*>(pImage->m_data)[ i ] );
+					mindepth = min( mindepth, reinterpret_cast<const TqFloat*>(pImage->m_data)[ i ] );
+					maxdepth = max( maxdepth, reinterpret_cast<const TqFloat*>(pImage->m_data)[ i ] );
 
 					totaldepth += reinterpret_cast<const TqFloat*>(pImage->m_data)[ i ];
 					samples++;
@@ -950,10 +964,10 @@ PtDspyError DspyImageDelayClose(PtDspyImageHandle image)
 }
 
 
-PtDspyError DspyImageQuery(PtDspyImageHandle image,
-                           PtDspyQueryType type,
-                           int size,
-                           void *data)
+extern "C" PtDspyError DspyImageQuery(PtDspyImageHandle image,
+                                      PtDspyQueryType type,
+                                      size_t size,
+                                      void *data)
 {
 	SqDisplayInstance* pImage;
 	pImage = reinterpret_cast<SqDisplayInstance*>(image);
