@@ -1,5 +1,5 @@
 // Aqsis
-// Copyright  1997 - 2001, Paul C. Gregory
+// Copyright (C) 1997 - 2001, Paul C. Gregory
 //
 // Contact: pgregory@aqsis.org
 //
@@ -26,7 +26,7 @@
 #include	"aqsis.h"
 #include	"raytrace.h"
 
-START_NAMESPACE( Aqsis )
+namespace Aqsis {
 
 
 /// Required function that implements Class Factory design pattern for Raytrace libraries
@@ -48,4 +48,4 @@ void CqRaytrace::Finalise()
 
 //---------------------------------------------------------------------
 
-END_NAMESPACE( Aqsis )
+} // namespace Aqsis
