@@ -4,8 +4,8 @@
  *	@author	Paul Gregory
  *	@brief	Implement the basic raytracer subsystem interface implementation.
  *
- *	Last change by:		$Author: pgregory $
- *	Last change date:	$Date: 2006-02-10 15:15:33 +0000 (Fri, 10 Feb 2006) $
+ *	Last change by:		$Author: c42f $
+ *	Last change date:	$Date: 2008-04-14 01:23:53 +0000 (Mon, 14 Apr 2008) $
  */
 //------------------------------------------------------------------------------
 
@@ -16,7 +16,7 @@
 #include	"aqsis.h"
 #include	"iraytrace.h"
 
-START_NAMESPACE( Aqsis )
+namespace Aqsis {
 
 
 struct CqRaytrace : public IqRaytrace
@@ -38,6 +38,6 @@ struct CqRaytrace : public IqRaytrace
 
 //-----------------------------------------------------------------------
 
-END_NAMESPACE( Aqsis )
+} // namespace Aqsis
 
 #endif	//	___raytrace_Loaded___
