@@ -1,5 +1,5 @@
 // Aqsis
-// Copyright  1997 - 2001, Paul C. Gregory
+// Copyright (C) 1997 - 2001, Paul C. Gregory
 //
 // Contact: pgregory@aqsis.org
 //
@@ -32,7 +32,7 @@
 #include	"imagebuffer.h"
 #include	"lights.h"
 
-START_NAMESPACE( Aqsis )
+namespace Aqsis {
 
 
 std::list<CqAttributes*>	Attribute_stack;
@@ -118,8 +118,9 @@ CqAttributes::CqAttributes()
 	ADD_SYSTEM_ATTR( Opacity, CqColor, CqColor, type_color, CqColor( 1.0f, 1.0f, 1.0f ) );	// the current opacity attribute.
 	ADD_SYSTEM_ATTR8( TextureCoordinates, TqFloat, TqFloat, type_float, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 1.0f, 1.0f, 1.0f );	// an array of 2D vectors representing the coordinate space.
 	ADD_SYSTEM_ATTR( ShadingRate, TqFloat, TqFloat, type_float, 1.0f );					// the current effective shading rate.
-	ADD_SYSTEM_ATTR( ShadingRateSqrt, TqFloat, TqFloat, type_float, 1.0f );					// the current effective sqrt(shading rate).
-	ADD_SYSTEM_ATTR( ShadingInterpolation, TqFloat, TqInt, type_integer, ShadingConstant );	// the current shading interpolation mode.
+	ADD_SYSTEM_ATTR( ShadingInterpolation, TqFloat, TqInt, type_integer, ShadingInterp_Constant );	// the current shading interpolation mode.
+	ADD_SYSTEM_ATTR( GeometricFlatness, TqFloat, TqFloat, type_float, 1.0f );				// the current screen-space geometric flatness.
+	ADD_SYSTEM_ATTR( GeometricFocusFactor, TqFloat, TqFloat, type_float, 0.0f );		// multiplier for CoC area to multiply the shading rate for depth of field
 	ADD_SYSTEM_ATTR( Matte, TqInt, TqFloat, type_integer, 0 );				// the current state of the matte flag.
 	ADD_SYSTEM_ATTR2( Basis, CqMatrix, CqMatrix, type_matrix, RiBezierBasis, RiBezierBasis );	// the basis matrix for the u direction.
 	ADD_SYSTEM_ATTR2( BasisStep, TqInt, TqFloat, type_integer, 3, 3 );	// the steps to advance the evaluation window in the u direction.
@@ -494,6 +495,6 @@ IqLightsource*	CqAttributes::pLight( TqInt index )
 
 //---------------------------------------------------------------------
 
-END_NAMESPACE( Aqsis )
+} // namespace Aqsis
 
 
