@@ -1,5 +1,5 @@
 // Aqsis
-// Copyright © 2006, Paul C. Gregory
+// Copyright (C) 2006, Paul C. Gregory
 //
 // Contact: pgregory@aqsis.org
 //
@@ -18,7 +18,6 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-
 /** \file
     \brief Implements Blobby polygonizer
     \author Romain Behar (romainbehar@yahoo.com)
@@ -28,30 +27,34 @@
 *          K-3D
 *
 */
-#include <stdio.h>
+
+#include "blobby.h"
+
+#include <cstring>
 #include <math.h>
 #include <vector>
 #include <list>
 #include <limits>
 
-#include "aqsis.h"
+#include "file.h"
+#include "itexturemap_old.h"
+#include "marchingcubes.h"
+#include "matrix.h"
+#include "plugins.h"
 #include "ri.h"
 #include "vector4d.h"
-#include "matrix.h"
-#include "blobby.h"
-#include "itexturemap.h"
-#include "marchingcubes.h"
-#include "plugins.h"
 
-#ifdef WIN32
+#if _MSC_VER
 #pragma warning(disable:4786)	// hide stl warnings (VS6)
+#endif
+#if _WIN32
 #include <windows.h>
 #endif
 
-START_NAMESPACE( Aqsis )
+namespace Aqsis {
 
 #define ZCLAMP  1e-6
-#define OPTIMUM_GRID_SIZE 127
+#define OPTIMUM_GRID_SIZE 15
 
 typedef struct {
 	int BlobbyId;
@@ -161,7 +164,7 @@ class blobby_vm_assembler
 				m_strings(strings),
 				m_instructions(Instructions),
 				m_bbox(BBox),
-				m_has_bounding_box(TqFalse)
+				m_has_bounding_box(false)
 		{
 
 
@@ -297,7 +300,7 @@ class blobby_vm_assembler
 			CqBound end_box( End.x() - r, End.y() - r, End.z() - r, End.x() + r, End.y() + r, End.z() + r );
 			end_box.Transform( transformation );
 
-			start_box.Encapsulate( end_box );
+			start_box.Encapsulate( &end_box );
 
 			encapsulate( start_box );
 		}
@@ -316,12 +319,12 @@ class blobby_vm_assembler
 		{
 			if(m_has_bounding_box)
 			{
-				m_bbox.Encapsulate(Bound);
+				m_bbox.Encapsulate(&Bound);
 				return;
 			}
 
 			m_bbox = Bound;
-			m_has_bounding_box = TqTrue;
+			m_has_bounding_box = true;
 		}
 
 
@@ -383,33 +386,21 @@ class blobby_vm_assembler
 						// Load the DBO plugin
 						if (!DBO_handle)
 						{
-							CqString dbo(m_strings[m_code[op.index+1]]);
-
-#if defined(AQSIS_SYSTEM_POSIX)
-							CqString plugin_path = DEFAULT_PLUGIN_PATH "/lib";
-							plugin_path += m_strings[m_code[op.index+1]];
-							dbo = plugin_path;
-							dbo += ".so";
-
-							if (access(dbo.c_str(), F_OK) != 0)
+							std::string dboName = std::string(m_strings[m_code[op.index+1]])
+										+ SHARED_LIBRARY_SUFFIX;
+							try
 							{
-								dbo = plugin_path;
-								dbo += ".dylib";
-							} 
-#elif defined(AQSIS_SYSTEM_WIN32)
-							char acPath[255];
-						
-							if ( GetModuleFileName( NULL, acPath, 256 ) != 0)
+								CqString fullName = findFileInPath(dboName,
+									*QGetRenderContext()->poptCurrent()
+										->GetStringOption("searchpath", "procedural") );
+								DBO_handle = DBO.SimpleDLOpen(&fullName);
+							}
+							catch(XqInvalidFile& e)
 							{
-								// guaranteed file name of at least one character after path
-								*( strrchr( acPath, '\\' ) + 1 ) = '\0';
+								Aqsis::log() << error
+									<< "Could not find dynamic blob object \""
+									<< dboName << "\"\n";
 							}
-
-							CqString plugin_path = acPath;
-							plugin_path.append( CqString("/" + dbo + ".dll") );
-							dbo = plugin_path;
-#endif
-							DBO_handle = DBO.SimpleDLOpen(&dbo);
 						}
 
 						// Attach each API DBO functions
@@ -554,7 +545,7 @@ class blobby_vm_assembler
 		char  ** m_strings;
 		CqBlobby::instructions_t& m_instructions;
 		CqBound& m_bbox;
-		TqBool m_has_bounding_box;
+		bool m_has_bounding_box;
 };
 
 //---------------------------------------------------------------------
@@ -562,7 +553,7 @@ class blobby_vm_assembler
  */
 CqBlobby::CqBlobby(TqInt nleaf, TqInt ncode, TqInt* code, TqInt nfloats, TqFloat* floats, TqInt nstrings, char** strings) : m_nleaf(nleaf), m_ncode(ncode), m_code(code), m_nfloats(nfloats), m_floats(floats), m_nstrings(nstrings), m_strings(strings)
 {
-	blobby_vm_assembler(nleaf, ncode, code, nfloats, floats, nstrings, strings, instructions, bbox);
+	blobby_vm_assembler(nleaf, ncode, code, nfloats, floats, nstrings, strings, m_instructions, m_bbox);
 }
 
 //---------------------------------------------------------------------
@@ -607,16 +598,16 @@ TqFloat CqBlobby::implicit_value( const CqVector3D& Point, TqInt n, std::vector
 
 	register unsigned long pc;
 
-	for(pc = 0; pc < instructions.size(); )
+	for(pc = 0; pc < m_instructions.size(); )
 	{
-		switch(instructions[pc++].opcode)
+		switch(m_instructions[pc++].opcode)
 		{
 				case NEGATE:
 				case IDEMPOTENTATE:
 					break;
 				case CONSTANT:
 				{
-					result = instructions[pc++].value;
+					result = m_instructions[pc++].value;
 					sum += result;
 					splits[int_index++] = result;
 				}
@@ -624,7 +615,7 @@ TqFloat CqBlobby::implicit_value( const CqVector3D& Point, TqInt n, std::vector
 
 				case ELLIPSOID:
 				{
-					const TqFloat r2 = (instructions[pc++].get_matrix() * Point).Magnitude2();
+					const TqFloat r2 = (m_instructions[pc++].get_matrix() * Point).Magnitude2();
 					result = r2 <= 1 ? 1 - 3*r2 + 3*r2*r2 - r2*r2*r2 : 0;
 					sum += result;
 					splits[int_index++] = result;
@@ -633,11 +624,15 @@ TqFloat CqBlobby::implicit_value( const CqVector3D& Point, TqInt n, std::vector
 
 				case PLANE:
 				{
-					TqInt which = (TqInt) instructions[pc++].value;
-					TqInt n = (TqInt) instructions[pc++].value;
+					TqInt which = (TqInt) m_instructions[pc++].value;
+					TqInt n = (TqInt) m_instructions[pc++].value;
 
 					CqString depthname = m_strings[which];
-					IqTextureMap* pMap = QGetRenderContextI() ->GetShadowMap( depthname );
+					/** \todo Fix to use the new-style texture maps.  Using
+					 * GetOcclusionMap happens to access the old texture
+					 * sampling machinary through GetShadowMap()
+					 */
+					IqTextureMapOld* pMap = QGetRenderContextI() ->GetOcclusionMap( depthname );
 
 					TqFloat A, B, C, D;
 					std::valarray<TqFloat> fv;
@@ -698,7 +693,7 @@ TqFloat CqBlobby::implicit_value( const CqVector3D& Point, TqInt n, std::vector
 					TqInt count, e, f, g, h, i, j;
 
 					e = f = g = h = i = j = 0;
-					count = instructions[pc++].count;
+					count = m_instructions[pc++].count;
 
 					if (m_code[count] >= 7)
 					{
@@ -719,10 +714,10 @@ TqFloat CqBlobby::implicit_value( const CqVector3D& Point, TqInt n, std::vector
 
 
 					TqFloat point[3];
-					const CqMatrix transformation = instructions[pc++].get_matrix();
-					const CqVector3D mid = instructions[pc++].get_vector();
-					const CqVector3D mx = instructions[pc++].get_vector();
-					const CqVector3D mn = instructions[pc++].get_vector();
+					const CqMatrix transformation = m_instructions[pc++].get_matrix();
+					const CqVector3D mid = m_instructions[pc++].get_vector();
+					const CqVector3D mx = m_instructions[pc++].get_vector();
+					const CqVector3D mn = m_instructions[pc++].get_vector();
 					const CqBound bound(mn, mx);
 
 					TqState s;
@@ -747,10 +742,10 @@ TqFloat CqBlobby::implicit_value( const CqVector3D& Point, TqInt n, std::vector
 
 				case SEGMENT:
 				{
-					const CqMatrix m = instructions[pc++].get_matrix();
-					const CqVector3D start = instructions[pc++].get_vector();
-					const CqVector3D end = instructions[pc++].get_vector();
-					const TqFloat radius = instructions[pc++].value;
+					const CqMatrix m = m_instructions[pc++].get_matrix();
+					const CqVector3D start = m_instructions[pc++].get_vector();
+					const CqVector3D end = m_instructions[pc++].get_vector();
+					const TqFloat radius = m_instructions[pc++].value;
 
 					// Nearest segment point
 					const CqVector3D segment_point = nearest_segment_point(Point, start, end);
@@ -794,22 +789,22 @@ TqFloat CqBlobby::implicit_value( const CqVector3D& Point )
 	register TqFloat result;
 	register unsigned long pc;
 
-	for(pc = 0; pc < instructions.size(); )
+	for(pc = 0; pc < m_instructions.size(); )
 	{
-		switch(instructions[pc++].opcode)
+		switch(m_instructions[pc++].opcode)
 		{
 				case NEGATE:
 				case IDEMPOTENTATE:
 					break;
 				case CONSTANT:
 				{
-					stack.push(instructions[pc++].value);
+					stack.push(m_instructions[pc++].value);
 				}
 				break;
 
 				case ELLIPSOID:
 				{
-					const TqFloat r2 = (instructions[pc++].get_matrix() * Point).Magnitude2();
+					const TqFloat r2 = (m_instructions[pc++].get_matrix() * Point).Magnitude2();
 					result = r2 <= 1 ? 1 - 3*r2 + 3*r2*r2 - r2*r2*r2 : 0;
 
 					//Aqsis::log() << info << "Ellipsoid: result " << result << std::endl;
@@ -819,11 +814,15 @@ TqFloat CqBlobby::implicit_value( const CqVector3D& Point )
 
 				case PLANE:
 				{
-					TqInt which = (TqInt) instructions[pc++].value;
-					TqInt n = (TqInt) instructions[pc++].value;
+					TqInt which = (TqInt) m_instructions[pc++].value;
+					TqInt n = (TqInt) m_instructions[pc++].value;
 
 					CqString depthname = m_strings[which];
-					IqTextureMap* pMap = QGetRenderContextI() ->GetShadowMap( depthname );
+					/** \todo Fix to use the new-style texture maps.  Using
+					 * GetOcclusionMap happens to access the old texture
+					 * sampling machinary through GetShadowMap()
+					 */
+					IqTextureMapOld* pMap = QGetRenderContextI() ->GetOcclusionMap( depthname );
 
 					TqFloat A, B, C, D;
 					std::valarray<TqFloat> fv;
@@ -859,7 +858,7 @@ TqFloat CqBlobby::implicit_value( const CqVector3D& Point )
 					TqInt count, e, f, g, h, i, j;
 
 					e = f = g = h = i = j = 0;
-					count = instructions[pc++].count;
+					count = m_instructions[pc++].count;
 
 					if (m_code[count] >= 7)
 					{
@@ -879,10 +878,10 @@ TqFloat CqBlobby::implicit_value( const CqVector3D& Point )
 
 
 					TqFloat point[3];
-					const CqMatrix transformation = instructions[pc++].get_matrix();
-					const CqVector3D mid = instructions[pc++].get_vector();
-					const CqVector3D mx = instructions[pc++].get_vector();
-					const CqVector3D mn = instructions[pc++].get_vector();
+					const CqMatrix transformation = m_instructions[pc++].get_matrix();
+					const CqVector3D mid = m_instructions[pc++].get_vector();
+					const CqVector3D mx = m_instructions[pc++].get_vector();
+					const CqVector3D mn = m_instructions[pc++].get_vector();
 					const CqBound bound(mn, mx);
 
 					TqState s;
@@ -908,10 +907,10 @@ TqFloat CqBlobby::implicit_value( const CqVector3D& Point )
 
 				case SEGMENT:
 				{
-					const CqMatrix m = instructions[pc++].get_matrix();
-					const CqVector3D start = instructions[pc++].get_vector();
-					const CqVector3D end = instructions[pc++].get_vector();
-					const TqFloat radius = instructions[pc++].value;
+					const CqMatrix m = m_instructions[pc++].get_matrix();
+					const CqVector3D start = m_instructions[pc++].get_vector();
+					const CqVector3D end = m_instructions[pc++].get_vector();
+					const TqFloat radius = m_instructions[pc++].value;
 
 					// Nearest segment point
 					const CqVector3D segment_point = nearest_segment_point(Point, start, end);
@@ -957,7 +956,7 @@ TqFloat CqBlobby::implicit_value( const CqVector3D& Point )
 
 				case ADD:
 				{
-					const TqInt count = instructions[pc++].count;
+					const TqInt count = m_instructions[pc++].count;
 					result = 0.0;
 					for(TqInt i = 0; i != count; ++i)
 					{
@@ -970,7 +969,7 @@ TqFloat CqBlobby::implicit_value( const CqVector3D& Point )
 
 				case MULTIPLY:
 				{
-					const TqInt count = instructions[pc++].count;
+					const TqInt count = m_instructions[pc++].count;
 					result = stack.top();
 					stack.pop();
 					for(TqInt i = 1; i != count; ++i)
@@ -984,12 +983,12 @@ TqFloat CqBlobby::implicit_value( const CqVector3D& Point )
 
 				case MIN:
 				{
-					const TqInt count = instructions[pc++].count;
+					const TqInt count = m_instructions[pc++].count;
 					result = stack.top();
 					stack.pop();
 					for(TqInt i = 1; i != count; ++i)
 					{
-						result = MIN(result, stack.top());
+						result = min(result, stack.top());
 						stack.pop();
 					}
 					stack.push(result);
@@ -997,12 +996,12 @@ TqFloat CqBlobby::implicit_value( const CqVector3D& Point )
 				break;
 				case MAX:
 				{
-					const TqInt count = instructions[pc++].count;
+					const TqInt count = m_instructions[pc++].count;
 					result = stack.top();
 					stack.pop();
 					for(TqInt i = 1; i != count; ++i)
 					{
-						result = MAX(result, stack.top());
+						result = max(result, stack.top());
 						stack.pop();
 					}
 					stack.push(result);
@@ -1038,8 +1037,8 @@ TqInt CqBlobby::polygonize( TqInt PixelsWidth, TqInt PixelsHeight, TqInt& NPoint
 
 
 	// Get bounding-box center and sizes
-	const CqVector3D center = ( bbox.vecMax() + bbox.vecMin() ) / 2.0;
-	const CqVector3D length = ( bbox.vecMax() - bbox.vecMin() );
+	const CqVector3D center = ( m_bbox.vecMax() + m_bbox.vecMin() ) / 2.0;
+	const CqVector3D length = ( m_bbox.vecMax() - m_bbox.vecMin() );
 
 	// Calculate voxel sizes and polygonization resolution
 	const TqFloat x_voxel_size = length.x() /  PixelsWidth;
@@ -1077,7 +1076,7 @@ TqInt CqBlobby::polygonize( TqInt PixelsWidth, TqInt PixelsHeight, TqInt& NPoint
 				TqFloat x,y,z;
 
 				z = z_start + (TqFloat) k1 * (TqFloat) OPTIMUM_GRID_SIZE * z_voxel_size;
-				TqBool isrequired = TqFalse;
+				bool isrequired = false;
 
 				// Initialize Marching Cubes algorithm
 				MarchingCubes mc(OPTIMUM_GRID_SIZE+1, OPTIMUM_GRID_SIZE+1, OPTIMUM_GRID_SIZE+1);
@@ -1228,7 +1227,7 @@ TqInt CqBlobby::polygonize( TqInt PixelsWidth, TqInt PixelsHeight, TqInt& NPoint
 }
 
 
-END_NAMESPACE( Aqsis )
+} // namespace Aqsis
 //---------------------------------------------------------------------
 
 
