@@ -1,5 +1,5 @@
 // Aqsis
-// Copyright  1997 - 2001, Paul C. Gregory
+// Copyright (C) 1997 - 2001, Paul C. Gregory
 //
 // Contact: pgregory@aqsis.org
 //
@@ -29,14 +29,14 @@
 
 #include	"aqsis.h"
 
-START_NAMESPACE( Aqsis )
+namespace Aqsis {
 
-TqBool TestCondition(RtString condition, RtString where);
+bool TestCondition(RtString condition, RtString where);
 
 
 //-----------------------------------------------------------------------
 
-END_NAMESPACE( Aqsis )
+} // namespace Aqsis
 
 #endif // CONDITION_H_INCLUDED
 
