@@ -1,5 +1,5 @@
 // Aqsis
-// Copyright  1997 - 2001, Paul C. Gregory
+// Copyright (C) 1997 - 2001, Paul C. Gregory
 //
 // Contact: pgregory@aqsis.org
 //
@@ -40,7 +40,7 @@
 #include	"lights.h"
 #include	"csgtree.h"
 
-START_NAMESPACE( Aqsis )
+namespace Aqsis {
 
 class CqSurface;
 class CqDeformingSurface;
@@ -55,7 +55,8 @@ enum EqModeBlock
     Transform,
     Solid,
     Object,
-    Motion
+    Motion,	
+    Resource
 };
 
 //----------------------------------------------------------------------
@@ -80,6 +81,7 @@ class CqModeBlock : public boost::enable_shared_from_this<CqModeBlock>
 		virtual	boost::shared_ptr<CqModeBlock>	BeginSolidModeBlock( CqString& type );
 		virtual	boost::shared_ptr<CqModeBlock>	BeginObjectModeBlock();
 		virtual	boost::shared_ptr<CqModeBlock>	BeginMotionModeBlock( TqInt N, TqFloat times[] );
+		virtual	boost::shared_ptr<CqModeBlock>	BeginResourceModeBlock();
 
 #ifdef _DEBUG
 
@@ -145,7 +147,13 @@ class CqModeBlock : public boost::enable_shared_from_this<CqModeBlock>
 		{
 			logInvalidNesting();
 		}
-
+		/** Delete the main context, overridable per derived class.
+		 * \warning If called at this level it is an error, as only the appropriate context can delete itself.
+		 */
+		virtual	void	EndResourceModeBlock()
+		{
+			logInvalidNesting();
+		}
 		virtual IqOptionsPtr	poptCurrent() const
 		{
 			return( m_poptCurrent );
@@ -221,9 +229,9 @@ class CqModeBlock : public boost::enable_shared_from_this<CqModeBlock>
 		/** Is this a motion block, should be overridden per derived class.
 		 * \return boolean indicating whether this is a motion block.
 		 */
-		virtual	TqBool	fMotionBlock() const
+		virtual	bool	fMotionBlock() const
 		{
-			return ( TqFalse );
+			return ( false );
 		}
 
 		/** Get a pointer to the previuos context.
@@ -259,9 +267,9 @@ class CqModeBlock : public boost::enable_shared_from_this<CqModeBlock>
 				pconParent() ->AddContextLightSource( pLS );
 		}
 
-		virtual	TqBool	isSolid()
+		virtual	bool	isSolid()
 		{
-			return ( ( pconParent() ) ? pconParent() ->isSolid() : TqFalse );
+			return ( ( pconParent() ) ? pconParent() ->isSolid() : false );
 		}
 		/** Get a pointer to the CSG tree node related to this context.
 		 * \return an options reference.
@@ -368,9 +376,9 @@ class CqFrameModeBlock : public CqModeBlock
 			return boost::shared_ptr<CqModeBlock>();
 		}	// Error
 		/** Create a solid context.
-		 * \warning It is an error to call this within a frame context.
+		 * \warning It is an error to call this within a frame context
 		 */
-		virtual	boost::shared_ptr<CqModeBlock>	BeginSolidModeBlock( CqString& type )
+		virtual	boost::shared_ptr<CqModeBlock>	BeginSolidModeBlock( CqString& /* type */ )
 		{
 			return boost::shared_ptr<CqModeBlock>();
 		}	// Error
@@ -649,9 +657,9 @@ class CqSolidModeBlock : public CqModeBlock
 		}
 
 
-		virtual	TqBool	isSolid()
+		virtual	bool	isSolid()
 		{
-			return ( TqTrue );
+			return ( true );
 		}
 		virtual	boost::shared_ptr<CqCSGTreeNode>	pCSGNode()
 		{
@@ -731,7 +739,7 @@ class CqObjectModeBlock : public CqModeBlock
 		 */
 		virtual	CqAttributes*	pattrWriteCurrent()
 		{
-			assert( TqFalse );
+			assert( false );
 			return ( 0 );
 		}	// Illegal to change attributes here.
 
@@ -787,7 +795,7 @@ class CqMotionModeBlock : public CqModeBlock
 		/** Create a solid context.
 		 * \warning It is an error to call this within a motion context.
 		 */
-		virtual	boost::shared_ptr<CqModeBlock>	BeginSolidModeBlock( CqString& type )
+		virtual	boost::shared_ptr<CqModeBlock>	BeginSolidModeBlock( CqString& /* type */ )
 		{
 			return boost::shared_ptr<CqModeBlock>();
 		}		// Error
@@ -801,7 +809,7 @@ class CqMotionModeBlock : public CqModeBlock
 		/** Create a motion context.
 		 * \warning It is an error to call this within a motion context.
 		 */
-		virtual	boost::shared_ptr<CqModeBlock>	BeginMotionModeBlock( TqInt N, TqFloat times[] )
+		virtual	boost::shared_ptr<CqModeBlock>	BeginMotionModeBlock( TqInt /* N */, TqFloat /* times */[] )
 		{
 			return boost::shared_ptr<CqModeBlock>();
 		}		// Error
@@ -851,9 +859,9 @@ class CqMotionModeBlock : public CqModeBlock
 		/** Indicate that this is a motion block.
 		 * \return boolean indicating whether this is a motion block.
 		 */
-		virtual	TqBool	fMotionBlock() const
+		virtual	bool	fMotionBlock() const
 		{
-			return ( TqTrue );
+			return ( true );
 		}
 		/** Get the CqDeformingSurface, if generating a deformation motion blur sequence.
 		 */
@@ -872,6 +880,64 @@ class CqMotionModeBlock : public CqModeBlock
 }
 ;
 
+//----------------------------------------------------------------------
+/** Define the context that exists between calls to RiResourceBegin/RiResourceEnd.
+ */
+
+class CqResourceModeBlock : public CqModeBlock
+{
+	public:
+		CqResourceModeBlock( const boost::shared_ptr<CqModeBlock>& pconParent );
+		virtual	~CqResourceModeBlock();
+
+		/** Create a main context.
+		 * \warning It is an error to call this within a attribute context.
+		 */
+		virtual	boost::shared_ptr<CqModeBlock>	BeginMainModeBlock()
+		{
+			return boost::shared_ptr<CqModeBlock>();
+		}	// Error
+		/** Create a frame context.
+		 * \warning It is an error to call this within a attribute context.
+		 */
+		virtual	boost::shared_ptr<CqModeBlock>	BeginFrameModeBlock()
+		{
+			return boost::shared_ptr<CqModeBlock>();
+		}	// Error
+		/** Create a world context.
+		 * \warning It is an error to call this within a attribute context.
+		 */
+		virtual	boost::shared_ptr<CqModeBlock>	BeginWorldModeBlock()
+		{
+			return boost::shared_ptr<CqModeBlock>();
+		}	// Error
+
+		/** Delete the attribute context.
+		 * \attention This is the only valid context deletion from within this block.
+		 */
+		virtual	void	EndResourceModeBlock()
+		{}
+
+		virtual IqOptionsPtr	poptCurrent() const
+		{
+			return( pconParent()->poptCurrent() );
+		}
+		virtual IqOptionsPtr	poptWriteCurrent()
+		{
+			return( pconParent()->poptWriteCurrent() );
+		}	
+		virtual IqOptionsPtr	pushOptions()
+		{
+			return(pconParent()->pushOptions());
+		}
+		virtual IqOptionsPtr	popOptions()
+		{
+			return(pconParent()->popOptions());
+		}
+
+	private:
+};
+
 
 //***
 // Note: These inline functions are defined here to allow full definition of all
@@ -965,8 +1031,17 @@ inline boost::shared_ptr<CqModeBlock> CqModeBlock::BeginMotionModeBlock( TqInt N
 	return boost::shared_ptr<CqModeBlock>( new CqMotionModeBlock( N, times, shared_from_this() ) );
 }
 
+//----------------------------------------------------------------------
+/** Create a new resource context, and link it to the current one.
+ * \return a pointer to the new context.
+ */
+
+inline boost::shared_ptr<CqModeBlock> CqModeBlock::BeginResourceModeBlock()
+{
+	return boost::shared_ptr<CqModeBlock>( new CqResourceModeBlock( shared_from_this() ) );
+}
 //-----------------------------------------------------------------------
 
-END_NAMESPACE( Aqsis )
+} // namespace Aqsis
 
 #endif
