@@ -1,5 +1,5 @@
 // Aqsis
-// Copyright  1997 - 2001, Paul C. Gregory
+// Copyright (C) 1997 - 2001, Paul C. Gregory
 //
 // Contact: pgregory@aqsis.org
 //
@@ -23,25 +23,23 @@
 		\author Paul C. Gregory (pgregory@aqsis.org)
 */
 
-#include	"MultiTimer.h"
-
-#include	"aqsis.h"
+#include	"imagebuffer.h"
 
 #ifdef WIN32
 #include    <windows.h>
 #endif
-#include	<math.h>
 
+#include	"multitimer.h"
+#include	"aqsismath.h"
 #include	"stats.h"
 #include	"options.h"
 #include	"renderer.h"
 #include	"surface.h"
 #include	"micropolygon.h"
-#include	"imagebuffer.h"
 #include	"occlusion.h"
 
 
-START_NAMESPACE( Aqsis )
+namespace Aqsis {
 
 static TqInt bucketmodulo = -1;
 static TqInt bucketdirection = -1;
@@ -146,10 +144,10 @@ void	CqImageBuffer::SetImage()
 
 	m_iXRes = QGetRenderContext() ->poptCurrent()->GetIntegerOption( "System", "Resolution" ) [ 0 ];
 	m_iYRes = QGetRenderContext() ->poptCurrent()->GetIntegerOption( "System", "Resolution" ) [ 1 ];
-	m_CropWindowXMin = static_cast<TqInt>( CLAMP( CEIL( m_iXRes * QGetRenderContext() ->poptCurrent()->GetFloatOption( "System", "CropWindow" ) [ 0 ] ), 0, m_iXRes ) );
-	m_CropWindowXMax = static_cast<TqInt>( CLAMP( CEIL( m_iXRes * QGetRenderContext() ->poptCurrent()->GetFloatOption( "System", "CropWindow" ) [ 1 ] ), 0, m_iXRes ) );
-	m_CropWindowYMin = static_cast<TqInt>( CLAMP( CEIL( m_iYRes * QGetRenderContext() ->poptCurrent()->GetFloatOption( "System", "CropWindow" ) [ 2 ] ), 0, m_iYRes ) );
-	m_CropWindowYMax = static_cast<TqInt>( CLAMP( CEIL( m_iYRes * QGetRenderContext() ->poptCurrent()->GetFloatOption( "System", "CropWindow" ) [ 3 ] ), 0, m_iYRes ) );
+	m_CropWindowXMin = clamp<TqInt>(lceil( m_iXRes * QGetRenderContext() ->poptCurrent()->GetFloatOption( "System", "CropWindow" ) [ 0 ] ), 0, m_iXRes);
+	m_CropWindowXMax = clamp<TqInt>(lceil( m_iXRes * QGetRenderContext() ->poptCurrent()->GetFloatOption( "System", "CropWindow" ) [ 1 ] ), 0, m_iXRes);
+	m_CropWindowYMin = clamp<TqInt>(lceil( m_iYRes * QGetRenderContext() ->poptCurrent()->GetFloatOption( "System", "CropWindow" ) [ 2 ] ), 0, m_iYRes);
+	m_CropWindowYMax = clamp<TqInt>(lceil( m_iYRes * QGetRenderContext() ->poptCurrent()->GetFloatOption( "System", "CropWindow" ) [ 3 ] ), 0, m_iYRes);
 	m_cXBuckets = ( ( m_iXRes + ( m_XBucketSize-1 ) ) / m_XBucketSize );
 	m_cYBuckets = ( ( m_iYRes + ( m_YBucketSize-1 ) ) / m_YBucketSize );
 	m_PixelXSamples = QGetRenderContext() ->poptCurrent()->GetIntegerOption( "System", "PixelSamples" ) [ 0 ];
@@ -172,7 +170,7 @@ void	CqImageBuffer::SetImage()
 		i->resize( m_cXBuckets );
 		std::vector<CqBucket>::iterator b;
 		for( b = i->begin(); b!=i->end(); b++)
-			b->SetProcessed( TqFalse );
+			b->SetProcessed( false );
 	}
 
 	m_CurrentBucketCol = m_CurrentBucketRow = 0;
@@ -218,34 +216,36 @@ void	CqImageBuffer::Release()
    inserts EVERY gprim into buckets (using a bound that is still in camera space).
  */
 
-TqBool CqImageBuffer::CullSurface( CqBound& Bound, const boost::shared_ptr<CqSurface>& pSurface )
+bool CqImageBuffer::CullSurface( CqBound& Bound, const boost::shared_ptr<CqSurface>& pSurface )
 {
 	// If the primitive is completely outside of the hither-yon z range, cull it.
 	if ( Bound.vecMin().z() >= QGetRenderContext() ->poptCurrent()->GetFloatOption( "System", "Clipping" ) [ 1 ] ||
 	        Bound.vecMax().z() <= QGetRenderContext() ->poptCurrent()->GetFloatOption( "System", "Clipping" ) [ 0 ] )
-		return ( TqTrue );
+		return ( true );
 
 	if(QGetRenderContext()->clippingVolume().whereIs(Bound) == CqBound::Side_Outside)
 	{
-		return(TqTrue);
+		return(true);
 	}
 
 	// If the primitive spans the epsilon plane and the hither plane and can be split,
-	if ( Bound.vecMin().z() <= 0.0f && Bound.vecMax().z() > FLT_EPSILON )
+	if ( Bound.vecMin().z() <= FLT_EPSILON )
 	{
 		// Mark the primitive as not dicable.
 		pSurface->ForceUndiceable();
 
-		if ( pSurface->EyeSplitCount() > m_MaxEyeSplits )
+		CqString objname( "unnamed" );
+		const CqString* pattrName = pSurface->pAttributes() ->GetStringAttribute( "identifier", "name" );
+		if ( pattrName != 0 )
+			objname = pattrName[ 0 ];
+		Aqsis::log() << info << "Object \"" << objname.c_str() << "\" spans the epsilon plane" << std::endl;
+
+		if ( pSurface->SplitCount() > m_MaxEyeSplits )
 		{
-			CqString objname( "unnamed" );
-			const CqString* pattrName = pSurface->pAttributes() ->GetStringAttribute( "identifier", "name" );
-			if ( pattrName != 0 )
-				objname = pattrName[ 0 ];
 			Aqsis::log() << warning << "Max eyesplits for object \"" << objname.c_str() << "\" exceeded" << std::endl;
-			return( TqTrue );
+			return( true );
 		}
-		return ( TqFalse );
+		return ( false );
 	}
 
 	TqFloat minz = Bound.vecMin().z();
@@ -253,15 +253,17 @@ TqBool CqImageBuffer::CullSurface( CqBound& Bound, const boost::shared_ptr<CqSur
 
 
 	// Convert the bounds to raster space.
-	Bound.Transform( QGetRenderContext() ->matSpaceToSpace( "camera", "raster", NULL, NULL, QGetRenderContext()->Time() ) );
+	CqMatrix mat;
+	QGetRenderContext() ->matSpaceToSpace( "camera", "raster", NULL, NULL, QGetRenderContext()->Time(), mat );
+	Bound.Transform( mat );
 
 	// Take into account depth-of-field
 	if ( QGetRenderContext() ->UsingDepthOfField() )
 	{
 		const CqVector2D minZCoc = QGetRenderContext()->GetCircleOfConfusion( minz );
 		const CqVector2D maxZCoc = QGetRenderContext()->GetCircleOfConfusion( maxz );
-		TqFloat cocX = MAX( minZCoc.x(), maxZCoc.x() );
-		TqFloat cocY = MAX( minZCoc.y(), maxZCoc.y() );
+		TqFloat cocX = max( minZCoc.x(), maxZCoc.x() );
+		TqFloat cocY = max( minZCoc.y(), maxZCoc.y() );
 		Bound.vecMin().x( Bound.vecMin().x() - cocX );
 		Bound.vecMin().y( Bound.vecMin().y() - cocY );
 		Bound.vecMax().x( Bound.vecMax().x() + cocX );
@@ -279,7 +281,7 @@ TqBool CqImageBuffer::CullSurface( CqBound& Bound, const boost::shared_ptr<CqSur
 	        Bound.vecMin().y() > CropWindowYMax() ||
 	        Bound.vecMax().x() < CropWindowXMin() ||
 	        Bound.vecMax().y() < CropWindowYMin() )
-		return ( TqTrue );
+		return ( true );
 
 	// Restore Z-Values to camera space.
 	Bound.vecMin().z( minz );
@@ -287,7 +289,7 @@ TqBool CqImageBuffer::CullSurface( CqBound& Bound, const boost::shared_ptr<CqSur
 
 	// Cache the Bound.
 	pSurface->CacheRasterBound( Bound );
-	return ( TqFalse );
+	return ( false );
 }
 
 
@@ -298,12 +300,13 @@ TqBool CqImageBuffer::CullSurface( CqBound& Bound, const boost::shared_ptr<CqSur
 
 void CqImageBuffer::PostSurface( const boost::shared_ptr<CqSurface>& pSurface )
 {
+	TIME_SCOPE("Post Surface")
 	// Count the number of total gprims
 	STATS_INC( GPR_created_total );
 
-
 	// Bound the primitive in its current space (camera) space taking into account any motion specification.
-	CqBound Bound( pSurface->Bound() );
+	CqBound Bound;
+	pSurface->Bound(&Bound);
 
 	// Take into account the displacement bound extension.
 	TqFloat db = 0.0f;
@@ -325,7 +328,9 @@ void CqImageBuffer::PostSurface( const boost::shared_ptr<CqSurface>& pSurface )
 			transShaderToWorld = pSurface->pAttributes() ->pshadDisplacement(QGetRenderContextI()->Time()) ->getTransform();
 		else if ( pSurface->pAttributes() ->pshadSurface(QGetRenderContextI()->Time()) )
 			transShaderToWorld = pSurface->pAttributes() ->pshadSurface(QGetRenderContextI()->Time()) ->getTransform();
-		vecDB = QGetRenderContext() ->matVSpaceToSpace( strCoordinateSystem.c_str(), "camera", transShaderToWorld, pSurface->pTransform().get(), QGetRenderContextI()->Time() ) * vecDB;
+		CqMatrix mat;
+		QGetRenderContext() ->matVSpaceToSpace( strCoordinateSystem.c_str(), "camera", transShaderToWorld, pSurface->pTransform().get(), QGetRenderContextI()->Time(), mat );
+		vecDB = mat * vecDB;
 		db = vecDB.Magnitude();
 
 		Bound.vecMax() += db;
@@ -357,8 +362,8 @@ void CqImageBuffer::PostSurface( const boost::shared_ptr<CqSurface>& pSurface )
 		if ( XMinb >= cXBuckets() || YMinb >= cYBuckets() )
 			return;
 
-		XMinb = CLAMP( XMinb, 0, cXBuckets() );
-		YMinb = CLAMP( YMinb, 0, cYBuckets() );
+		XMinb = clamp( XMinb, 0, cXBuckets() );
+		YMinb = clamp( YMinb, 0, cYBuckets() );
 
 		if( Bucket(XMinb, YMinb).IsProcessed() )
 		{
@@ -382,7 +387,7 @@ void CqImageBuffer::PostSurface( const boost::shared_ptr<CqSurface>& pSurface )
  * \return Boolean indicating that the GPrim has been culled.
 */
 
-TqBool CqImageBuffer::OcclusionCullSurface( const boost::shared_ptr<CqSurface>& pSurface )
+bool CqImageBuffer::OcclusionCullSurface( const boost::shared_ptr<CqSurface>& pSurface )
 {
 	CqBound RasterBound( pSurface->GetCachedRasterBound() );
 
@@ -397,14 +402,14 @@ TqBool CqImageBuffer::OcclusionCullSurface( const boost::shared_ptr<CqSurface>&
 		        ( RasterBound.vecMax().x() >= pos.x() ) )
 		{
 			Bucket( nextBucket, CurrentBucketRow() ).AddGPrim( pSurface );
-			return TqTrue;
+			return true;
 		}
 
 		// next row
 		nextBucket = CurrentBucketRow() + 1;
 		// find bucket containing left side of bound
 		TqInt nextBucketX = static_cast<TqInt>( RasterBound.vecMin().x() ) / XBucketSize();
-		nextBucketX = MAX( nextBucketX, 0 );
+		nextBucketX = max( nextBucketX, 0 );
 		pos = BucketPosition( nextBucketX, nextBucket );
 
 		if ( ( nextBucketX < cXBuckets() ) &&
@@ -412,7 +417,7 @@ TqBool CqImageBuffer::OcclusionCullSurface( const boost::shared_ptr<CqSurface>&
 		        ( RasterBound.vecMax().y() >= pos.y() ) )
 		{
 			Bucket( nextBucketX, nextBucket ).AddGPrim( pSurface );
-			return TqTrue;
+			return true;
 		}
 
 		// Bound covers no more buckets therefore we can delete the surface completely.
@@ -422,11 +427,11 @@ TqBool CqImageBuffer::OcclusionCullSurface( const boost::shared_ptr<CqSurface>&
 			objname = *pattrName;
 		Aqsis::log() << info << "GPrim: \"" << objname << "\" occlusion culled" << std::endl;
 		STATS_INC( GPR_culled );
-		return TqTrue;
+		return true;
 	}
 	else
 	{
-		return TqFalse;
+		return false;
 	}
 }
 
@@ -443,7 +448,7 @@ void CqImageBuffer::AddMPG( CqMicroPolygon* pmpgNew )
 	ADDREF( pmpgNew );
 
 	if ( B.vecMax().x() < m_CropWindowXMin - m_FilterXWidth / 2.0f || B.vecMax().y() < m_CropWindowYMin - m_FilterYWidth / 2.0f ||
-	        B.vecMin().x() > m_CropWindowXMax + m_FilterXWidth / 2.0f || B.vecMin().y() > m_CropWindowYMax + m_FilterYWidth / 2.0f )
+	     B.vecMin().x() > m_CropWindowXMax + m_FilterXWidth / 2.0f || B.vecMin().y() > m_CropWindowYMax + m_FilterYWidth / 2.0f )
 	{
 		RELEASEREF( pmpgNew );
 		return ;
@@ -503,7 +508,7 @@ void CqImageBuffer::AddMPG( CqMicroPolygon* pmpgNew )
  * \param pmpg Pointer to a CqMicroPolygon derived class.
  */
 
-TqBool CqImageBuffer::PushMPGForward( CqMicroPolygon* pmpg, TqInt Col, TqInt Row )
+bool CqImageBuffer::PushMPGForward( CqMicroPolygon* pmpg, TqInt Col, TqInt Row )
 {
 	// Should always mark as pushed forward even if not. As this is an idicator
 	// that the attempt has been made, used by the PushDown function. If this wasn't set
@@ -512,7 +517,7 @@ TqBool CqImageBuffer::PushMPGForward( CqMicroPolygon* pmpg, TqInt Col, TqInt Row
 
 	// Check if there is anywhere to push forward to.
 	if ( Col == ( cXBuckets() - 1 ) )
-		return ( TqFalse );
+		return ( false );
 
 	TqInt NextBucketForward = Col + 1;
 
@@ -536,15 +541,15 @@ TqBool CqImageBuffer::PushMPGForward( CqMicroPolygon* pmpg, TqInt Col, TqInt Row
 	        ( vMax.x() < BucketMin.x() ) ||
 	        ( vMax.y() < BucketMin.y() ) )
 	{
-		return ( TqFalse );
+		return ( false );
 	}
 	else
 	{
 		ADDREF( pmpg );
 		Bucket( NextBucketForward, Row ).AddMPG( pmpg );
-		return ( TqTrue );
+		return ( true );
 	}
-	return ( TqFalse );
+	return ( false );
 }
 
 
@@ -554,14 +559,14 @@ TqBool CqImageBuffer::PushMPGForward( CqMicroPolygon* pmpg, TqInt Col, TqInt Row
  * \param CurrBucketIndex Index of the bucket from which we are pushing down.
  */
 
-TqBool CqImageBuffer::PushMPGDown( CqMicroPolygon* pmpg, TqInt Col, TqInt Row )
+bool CqImageBuffer::PushMPGDown( CqMicroPolygon* pmpg, TqInt Col, TqInt Row )
 {
 	if ( pmpg->IsPushedForward() )
-		return ( TqFalse );
+		return ( false );
 
 	// Check if there is anywhere to push down to.
 	if ( Row == ( m_cYBuckets - 1 ) )
-		return ( TqFalse );
+		return ( false );
 
 	TqInt NextBucketDown = Row + 1;
 
@@ -570,7 +575,7 @@ TqBool CqImageBuffer::PushMPGDown( CqMicroPolygon* pmpg, TqInt Col, TqInt Row )
 	if( Bucket( Col, NextBucketDown ).IsProcessed() )
 	{
 		if( PushMPGForward( pmpg, Col, NextBucketDown ) )
-			return( TqTrue );
+			return( true );
 		else
 			// If that fails, push down again.
 			return( PushMPGDown( pmpg, Col, NextBucketDown ) );
@@ -592,7 +597,7 @@ TqBool CqImageBuffer::PushMPGDown( CqMicroPolygon* pmpg, TqInt Col, TqInt Row )
 	        ( vMax.x() < BucketMin.x() ) ||
 	        ( vMax.y() < BucketMin.y() ) )
 	{
-		return ( TqFalse );
+		return ( false );
 	}
 	else
 	{
@@ -601,9 +606,9 @@ TqBool CqImageBuffer::PushMPGDown( CqMicroPolygon* pmpg, TqInt Col, TqInt Row )
 		// See if it needs to be pushed further down (extreme Motion Blur)
 		if ( PushMPGDown( pmpg, Col, NextBucketDown ) )
 			STATS_INC( MPG_pushed_far_down );
-		return ( TqTrue );
+		return ( true );
 	}
-	return ( TqFalse );
+	return ( false );
 }
 
 
@@ -620,7 +625,6 @@ void CqImageBuffer::CacheGridInfo(CqMicroPolyGridBase* pGrid)
 
 	m_CurrentGridInfo.m_UsesDataMap = !(QGetRenderContext() ->GetMapOfOutputDataEntries().empty());
 
-	m_CurrentGridInfo.m_ShadingRate = pGrid->pAttributes() ->GetFloatAttribute( "System", "ShadingRate" ) [ 0 ];
 	m_CurrentGridInfo.m_ShutterOpenTime = QGetRenderContext() ->poptCurrent()->GetFloatOption( "System", "Shutter" ) [ 0 ];
 	m_CurrentGridInfo.m_ShutterCloseTime = QGetRenderContext() ->poptCurrent()->GetFloatOption( "System", "Shutter" ) [ 1 ];
 
@@ -643,27 +647,30 @@ void CqImageBuffer::CacheGridInfo(CqMicroPolyGridBase* pGrid)
 
 void CqImageBuffer::RenderMPGs( long xmin, long xmax, long ymin, long ymax )
 {
-	// Render any waiting MPGs
-	std::vector<CqMicroPolygon*>::iterator lastmpg = CurrentBucket().aMPGs().end();
-	CqMicroPolyGridBase* pPrevGrid = NULL;
-	for ( std::vector<CqMicroPolygon*>::iterator impg = CurrentBucket().aMPGs().begin(); impg != lastmpg; impg++ )
 	{
-		CqMicroPolygon* pMpg = *impg;
-
-		if(pMpg->pGrid() != pPrevGrid)
+		TIME_SCOPE("Render MPGs")
+		// Render any waiting MPGs
+		std::vector<CqMicroPolygon*>::iterator lastmpg = CurrentBucket().aMPGs().end();
+		CqMicroPolyGridBase* pPrevGrid = NULL;
+		for ( std::vector<CqMicroPolygon*>::iterator impg = CurrentBucket().aMPGs().begin(); impg != lastmpg; impg++ )
 		{
-			pPrevGrid = pMpg->pGrid();
-			CacheGridInfo(pPrevGrid);
-		}
+			CqMicroPolygon* pMpg = *impg;
 
-		RenderMicroPoly( pMpg, xmin, xmax, ymin, ymax );
-		if ( PushMPGDown( ( pMpg ), CurrentBucketCol(), CurrentBucketRow() ) )
-			STATS_INC( MPG_pushed_down );
-		if ( PushMPGForward( ( pMpg ), CurrentBucketCol(), CurrentBucketRow() ) )
-			STATS_INC( MPG_pushed_forward );
-		RELEASEREF( ( pMpg ) );
+			if(pMpg->pGrid() != pPrevGrid)
+			{
+				pPrevGrid = pMpg->pGrid();
+				CacheGridInfo(pPrevGrid);
+			}
+
+			RenderMicroPoly( pMpg, xmin, xmax, ymin, ymax );
+			if ( PushMPGDown( ( pMpg ), CurrentBucketCol(), CurrentBucketRow() ) )
+				STATS_INC( MPG_pushed_down );
+			if ( PushMPGForward( ( pMpg ), CurrentBucketCol(), CurrentBucketRow() ) )
+				STATS_INC( MPG_pushed_forward );
+			RELEASEREF( ( pMpg ) );
+		}
+		CurrentBucket().aMPGs().clear();
 	}
-	CurrentBucket().aMPGs().clear();
 
 	// Split any grids in this bucket waiting to be processed.
 	if ( !CurrentBucket().aGrids().empty() )
@@ -721,40 +728,31 @@ void CqImageBuffer::RenderMicroPoly( CqMicroPolygon* pMPG, long xmin, long xmax,
 		return;
 	}
 
-	// fill in sample info for this mpg so we don't have to keep fetching it for each sample.
-	// Must check if colour is needed, as if not, the variable will have been deleted from the grid.
-	if ( QGetRenderContext() ->pDDmanager() ->fDisplayNeeds( "Ci" ) )
-	{
-		m_CurrentMpgSampleInfo.m_Colour = pMPG->colColor()[0];
-	}
-	else
-	{
-		m_CurrentMpgSampleInfo.m_Colour = gColWhite;
-	}
+	bool UsingDof = QGetRenderContext() ->UsingDepthOfField();
+	bool IsMoving = pMPG->IsMoving();
 
-	// Must check if opacity is needed, as if not, the variable will have been deleted from the grid.
-	if ( QGetRenderContext() ->pDDmanager() ->fDisplayNeeds( "Oi" ) )
-	{
-		m_CurrentMpgSampleInfo.m_Opacity = pMPG->colOpacity()[0];
-		m_CurrentMpgSampleInfo.m_Occludes = m_CurrentMpgSampleInfo.m_Opacity >= gColWhite;
-	}
-	else
-	{
-		m_CurrentMpgSampleInfo.m_Opacity = gColWhite;
-		m_CurrentMpgSampleInfo.m_Occludes = TqTrue;
-	}
+	// Cache the shading interpolation type.  Ideally this should really be
+	// done by the CacheOutputInterpCoeffs(), or possibly once per grid...
+	const TqInt* interpType = pMPG->pGrid()->pAttributes()
+		->GetIntegerAttribute("System", "ShadingInterpolation");
+	// At this stage, only use smooth shading interpolation for stationary
+	// grids without DoF.
+	/// \todo Allow smooth shading with MB or DoF.
+	m_CurrentMpgSampleInfo.smoothInterpolation
+		= !(UsingDof || IsMoving) && (*interpType == ShadingInterp_Smooth);
+
+	// Cache output sample info for this mpg so we don't have to keep fetching
+	// it for each sample.
+	pMPG->CacheOutputInterpCoeffs(m_CurrentMpgSampleInfo);
 
 	// use the single imagesample rather than the list if possible.
 	// transparent, matte or csg samples, or if we need more than the first depth
 	// value have to use the (slower) list.
-	m_CurrentMpgSampleInfo.m_IsOpaque = m_CurrentMpgSampleInfo.m_Occludes &&
+	m_CurrentMpgSampleInfo.isOpaque = m_CurrentMpgSampleInfo.occludes &&
 	                                    !pMPG->pGrid()->pCSGNode() &&
 	                                    !( DisplayMode() & ModeZ ) &&
 	                                    !m_CurrentGridInfo.m_IsMatte;
 
-	TqBool UsingDof = QGetRenderContext() ->UsingDepthOfField();
-	TqBool IsMoving = pMPG->IsMoving();
-
 	if(IsMoving || UsingDof)
 	{
 		RenderMPG_MBOrDof( pMPG, xmin, xmax, ymin, ymax, IsMoving, UsingDof );
@@ -768,71 +766,69 @@ void CqImageBuffer::RenderMicroPoly( CqMicroPolygon* pMPG, long xmin, long xmax,
 
 // this function assumes that either dof or mb or both are being used.
 void CqImageBuffer::RenderMPG_MBOrDof( CqMicroPolygon* pMPG,
-                                       long xmin, long xmax, long ymin, long ymax,
-                                       TqBool IsMoving, TqBool UsingDof )
+					long xmin, long xmax, long ymin, long ymax,
+					bool IsMoving, bool UsingDof )
 {
 	CqHitTestCache hitTestCache;
-	TqBool cachedHitData = TqFalse;
 	pMPG->CacheHitTestValues(&hitTestCache);
-	cachedHitData = TqTrue;
 
 	TqInt iXSamples = PixelXSamples();
-	TqInt iYSamples = PixelYSamples();
+    TqInt iYSamples = PixelYSamples();
 
 	TqFloat opentime = m_CurrentGridInfo.m_ShutterOpenTime;
 	TqFloat closetime = m_CurrentGridInfo.m_ShutterCloseTime;
-	TqFloat timePerSample;
+	TqFloat timePerSample = 0;
 	if(IsMoving)
 	{
 		TqInt numSamples = iXSamples * iYSamples;
 		timePerSample = (float)numSamples / ( closetime - opentime );
 	}
 
-	TqInt bound_maxMB = pMPG->cSubBounds();
-	TqInt bound_maxMB_1 = bound_maxMB - 1;
-	for ( TqInt bound_numMB = 0; bound_numMB < bound_maxMB; bound_numMB++ )
-	{
+    TqInt bound_maxMB = pMPG->cSubBounds();
+    TqInt bound_maxMB_1 = bound_maxMB - 1;
+    for ( TqInt bound_numMB = 0; bound_numMB < bound_maxMB; bound_numMB++ )
+    {
 		TqFloat time0 = m_CurrentGridInfo.m_ShutterOpenTime;
 		TqFloat time1 = m_CurrentGridInfo.m_ShutterCloseTime;
-		const CqBound& Bound = pMPG->SubBound( bound_numMB, time0 );
+        const CqBound& Bound = pMPG->SubBound( bound_numMB, time0 );
 
 		// get the index of the first and last samples that can fall inside
 		// the time range of this bound
-		TqInt indexT0;
-		TqInt indexT1;
+		TqInt indexT0 = 0;
+		TqInt indexT1 = 0;
 		if(IsMoving)
 		{
 			if ( bound_numMB != bound_maxMB_1 )
 				pMPG->SubBound( bound_numMB + 1, time1 );
 			else
-				time1 = closetime;//QGetRenderContext() ->poptCurrent()->GetFloatOptionWrite( "System", "Shutter" ) [ 1 ];
-
-			indexT0 = static_cast<TqInt>(FLOOR((time0 - opentime) * timePerSample));
-			indexT1 = static_cast<TqInt>(CEIL((time1 - opentime) * timePerSample));
+				time1 = closetime;//QGetRenderContext() ->optCurrent().GetFloatOptionWrite( "System", "Shutter" ) [ 1 ];
+	
+			indexT0 = static_cast<TqInt>(std::floor((time0 - opentime) * timePerSample));
+			indexT1 = static_cast<TqInt>(lceil((time1 - opentime) * timePerSample));
 		}
 
-		TqFloat maxCocX = 0.0f;
-		TqFloat maxCocY = 0.0f;
+		TqFloat maxCocX = 0;
+		TqFloat maxCocY = 0;
 
-		TqFloat bminx = 0.0f;
-		TqFloat bmaxx = 0.0f;
-		TqFloat bminy = 0.0f;
-		TqFloat bmaxy = 0.0f;
-		TqFloat bminz = 0.0f;
-		TqFloat bmaxz = 0.0f;
+		TqFloat bminx = 0;
+		TqFloat bmaxx = 0;
+		TqFloat bminy = 0;
+		TqFloat bmaxy = 0;
+		TqFloat bminz = 0;
+		TqFloat bmaxz = 0;
 		// these values are the bound of the mpg not including dof extension.
 		// reduce the mpg bound so it doesn't include the coc.
-		TqFloat mpgbminx = 0.0f;
-		TqFloat mpgbmaxx = 0.0f;
-		TqFloat mpgbminy = 0.0f;
-		TqFloat mpgbmaxy = 0.0f;
-		TqInt bound_maxDof = 0;
+		TqFloat mpgbminx = 0;
+		TqFloat mpgbmaxx = 0;
+		TqFloat mpgbminy = 0;
+		TqFloat mpgbmaxy = 0;
+		TqInt bound_maxDof = 1;
 		if(UsingDof)
 		{
 			const CqVector2D& minZCoc = QGetRenderContext()->GetCircleOfConfusion( Bound.vecMin().z() );
 			const CqVector2D& maxZCoc = QGetRenderContext()->GetCircleOfConfusion( Bound.vecMax().z() );
-			maxCocX = MAX( minZCoc.x(), maxZCoc.x() );
-			maxCocY = MAX( minZCoc.y(), maxZCoc.y() );
+			maxCocX = max( minZCoc.x(), maxZCoc.x() );
+			maxCocY = max( minZCoc.y(), maxZCoc.y() );
 
 			mpgbminx = Bound.vecMin().x() + maxCocX;
 			mpgbmaxx = Bound.vecMax().x() - maxCocX;
@@ -874,25 +870,25 @@ void CqImageBuffer::RenderMPG_MBOrDof( CqMicroPolygon* pMPG,
 			}
 
 			// if bounding box is outside our viewing range, then cull it.
-			if ( bmaxx < (float)xmin || bmaxy < (float)ymin ||
-			        bminx > (float)xmax || bminy > (float)ymax ||
-			        bminz > ClippingFar() || bmaxz < ClippingNear())
+			if ( bmaxx <= (float)xmin || bmaxy <= (float)ymin ||
+				bminx >= (float)xmax || bminy >= (float)ymax ||
+				bminz >= ClippingFar() || bmaxz <= ClippingNear())
 			{
 				continue;
 			}
 
-			if(UsingDof)
-			{
-				CqBound DofBound(bminx, bminy, bminz, bmaxx, bmaxy, bmaxz);
-				CqOcclusionBox::KDTree()->SampleMPG(pMPG, DofBound, IsMoving, time0, time1, TqTrue, bound_numDof, m_CurrentMpgSampleInfo, m_CurrentGridInfo.m_LodBounds[0] >= 0.0f, m_CurrentGridInfo);
-			}
-			else
-			{
-				CqOcclusionBox::KDTree()->SampleMPG(pMPG, Bound, IsMoving, time0, time1, TqFalse, 0, m_CurrentMpgSampleInfo, m_CurrentGridInfo.m_LodBounds[0] >= 0.0f, m_CurrentGridInfo);
-			}
-		}
-	}
-}
+					if(UsingDof)
+					{
+								CqBound DofBound(bminx, bminy, bminz, bmaxx, bmaxy, bmaxz);
+				CqOcclusionBox::KDTree()->SampleMPG(pMPG, DofBound, IsMoving, time0, time1, true, bound_numDof, m_CurrentMpgSampleInfo, m_CurrentGridInfo.m_LodBounds[0] >= 0.0f, m_CurrentGridInfo);
+									}
+							else
+							{
+				CqOcclusionBox::KDTree()->SampleMPG(pMPG, Bound, IsMoving, time0, time1, false, 0, m_CurrentMpgSampleInfo, m_CurrentGridInfo.m_LodBounds[0] >= 0.0f, m_CurrentGridInfo);
+									}
+								}
+								}
+							}
 
 
 
@@ -901,12 +897,12 @@ void CqImageBuffer::RenderMPG_MBOrDof( CqMicroPolygon* pMPG,
 void CqImageBuffer::RenderMPG_Static( CqMicroPolygon* pMPG, long xmin, long xmax, long ymin, long ymax )
 {
 	CqHitTestCache hitTestCache;
-	pMPG->CacheHitTestValues(&hitTestCache);
+					pMPG->CacheHitTestValues(&hitTestCache);
 
 	const CqBound& Bound = pMPG->GetTotalBound();
 
-	CqOcclusionBox::KDTree()->SampleMPG(pMPG, Bound, TqFalse, 0, 0, TqFalse, 0, m_CurrentMpgSampleInfo, m_CurrentGridInfo.m_LodBounds[0] >= 0.0f, m_CurrentGridInfo);
-}
+	CqOcclusionBox::KDTree()->SampleMPG(pMPG, Bound, false, 0, 0, false, 0, m_CurrentMpgSampleInfo, m_CurrentGridInfo.m_LodBounds[0] >= 0.0f, m_CurrentGridInfo);
+								}
 
 
 
@@ -1013,15 +1009,12 @@ void CqImageBuffer::StoreExtraData( CqMicroPolygon* pMPG, SqImageSample& sample)
  * \param ymax Integer maximum extend of the image part being rendered, takes into account buckets and clipping.
  */
 
-void CqImageBuffer::RenderSurfaces( long xmin, long xmax, long ymin, long ymax, TqBool fImager, enum EqFilterDepth depthfilter, CqColor zThreshold)
+void CqImageBuffer::RenderSurfaces( long xmin, long xmax, long ymin, long ymax, bool fImager, enum EqFilterDepth depthfilter, CqColor zThreshold)
 {
-	TqBool bIsEmpty = IsCurrentBucketEmpty();
+	bool bIsEmpty = IsCurrentBucketEmpty();
 
 	// Render any waiting micro polygon grids.
-	{
-		TIME_SCOPE("Render MPGs")
-		RenderMPGs( xmin, xmax, ymin, ymax );
-	}
+	RenderMPGs( xmin, xmax, ymin, ymax );
 
 	CqBucket& Bucket = CurrentBucket();
 
@@ -1036,10 +1029,12 @@ void CqImageBuffer::RenderSurfaces( long xmin, long xmax, long ymin, long ymax,
 		//Cull surface if it's hidden
 		if ( !( DisplayMode() & ModeZ ) && !pSurface->pCSGNode() )
 		{
-			TIME_SCOPE("Occlusion culling")
-			TqBool fCull = TqFalse;
+			bool fCull = false;
 			if ( !bIsEmpty && pSurface->fCachedBound() )
+			{
+				TIME_SCOPE("Occlusion culling surfaces")
 				fCull = OcclusionCullSurface( pSurface );
+			}
 			if ( fCull )
 			{
 				Bucket.popSurface();
@@ -1049,7 +1044,7 @@ void CqImageBuffer::RenderSurfaces( long xmin, long xmax, long ymin, long ymax,
 		}
 
 		// If the epsilon check has deemed this surface to be undiceable, don't bother asking.
-		TqBool fDiceable = TqFalse;
+		bool fDiceable = false;
 		// Dice & shade the surface if it's small enough...
 		{
 			TIME_SCOPE("Dicable check")
@@ -1073,15 +1068,12 @@ void CqImageBuffer::RenderSurfaces( long xmin, long xmax, long ymin, long ymax,
 				pGrid->Shade();
 				pGrid->TransferOutputVariables();
 
-				if ( pGrid->vfCulled() == TqFalse )
+				if ( pGrid->vfCulled() == false )
 				{
 					// Only project micropolygon not culled
 					Bucket.AddGrid( pGrid );
 					// Render any waiting micro polygon grids.
-					{
-						TIME_SCOPE("Render MPGs")
-						RenderMPGs( xmin, xmax, ymin, ymax );
-					}
+					RenderMPGs( xmin, xmax, ymin, ymax );
 				}
 				RELEASEREF( pGrid );
 			}
@@ -1096,9 +1088,12 @@ void CqImageBuffer::RenderSurfaces( long xmin, long xmax, long ymin, long ymax,
 
 			// Split it
 			{
-				TIME_SCOPE("Splits")
+				TqInt cSplits = 0;
 				std::vector<boost::shared_ptr<CqSurface> > aSplits;
-				TqInt cSplits = pSurface->Split( aSplits );
+				{
+					TIME_SCOPE("Splits")
+					cSplits = pSurface->Split( aSplits );
+				}
 				TqInt i;
 				for ( i = 0; i < cSplits; i++ )
 					PostSurface( aSplits[ i ] );
@@ -1126,10 +1121,7 @@ void CqImageBuffer::RenderSurfaces( long xmin, long xmax, long ymin, long ymax,
 
 		pSurface = Bucket.pTopSurface();
 		// Render any waiting micro polygon grids.
-		{
-			TIME_SCOPE("Render MPGs")
-			RenderMPGs( xmin, xmax, ymin, ymax );
-		}
+		RenderMPGs( xmin, xmax, ymin, ymax );
 	}
 
 	// Now combine the colors at each pixel sample for any micropolygons rendered to that pixel.
@@ -1144,7 +1136,7 @@ void CqImageBuffer::RenderSurfaces( long xmin, long xmax, long ymin, long ymax,
 
 	TIMER_START("Filter")
 	if (fImager)
-		bIsEmpty = TqFalse;
+		bIsEmpty = false;
 
 	Bucket.FilterBucket(bIsEmpty);
 	if(!bIsEmpty)
@@ -1171,16 +1163,16 @@ void CqImageBuffer::RenderSurfaces( long xmin, long xmax, long ymin, long ymax,
      It is empty only when this bucket doesn't contain any surface, 
 	 micropolygon or grids.
  */
-TqBool CqImageBuffer::IsCurrentBucketEmpty()
+bool CqImageBuffer::IsCurrentBucketEmpty()
 {
-	TqBool retval = TqFalse;
+	bool retval = false;
 
 	CqBucket& Bucket = CurrentBucket();
 
 	if ( ( !Bucket.pTopSurface() ) &&
 	        Bucket.aGrids().empty() &&
 	        Bucket.aMPGs().empty() )
-		retval = TqTrue;
+		retval = true;
 
 	return retval;
 }
@@ -1224,11 +1216,11 @@ void CqImageBuffer::RenderImage()
 			bucketmodulo = m_cXBuckets;
 	}
 
-	TqBool fImager = TqFalse;
+	bool fImager = false;
 	const CqString* systemOptions;
 	if( ( systemOptions = QGetRenderContext() ->poptCurrent()->GetStringOption( "System", "Imager" ) ) != 0 )
 		if( systemOptions[ 0 ].compare("null") != 0 )
-			fImager = TqTrue;
+			fImager = true;
 
 	const CqString* pstrDepthFilter = QGetRenderContext() ->poptCurrent()->GetStringOption( "Hider", "depthfilter" );
 	const CqColor* pzThreshold = QGetRenderContext() ->poptCurrent()->GetColorOption( "limits", "zthreshold" );
@@ -1252,9 +1244,9 @@ void CqImageBuffer::RenderImage()
 	}
 
 	// Render the surface at the front of the list.
-	m_fDone = TqFalse;
+	m_fDone = false;
 
-	CqVector2D bHalf = CqVector2D( FLOOR(m_FilterXWidth / 2.0f), FLOOR(m_FilterYWidth / 2.0f) );
+	CqVector2D bHalf = CqVector2D( std::floor(m_FilterXWidth / 2.0f), std::floor(m_FilterYWidth / 2.0f) );
 
 	RtProgressFunc pProgressHandler = NULL;
 	pProgressHandler = QGetRenderContext()->pProgressHandler();
@@ -1283,26 +1275,30 @@ void CqImageBuffer::RenderImage()
 	// A counter for the number of processed buckets (used for progress reporting)
 	TqInt iBucket = 0;
 
+	bool bJitter = true;
+	if(const TqInt* useJitterPtr = QGetRenderContext()->poptCurrent()->GetIntegerOption( "Hider", "jitter" ))
+		bJitter = static_cast<bool>(*useJitterPtr);
+
 	// Iterate over all buckets...
 	do
 	{
-		TqBool bIsEmpty = IsCurrentBucketEmpty();
+		bool bIsEmpty = IsCurrentBucketEmpty();
 		// Prepare the bucket.
 		CqVector2D bPos = BucketPosition();
 		CqVector2D bSize = BucketSize();
 
-		TqBool fImager = TqFalse;
+		bool fImager = false;
 		const CqString* systemOptions;
 		if( ( systemOptions = QGetRenderContext() ->poptCurrent()->GetStringOption( "System", "Imager" ) ) != 0 )
 			if( systemOptions[ 0 ].compare("null") != 0 )
-				fImager = TqTrue;
+				fImager = true;
 
 		if (fImager)
-			bIsEmpty = TqFalse;
+			bIsEmpty = false;
 
 		{
 			TIME_SCOPE("Prepare bucket")
-			CqBucket::PrepareBucket( static_cast<TqInt>( bPos.x() ), static_cast<TqInt>( bPos.y() ), static_cast<TqInt>( bSize.x() ), static_cast<TqInt>( bSize.y() ), true, bIsEmpty );
+			CqBucket::PrepareBucket( static_cast<TqInt>( bPos.x() ), static_cast<TqInt>( bPos.y() ), static_cast<TqInt>( bSize.x() ), static_cast<TqInt>( bSize.y() ), bJitter, bIsEmpty );
 			CqBucket::InitialiseFilterValues();
 		}
 
@@ -1336,7 +1332,7 @@ void CqImageBuffer::RenderImage()
 
 		if ( !bIsEmpty )
 		{
-			TIME_SCOPE("Occlusion culling")
+			TIME_SCOPE("Occlusion culling initialisation")
 			CqOcclusionBox::SetupHierarchy( &CurrentBucket(), xmin, ymin, xmax, ymax );
 		}
 
@@ -1351,24 +1347,15 @@ void CqImageBuffer::RenderImage()
 		}
 
 
-		RenderSurfaces( xmin, xmax, ymin, ymax, fImager, depthfilter, zThreshold );
+			RenderSurfaces( xmin, xmax, ymin, ymax, fImager, depthfilter, zThreshold );
 		if ( m_fQuit )
 		{
-			m_fDone = TqTrue;
+			m_fDone = true;
 			return ;
 		}
 #ifdef WIN32
 		if ( !( iBucket % bucketmodulo ) )
 			SetProcessWorkingSetSize( GetCurrentProcess(), 0xffffffff, 0xffffffff );
-#endif
-#if defined(REQUIRED)
-
-		TqInt iB2, NumGrids = 0, NumPolys = 0;
-		for ( iB2 = 0; iB2 < nBuckets; iB2++ )
-		{
-			NumGrids += m_aBuckets[ iB2 ].aGrids().size();
-			NumPolys += m_aBuckets[ iB2 ].aMPGs().size();
-		}
 #endif
 		CurrentBucket().SetProcessed();
 		// Increase the bucket counter...
@@ -1385,7 +1372,7 @@ void CqImageBuffer::RenderImage()
 	{
 		( *pProgressHandler ) ( 100.0f, QGetRenderContext() ->CurrentFrame() );
 	}
-	m_fDone = TqTrue;
+	m_fDone = true;
 }
 
 
@@ -1395,7 +1382,7 @@ void CqImageBuffer::RenderImage()
 
 void CqImageBuffer::Quit()
 {
-	m_fQuit = TqTrue;
+	m_fQuit = true;
 }
 
 //----------------------------------------------------------------------
@@ -1405,7 +1392,7 @@ void CqImageBuffer::Quit()
 
   \return True if there is still an unprocessed bucket left, otherwise False.
  */
-TqBool CqImageBuffer::NextBucket(EqBucketOrder order)
+bool CqImageBuffer::NextBucket(EqBucketOrder order)
 {
 
 	TqInt cnt = 0;
@@ -1420,7 +1407,7 @@ TqBool CqImageBuffer::NextBucket(EqBucketOrder order)
 	if ((order != Bucket_Vertical) &&
 	        (order != Bucket_Horizontal) &&
 	        (cnt == total))
-		return TqFalse;
+		return false;
 
 	switch (order)
 	{
@@ -1432,8 +1419,8 @@ TqBool CqImageBuffer::NextBucket(EqBucketOrder order)
 			{
 				m_CurrentBucketCol = (TqInt) rg.RandomFloat(m_cXBuckets);
 				m_CurrentBucketRow = (TqInt) rg.RandomFloat(m_cYBuckets);
-				m_CurrentBucketCol = CLAMP(m_CurrentBucketCol, 0, m_cXBuckets - 1);
-				m_CurrentBucketRow = CLAMP(m_CurrentBucketRow, 0, m_cYBuckets - 1);
+				m_CurrentBucketCol = clamp(m_CurrentBucketCol, 0, m_cXBuckets - 1);
+				m_CurrentBucketRow = clamp(m_CurrentBucketRow, 0, m_cYBuckets - 1);
 			}
 			while ( Bucket(m_CurrentBucketCol, m_CurrentBucketRow).IsProcessed() );
 
@@ -1474,8 +1461,8 @@ TqBool CqImageBuffer::NextBucket(EqBucketOrder order)
 				if (radius > r)
 					break;
 
-				m_CurrentBucketCol = CLAMP(m_CurrentBucketCol, 0, m_cXBuckets - 1);
-				m_CurrentBucketRow = CLAMP(m_CurrentBucketRow, 0, m_cYBuckets - 1);
+				m_CurrentBucketCol = clamp(m_CurrentBucketCol, 0, m_cXBuckets - 1);
+				m_CurrentBucketRow = clamp(m_CurrentBucketRow, 0, m_cYBuckets - 1);
 			}
 			while (Bucket(m_CurrentBucketCol, m_CurrentBucketRow).IsProcessed());
 
@@ -1511,7 +1498,7 @@ TqBool CqImageBuffer::NextBucket(EqBucketOrder order)
 				m_CurrentBucketRow++;
 				bucketdirection = -1;
 				if( m_CurrentBucketRow >= m_cYBuckets )
-					return( TqFalse );
+					return( false );
 
 			}
 			else if((bucketdirection == -1) && ( m_CurrentBucketCol < 0 ))
@@ -1522,7 +1509,7 @@ TqBool CqImageBuffer::NextBucket(EqBucketOrder order)
 				bucketdirection = 1;
 
 				if( m_CurrentBucketRow >= m_cYBuckets )
-					return( TqFalse );
+					return( false );
 
 			}
 		}
@@ -1539,7 +1526,7 @@ TqBool CqImageBuffer::NextBucket(EqBucketOrder order)
 				m_CurrentBucketCol++;
 
 				if( m_CurrentBucketCol >= m_cXBuckets )
-					return( TqFalse );
+					return( false );
 
 			}
 		}
@@ -1556,17 +1543,18 @@ TqBool CqImageBuffer::NextBucket(EqBucketOrder order)
 				m_CurrentBucketCol = 0;
 				m_CurrentBucketRow++;
 				if( m_CurrentBucketRow >= m_cYBuckets )
-					return( TqFalse );
+					return( false );
 
 			}
 		}
 		break;
 	}
-	return( TqTrue );
+	return( true );
 }
 
 //---------------------------------------------------------------------
 
-END_NAMESPACE( Aqsis )
+} // namespace Aqsis
+
 
 
