@@ -1,5 +1,5 @@
 // Aqsis
-// Copyright  1997 - 2001, Paul C. Gregory
+// Copyright (C) 1997 - 2001, Paul C. Gregory
 //
 // Contact: pgregory@aqsis.org
 //
@@ -34,7 +34,7 @@
 #include "vector2d.h"
 #include "vector3d.h"
 #include "curves.h"
-START_NAMESPACE( Aqsis )
+namespace Aqsis {
 
 
 static TqUlong hp = CqString::hash("P");
@@ -48,28 +48,12 @@ static TqUlong hv = CqString::hash("v");
 CqLinearCurveSegment::CqLinearCurveSegment() : CqCurve()
 { }
 
-
-
-/**
- * CqLinearCurveSegment copy constructor.
- */
-/* CqLinearCurveSegment::CqLinearCurveSegment( const CqLinearCurveSegment &from ) :
- * 		CqCurve()
- * {
- * 	( *this ) = from;
- * }
- */
-
-
-
 /**
  * CqLinearCurveSegment destructor.
  */
 CqLinearCurveSegment::~CqLinearCurveSegment()
 { }
 
-
-
 /**
  * Create a clone of this curve surface
  *
@@ -82,103 +66,69 @@ CqSurface* CqLinearCurveSegment::Clone() const
 }
 
 
+namespace {
 
-
-/**
- * Implements natural subdivision for this curve segment.
+/** \brief Helper function for natural subdivision of linear curve parameters.
  *
- * @param pParam        Original parameter.
- * @param pParam1       First new parameter.
- * @param pParam2       Second new parameter.
- * @param u             true if the split is along u (should
- *                              always be false!)
+ * \param pParam - pointer to source parameter data
+ * \param pResult1 - pointer to output parameter for first result curve
+ * \param pResult1 - pointer to output parameter for second result curve
  */
+template <class T, class SLT>
+void linCurveNatSubdiv(CqParameter* pParam,
+		CqParameter* pResult1,
+		CqParameter* pResult2)
+{
+	CqParameterTyped<T, SLT>* pTParam = static_cast<CqParameterTyped<T, SLT>*>( pParam );
+	CqParameterTyped<T, SLT>* pTResult1 = static_cast<CqParameterTyped<T, SLT>*>( pResult1 );
+	CqParameterTyped<T, SLT>* pTResult2 = static_cast<CqParameterTyped<T, SLT>*>( pResult2 );
+
+	pTResult1->pValue() [ 0 ] = pTParam->pValue() [ 0 ];
+	pTResult1->pValue() [ 1 ] = pTResult2->pValue() [ 0 ] = static_cast<T>( ( pTParam->pValue() [ 0 ] + pTParam->pValue() [ 1 ] ) * 0.5f );
+	pTResult2->pValue() [ 1 ] = pTParam->pValue() [ 1 ];
+}
+
+} // unnamed namespace 
+
 void CqLinearCurveSegment::NaturalSubdivide(
     CqParameter* pParam,
     CqParameter* pParam1, CqParameter* pParam2,
-    TqBool u
+    bool u
 )
 {
-
 	assert( u == false );
-	switch ( pParam->Type() )
+	switch(pParam->Type())
 	{
-			case type_float:
-			{
-				CqParameterTyped<TqFloat, TqFloat>* pTParam = static_cast<CqParameterTyped<TqFloat, TqFloat>*>( pParam );
-				CqParameterTyped<TqFloat, TqFloat>* pTResult1 = static_cast<CqParameterTyped<TqFloat, TqFloat>*>( pParam1 );
-				CqParameterTyped<TqFloat, TqFloat>* pTResult2 = static_cast<CqParameterTyped<TqFloat, TqFloat>*>( pParam2 );
-				TypedNaturalSubdivide( pTParam, pTResult1, pTResult2, u );
-				break;
-			}
-
-			case type_integer:
-			{
-				CqParameterTyped<TqInt, TqFloat>* pTParam = static_cast<CqParameterTyped<TqInt, TqFloat>*>( pParam );
-				CqParameterTyped<TqInt, TqFloat>* pTResult1 = static_cast<CqParameterTyped<TqInt, TqFloat>*>( pParam1 );
-				CqParameterTyped<TqInt, TqFloat>* pTResult2 = static_cast<CqParameterTyped<TqInt, TqFloat>*>( pParam2 );
-				TypedNaturalSubdivide( pTParam, pTResult1, pTResult2, u );
-				break;
-			}
-
-			case type_point:
-			case type_vector:
-			case type_normal:
-			{
-				CqParameterTyped<CqVector3D, CqVector3D>* pTParam = static_cast<CqParameterTyped<CqVector3D, CqVector3D>*>( pParam );
-				CqParameterTyped<CqVector3D, CqVector3D>* pTResult1 = static_cast<CqParameterTyped<CqVector3D, CqVector3D>*>( pParam1 );
-				CqParameterTyped<CqVector3D, CqVector3D>* pTResult2 = static_cast<CqParameterTyped<CqVector3D, CqVector3D>*>( pParam2 );
-				TypedNaturalSubdivide( pTParam, pTResult1, pTResult2, u );
-				break;
-			}
-
-			case type_hpoint:
-			{
-				CqParameterTyped<CqVector4D, CqVector3D>* pTParam = static_cast<CqParameterTyped<CqVector4D, CqVector3D>*>( pParam );
-				CqParameterTyped<CqVector4D, CqVector3D>* pTResult1 = static_cast<CqParameterTyped<CqVector4D, CqVector3D>*>( pParam1 );
-				CqParameterTyped<CqVector4D, CqVector3D>* pTResult2 = static_cast<CqParameterTyped<CqVector4D, CqVector3D>*>( pParam2 );
-				TypedNaturalSubdivide( pTParam, pTResult1, pTResult2, u );
-				break;
-			}
-
-
-			case type_color:
-			{
-				CqParameterTyped<CqColor, CqColor>* pTParam = static_cast<CqParameterTyped<CqColor, CqColor>*>( pParam );
-				CqParameterTyped<CqColor, CqColor>* pTResult1 = static_cast<CqParameterTyped<CqColor, CqColor>*>( pParam1 );
-				CqParameterTyped<CqColor, CqColor>* pTResult2 = static_cast<CqParameterTyped<CqColor, CqColor>*>( pParam2 );
-				TypedNaturalSubdivide( pTParam, pTResult1, pTResult2, u );
-				break;
-			}
-
-			case type_string:
-			{
-				CqParameterTyped<CqString, CqString>* pTParam = static_cast<CqParameterTyped<CqString, CqString>*>( pParam );
-				CqParameterTyped<CqString, CqString>* pTResult1 = static_cast<CqParameterTyped<CqString, CqString>*>( pParam1 );
-				CqParameterTyped<CqString, CqString>* pTResult2 = static_cast<CqParameterTyped<CqString, CqString>*>( pParam2 );
-				TypedNaturalSubdivide( pTParam, pTResult1, pTResult2, u );
-				break;
-			}
-
-			case type_matrix:
-			{
-				//			CqParameterTyped<CqMatrix, CqMatrix>* pTParam = static_cast<CqParameterTyped<CqMatrix, CqMatrix>*>( pParam );
-				//			CqParameterTyped<CqMatrix, CqMatrix>* pTResult1 = static_cast<CqParameterTyped<CqMatrix, CqMatrix>*>( pParam1 );
-				//			CqParameterTyped<CqMatrix, CqMatrix>* pTResult2 = static_cast<CqParameterTyped<CqMatrix, CqMatrix>*>( pParam2 );
-				//			TypedNaturalSubdivide( pTParam, pTResult1, pTResult2, u );
-				//			break;
-			}
-
-			default:
-			{
-				break;
-			}
+		case type_float:
+			linCurveNatSubdiv<TqFloat, TqFloat>(pParam, pParam1, pParam2);
+			break;
+		case type_integer:
+			linCurveNatSubdiv<TqInt, TqFloat>(pParam, pParam1, pParam2);
+			break;
+		case type_point:
+		case type_vector:
+		case type_normal:
+			linCurveNatSubdiv<CqVector3D, CqVector3D>(pParam, pParam1, pParam2);
+			break;
+		case type_hpoint:
+			linCurveNatSubdiv<CqVector4D, CqVector3D>(pParam, pParam1, pParam2);
+			break;
+		case type_color:
+			linCurveNatSubdiv<CqColor, CqColor>(pParam, pParam1, pParam2);
+			break;
+		case type_string:
+			linCurveNatSubdiv<CqString, CqString>( pParam, pParam1, pParam2);
+			break;
+		case type_matrix:
+			/// \todo Why is this removed?
+			//linCurveNatSubdiv<CqMatrix, CqMatrix>( pParam, pParam1, pParam2);
+			//break;
+		default:
+			break;
 	}
-
 }
 
 
-
 /**
  * Splits a CqLinearCurveSegment into either two smaller segments or a
  * patch.
@@ -220,7 +170,6 @@ TqInt CqLinearCurveSegment::Split( std::vector<boost::shared_ptr<CqSurface> >& a
 }
 
 
-
 /**
  * Splits a linear curve segment into two smaller curves.
  *
@@ -232,7 +181,6 @@ TqInt CqLinearCurveSegment::SplitToCurves(
     std::vector<boost::shared_ptr<CqSurface> >& aSplits
 )
 {
-
 	// split into more curves
 	//  This bit right here looks a lot like CqSurface::Split().
 	//  The difference is that we *don't* want the default splitter
@@ -243,10 +191,10 @@ TqInt CqLinearCurveSegment::SplitToCurves(
 	aSplits.push_back( boost::shared_ptr<CqSurface>( new CqLinearCurveSegment ) );
 
 	aSplits[ 0 ] ->SetSurfaceParameters( *this );
-	aSplits[ 0 ] ->SetEyeSplitCount( EyeSplitCount() );
+	aSplits[ 0 ] ->SetSplitCount( SplitCount() + 1 );
 
 	aSplits[ 1 ] ->SetSurfaceParameters( *this );
-	aSplits[ 1 ] ->SetEyeSplitCount( EyeSplitCount() );
+	aSplits[ 1 ] ->SetSplitCount( SplitCount() + 1 );
 
 	// Iterate through any user parameters, subdividing and storing
 	//  the second value in the target surface.
@@ -266,7 +214,7 @@ TqInt CqLinearCurveSegment::SplitToCurves(
 			//  handle them the same way as vertex class
 			//  variables for the simple case of a
 			//  CqSingleCurveLinear
-			NaturalSubdivide( ( *iUP ), pNewA, pNewB, TqFalse );
+			NaturalSubdivide( ( *iUP ), pNewA, pNewB, false );
 		}
 		else
 		{
@@ -438,7 +386,7 @@ TqInt CqLinearCurveSegment::SplitToPatch(
  * @param periodic      true if the curves in the group are periodic.
  */
 CqLinearCurvesGroup::CqLinearCurvesGroup(
-    TqInt ncurves, TqInt nvertices[], TqBool periodic
+    TqInt ncurves, TqInt nvertices[], bool periodic
 ) : CqCurvesGroup()
 {
 
@@ -650,57 +598,4 @@ TqInt CqLinearCurvesGroup::Split( std::vector<boost::shared_ptr<CqSurface> >& aS
 
 }
 
-
-
-/**
- * Transforms this GPrim using the specified matrices.
- *
- * @param matTx         Reference to the transformation matrix.
- * @param matITTx       Reference to the inverse transpose of the 
- *                        transformation matrix, used to transform normals.
- * @param matRTx        Reference to the rotation only transformation matrix, 
- *                        used to transform vectors.
- * @param iTime			The frame time at which to perform the transformation.
- */
-void CqLinearCurvesGroup::Transform(
-    const CqMatrix& matTx,
-    const CqMatrix& matITTx,
-    const CqMatrix& matRTx,
-    TqInt iTime
-)
-{
-	// First, we want to transform the width array.  For each curve in the
-	//  group, there are as many width parameters as there are vertices,
-	//  so each vertex matches exactly with a width; no stuffing around is
-	//  required.
-	PopulateWidth();
-	assert( cVarying() == cVertex() );
-	for ( TqUint i = 0; i < cVarying(); i++ )
-	{
-		// first, create a horizontal vector in the new space which is
-		//  the length of the current width in current space
-		CqVector3D horiz( 1, 0, 0 );
-		horiz = matITTx * horiz;
-		horiz *= width()->pValue( i )[0] / horiz.Magnitude();
-
-		// now, create two points; one at the vertex in current space
-		//  and one which is offset horizontally in the new space by
-		//  the width in the current space.  transform both points
-		//  into the new space
-		CqVector3D pt = P()->pValue( i )[0];
-		CqVector3D pt_delta = pt + horiz;
-		pt = matTx * pt;
-		pt_delta = matTx * pt_delta;
-
-		// finally, find the difference between the two points in
-		//  the new space - this is the transformed width
-		CqVector3D widthVector = pt_delta - pt;
-		width()->pValue( i )[0] = widthVector.Magnitude();
-	}
-
-	// finally, we want to call the base class transform
-	CqCurve::Transform( matTx, matITTx, matRTx, iTime );
-}
-
-
-END_NAMESPACE( Aqsis )
+} // namespace Aqsis
