@@ -1,5 +1,5 @@
 // Aqsis
-// Copyright  1997 - 2001, Paul C. Gregory
+// Copyright (C) 1997 - 2001, Paul C. Gregory
 //
 // Contact: pgregory@aqsis.org
 //
@@ -45,13 +45,66 @@
 #include	"refcount.h"
 #include	"logging.h"
 
-START_NAMESPACE( Aqsis )
+namespace Aqsis {
 
 class CqVector3D;
 class CqImageBuffer;
 class CqSurface;
 struct SqSampleData;
 
+
+// This struct holds info about a grid that can be cached and used for all its mpgs.
+//
+// This version of SqGridInfo comes from the multithreading branch, but is
+// backported here to support the "focusfactor" optimizations with the smallest
+// changes from the trunk implementation.
+struct SqGridInfo2
+{
+	TqFloat			m_ShadingRate;
+};
+
+//----------------------------------------------------------------------
+/** \brief Cache of output sample data for a micropoly
+ *
+ * This cache holds colour and opacity output data for a micropolygon.  The
+ * coefficients allow colour and opacity to be interpolated across the face of
+ * the micropolygon.  For CqMicroPolygon, interpolation may be either "smooth"
+ * (uses a linear approximation over the micropoly) or "constant" - using a
+ * constant value for each micropolygon.
+ *
+ * AOV's are handled seperately - they do not support smooth shading in the
+ * current implementation.
+ *
+ * \todo Code Review: This struct is a bit kludgy, holding stuff which should
+ * be only visible to the micropolygon implementation classes.  It's made worse
+ * by the fact that the smooth shading interpolation coefficients are
+ * irrelevant to some micropolygon subclasses (CqMicroPolygonPoints for eg).
+ */
+struct SqMpgSampleInfo
+{
+	/// Color for flat shading and for the "zero" point of smooth shading.
+	CqColor color;
+	/// Color multipilers used for smooth shading interpolation
+	CqColor colorMultX;
+	CqColor colorMultY;
+
+	/// Opacity
+	CqColor opacity;
+	/// Opacity multipilers used for smooth shading interpolation
+	CqColor opacityMultX;
+	CqColor opacityMultY;
+
+	/// Whether to use smooth shading interpolation or not
+	bool smoothInterpolation;
+
+	/// Whether the opacity is full.
+	bool occludes;
+	/// Whether to store samples deriving from this micropoly in the opaque
+	/// sample slot or not.
+	bool isOpaque;
+};
+
+
 //----------------------------------------------------------------------
 /** \class CqMicroPolyGridBase
  * Base class from which all MicroPolyGrids are derived.
@@ -60,7 +113,7 @@ struct SqSampleData;
 class CqMicroPolyGridBase : public CqRefCount
 {
 	public:
-		CqMicroPolyGridBase() : m_fCulled( TqFalse ), m_fTriangular( TqFalse )
+		CqMicroPolyGridBase() : m_fCulled( false ), m_fTriangular( false )
 		{}
 		virtual	~CqMicroPolyGridBase()
 		{}
@@ -75,12 +128,12 @@ class CqMicroPolyGridBase : public CqRefCount
 		virtual	void	Split( CqImageBuffer* pImage, long xmin, long xmax, long ymin, long ymax ) = 0;
 		/** Pure virtual, shade the grid.
 		 */
-		virtual	void	Shade() = 0;
+		virtual	void	Shade(bool canCullGrid = true ) = 0;
 		virtual	void	TransferOutputVariables() = 0;
 		/*
 		 * Delete all the variables per grid 
 		 */
-		virtual void DeleteVariables( TqBool all ) = 0;
+		virtual void DeleteVariables( bool all ) = 0;
 		/** Pure virtual, get a pointer to the surface this grid belongs.
 		 * \return Pointer to surface, only valid during grid shading.
 		 */
@@ -88,21 +141,21 @@ class CqMicroPolyGridBase : public CqRefCount
 
 		virtual	const IqAttributes* pAttributes() const = 0;
 
-		virtual TqBool	usesCSG() const = 0;
+		virtual bool	usesCSG() const = 0;
 		virtual	boost::shared_ptr<CqCSGTreeNode> pCSGNode() const = 0;
-		TqBool vfCulled()
+		bool vfCulled()
 		{
 			return m_fCulled;
 		}
 		/** Query whether this grid is being rendered as a triangle.
 		 */
-		virtual TqBool fTriangular() const
+		virtual bool fTriangular() const
 		{
 			return ( m_fTriangular );
 		}
 		/** Set this grid as being rendered as a triangle or not.
 		 */
-		virtual void SetfTriangular( TqBool fTriangular )
+		virtual void SetfTriangular( bool fTriangular )
 		{
 			m_fTriangular = fTriangular;
 		}
@@ -110,6 +163,7 @@ class CqMicroPolyGridBase : public CqRefCount
 		virtual	TqInt	vGridRes() const = 0;
 		virtual	TqUint	numMicroPolygons(TqInt cu, TqInt cv) const = 0;
 		virtual	TqUint	numShadingPoints(TqInt cu, TqInt cv) const = 0;
+		virtual bool	hasValidDerivatives() const = 0;
 		virtual IqShaderData* pVar(TqInt index) = 0;
 		/** Get the points of the triangle split line if this grid represents a triangle.
 		 */
@@ -125,8 +179,15 @@ class CqMicroPolyGridBase : public CqRefCount
 				CqTriangleSplitLine( const SqTriangleSplitLine& def = SqTriangleSplitLine() ) : CqMotionSpec<SqTriangleSplitLine>( def )
 				{}
 
+				/**
+				* \todo Review: Unused parameter A
+				*/		
 				virtual	void	ClearMotionObject( SqTriangleSplitLine& A ) const
-					{}
+				{}
+
+				/**
+				* \todo Review: Unused parameter B
+				*/		
 				virtual	SqTriangleSplitLine	ConcatMotionObjects( const SqTriangleSplitLine& A, const SqTriangleSplitLine& B ) const
 				{
 					return( A );
@@ -142,10 +203,23 @@ class CqMicroPolyGridBase : public CqRefCount
 		virtual	IqShaderData* FindStandardVar( const char* pname ) = 0;
 		virtual boost::shared_ptr<IqShaderExecEnv> pShaderExecEnv() = 0; 
 
-	public:
-		TqBool m_fCulled; ///< Boolean indicating the entire grid is culled.
+		const SqGridInfo2& GetCachedGridInfo() const
+		{
+			return m_CurrentGridInfo;
+		}
+
+	protected:
+		bool m_fCulled; ///< Boolean indicating the entire grid is culled.
 		CqTriangleSplitLine	m_TriangleSplitLine;	///< Two endpoints of the line that is used to turn the quad into a triangle at sample time.
-		TqBool	m_fTriangular;			///< Flag indicating that this grid should be rendered as a triangular grid with a phantom fourth corner.
+		bool	m_fTriangular;			///< Flag indicating that this grid should be rendered as a triangular grid with a phantom fourth corner.
+
+		/** Cached info about the given grid so it can be
+		 * referenced by multiple mpgs. */
+		SqGridInfo2 m_CurrentGridInfo;
+
+		/** Cache some info about the given grid so it can be
+		 * referenced by multiple mpgs. */
+		void CacheGridInfo(const boost::shared_ptr<const CqSurface>& surface);
 };
 
 
@@ -168,31 +242,43 @@ class CqMicroPolyGrid : public CqMicroPolyGridBase
 		}
 #endif
 
-		void	CalcNormals();
-		void	CalcSurfaceDerivatives();
+		virtual void CalcNormals();
+		virtual void CalcSurfaceDerivatives();
+		/** \brief Expand the boundary micropolygons to cover grid cracks.
+		 *
+		 * This function expands the boundary of a grid by moving the boundary
+		 * vertices outward along the vectors which connect them to the
+		 * interior vertices of the grid.  This is a computationally cheap and
+		 * easy way to cover cracks between adjacent grids which result from
+		 * differing dicing rates.
+		 *
+		 * \param amount - fraction of a micropolygon to expand the boundary
+		 *                 outward by.
+		 */
+		void ExpandGridBoundaries(TqFloat amount);
 		/** Set the shading normals flag, indicating this grid has shading (N) normals already specified.
 		 * \param f The new state of the flag.
 		 */
-		void	SetbShadingNormals( TqBool f )
+		void	SetbShadingNormals( bool f )
 		{
 			m_bShadingNormals = f;
 		}
 		/** Set the geometric normals flag, indicating this grid has geometric (Ng) normals already specified.
 		 * \param f The new state of the flag.
 		 */
-		void	SetbGeometricNormals( TqBool f )
+		void	SetbGeometricNormals( bool f )
 		{
 			m_bGeometricNormals = f;
 		}
 		/** Query whether shading (N) normals have been filled in by the surface at dice time.
 		 */
-		TqBool bShadingNormals() const
+		bool bShadingNormals() const
 		{
 			return ( m_bShadingNormals );
 		}
 		/** Query whether geometric (Ng) normals have been filled in by the surface at dice time.
 		 */
-		TqBool bGeometricNormals() const
+		bool bGeometricNormals() const
 		{
 			return ( m_bGeometricNormals );
 		}
@@ -212,11 +298,11 @@ class CqMicroPolyGrid : public CqMicroPolyGridBase
 
 		void	Initialise( TqInt cu, TqInt cv, const boost::shared_ptr<CqSurface>& pSurface );
 
-		void DeleteVariables( TqBool all );
+		void DeleteVariables( bool all );
 
 		// Overrides from CqMicroPolyGridBase
 		virtual	void	Split( CqImageBuffer* pImage, long xmin, long xmax, long ymin, long ymax );
-		virtual	void	Shade();
+		virtual	void	Shade( bool canCullGrid = true );
 		virtual	void	TransferOutputVariables();
 
 		/** Get a pointer to the surface which this grid belongs.
@@ -231,9 +317,9 @@ class CqMicroPolyGrid : public CqMicroPolyGridBase
 			assert( m_pShaderExecEnv );
 			return ( m_pShaderExecEnv->pAttributes() );
 		}
-		virtual TqBool	usesCSG() const
+		virtual bool	usesCSG() const
 		{
-			TqBool result = (m_pCSGNode.get() != NULL);
+			bool result = (m_pCSGNode.get() != NULL);
 			return(result);
 		}
 		virtual	boost::shared_ptr<CqCSGTreeNode> pCSGNode() const
@@ -260,6 +346,10 @@ class CqMicroPolyGrid : public CqMicroPolyGridBase
 		{
 			return ( ( cu + 1 ) * ( cv + 1 ) );
 		}
+		virtual bool	hasValidDerivatives() const
+		{
+			return true;
+		}
 		virtual	const CqMatrix&	matObjectToWorld() const
 		{
 			assert( m_pShaderExecEnv );
@@ -292,9 +382,20 @@ class CqMicroPolyGrid : public CqMicroPolyGridBase
 			return(m_pShaderExecEnv);
 		}
 
+		/** \brief Set surface derivative in u.
+		 * 
+		 *  Set the value of du if needed, du is constant across the grid being shaded.
+		 */
+		virtual void setDu();
+		/** \brief Set surface derivative in v.
+		 * 
+		 *  Set the value of dv if needed, dv is constant across the grid being shaded.
+		 */
+		virtual void setDv();
+
 	private:
-		TqBool	m_bShadingNormals;		///< Flag indicating shading normals have been filled in and don't need to be calculated during shading.
-		TqBool	m_bGeometricNormals;	///< Flag indicating geometric normals have been filled in and don't need to be calculated during shading.
+		bool	m_bShadingNormals;		///< Flag indicating shading normals have been filled in and don't need to be calculated during shading.
+		bool	m_bGeometricNormals;	///< Flag indicating geometric normals have been filled in and don't need to be calculated during shading.
 		boost::shared_ptr<CqSurface> m_pSurface;	///< Pointer to the surface for this grid.
 		boost::shared_ptr<CqCSGTreeNode> m_pCSGNode;	///< Pointer to the CSG tree node this grid belongs to, NULL if not part of a solid.
 		CqBitVector	m_CulledPolys;		///< Bitvector indicating whether the individual micro polygons are culled.
@@ -321,12 +422,15 @@ class CqMotionMicroPolyGrid : public CqMicroPolyGridBase, public CqMotionSpec<Cq
 
 
 		virtual	void	Split( CqImageBuffer* pImage, long xmin, long xmax, long ymin, long ymax );
-		virtual	void	Shade();
+		virtual	void	Shade( bool canCullGrid = true );
 		virtual	void	TransferOutputVariables();
-		void DeleteVariables( TqBool all )
+		
+		/**
+		* \todo Review: Unused parameter all
+		*/		
+		void DeleteVariables( bool all )
 		{}
 
-
 		// Redirect acces via IqShaderExecEnv
 		virtual	TqInt	uGridRes() const
 		{
@@ -348,6 +452,12 @@ class CqMotionMicroPolyGrid : public CqMicroPolyGridBase, public CqMotionSpec<Cq
 			assert( GetMotionObject( Time( 0 ) ) );
 			return ( static_cast<CqMicroPolyGrid*>( GetMotionObject( Time( 0 ) ) ) ->numShadingPoints(cu, cv) );
 		}
+		virtual bool	hasValidDerivatives() const
+		{
+			assert( GetMotionObject( Time( 0 ) ) );
+			return ( static_cast<CqMicroPolyGrid*>( GetMotionObject( Time( 0 ) ) ) ->hasValidDerivatives() );
+		}
+
 		virtual IqShaderData* pVar(TqInt index)
 		{
 			assert( GetMotionObject( Time( 0 ) ) );
@@ -379,7 +489,7 @@ class CqMotionMicroPolyGrid : public CqMicroPolyGridBase, public CqMotionSpec<Cq
 			return ( static_cast<CqMicroPolyGrid*>( GetMotionObject( Time( 0 ) ) ) ->pAttributes() );
 		}
 
-		virtual TqBool	usesCSG() const
+		virtual bool	usesCSG() const
 		{
 			return(static_cast<CqMicroPolyGrid*>( GetMotionObject( Time( 0 ) ) ) ->usesCSG());
 		}
@@ -388,20 +498,30 @@ class CqMotionMicroPolyGrid : public CqMicroPolyGridBase, public CqMotionSpec<Cq
 			return ( static_cast<CqMicroPolyGrid*>( GetMotionObject( Time( 0 ) ) ) ->pCSGNode() );
 		}
 
+		/**
+		* \todo Review: Unused parameter A
+		*/		
 		virtual	void	ClearMotionObject( CqMicroPolyGridBase*& A ) const
-			{}
+		{}
+
 		/** Overridden from CqMotionSpec, does nothing.
 		 */
-		virtual	CqMicroPolyGridBase* ConcatMotionObjects( CqMicroPolyGridBase* const & A, CqMicroPolyGridBase* const & B ) const
+		virtual	CqMicroPolyGridBase* ConcatMotionObjects( CqMicroPolyGridBase* const & /* A */, CqMicroPolyGridBase* const & B ) const
 		{
 			return ( B );
 		}
 		/** Overridden from CqMotionSpec, does nothing.
 		 */
-		virtual	CqMicroPolyGridBase* LinearInterpolateMotionObjects( TqFloat Fraction, CqMicroPolyGridBase* const & A, CqMicroPolyGridBase* const & B ) const
+		virtual	CqMicroPolyGridBase* LinearInterpolateMotionObjects( TqFloat /* Fraction */, CqMicroPolyGridBase* const & A, CqMicroPolyGridBase* const & /* B */ ) const
 		{
 			return ( A );
 		}
+		void	Initialise( TqInt cu, TqInt cv, const boost::shared_ptr<CqSurface>& pSurface )
+		{
+			//assert( GetMotionObject( Time( 0 ) ) );
+			//static_cast<CqMicroPolyGrid*>( GetMotionObject( Time( 0 ) ) ) ->Initialise(cu, cv, pSurface);
+			CacheGridInfo(pSurface);
+		}
 
 	private:
 };
@@ -441,6 +561,7 @@ class CqMicroPolygon : public CqRefCount
 		virtual	~CqMicroPolygon();
 
 		/** Overridden operator new to allocate micropolys from a pool.
+		 * \todo Review: Unused parameter size
 		 */
 		void* operator new( size_t size )
 		{
@@ -480,10 +601,10 @@ class CqMicroPolygon : public CqRefCount
 	private:
 		enum EqMicroPolyFlags
 		{
-		    MicroPolyFlags_Trimmed	= 0x0001,
-		    MicroPolyFlags_Hit	= 0x0002,
-		    MicroPolyFlags_PushedForward	= 0x0004,
-	};
+			MicroPolyFlags_Trimmed		= 0x0001,
+			MicroPolyFlags_Hit		= 0x0002,
+			MicroPolyFlags_PushedForward	= 0x0004,
+		};
 
 	public:
 		/** Set up the pointer to the grid this micropoly came from.
@@ -515,7 +636,7 @@ class CqMicroPolygon : public CqRefCount
 		 */
 		void	SetIndex( TqInt Index )
 		{
-			assert( m_pGrid != 0 && m_pGrid->pShaderExecEnv()->shadingPointCount() > Index );
+			assert( m_pGrid != 0 && static_cast<TqInt>(m_pGrid->pShaderExecEnv()->shadingPointCount()) > Index );
 			m_Index = Index;
 		}
 		/** Release this micropolys reference to the donor grid.
@@ -568,6 +689,10 @@ class CqMicroPolygon : public CqRefCount
 		{
 			return ( 1 );
 		}
+
+		/**
+		* \todo Review: Unused parameter iIndex
+		*/				
 		virtual	CqBound SubBound( TqInt iIndex, TqFloat& time )
 		{
 			time = 0.0f;
@@ -576,7 +701,7 @@ class CqMicroPolygon : public CqRefCount
 
 		/** Query if the micropolygon has been successfully hit by a pixel sample.
 		 */
-		TqBool IsHit() const
+		bool IsHit() const
 		{
 			return ( ( m_Flags & MicroPolyFlags_Hit ) != 0 );
 		}
@@ -588,7 +713,7 @@ class CqMicroPolygon : public CqRefCount
 		}
 		/** Get the flag indicating if the micropoly has already beed pushed forward to the next bucket.
 		 */
-		TqBool	IsPushedForward() const
+		bool	IsPushedForward() const
 		{
 			return ( ( m_Flags & MicroPolyFlags_PushedForward ) != 0 );
 		}
@@ -602,14 +727,14 @@ class CqMicroPolygon : public CqRefCount
 		{
 			m_Flags |= MicroPolyFlags_Trimmed;
 		}
-		virtual TqBool	IsTrimmed() const
+		virtual bool	IsTrimmed() const
 		{
 			return ( ( m_Flags & MicroPolyFlags_Trimmed ) != 0 );
 		}
 
-		virtual TqBool IsMoving()
+		virtual bool IsMoving()
 		{
-			return TqFalse;
+			return false;
 		}
 
 		/** Check if the sample point is within the micropoly.
@@ -618,12 +743,34 @@ class CqMicroPolygon : public CqRefCount
 		 * \param D storage to put the depth at the sample point if success.
 		 * \return Boolean success.
 		 */
-		virtual	TqBool	Sample( const SqSampleData& sample, TqFloat& D, TqFloat time, TqBool UsingDof = TqFalse );
+		virtual	bool	Sample( const SqSampleData& sample, TqFloat& D, TqFloat time, bool UsingDof = false );
 
-		virtual TqBool	fContains( const CqVector2D& vecP, TqFloat& Depth, TqFloat time ) const;
+		virtual bool	fContains( const CqVector2D& vecP, TqFloat& Depth, TqFloat time ) const;
 		virtual void	CacheHitTestValues(CqHitTestCache* cache, CqVector3D* points);
 		virtual void	CacheHitTestValues(CqHitTestCache* cache);
 		virtual void	CacheHitTestValuesDof(CqHitTestCache* cache, const CqVector2D& DofOffset, CqVector2D* coc);
+
+		/** \brief Cache information needed to interpolate colour and opacity
+		 * across the micropolygon.
+		 *
+		 * This function should caches constant values for the micropolygon, or
+		 * computes the interpolation coefficients necessary for smooth
+		 * shading.
+		 *
+		 * \param cache - location into which to store the interpolation coefficients.
+		 */
+		virtual void CacheOutputInterpCoeffs(SqMpgSampleInfo& cache) const;
+
+		/** \brief Get colour and opacity at pos using cached coefficients
+		 *
+		 * \param cache - previously cached coefficients for this micropolygon
+		 * \param pos - position to evaluate color and opacity at
+		 * \param outCol - interpolated colour output will be placed here.
+		 * \param outOpac - interpolated opacity output will be placed here.
+		 */
+		virtual void InterpolateOutputs(const SqMpgSampleInfo& cache,
+				const CqVector2D& pos, CqColor& outCol, CqColor& outOpac) const;
+
 		void	Initialise();
 		CqVector2D ReverseBilinear( const CqVector2D& v );
 
@@ -651,12 +798,21 @@ class CqMicroPolygon : public CqRefCount
 			m_pGrid->pVar(EnvVars_P) ->GetPointPtr( pP );
 			return ( pP[ GetCodedIndex( m_IndexCode, 3 ) ] );
 		}
-		virtual const TqBool IsDegenerate() const
+		virtual const bool IsDegenerate() const
 		{
 			return ( ( m_IndexCode & 0x8000000 ) != 0 );
 		}
 
 	protected:
+		/** \brief Cache output interpolation coefficients for constant shading
+		 * \see CacheOutputInterpCoeffs
+		 */
+		virtual void CacheOutputInterpCoeffsConstant(SqMpgSampleInfo& cache) const;
+		/** \brief Cache output interpolation coefficients for smooth shading
+		 * \see CacheOutputInterpCoeffs
+		 */
+		virtual void CacheOutputInterpCoeffsSmooth(SqMpgSampleInfo& cache) const;
+
 		TqInt GetCodedIndex( TqShort code, TqShort shift ) const
 		{
 			switch ( ( ( code >> ( shift << 1 ) ) & 0x3 ) )
@@ -682,8 +838,11 @@ class CqMicroPolygon : public CqRefCount
 
 		CqHitTestCache* m_pHitTestCache; // struct to hold cached values used in the point-in-poly test
 	private:
-		CqMicroPolygon( const CqMicroPolygon& From )
-	{}
+		/**
+		* \todo Review: operator= defined, but empty copy-ctor.
+		*/
+		CqMicroPolygon( const CqMicroPolygon& From)
+		{}
 
 		static	CqObjectPool<CqMicroPolygon> m_thePool;
 }
@@ -709,6 +868,7 @@ class CqMovingMicroPolygonKey
 		{}
 
 		/** Overridden operator new to allocate micropolys from a pool.
+		 * \todo Review: Unused parameter size
 		 */
 		void* operator new( size_t size )
 		{
@@ -728,7 +888,7 @@ class CqMovingMicroPolygonKey
 		void	Initialise( const CqVector3D& vA, const CqVector3D& vB, const CqVector3D& vC, const CqVector3D& vD );
 		CqVector2D ReverseBilinear( const CqVector2D& v );
 
-		const TqBool IsDegenerate() const
+		const bool IsDegenerate() const
 		{
 			return ( m_Point2 == m_Point3 );
 		}
@@ -738,10 +898,12 @@ class CqMovingMicroPolygonKey
 		CqVector3D	m_Point2;
 		CqVector3D	m_Point3;
 		CqVector3D	m_N;			///< The normal to the micropoly.
+
+	protected:
 		TqFloat	m_D;				///< Distance of the plane from the origin, used for calculating sample depth.
 
 		CqBound m_Bound;
-		TqBool	m_BoundReady;
+		bool	m_BoundReady;
 
 		static	CqObjectPool<CqMovingMicroPolygonKey>	m_thePool;
 }
@@ -756,7 +918,7 @@ class CqMovingMicroPolygonKey
 class CqMicroPolygonMotion : public CqMicroPolygon
 {
 	public:
-		CqMicroPolygonMotion() : CqMicroPolygon(), m_BoundReady( TqFalse )
+		CqMicroPolygonMotion() : CqMicroPolygon(), m_BoundReady( false )
 		{ }
 		virtual	~CqMicroPolygonMotion()
 		{
@@ -782,7 +944,7 @@ class CqMicroPolygonMotion : public CqMicroPolygon
 
 	public:
 		void	AppendKey( const CqVector3D& vA, const CqVector3D& vB, const CqVector3D& vC, const CqVector3D& vD, TqFloat time );
-		void	DeleteVariables( TqBool all )
+		void	DeleteVariables( bool all )
 		{}
 
 		// Overrides from CqMicroPolygon
@@ -805,25 +967,25 @@ class CqMicroPolygonMotion : public CqMicroPolygon
 		{
 			if ( !m_BoundReady )
 				BuildBoundList();
-			assert( iIndex < m_BoundList.Size() );
+			assert( iIndex < static_cast<TqInt>(m_BoundList.Size()) );
 			time = m_BoundList.GetTime( iIndex );
 			return ( m_BoundList.GetBound( iIndex ) );
 		}
 		virtual void	BuildBoundList();
 
-		virtual	TqBool	Sample( const SqSampleData& sample, TqFloat& D, TqFloat time, TqBool UsingDof = TqFalse );
+		virtual	bool	Sample( const SqSampleData& sample, TqFloat& D, TqFloat time, bool UsingDof = false );
 
 		virtual void	MarkTrimmed()
 		{
-			m_fTrimmed = TqTrue;
+			m_fTrimmed = true;
 		}
 
-		virtual TqBool IsMoving()
+		virtual bool IsMoving()
 		{
-			return TqTrue;
+			return true;
 		}
 
-		virtual const TqBool IsDegenerate() const
+		virtual const bool IsDegenerate() const
 		{
 			return ( m_Keys[0]->IsDegenerate() );
 		}
@@ -835,32 +997,56 @@ class CqMicroPolygonMotion : public CqMicroPolygon
 
 		virtual TqFloat Time(TqInt index) const
 		{
-			assert(index < m_Times.size());
+			assert( index < static_cast<TqInt>(m_Times.size()) );
 			return(m_Times[index]);
 		}
 
 		virtual CqMovingMicroPolygonKey* Key(TqInt index) const
 		{
-			assert(index < m_Keys.size());
+			assert( index < static_cast<TqInt>(m_Keys.size()) );
 			return(m_Keys[index]);
 		}
 
-	private:
+	protected:
 		CqBoundList	m_BoundList;			///< List of bounds to get a tighter fit.
-		TqBool	m_BoundReady;				///< Flag indicating the boundary has been initialised.
+		bool	m_BoundReady;				///< Flag indicating the boundary has been initialised.
 		std::vector<TqFloat> m_Times;
 		std::vector<CqMovingMicroPolygonKey*>	m_Keys;
-		TqBool	m_fTrimmed;		///< Flag indicating that the MPG spans a trim curve.
+		bool	m_fTrimmed;		///< Flag indicating that the MPG spans a trim curve.
 
+		/**
+		* \todo Review Empty copy-ctor, but default operator=!
+		*/
 		CqMicroPolygonMotion( const CqMicroPolygonMotion& From )
 		{}
 }
 ;
 
+//==============================================================================
+// Implementation details
+//==============================================================================
+inline void CqMicroPolyGrid::setDu()
+{
+	float f0 = 0;
+	float f1 = 0;
+	pVar(EnvVars_u)->GetValue(f0, 0);
+	pVar(EnvVars_u)->GetValue(f1, 1);
+	pVar(EnvVars_du)->SetFloat(f1 - f0);
+}
+
+inline void CqMicroPolyGrid::setDv()
+{
+	float f0 = 0;
+	float f1 = 0;
+	pVar(EnvVars_v)->GetValue(f0, 0);
+	pVar(EnvVars_v)->GetValue(f1, uGridRes() + 1);
+	pVar(EnvVars_dv)->SetFloat(f1 - f0);
+}
+
 
 //-----------------------------------------------------------------------
 
-END_NAMESPACE( Aqsis )
+} // namespace Aqsis
 
 #endif	// !MICROPOLYGON_H_INCLUDED
 
