@@ -35,14 +35,15 @@
 #include	"imagebuffer.h"
 #include	"bilinear.h"
 #include	"attributes.h"
+#include	"aqsismath.h"
 
-START_NAMESPACE( Aqsis )
+namespace Aqsis {
 
 //---------------------------------------------------------------------
 /** Constructor.
  */
 
-CqSurfaceNURBS::CqSurfaceNURBS() : CqSurface(), m_uOrder( 0 ), m_vOrder( 0 ), m_cuVerts( 0 ), m_cvVerts( 0 ), m_umin( 0.0f ), m_umax( 1.0f ), m_vmin( 0.0f ), m_vmax( 1.0f ), m_fPatchMesh( TqFalse )
+CqSurfaceNURBS::CqSurfaceNURBS() : CqSurface(), m_uOrder( 0 ), m_vOrder( 0 ), m_cuVerts( 0 ), m_cvVerts( 0 ), m_umin( 0.0f ), m_umax( 1.0f ), m_vmin( 0.0f ), m_vmax( 1.0f ), m_fPatchMesh( false )
 {
 	TrimLoops() = static_cast<const CqAttributes*>( pAttributes() ) ->TrimLoops();
 
@@ -314,6 +315,8 @@ void CqSurfaceNURBS::DersBasisFunctions( TqFloat u, TqUint i, std::vector<TqFloa
 
 //---------------------------------------------------------------------
 /** Evaluate the nurbs surface at parameter values u,v.
+ *
+ * \todo Code Review: Unused function [except by deprecated CqSurfaceNURBS::GenerateGeometricNormals() ]
  */
 
 CqVector4D	CqSurfaceNURBS::EvaluateWithNormal( TqFloat u, TqFloat v, CqVector4D& P )
@@ -330,12 +333,12 @@ CqVector4D	CqSurfaceNURBS::EvaluateWithNormal( TqFloat u, TqFloat v, CqVector4D&
 	std::vector<std::vector<TqFloat> > Nu, Nv;
 	std::vector<CqVector4D> temp( q + 1 );
 
-	TqInt du = MIN( d, p );
+	TqInt du = min( d, p );
 	for ( k = p + 1; k <= d; k++ )
 		for ( l = 0; l <= d - k; l++ )
 			SKL[ k ][ l ] = CqVector4D( 0.0f, 0.0f, 0.0f, 1.0f );
 
-	TqInt dv = MIN( d, q );
+	TqInt dv = min( d, q );
 	for ( l = q + 1; l <= d; l++ )
 		for ( k = 0; k <= d - l; k++ )
 			SKL[ k ][ l ] = CqVector4D( 0.0f, 0.0f, 0.0f, 1.0f );
@@ -353,7 +356,7 @@ CqVector4D	CqSurfaceNURBS::EvaluateWithNormal( TqFloat u, TqFloat v, CqVector4D&
 			for ( r = 0; r <= p; r++ )
 				temp[ s ] = temp[ s ] + Nu[ k ][ r ] * CP( uspan - p + r, vspan - q + s );
 		}
-		TqInt dd = MIN( d - k, dv );
+		TqInt dd = min( d - k, dv );
 		for ( l = 0; l <= dd; l++ )
 		{
 			SKL[ k ][ l ] = CqVector4D( 0.0f, 0.0f, 0.0f, 1.0f );
@@ -421,22 +424,13 @@ TqUint CqSurfaceNURBS::InsertKnotU( TqFloat u, TqInt r )
 	if ( r <= 0 )
 		return ( 0 );
 
-
-	m_cuVerts = m_cuVerts + r;
-	m_auKnots.resize( m_cuVerts + m_uOrder );
-
-	std::vector<TqFloat>	auHold( m_auKnots );
-	// Load new knot vector
-	// Copy up to the insertion point.
-	for ( i = 0;i <= k;i++ )
-		m_auKnots[ i ] = auHold[ i ];
-	// Add the specified value 'r' times at the insertion point.
-	for ( i = 1;i <= r;i++ )
-		m_auKnots[ k + i ] = u;
-	// Copy after the insertion point up to the end.
-	for ( i = k + 1; i < static_cast<TqInt>( m_auKnots.size() ); i++ )
-		m_auKnots[ i + r ] = auHold[ i ];
-
+	std::vector<TqFloat> auHold(m_auKnots);
+	// Reserve more space for new knots.
+	m_cuVerts += r;
+	m_auKnots.reserve( m_cuVerts + m_uOrder );
+	// Insert r new knots.
+	std::vector<TqFloat> newKnots(r, u);
+	m_auKnots.insert(m_auKnots.begin()+(k+1), newKnots.begin(), newKnots.end());
 
 	// Now process all the 'vertex' class variables.
 	std::vector<CqParameter*>::iterator iUP;
@@ -610,19 +604,13 @@ TqUint CqSurfaceNURBS::InsertKnotV( TqFloat v, TqInt r )
 	if ( r <= 0 )
 		return ( 0 );
 
-	// Work on a copy.
-	m_cvVerts = m_cvVerts + r;
-	m_avKnots.resize( m_cvVerts + m_vOrder );
-	std::vector<TqFloat>	avHold( m_avKnots );
-
-	// Load new knot vector
-	for ( i = 0;i <= k;i++ )
-		m_avKnots[ i ] = avHold[ i ];
-	for ( i = 1;i <= r;i++ )
-		m_avKnots[ k + i ] = v;
-	size = static_cast<TqInt>( m_avKnots.size() );
-	for ( i = k + 1;i < size; i++ )
-		m_avKnots[ i + r ] = avHold[ i ];
+	std::vector<TqFloat> avHold(m_avKnots);
+	// Reserve more space for new knots.
+	m_cvVerts += r;
+	m_avKnots.reserve( m_cvVerts + m_vOrder );
+	// Insert r new knots
+	std::vector<TqFloat> newKnots(r, v);
+	m_avKnots.insert(m_avKnots.begin()+(k+1), newKnots.begin(), newKnots.end());
 
 	// Now process all the 'vertex' class variables.
 	std::vector<CqParameter*>::iterator iUP;
@@ -1210,7 +1198,7 @@ void CqSurfaceNURBS::ClampV()
 /** Split this NURBS surface into two subsurfaces along u or v depending on dirflag (TRUE=u)
  */
 
-void CqSurfaceNURBS::SplitNURBS( CqSurfaceNURBS& nrbA, CqSurfaceNURBS& nrbB, TqBool dirflag )
+void CqSurfaceNURBS::SplitNURBS( CqSurfaceNURBS& nrbA, CqSurfaceNURBS& nrbB, bool dirflag )
 {
 	std::vector<TqFloat>& aKnots = ( dirflag ) ? m_auKnots : m_avKnots;
 	TqUint Order = ( dirflag ) ? m_uOrder : m_vOrder;
@@ -1335,7 +1323,7 @@ void CqSurfaceNURBS::uSubdivide( CqSurfaceNURBS*& pnrbA, CqSurfaceNURBS*& pnrbB
 	pnrbA = new CqSurfaceNURBS();
 	pnrbB = new CqSurfaceNURBS();
 
-	SplitNURBS( *pnrbA, *pnrbB, TqTrue );
+	SplitNURBS( *pnrbA, *pnrbB, true );
 
 	uSubdivideUserParameters( pnrbA, pnrbB );
 }
@@ -1350,7 +1338,7 @@ void CqSurfaceNURBS::vSubdivide( CqSurfaceNURBS*& pnrbA, CqSurfaceNURBS*& pnrbB
 	pnrbA = new CqSurfaceNURBS();
 	pnrbB = new CqSurfaceNURBS();
 
-	SplitNURBS( *pnrbA, *pnrbB, TqFalse );
+	SplitNURBS( *pnrbA, *pnrbB, false );
 
 	vSubdivideUserParameters( pnrbA, pnrbB );
 }
@@ -1360,7 +1348,7 @@ void CqSurfaceNURBS::vSubdivide( CqSurfaceNURBS*& pnrbA, CqSurfaceNURBS*& pnrbB
 /** Return the boundary extents in object space of the surface patch
  */
 
-CqBound CqSurfaceNURBS::Bound() const
+void CqSurfaceNURBS::Bound(IqBound* bound) const
 {
 	// Get the boundary in camera space.
 	CqVector3D	vecA( FLT_MAX, FLT_MAX, FLT_MAX );
@@ -1382,11 +1370,10 @@ CqBound CqSurfaceNURBS::Bound() const
 		if ( vecV.z() > vecB.z() )
 			vecB.z( vecV.z() );
 	}
-	CqBound	B;
-	B.vecMin() = vecA;
-	B.vecMax() = vecB;
+	bound->vecMin() = vecA;
+	bound->vecMax() = vecB;
 
-	return ( AdjustBoundForTransformationMotion( B ) );
+	AdjustBoundForTransformationMotion( bound );
 }
 
 
@@ -1522,14 +1509,34 @@ void CqSurfaceNURBS::NaturalDice( CqParameter* pParameter, TqInt uDiceSize, TqIn
 /** Generate the vertex normals if not specified.
  */
 
+/*
+
+// This function used to generate the normals for NURBS, but was disabled in
+// the old cvs repository:
+//
+//   Revision 1.19 - (view) (download) (annotate) - [select for diffs]
+//   Tue Oct 8 20:10:41 2002 UTC (5 years, 8 months ago) by pgregory
+//   Branch: MAIN
+//   Changes since 1.18: +2 -1 lines
+//   Diff to previous 1.18
+//
+//   Fix some bugs in the 'clamping' of NURBS surfaces.
+//   Disable generation of normals as there is a problem with it at the moment. Let the grid generate normals until it is fixed.
+//
+// If this code was to be resurrected, it should be moved into the more modern
+// DiceAll() function, which would also resolve the todo.  We'd need to recall
+// what the "problem" was with the old version - possibly something to do with
+// degenerate control points and tangents?
+
+
 void CqSurfaceNURBS::GenerateGeometricNormals( TqInt uDiceSize, TqInt vDiceSize, IqShaderData* pNormals )
 {
 	// Get the handedness of the coordinate system (at the time of creation) and
 	// the coordinate system specified, to check for normal flipping.
 	assert( NULL != P() );
 
-	TqBool CSO = pTransform()->GetHandedness(pTransform()->Time(0));
-	TqBool O = pAttributes() ->GetIntegerAttribute( "System", "Orientation" ) [ 0 ] != 0;
+	bool CSO = pTransform()->GetHandedness(pTransform()->Time(0));
+	bool O = pAttributes() ->GetIntegerAttribute( "System", "Orientation" ) [ 0 ] != 0;
 
 	CqVector3D	N;
 	CqVector4D P;
@@ -1554,6 +1561,7 @@ void CqSurfaceNURBS::GenerateGeometricNormals( TqInt uDiceSize, TqInt vDiceSize,
 	}
 }
 
+*/
 
 //---------------------------------------------------------------------
 /** Split the patch into smaller patches.
@@ -1573,9 +1581,9 @@ TqInt CqSurfaceNURBS::Split( std::vector<boost::shared_ptr<CqSurface> >& aSplits
 		{
 			S[ i ] ->SetSurfaceParameters( *this );
 			S[ i ] ->TrimLoops() = TrimLoops();
-			S[ i ] ->m_fDiceable = TqTrue;
+			S[ i ] ->m_fDiceable = true;
 			S[ i ] ->m_SplitDir = m_SplitDir;
-			S[ i ] ->m_EyeSplitCount = m_EyeSplitCount;
+			S[ i ] ->SetSplitCount( SplitCount() + 1 );
 			//ADDREF( S[ i ] );
 			aSplits.push_back( S[ i ] );
 		}
@@ -1607,14 +1615,14 @@ TqInt CqSurfaceNURBS::Split( std::vector<boost::shared_ptr<CqSurface> >& aSplits
 	pNew2->SetSurfaceParameters( *this );
 	pNew1->TrimLoops() = TrimLoops();
 	pNew2->TrimLoops() = TrimLoops();
-	pNew1->m_fDiceable = TqTrue;
-	pNew2->m_fDiceable = TqTrue;
+	pNew1->m_fDiceable = true;
+	pNew2->m_fDiceable = true;
 	pNew1->m_SplitDir = ( m_SplitDir == SplitDir_U )? SplitDir_V : SplitDir_U;
 	pNew2->m_SplitDir = ( m_SplitDir == SplitDir_U )? SplitDir_V : SplitDir_U;
-	pNew1->m_EyeSplitCount = m_EyeSplitCount;
-	pNew2->m_EyeSplitCount = m_EyeSplitCount;
-	pNew1->SetfPatchMesh( TqFalse );
-	pNew2->SetfPatchMesh( TqFalse );
+	pNew1->SetSplitCount( SplitCount() + 1 );
+	pNew2->SetSplitCount( SplitCount() + 1 );
+	pNew1->SetfPatchMesh( false );
+	pNew2->SetfPatchMesh( false );
 
 	aSplits.push_back( pNew1 );
 	aSplits.push_back( pNew2 );
@@ -1641,14 +1649,14 @@ TqInt CqSurfaceNURBS::Split( std::vector<boost::shared_ptr<CqSurface> >& aSplits
 /** Return whether or not the patch is diceable
  */
 
-TqBool	CqSurfaceNURBS::Diceable()
+bool	CqSurfaceNURBS::Diceable()
 {
 	assert( NULL != P() );
 
 	// If the cull check showed that the primitive cannot be diced due to crossing the e and hither planes,
 	// then we can return immediately.
 	if ( !m_fDiceable )
-		return ( TqFalse );
+		return ( false );
 
 	// Otherwise we should continue to try to find the most advantageous split direction, OR the dice size.
 	// Convert the control hull to raster space.
@@ -1665,9 +1673,8 @@ TqBool	CqSurfaceNURBS::Diceable()
 	if (gs >= 1.0)
 		gridsize = gs * gs;
 
-	TqFloat ShadingRateSqrt = pAttributes() ->GetFloatAttribute( "System", "ShadingRateSqrt" ) [ 0 ];
-
-	const CqMatrix& matCtoR = QGetRenderContext() ->matSpaceToSpace( "camera", "raster", NULL, pTransform().get(), QGetRenderContext()->Time() );
+	CqMatrix matCtoR;
+	QGetRenderContext() ->matSpaceToSpace( "camera", "raster", NULL, pTransform().get(), QGetRenderContext()->Time(), matCtoR );
 	for ( i = 0; i < m_cuVerts*m_cvVerts; i++ )
 	{
 		CqVector3D vT = P()->pValue( i )[0];
@@ -1707,27 +1714,28 @@ TqBool	CqSurfaceNURBS::Diceable()
 	{
 		m_SplitDir = ( MaxuLen > MaxvLen ) ? SplitDir_U : SplitDir_V;
 		delete[] ( avecHull );
-		return ( TqFalse );
+		return ( false );
 	}
 
-	MaxuLen /= ShadingRateSqrt;
-	MaxvLen /= ShadingRateSqrt;
-	m_uDiceSize = static_cast<TqUint>( MAX( ROUND( MaxuLen ), 1 ) );
-	m_vDiceSize = static_cast<TqUint>( MAX( ROUND( MaxvLen ), 1 ) );
+	TqFloat sqrtShadingRate = sqrt(AdjustedShadingRate());
+	MaxuLen /= sqrtShadingRate;
+	MaxvLen /= sqrtShadingRate;
+	m_uDiceSize = max<TqInt>(lround(MaxuLen), 1);
+	m_vDiceSize = max<TqInt>(lround(MaxvLen), 1);
 
 	// Ensure power of 2 to avoid cracking
 	const TqInt *binary = pAttributes() ->GetIntegerAttribute( "dice", "binary" );
 	if ( binary && *binary)
 	{
-		m_uDiceSize = CEIL_POW2( m_uDiceSize );
-		m_vDiceSize = CEIL_POW2( m_vDiceSize );
+		m_uDiceSize = ceilPow2( m_uDiceSize );
+		m_vDiceSize = ceilPow2( m_vDiceSize );
 	}
 
 	if ( MaxuLen < FLT_EPSILON || MaxvLen < FLT_EPSILON )
 	{
-		m_fDiscard = TqTrue;
+		m_fDiscard = true;
 		delete[] ( avecHull );
-		return ( TqFalse );
+		return ( false );
 	}
 
 
@@ -1735,11 +1743,11 @@ TqBool	CqSurfaceNURBS::Diceable()
 	m_SplitDir = ( MaxuLen > MaxvLen ) ? SplitDir_U : SplitDir_V;
 
 	if ( m_uDiceSize > gs)
-		return TqFalse;
+		return false;
 	if ( m_vDiceSize > gs)
-		return TqFalse;
+		return false;
 
-	return ( TqTrue );
+	return ( true );
 }
 
 
@@ -1753,7 +1761,8 @@ TqInt	CqSurfaceNURBS::TrimDecimation( const CqTrimCurve& Curve )
 	TqFloat Len = 0;
 	TqFloat MaxLen = 0;
 	TqInt cSegments = 0;
-	CqMatrix matCtoR = QGetRenderContext() ->matSpaceToSpace( "camera", "raster", NULL, pTransform().get(), QGetRenderContext()->Time() );
+	CqMatrix matCtoR;
+	QGetRenderContext() ->matSpaceToSpace( "camera", "raster", NULL, pTransform().get(), QGetRenderContext()->Time(), matCtoR );
 
 	TqUint iTrimCurvePoint;
 	for ( iTrimCurvePoint = 0; iTrimCurvePoint < Curve.cVerts() - 1; iTrimCurvePoint++ )
@@ -1781,10 +1790,9 @@ TqInt	CqSurfaceNURBS::TrimDecimation( const CqTrimCurve& Curve )
 			MaxLen = Len;
 		cSegments++;
 	}
-	TqFloat ShadingRateSqrt = pAttributes() ->GetFloatAttribute( "System", "ShadingRateSqrt" ) [ 0 ];
-	MaxLen /= ShadingRateSqrt;
+	MaxLen /= sqrt(AdjustedShadingRate());
 
-	TqInt SplitCount = static_cast<TqUint>( MAX( MaxLen, 1 ) );
+	TqInt SplitCount = static_cast<TqUint>( max(MaxLen, 1.0f) );
 
 	return ( SplitCount * cSegments );
 }
@@ -1941,7 +1949,7 @@ void CqSurfaceNURBS::Output( const char* name )
 }
 
 
-void CqSurfaceNURBS::SetDefaultPrimitiveVariables( TqBool bUseDef_st )
+void CqSurfaceNURBS::SetDefaultPrimitiveVariables( bool bUseDef_st )
 {
 	TqInt bUses = Uses();
 
@@ -2177,7 +2185,7 @@ void CqSurfaceNURBS::SubdivideSegments( std::vector<boost::shared_ptr<CqSurfaceN
 			// The index of the patch we are working on.
 			TqInt iS = ( vPatch * uSplits ) + uPatch;
 			S[ iS ] = boost::shared_ptr<CqSurfaceNURBS>( new CqSurfaceNURBS );
-			S[ iS ] ->SetfPatchMesh( TqFalse );
+			S[ iS ] ->SetfPatchMesh( false );
 			// Initialise it to the same orders as us, with the calculated control point densities.
 			S[ iS ] ->Init( m_uOrder, m_vOrder, ( uEnd + 1 ) - uOffset, ( vEnd + 1 ) - vOffset );
 
@@ -2267,4 +2275,4 @@ void CqSurfaceNURBS::SubdivideSegments( std::vector<boost::shared_ptr<CqSurfaceN
 
 //-------------------------------------------------------
 
-END_NAMESPACE( Aqsis )
+} // namespace Aqsis
