@@ -1,5 +1,5 @@
 // Aqsis
-// Copyright  1997 - 2002, Paul C. Gregory
+// Copyright (C) 1997 - 2002, Paul C. Gregory
 //
 // Contact: pgregory@aqsis.org
 //
@@ -25,7 +25,7 @@
 
 #include "aqsis.h"
 
-#ifdef	AQSIS_SYSTEM_WIN32
+#if _MSC_VER
 #pragma warning(disable : 4786)
 #endif
 
@@ -34,10 +34,8 @@
 #include "imagebuffer.h"
 #include <deque>
 #include <fstream>
-#undef	min
-#undef	max
 
-START_NAMESPACE( Aqsis )
+namespace Aqsis {
 
 TqInt CqOcclusionTree::m_Tab = 0;
 
@@ -95,12 +93,12 @@ CqOcclusionTree::SplitNode(CqOcclusionTreePtr& a, CqOcclusionTreePtr& b)
 	{
 		a->m_SampleIndices.push_back(m_SampleIndices[i]);
 		const SqSampleData& sample = CqBucket::ImageElement(m_SampleIndices[i].first).SampleData(m_SampleIndices[i].second);
-		minTime = MIN(minTime, sample.m_Time);
-		maxTime = MAX(maxTime, sample.m_Time);
-		minDofIndex = MIN(minDofIndex, sample.m_DofOffsetIndex);
-		maxDofIndex = MAX(maxDofIndex, sample.m_DofOffsetIndex);
-		minDetailLevel = MIN(minDetailLevel, sample.m_DetailLevel);
-		maxDetailLevel = MAX(maxDetailLevel, sample.m_DetailLevel);
+		minTime = min(minTime, sample.m_Time);
+		maxTime = max(maxTime, sample.m_Time);
+		minDofIndex = min(minDofIndex, sample.m_DofOffsetIndex);
+		maxDofIndex = max(maxDofIndex, sample.m_DofOffsetIndex);
+		minDetailLevel = min(minDetailLevel, sample.m_DetailLevel);
+		maxDetailLevel = max(maxDetailLevel, sample.m_DetailLevel);
 	}
 	a->m_MinTime = minTime;
 	a->m_MaxTime = maxTime;
@@ -116,12 +114,12 @@ CqOcclusionTree::SplitNode(CqOcclusionTreePtr& a, CqOcclusionTreePtr& b)
 	{
 		b->m_SampleIndices.push_back(m_SampleIndices[i]);
 		const SqSampleData& sample = CqBucket::ImageElement(m_SampleIndices[i].first).SampleData(m_SampleIndices[i].second);
-		minTime = MIN(minTime, sample.m_Time);
-		maxTime = MAX(maxTime, sample.m_Time);
-		minDofIndex = MIN(minDofIndex, sample.m_DofOffsetIndex);
-		maxDofIndex = MAX(maxDofIndex, sample.m_DofOffsetIndex);
-		minDetailLevel = MIN(minDetailLevel, sample.m_DetailLevel);
-		maxDetailLevel = MAX(maxDetailLevel, sample.m_DetailLevel);
+		minTime = min(minTime, sample.m_Time);
+		maxTime = max(maxTime, sample.m_Time);
+		minDofIndex = min(minDofIndex, sample.m_DofOffsetIndex);
+		maxDofIndex = max(maxDofIndex, sample.m_DofOffsetIndex);
+		minDetailLevel = min(minDetailLevel, sample.m_DetailLevel);
+		maxDetailLevel = max(maxDetailLevel, sample.m_DetailLevel);
 	}
 	b->m_MinTime = minTime;
 	b->m_MaxTime = maxTime;
@@ -222,16 +220,16 @@ void CqOcclusionTree::InitialiseBounds()
 	for(i = m_SampleIndices.begin()+1; i!=m_SampleIndices.end(); ++i)
 	{
 		const SqSampleData& sample = CqBucket::ImageElement(i->first).SampleData(i->second);
-		minXVal = MIN(minXVal, sample.m_Position.x());
-		maxXVal = MAX(maxXVal, sample.m_Position.x());
-		minYVal = MIN(minYVal, sample.m_Position.y());
-		maxYVal = MAX(maxYVal, sample.m_Position.y());
-		minTime = MIN(minTime, sample.m_Time);
-		maxTime = MAX(maxTime, sample.m_Time);
-		minDofIndex = MIN(minDofIndex, sample.m_DofOffsetIndex);
-		maxDofIndex = MAX(maxDofIndex, sample.m_DofOffsetIndex);
-		minDetailLevel = MIN(minDetailLevel, sample.m_DetailLevel);
-		maxDetailLevel = MAX(maxDetailLevel, sample.m_DetailLevel);
+		minXVal = min(minXVal, sample.m_Position.x());
+		maxXVal = max(maxXVal, sample.m_Position.x());
+		minYVal = min(minYVal, sample.m_Position.y());
+		maxYVal = max(maxYVal, sample.m_Position.y());
+		minTime = min(minTime, sample.m_Time);
+		maxTime = max(maxTime, sample.m_Time);
+		minDofIndex = min(minDofIndex, sample.m_DofOffsetIndex);
+		maxDofIndex = max(maxDofIndex, sample.m_DofOffsetIndex);
+		minDetailLevel = min(minDetailLevel, sample.m_DetailLevel);
+		maxDetailLevel = max(maxDetailLevel, sample.m_DetailLevel);
 	}
 	m_MinSamplePoint[0] = minXVal;
 	m_MaxSamplePoint[0] = maxXVal;
@@ -318,7 +316,7 @@ void CqOcclusionTree::PropagateChanges()
 			{
 				if (*child)
 				{
-					maxdepth = MAX((*child)->m_MaxOpaqueZ, maxdepth);
+					maxdepth = max((*child)->m_MaxOpaqueZ, maxdepth);
 				}
 			}
 			// Only if this has resulted in a change at this level, should we process the parent.
@@ -340,13 +338,13 @@ void CqOcclusionTree::PropagateChanges()
 }
 
 
-TqBool CqOcclusionTree::CanCull( CqBound* bound )
+bool CqOcclusionTree::CanCull( CqBound* bound )
 {
 	// Recursively call each level to see if it can be culled at that point.
 	// Stop recursing at a level that doesn't contain the whole bound.
 	std::deque<CqOcclusionTree*> stack;
 	stack.push_front(this);
-	TqBool	top_level = TqTrue;
+	bool	top_level = true;
 	while(!stack.empty())
 	{
 		CqOcclusionTree* node = stack.front();
@@ -354,13 +352,13 @@ TqBool CqOcclusionTree::CanCull( CqBound* bound )
 		// Check the bound against the 2D limits of this level, if not entirely contained, then we
 		// cannot cull at this level, nor at any of the children.
 		CqBound b1(node->MinSamplePoint(), node->MaxSamplePoint());
-		if(b1.Contains2D(*bound) || top_level)
+		if(b1.Contains2D(bound) || top_level)
 		{
-			top_level = TqFalse;
+			top_level = false;
 			if( bound->vecMin().z() > node->MaxOpaqueZ() )
 				// If the bound is entirely contained within this node's 2D bound, and is further
 				// away than the furthest opaque point, then cull.
-				return(TqTrue);
+				return(true);
 			// If contained, but not behind the furthest point, push the children nodes onto the stack for
 			// processing.
 			CqOcclusionTree::TqChildArray::iterator childNode;
@@ -373,7 +371,7 @@ TqBool CqOcclusionTree::CanCull( CqBound* bound )
 			}
 		}
 	}
-	return(TqFalse);
+	return(false);
 }
 
 
@@ -414,10 +412,10 @@ CqOcclusionBox::~CqOcclusionBox()
 /** Delete the static hierarchy created in CreateHierachy(). static.
 */
 void CqOcclusionBox::DeleteHierarchy()
-{
+	{
 	delete m_KDTree;
 	m_KDTree = NULL;
-}
+		}
 
 
 //----------------------------------------------------------------------
@@ -447,19 +445,19 @@ void CqOcclusionBox::SetupHierarchy( CqBucket* bucket, TqInt xMin, TqInt yMin, T
 			for ( TqInt i = 0; i < numsamples; i++ )
 			{
 				m_KDTree->AddSample(std::pair<TqInt, TqInt>(j,i));
-			}
 		}
+	} 
 		// Now split the tree down until each leaf has only one sample.
 		m_KDTree->InitialiseBounds();
 		m_KDTree->ConstructTree();
-	}
+}
 
 	m_KDTree->UpdateBounds();
 
 	/*
 		static TqInt i__ = 0;
 		if(i__ == 800)
-		{
+{
 			std::ofstream strFile("test.out");
 			strFile << "xmin = " << xMin << std::endl << "ymin = " << yMin << std::endl << "xmax = " <<  xMax << std::endl << "ymax = " << yMax << std::endl << std::endl;
 			strFile << "points = [" << std::endl;
@@ -474,21 +472,21 @@ void CqOcclusionBox::SetupHierarchy( CqBucket* bucket, TqInt xMin, TqInt yMin, T
 }
 
 
-TqBool CqOcclusionBox::CanCull( CqBound* bound )
+bool CqOcclusionBox::CanCull( CqBound* bound )
 {
 	return(m_KDTree->CanCull(bound));
 }
 
-
+	
 void StoreExtraData( CqMicroPolygon* pMPG, SqImageSample& sample)
-{
+	{
 	std::map<std::string, CqRenderer::SqOutputDataEntry>& DataMap = QGetRenderContext() ->GetMapOfOutputDataEntries();
 	std::map<std::string, CqRenderer::SqOutputDataEntry>::iterator entry;
 	for ( entry = DataMap.begin(); entry != DataMap.end(); ++entry )
-	{
+		{
 		IqShaderData* pData;
 		if ( ( pData = pMPG->pGrid() ->FindStandardVar( entry->first.c_str() ) ) != NULL )
-		{
+			{
 			switch ( pData->Type() )
 			{
 					case type_float:
@@ -497,29 +495,29 @@ void StoreExtraData( CqMicroPolygon* pMPG, SqImageSample& sample)
 						TqFloat f;
 						pData->GetFloat( f, pMPG->GetIndex() );
 						sample.Data()[ entry->second.m_Offset ] = f;
-						break;
-					}
+				break;
+			}
 					case type_point:
 					case type_normal:
 					case type_vector:
 					case type_hpoint:
-					{
+			{
 						CqVector3D v;
 						pData->GetPoint( v, pMPG->GetIndex() );
 						sample.Data()[ entry->second.m_Offset ] = v.x();
 						sample.Data()[ entry->second.m_Offset + 1 ] = v.y();
 						sample.Data()[ entry->second.m_Offset + 2 ] = v.z();
 						break;
-					}
+			}
 					case type_color:
-					{
+			{
 						CqColor c;
 						pData->GetColor( c, pMPG->GetIndex() );
 						sample.Data()[ entry->second.m_Offset ] = c.fRed();
 						sample.Data()[ entry->second.m_Offset + 1 ] = c.fGreen();
 						sample.Data()[ entry->second.m_Offset + 2 ] = c.fBlue();
 						break;
-					}
+			}
 					case type_matrix:
 					{
 						CqMatrix m;
@@ -542,19 +540,19 @@ void StoreExtraData( CqMicroPolygon* pMPG, SqImageSample& sample)
 						sample.Data()[ entry->second.m_Offset + 14 ] = pElements[ 14 ];
 						sample.Data()[ entry->second.m_Offset + 15 ] = pElements[ 15 ];
 						break;
-					}
+		}
 					default:
 					// left blank to avoid compiler warnings about unhandled
 					//  types
 					break;
-			}
-		}
+	}
+}
 	}
 }
 
 
 
-void CqOcclusionTree::SampleMPG( CqMicroPolygon* pMPG, const CqBound& bound, TqBool usingMB, TqFloat time0, TqFloat time1, TqBool usingDof, TqInt dofboundindex, SqMpgSampleInfo& MpgSampleInfo, TqBool usingLOD, SqGridInfo& gridInfo)
+void CqOcclusionTree::SampleMPG( CqMicroPolygon* pMPG, const CqBound& bound, bool usingMB, TqFloat time0, TqFloat time1, bool usingDof, TqInt dofboundindex, SqMpgSampleInfo& MpgSampleInfo, bool usingLOD, SqGridInfo& gridInfo)
 {
 	// Check the current tree level, and if only one leaf, sample the MP, otherwise, pass it down to the left
 	// and/or right side of the tree if it crosses.
@@ -562,7 +560,7 @@ void CqOcclusionTree::SampleMPG( CqMicroPolygon* pMPG, const CqBound& bound, TqB
 	{
 		// Sample the MPG
 		SqSampleData& sample = Sample();
-		TqBool SampleHit;
+		bool SampleHit;
 		TqFloat D;
 
 		CqStats::IncI( CqStats::SPL_count );
@@ -570,8 +568,8 @@ void CqOcclusionTree::SampleMPG( CqMicroPolygon* pMPG, const CqBound& bound, TqB
 
 		if ( SampleHit )
 		{
-			TqBool Occludes = MpgSampleInfo.m_Occludes;
-			TqBool opaque =  MpgSampleInfo.m_IsOpaque;
+			bool Occludes = MpgSampleInfo.occludes;
+			bool opaque =  MpgSampleInfo.isOpaque;
 
 			SqImageSample& currentOpaqueSample = sample.m_OpaqueSample;
 			static SqImageSample localImageVal;
@@ -587,15 +585,17 @@ void CqOcclusionTree::SampleMPG( CqMicroPolygon* pMPG, const CqBound& bound, TqB
 				        currentOpaqueSample.Data()[Sample_Depth] <= D)
 				{
 					return;
-				}
-			}
+	}
+}
 
 			CqStats::IncI( CqStats::SPL_hits );
 			pMPG->MarkHit();
 
 			TqFloat* val = ImageVal.Data();
-			const CqColor& col = MpgSampleInfo.m_Colour;
-			const CqColor& opa = MpgSampleInfo.m_Opacity;
+			CqColor col;
+			CqColor opa;
+			pMPG->InterpolateOutputs(MpgSampleInfo, sample.m_Position, col, opa);
+
 			val[ 0 ] = col[0];
 			val[ 1 ] = col[1];
 			val[ 2 ] = col[2];
@@ -606,7 +606,7 @@ void CqOcclusionTree::SampleMPG( CqMicroPolygon* pMPG, const CqBound& bound, TqB
 
 			// Now store any other data types that have been registered.
 			if(gridInfo.m_UsesDataMap)
-			{
+{
 				StoreExtraData(pMPG, ImageVal);
 			}
 
@@ -618,9 +618,9 @@ void CqOcclusionTree::SampleMPG( CqMicroPolygon* pMPG, const CqBound& bound, TqB
 			// If the sample depth is closer than the current closest one, and is opaques
 			// we can just replace, as we know we are in a treenode that is a leaf.
 			if ( opaque )
-			{
+	{
 				if(D < MaxOpaqueZ())
-				{
+		{
 					SetMaxOpaqueZ(D);
 					PropagateChanges();
 				}
@@ -635,9 +635,9 @@ void CqOcclusionTree::SampleMPG( CqMicroPolygon* pMPG, const CqBound& bound, TqB
 				ImageVal.m_flags |= SqImageSample::Flag_Occludes;
 			}
 			if( gridInfo.m_IsMatte )
-			{
+				{
 				ImageVal.m_flags |= SqImageSample::Flag_Matte;
-			}
+				}
 
 			if(!opaque)
 			{
@@ -647,9 +647,9 @@ void CqOcclusionTree::SampleMPG( CqMicroPolygon* pMPG, const CqBound& bound, TqB
 			{
 				// mark this sample as having been written into.
 				ImageVal.m_flags |= SqImageSample::Flag_Valid;
-			}
 		}
 	}
+	}
 	else
 	{
 		TqChildArray::iterator child;
@@ -658,7 +658,7 @@ void CqOcclusionTree::SampleMPG( CqMicroPolygon* pMPG, const CqBound& bound, TqB
 		{
 			if (!*child)
 				continue;
-
+	
 			if(	   (!usingDof || ((dofboundindex >= (*child)->m_MinDofBoundIndex) && (dofboundindex <= (*child)->m_MaxDofBoundIndex )) )
 			        && (!usingMB || ((time0 <= (*child)->m_MaxTime) && (time1 >= (*child)->m_MinTime)) )
 			        && (!usingLOD || ((gridInfo.m_LodBounds[0] <= (*child)->m_MaxDetailLevel) && (gridInfo.m_LodBounds[1] >= (*child)->m_MinDetailLevel)) )
@@ -667,7 +667,7 @@ void CqOcclusionTree::SampleMPG( CqMicroPolygon* pMPG, const CqBound& bound, TqB
 				if(bound.vecMin().z() <= (*child)->m_MaxOpaqueZ || !gridInfo.m_IsCullable)
 				{
 					(*child)->SampleMPG(pMPG, bound, usingMB, time0, time1, usingDof, dofboundindex, MpgSampleInfo, usingLOD, gridInfo);
-				}
+}
 			}
 		}
 	}
@@ -704,5 +704,5 @@ void CqOcclusionTree::SampleMPG( CqMicroPolygon* pMPG, const CqBound& bound, TqB
 }
 */
 
-END_NAMESPACE( Aqsis )
+} // namespace Aqsis
 
