@@ -1,5 +1,5 @@
 // Aqsis
-// Copyright  1997 - 2001, Paul C. Gregory
+// Copyright (C) 1997 - 2001, Paul C. Gregory
 //
 // Contact: pgregory@aqsis.org
 //
@@ -32,7 +32,7 @@
 #include	"imagers.h"
 #include	"ri.h"
 
-START_NAMESPACE( Aqsis )
+namespace Aqsis {
 
 
 //---------------------------------------------------------------------
@@ -63,7 +63,7 @@ void CqOptions::InitialiseCamera()
 				TqFloat f = GetFloatOption( "System", "Clipping" ) [ 1 ];
 
 				matCameraToScreen.Identity();
-				matCameraToScreen.SetfIdentity( TqFalse );
+				matCameraToScreen.SetfIdentity( false );
 				matCameraToScreen.SetElement( 0, 0, 2.0f / ( r - l ) );
 				matCameraToScreen.SetElement( 3, 0, -( r + l ) / ( r - l ) );
 				matCameraToScreen.SetElement( 1, 1, 2.0f / ( t - b ) );
@@ -124,7 +124,7 @@ void CqOptions::InitialiseCamera()
 
 			case	ProjectionPerspective:
 			{
-				TqFloat fov = GetFloatOption( "System", "Clipping" ) [ 0 ] * ( tan( RAD( GetFloatOption( "System", "FOV" ) [ 0 ] / 2.0f ) ) );
+				TqFloat fov = GetFloatOption( "System", "Clipping" ) [ 0 ] * ( tan( degToRad( GetFloatOption( "System", "FOV" ) [ 0 ] / 2.0f ) ) );
 				TqFloat l = GetFloatOption( "System", "ScreenWindow" ) [ 0 ] * fov;
 				TqFloat r = GetFloatOption( "System", "ScreenWindow" ) [ 1 ] * fov;
 				TqFloat t = GetFloatOption( "System", "ScreenWindow" ) [ 2 ] * fov;
@@ -133,11 +133,11 @@ void CqOptions::InitialiseCamera()
 				TqFloat f = GetFloatOption( "System", "Clipping" ) [ 1 ];
 
 				matCameraToScreen.Identity();
-				matCameraToScreen.SetfIdentity( TqFalse );
+				matCameraToScreen.SetfIdentity( false );
 				matCameraToScreen.SetElement( 0, 0, ( 2.0f * n ) / ( r - l ) );
-				matCameraToScreen.SetElement( 2, 0, ( r + l ) / ( r - l ) );
+				matCameraToScreen.SetElement( 2, 0, (-( r + l )) / ( r - l ) );
 				matCameraToScreen.SetElement( 1, 1, ( 2.0f * n ) / ( t - b ) );
-				matCameraToScreen.SetElement( 2, 1, ( t + b ) / ( t - b ) );
+				matCameraToScreen.SetElement( 2, 1, (-( t + b )) / ( t - b ) );
 				TqFloat a = f / ( f - n );
 				//			matCameraToScreen.SetElement(2,2,-((f+n)/(f-n)));
 				matCameraToScreen.SetElement( 2, 2, a );
@@ -211,14 +211,16 @@ void CqOptions::InitialiseCamera()
 				break;
 			}
 	}
-	CqMatrix matWorldToCamera( QGetRenderContext() ->matSpaceToSpace( "world", "camera", NULL, NULL, QGetRenderContext()->Time() ) );
+	CqMatrix matWorldToCamera;
+	QGetRenderContext() ->matSpaceToSpace( "world", "camera", NULL, NULL, QGetRenderContext()->Time(), matWorldToCamera );
 	QGetRenderContext() ->SetmatScreen( matCameraToScreen * matWorldToCamera );
 	QGetRenderContext() ->SetmatNDC( matScreenToNDC * matCameraToScreen * matWorldToCamera );
 	QGetRenderContext() ->SetmatRaster( matNDCToRaster * matScreenToNDC * matCameraToScreen * matWorldToCamera );
 
 	CqMatrix matWorldToScreen = matCameraToScreen * matWorldToCamera;
 
-	CqMatrix dofm = QGetRenderContext() ->matVSpaceToSpace( "camera", "raster", NULL, NULL, QGetRenderContext()->Time() );
+	CqMatrix dofm;
+	QGetRenderContext() ->matVSpaceToSpace( "camera", "raster", NULL, NULL, QGetRenderContext()->Time(), dofm );
 
 	CqVector3D	dofe( 1, 1, -1 );
 	CqVector3D	dofc( 0, 0, -1 );
@@ -227,6 +229,19 @@ void CqOptions::InitialiseCamera()
 	dofc = dofm *  dofc;
 
 	QGetRenderContext() ->SetDepthOfFieldScale( fabs(dofe.x()-dofc.x()), fabs(dofe.y()-dofc.y()) );
+
+	// Setup the cached data for use during DoF calculations.
+	const TqFloat* dofOptions = QGetRenderContext() ->poptCurrent()->GetFloatOption( "System", "DepthOfField" );
+	// Set the default values to indicate DOF is off.
+	TqFloat fstop = FLT_MAX, focallength = 0.0f, focaldistance = 0.0f; 
+	if(dofOptions)
+	{
+		fstop = dofOptions[0];
+		focallength = dofOptions[1];
+		focaldistance = dofOptions[2];
+	}
+	QGetRenderContext() ->SetDepthOfFieldData( fstop, focallength, focaldistance );
+
 }
 
 
@@ -679,7 +694,7 @@ void CqOptions::SetpshadImager( const boost::shared_ptr<IqShader>& pshadImager )
 {
 	delete m_pshadImager;
 
-	m_pshadImager = new CqImagersource(pshadImager, TqTrue);
+	m_pshadImager = new CqImagersource(pshadImager, true);
         m_pshadImager->pShader()->PrepareDefArgs();
 }
 
@@ -779,4 +794,4 @@ CqColor CqOptions::GetOpacityImager( TqFloat x, TqFloat y )
 
 //---------------------------------------------------------------------
 
-END_NAMESPACE( Aqsis )
+} // namespace Aqsis
