@@ -33,10 +33,11 @@
 
 #include	"isurface.h"
 #include	"ishaderdata.h"
+#include	"iparameter.h"
 #include	"bilinear.h"
 #include	<boost/shared_ptr.hpp>
 
-START_NAMESPACE( Aqsis )
+namespace Aqsis {
 
 
 //----------------------------------------------------------------------
@@ -44,7 +45,7 @@ START_NAMESPACE( Aqsis )
  * Class storing a parameter with a name and value.
  */
 
-class CqParameter
+class CqParameter : public IqParameter
 {
 	public:
 		CqParameter( const char* strName, TqInt Count = 1 );
@@ -89,7 +90,7 @@ class CqParameter
 		 * \param u Boolean indicating whether to split in the u direction, false indicates split in v.
 		 * \param pSurface Pointer to the surface which this paramter belongs, used if the surface has special handling of parameter splitting.
 		 */
-		virtual void	Subdivide( CqParameter* pResult1, CqParameter* pResult2, TqBool u, IqSurface* pSurface = 0 )
+		virtual void	Subdivide( CqParameter* /* pResult1 */, CqParameter* /* pResult2 */, bool /* u */, IqSurface* /* pSurface */ = 0 )
 		{}
 		/** Pure virtual, dice the value into a grid using appropriate interpolation for the class.
 		 * \param u Integer dice count for the u direction.
@@ -239,7 +240,7 @@ class CqParameterTypedVarying : public CqParameterTyped<T, SLT>
 		{
 			m_aValues.clear();
 		}
-		virtual void	Subdivide( CqParameter* pResult1, CqParameter* pResult2, TqBool u, IqSurface* pSurface = 0 )
+		virtual void	Subdivide( CqParameter* pResult1, CqParameter* pResult2, bool u, IqSurface* pSurface = 0 )
 		{
 			assert( pResult1->Type() == this->Type() && pResult1->Type() == this->Type() &&
 			        pResult1->Class() == this->Class() && pResult1->Class() == this->Class() );
@@ -272,7 +273,7 @@ class CqParameterTypedVarying : public CqParameterTyped<T, SLT>
 
 		virtual	void	DiceOne( TqInt u, TqInt v, IqShaderData* pResult, IqSurface* pSurface = 0, TqInt ArrayIndex = 0 )
 		{
-			assert( TqFalse );
+			assert( false );
 			return;
 		}
 
@@ -382,7 +383,7 @@ class CqParameterTypedUniform : public CqParameterTyped<T, SLT>
 			m_aValues.clear();
 		}
 
-		virtual void	Subdivide( CqParameter* pResult1, CqParameter* pResult2, TqBool u, IqSurface* pSurface = 0 )
+		virtual void	Subdivide( CqParameter* pResult1, CqParameter* pResult2, bool u, IqSurface* pSurface = 0 )
 		{
 			assert( pResult1->Type() == this->Type() && pResult1->Type() == this->Type() &&
 			        pResult1->Class() == Class() && pResult1->Class() == Class() );
@@ -400,8 +401,8 @@ class CqParameterTypedUniform : public CqParameterTyped<T, SLT>
 			// Also note that the only time a Uniform value is diced is when it is on a single element, i.e. the patchmesh
 			// has been split into isngle patches, or the polymesh has been split into polys.
 			TqUint i;
-			TqUint max = MAX( (TqUint)u * (TqUint)v, pResult->Size() );
-			for ( i = 0; i < max; i++ )
+			TqUint size = max<TqInt>(u*v, pResult->Size());
+			for ( i = 0; i < size; i++ )
 				pResult->SetValue( m_aValues[ 0 ], i );
 		}
 
@@ -414,14 +415,14 @@ class CqParameterTypedUniform : public CqParameterTyped<T, SLT>
 			// Also note that the only time a Uniform value is diced is when it is on a single element, i.e. the patchmesh
 			// has been split into isngle patches, or the polymesh has been split into polys.
 			TqUint i;
-			TqUint max = pResult->Size();
-			for ( i = 0; i < max; i++ )
+			TqUint size = pResult->Size();
+			for ( i = 0; i < size; i++ )
 				pResult->SetValue( m_aValues[ 0 ], i );
 		}
 
 		virtual	void	DiceOne( TqInt u, TqInt v, IqShaderData* pResult, IqSurface* pSurface = 0, TqInt ArrayIndex = 0 )
 		{
-			assert( TqFalse );
+			assert( false );
 			return;
 		}
 
@@ -519,7 +520,7 @@ class CqParameterTypedConstant : public CqParameterTyped<T, SLT>
 		virtual	void	Clear()
 		{}
 
-		virtual void	Subdivide( CqParameter* pResult1, CqParameter* pResult2, TqBool u, IqSurface* pSurface = 0 )
+		virtual void	Subdivide( CqParameter* pResult1, CqParameter* pResult2, bool u, IqSurface* pSurface = 0 )
 		{
 			assert( pResult1->Type() == this->Type() && pResult1->Type() == this->Type() &&
 			        pResult1->Class() == this->Class() && pResult1->Class() == this->Class() );
@@ -535,8 +536,8 @@ class CqParameterTypedConstant : public CqParameterTyped<T, SLT>
 			// Note it is assumed that the variable has been
 			// initialised to the correct size prior to calling.
 			TqUint i;
-			TqUint max = MAX( (TqUint) u * (TqUint) v, pResult->Size() );
-			for ( i = 0; i < max ; i++ )
+			TqUint size = max<TqInt>(u*v, pResult->Size());
+			for ( i = 0; i < size ; i++ )
 				pResult->SetValue( m_Value, i );
 		}
 		virtual	void	CopyToShaderVariable( IqShaderData* pResult )
@@ -546,14 +547,14 @@ class CqParameterTypedConstant : public CqParameterTyped<T, SLT>
 			// Note it is assumed that the variable has been
 			// initialised to the correct size prior to calling.
 			TqUint i;
-			TqUint max = pResult->Size();
-			for ( i = 0; i < max ; i++ )
+			TqUint size = pResult->Size();
+			for ( i = 0; i < size ; i++ )
 				pResult->SetValue( m_Value, i );
 		}
 
 		virtual	void	DiceOne( TqInt u, TqInt v, IqShaderData* pResult, IqSurface* pSurface = 0, TqInt ArrayIndex = 0 )
 		{
-			assert( TqFalse );
+			assert( false );
 			return;
 		}
 
@@ -628,7 +629,7 @@ class CqParameterTypedVertex : public CqParameterTypedVarying<T, I, SLT>
 		{
 			return ( class_vertex );
 		}
-		virtual void	Subdivide( CqParameter* pResult1, CqParameter* pResult2, TqBool u, IqSurface* pSurface = 0 )
+		virtual void	Subdivide( CqParameter* pResult1, CqParameter* pResult2, bool u, IqSurface* pSurface = 0 )
 		{
 			assert( pResult1->Type() == this->Type() && pResult1->Type() == this->Type() &&
 			        pResult1->Class() == this->Class() && pResult1->Class() == this->Class() );
@@ -648,14 +649,14 @@ class CqParameterTypedVertex : public CqParameterTypedVarying<T, I, SLT>
 		{
 			assert( pResult->Type() == this->Type() );
 			TqUint i;
-			TqUint max = pResult->Size();
-			for ( i = 0; i < max ; i++ )
+			TqUint size = pResult->Size();
+			for ( i = 0; i < size ; i++ )
 				pResult->SetValue( this->pValue(i)[0], i );
 		}
 
 		virtual	void	DiceOne( TqInt u, TqInt v, IqShaderData* pResult, IqSurface* pSurface = 0, TqInt ArrayIndex = 0 )
 		{
-			assert( TqFalse );
+			assert( false );
 			return;
 		}
 
@@ -809,7 +810,7 @@ class CqParameterTypedVaryingArray : public CqParameterTyped<T, SLT>
 		{
 			m_aValues.clear();
 		}
-		virtual void	Subdivide( CqParameter* pResult1, CqParameter* pResult2, TqBool u, IqSurface* pSurface = 0 )
+		virtual void	Subdivide( CqParameter* pResult1, CqParameter* pResult2, bool u, IqSurface* pSurface = 0 )
 		{
 			assert( pResult1->Type() == this->Type() && pResult1->Type() == this->Type() &&
 			        pResult1->Class() == Class() && pResult1->Class() == Class() );
@@ -970,7 +971,7 @@ class CqParameterTypedUniformArray : public CqParameterTyped<T, SLT>
 		virtual	void	Clear()
 		{}
 
-		virtual void	Subdivide( CqParameter* pResult1, CqParameter* pResult2, TqBool u, IqSurface* pSurface = 0 )
+		virtual void	Subdivide( CqParameter* pResult1, CqParameter* pResult2, bool u, IqSurface* pSurface = 0 )
 		{}
 		virtual	void	Dice( TqInt u, TqInt v, IqShaderData* pResult, IqSurface* pSurface = 0 )
 		{
@@ -980,8 +981,8 @@ class CqParameterTypedUniformArray : public CqParameterTyped<T, SLT>
 			// initialised to the correct size prior to calling.
 			TqUint i; 
 			TqInt  j;
-			TqUint max = ( MAX( (TqUint)u * (TqUint) v, pResult->Size() ) );
-			for ( i = 0; i < max; ++i )
+			TqUint size = max<TqInt>(u*v, pResult->Size());
+			for ( i = 0; i < size; ++i )
 				for( j = 0; j < this->Count(); ++j )
 					pResult->SetValue( pValue( 0 ) [ j ], i );
 		}
@@ -992,8 +993,8 @@ class CqParameterTypedUniformArray : public CqParameterTyped<T, SLT>
 			// Note it is assumed that the variable has been
 			// initialised to the correct size prior to calling.
 			TqUint i;
-			TqUint max = pResult->Size();
-			for ( i = 0; i < max; i++ )
+			TqUint size = pResult->Size();
+			for ( i = 0; i < size; i++ )
 				pResult->SetValue( pValue( 0 ) [ 0 ], i );
 		}
 
@@ -1005,8 +1006,8 @@ class CqParameterTypedUniformArray : public CqParameterTyped<T, SLT>
 			// Note it is assumed that the variable has been
 			// initialised to the correct size prior to calling.
 			TqUint i;
-			TqUint max = ( MAX( (TqUint)u * (TqUint) v, pResult->Size() ) );
-			for ( i = 0; i < max; i++ )
+			TqUint size = max<TqInt>(u*v, pResult->Size());
+			for ( i = 0; i < size; i++ )
 				pResult->SetValue( pValue( 0 ) [ ArrayIndex ], i );
 		}
 
@@ -1106,7 +1107,7 @@ class CqParameterTypedConstantArray : public CqParameterTyped<T, SLT>
 		virtual	void	Clear()
 		{}
 
-		virtual void	Subdivide( CqParameter* pResult1, CqParameter* pResult2, TqBool u, IqSurface* pSurface = 0 )
+		virtual void	Subdivide( CqParameter* pResult1, CqParameter* pResult2, bool u, IqSurface* pSurface = 0 )
 		{}
 		virtual	void	Dice( TqInt u, TqInt v, IqShaderData* pResult, IqSurface* pSurface = 0 )
 		{
@@ -1116,8 +1117,8 @@ class CqParameterTypedConstantArray : public CqParameterTyped<T, SLT>
 			// initialised to the correct size prior to calling.
 			TqUint i; 
 			TqInt  j;
-			TqUint max = ( MAX( (TqUint) u * (TqUint) v, pResult->Size() ) );
-			for ( i = 0; i < max; ++i )
+			TqUint size = max<TqInt>(u*v, pResult->Size());
+			for ( i = 0; i < size; ++i )
 				for( j = 0; j < this->Count(); ++j )
 					pResult->SetValue( pValue( 0 ) [ j ], i );
 		}
@@ -1128,8 +1129,8 @@ class CqParameterTypedConstantArray : public CqParameterTyped<T, SLT>
 			// Note it is assumed that the variable has been
 			// initialised to the correct size prior to calling.
 			TqUint i;
-			TqUint max = pResult->Size();
-			for ( i = 0; i < max; i++ )
+			TqUint size = pResult->Size();
+			for ( i = 0; i < size; i++ )
 				pResult->SetValue( pValue( 0 ) [ 0 ], i );
 		}
 
@@ -1141,8 +1142,8 @@ class CqParameterTypedConstantArray : public CqParameterTyped<T, SLT>
 			// Note it is assumed that the variable has been
 			// initialised to the correct size prior to calling.
 			TqUint i;
-			TqUint max = ( MAX( (TqUint) u * (TqUint) v, pResult->Size() ) );
-			for ( i = 0; i < max; i++ )
+			TqUint size = max<TqInt>(u*v, pResult->Size());
+			for ( i = 0; i < size; i++ )
 				pResult->SetValue( pValue( 0 ) [ ArrayIndex ], i );
 		}
 
@@ -1239,8 +1240,8 @@ class CqParameterTypedVertexArray : public CqParameterTypedVaryingArray<T, I, SL
 			// Note it is assumed that the variable has been
 			// initialised to the correct size prior to calling.
 			TqUint i;
-			TqUint max = pResult->Size();
-			for ( i = 0; i < max; i++ )
+			TqUint size = pResult->Size();
+			for ( i = 0; i < size; i++ )
 				pResult->SetValue( this->pValue( 0 ) [ 0 ], i );
 		}
 
@@ -1923,6 +1924,6 @@ extern CqParameter* ( *gVariableCreateFuncsFaceVertexArray[] ) ( const char* str
 
 //-----------------------------------------------------------------------
 
-END_NAMESPACE( Aqsis )
+} // namespace Aqsis
 
 #endif	// !PARAMETERS_H_INCLUDED
