@@ -1,6 +1,6 @@
 
 // Aqsis
-// Copyright  1997 - 2001, Paul C. Gregory
+// Copyright (C) 1997 - 2001, Paul C. Gregory
 //
 // Contact: pgregory@aqsis.org
 //
@@ -24,16 +24,16 @@
 		\author Paul C. Gregory (pgregory@aqsis.org)
 */
 
-#include	<math.h>
+#include	"patch.h"
 
-#include	"aqsis.h"
+#include	"forwarddiff.h"
 #include	"imagebuffer.h"
 #include	"micropolygon.h"
 #include	"renderer.h"
-#include	"patch.h"
 #include	"vector2d.h"
+#include	"aqsismath.h"
 
-START_NAMESPACE( Aqsis )
+namespace Aqsis {
 
 //---------------------------------------------------------------------
 /** Constructor both u and vbasis matrices default to bezier.
@@ -80,83 +80,87 @@ CqSurface* CqSurfacePatchBicubic::Clone() const
 }
 
 
+namespace {
 
+/** \brief Implementation of natural subdivision for bicubic patches.
+ */
+template <class T, class SLT>
+void bicubicPatchNatSubdiv(CqParameter* pParam, CqParameter* pResult1,
+		CqParameter* pResult2, bool u)
+{
+	TqInt iu, iv;
 
-void CqSurfacePatchBicubic::NaturalSubdivide( CqParameter* pParam, CqParameter* pParam1, CqParameter* pParam2, TqBool u )
+	CqParameterTyped<T, SLT>* pTParam = static_cast<CqParameterTyped<T, SLT>*>( pParam );
+	CqParameterTyped<T, SLT>* pTResult1 = static_cast<CqParameterTyped<T, SLT>*>( pResult1 );
+	CqParameterTyped<T, SLT>* pTResult2 = static_cast<CqParameterTyped<T, SLT>*>( pResult2 );
+	if ( u )
+	{
+		for ( iv = 0; iv < 4; iv++ )
+		{
+			TqUint ivo = ( iv * 4 );
+			pTResult1->pValue() [ ivo + 0 ] = pTParam->pValue() [ ivo + 0 ];
+			pTResult1->pValue() [ ivo + 1 ] = static_cast<T>( ( pTParam->pValue() [ ivo + 0 ] + pTParam->pValue() [ ivo + 1 ] ) / 2.0f );
+			pTResult1->pValue() [ ivo + 2 ] = static_cast<T>( pTResult1->pValue() [ ivo + 1 ] / 2.0f + ( pTParam->pValue() [ ivo + 1 ] + pTParam->pValue() [ ivo + 2 ] ) / 4.0f );
+
+			pTResult2->pValue() [ ivo + 3 ] = pTParam->pValue() [ ivo + 3 ];
+			pTResult2->pValue() [ ivo + 2 ] = static_cast<T>( ( pTParam->pValue() [ ivo + 2 ] + pTParam->pValue() [ ivo + 3 ] ) / 2.0f );
+			pTResult2->pValue() [ ivo + 1 ] = static_cast<T>( pTResult2->pValue() [ ivo + 2 ] / 2.0f + ( pTParam->pValue() [ ivo + 1 ] + pTParam->pValue() [ ivo + 2 ] ) / 4.0f );
+
+			pTResult1->pValue() [ ivo + 3 ] = static_cast<T>( ( pTResult1->pValue() [ ivo + 2 ] + pTResult2->pValue() [ ivo + 1 ] ) / 2.0f );
+			pTResult2->pValue() [ ivo + 0 ] = pTResult1->pValue() [ ivo + 3 ];
+		}
+	}
+	else
+	{
+		for ( iu = 0; iu < 4; iu++ )
+		{
+			pTResult1->pValue() [ 0 + iu ] = pTParam->pValue() [ 0 + iu ];
+			pTResult1->pValue() [ 4 + iu ] = static_cast<T>( ( pTParam->pValue() [ 0 + iu ] + pTParam->pValue() [ 4 + iu ] ) / 2.0f );
+			pTResult1->pValue() [ 8 + iu ] = static_cast<T>( pTResult1->pValue() [ 4 + iu ] / 2.0f + ( pTParam->pValue() [ 4 + iu ] + pTParam->pValue() [ 8 + iu ] ) / 4.0f );
+
+			pTResult2->pValue() [ 12 + iu ] = pTParam->pValue() [ 12 + iu ];
+			pTResult2->pValue() [ 8 + iu ] = static_cast<T>( ( pTParam->pValue() [ 8 + iu ] + pTParam->pValue() [ 12 + iu ] ) / 2.0f );
+			pTResult2->pValue() [ 4 + iu ] = static_cast<T>( pTResult2->pValue() [ 8 + iu ] / 2.0f + ( pTParam->pValue() [ 4 + iu ] + pTParam->pValue() [ 8 + iu ] ) / 4.0f );
+
+			pTResult1->pValue() [ 12 + iu ] = static_cast<T>( ( pTResult1->pValue() [ 8 + iu ] + pTResult2->pValue() [ 4 + iu ] ) / 2.0f );
+			pTResult2->pValue() [ 0 + iu ] = pTResult1->pValue() [ 12 + iu ];
+		}
+	}
+}
+
+} // unnamed namespace
+
+void CqSurfacePatchBicubic::NaturalSubdivide( CqParameter* pParam, CqParameter* pParam1, CqParameter* pParam2, bool u )
 {
 	switch ( pParam->Type() )
 	{
-			case type_float:
-			{
-				CqParameterTyped<TqFloat, TqFloat>* pTParam = static_cast<CqParameterTyped<TqFloat, TqFloat>*>( pParam );
-				CqParameterTyped<TqFloat, TqFloat>* pTResult1 = static_cast<CqParameterTyped<TqFloat, TqFloat>*>( pParam1 );
-				CqParameterTyped<TqFloat, TqFloat>* pTResult2 = static_cast<CqParameterTyped<TqFloat, TqFloat>*>( pParam2 );
-				TypedNaturalSubdivide( pTParam, pTResult1, pTResult2, u );
-				break;
-			}
-
-			case type_integer:
-			{
-				CqParameterTyped<TqInt, TqFloat>* pTParam = static_cast<CqParameterTyped<TqInt, TqFloat>*>( pParam );
-				CqParameterTyped<TqInt, TqFloat>* pTResult1 = static_cast<CqParameterTyped<TqInt, TqFloat>*>( pParam1 );
-				CqParameterTyped<TqInt, TqFloat>* pTResult2 = static_cast<CqParameterTyped<TqInt, TqFloat>*>( pParam2 );
-				TypedNaturalSubdivide( pTParam, pTResult1, pTResult2, u );
-				break;
-			}
-
-			case type_point:
-			case type_vector:
-			case type_normal:
-			{
-				CqParameterTyped<CqVector3D, CqVector3D>* pTParam = static_cast<CqParameterTyped<CqVector3D, CqVector3D>*>( pParam );
-				CqParameterTyped<CqVector3D, CqVector3D>* pTResult1 = static_cast<CqParameterTyped<CqVector3D, CqVector3D>*>( pParam1 );
-				CqParameterTyped<CqVector3D, CqVector3D>* pTResult2 = static_cast<CqParameterTyped<CqVector3D, CqVector3D>*>( pParam2 );
-				TypedNaturalSubdivide( pTParam, pTResult1, pTResult2, u );
-				break;
-			}
-
-			case type_hpoint:
-			{
-				CqParameterTyped<CqVector4D, CqVector3D>* pTParam = static_cast<CqParameterTyped<CqVector4D, CqVector3D>*>( pParam );
-				CqParameterTyped<CqVector4D, CqVector3D>* pTResult1 = static_cast<CqParameterTyped<CqVector4D, CqVector3D>*>( pParam1 );
-				CqParameterTyped<CqVector4D, CqVector3D>* pTResult2 = static_cast<CqParameterTyped<CqVector4D, CqVector3D>*>( pParam2 );
-				TypedNaturalSubdivide( pTParam, pTResult1, pTResult2, u );
-				break;
-			}
-
-
-			case type_color:
-			{
-				CqParameterTyped<CqColor, CqColor>* pTParam = static_cast<CqParameterTyped<CqColor, CqColor>*>( pParam );
-				CqParameterTyped<CqColor, CqColor>* pTResult1 = static_cast<CqParameterTyped<CqColor, CqColor>*>( pParam1 );
-				CqParameterTyped<CqColor, CqColor>* pTResult2 = static_cast<CqParameterTyped<CqColor, CqColor>*>( pParam2 );
-				TypedNaturalSubdivide( pTParam, pTResult1, pTResult2, u );
-				break;
-			}
-
-			case type_string:
-			{
-				CqParameterTyped<CqString, CqString>* pTParam = static_cast<CqParameterTyped<CqString, CqString>*>( pParam );
-				CqParameterTyped<CqString, CqString>* pTResult1 = static_cast<CqParameterTyped<CqString, CqString>*>( pParam1 );
-				CqParameterTyped<CqString, CqString>* pTResult2 = static_cast<CqParameterTyped<CqString, CqString>*>( pParam2 );
-				TypedNaturalSubdivide( pTParam, pTResult1, pTResult2, u );
-				break;
-			}
-
-			case type_matrix:
-			{
-				//			CqParameterTyped<CqMatrix, CqMatrix>* pTParam = static_cast<CqParameterTyped<CqMatrix, CqMatrix>*>( pParam );
-				//			CqParameterTyped<CqMatrix, CqMatrix>* pTResult1 = static_cast<CqParameterTyped<CqMatrix, CqMatrix>*>( pParam1 );
-				//			CqParameterTyped<CqMatrix, CqMatrix>* pTResult2 = static_cast<CqParameterTyped<CqMatrix, CqMatrix>*>( pParam2 );
-				//			TypedNaturalSubdivide( pTParam, pTResult1, pTResult2, u );
-				//			break;
-			}
-
-			default:
-			{
-				// left blank to avoid compiler warnings about unhandled types
-				break;
-			}
+		case type_float:
+			bicubicPatchNatSubdiv<TqFloat, TqFloat>(pParam, pParam1, pParam2, u);
+			break;
+		case type_integer:
+			bicubicPatchNatSubdiv<TqInt, TqFloat>(pParam, pParam1, pParam2, u);
+			break;
+		case type_point:
+		case type_vector:
+		case type_normal:
+			bicubicPatchNatSubdiv<CqVector3D, CqVector3D>(pParam, pParam1, pParam2, u);
+			break;
+		case type_hpoint:
+			bicubicPatchNatSubdiv<CqVector4D, CqVector3D>(pParam, pParam1, pParam2, u);
+			break;
+		case type_color:
+			bicubicPatchNatSubdiv<CqColor, CqColor>(pParam, pParam1, pParam2, u);
+			break;
+		case type_string:
+			bicubicPatchNatSubdiv<CqString, CqString>(pParam, pParam1, pParam2, u);
+			break;
+		case type_matrix:
+			/// \todo why is this commented out?
+			//bicubicPatchNatSubdiv<CqMatrix, CqMatrix>(pParam, pParam1, pParam2, u);
+			//break;
+		default:
+			// left blank to avoid compiler warnings about unhandled types
+			break;
 	}
 }
 
@@ -165,7 +169,7 @@ void CqSurfacePatchBicubic::NaturalSubdivide( CqParameter* pParam, CqParameter*
 /** Get the boundary extents in camera space of the surface patch
  */
 
-CqBound CqSurfacePatchBicubic::Bound() const
+void CqSurfacePatchBicubic::Bound(IqBound* bound) const
 {
 	// Get the boundary in camera space.
 	CqVector3D	vecA( FLT_MAX, FLT_MAX, FLT_MAX );
@@ -187,78 +191,92 @@ CqBound CqSurfacePatchBicubic::Bound() const
 		if ( vecV.z() > vecB.z() )
 			vecB.z( vecV.z() );
 	}
-	CqBound	B;
-	B.vecMin() = vecA;
-	B.vecMax() = vecB;
-	return ( AdjustBoundForTransformationMotion( B ) );
+	bound->vecMin() = vecA;
+	bound->vecMax() = vecB;
+	AdjustBoundForTransformationMotion( bound );
 }
 
 
 //---------------------------------------------------------------------
+namespace {
+
+/** \brief Implementation of dicing for bicubic patches.
+ */
+template <class T, class SLT>
+void bicubicPatchNatDice(TqFloat uSize, TqFloat vSize, CqParameter* pParam,
+		IqShaderData* pData)
+{
+	CqParameterTyped<T, SLT>* pTParam = static_cast<CqParameterTyped<T, SLT>*>(pParam);
+	CqForwardDiffBezier<T> vFD0( 1.0f / vSize );
+	CqForwardDiffBezier<T> vFD1( 1.0f / vSize );
+	CqForwardDiffBezier<T> vFD2( 1.0f / vSize );
+	CqForwardDiffBezier<T> vFD3( 1.0f / vSize );
+	CqForwardDiffBezier<T> uFD0( 1.0f / uSize );
+
+	IqShaderData* arrayValue;
+	TqInt i;
+	for(i = 0; i<pTParam->Count(); i++)
+	{
+		vFD0.CalcForwardDiff( pTParam->pValue(0) [ i ], pTParam->pValue(4) [ i ], pTParam->pValue(8) [ i ], pTParam->pValue(12) [ i ] );
+		vFD1.CalcForwardDiff( pTParam->pValue(1) [ i ], pTParam->pValue(5) [ i ], pTParam->pValue(9) [ i ], pTParam->pValue(13) [ i ] );
+		vFD2.CalcForwardDiff( pTParam->pValue(2) [ i ], pTParam->pValue(6) [ i ], pTParam->pValue(10) [ i ], pTParam->pValue(14) [ i ] );
+		vFD3.CalcForwardDiff( pTParam->pValue(3) [ i ], pTParam->pValue(7) [ i ], pTParam->pValue(11) [ i ], pTParam->pValue(15) [ i ] );
+
+		TqInt iv, iu;
+		for ( iv = 0; iv <= vSize; iv++ )
+		{
+			T vA = vFD0.GetValue();
+			T vB = vFD1.GetValue();
+			T vC = vFD2.GetValue();
+			T vD = vFD3.GetValue();
+			uFD0.CalcForwardDiff( vA, vB, vC, vD );
+
+			for ( iu = 0; iu <= uSize; iu++ )
+			{
+				T vec = uFD0.GetValue();
+				TqInt igrid = static_cast<TqInt>( ( iv * ( uSize + 1 ) ) + iu );
+				arrayValue = pData->ArrayEntry(i);
+				arrayValue->SetValue( static_cast<SLT>( vec ), igrid );
+			}
+		}
+	}
+}
+
+} // unnamed namespace
+
 /** Dice the patch into a mesh of micropolygons.
  */
-
-
-void CqSurfacePatchBicubic::NaturalDice( CqParameter* pParameter, TqInt uDiceSize, TqInt vDiceSize, IqShaderData* pData )
+void CqSurfacePatchBicubic::NaturalDice(CqParameter* pParam, TqInt uDiceSize,
+		TqInt vDiceSize, IqShaderData* pData)
 {
-	switch ( pParameter->Type() )
+	switch(pParam->Type())
 	{
-			case type_float:
-			{
-				CqParameterTyped<TqFloat, TqFloat>* pTParam = static_cast<CqParameterTyped<TqFloat, TqFloat>*>( pParameter );
-				TypedNaturalDice( uDiceSize, vDiceSize, pTParam, pData );
-				break;
-			}
-
-			case type_integer:
-			{
-				CqParameterTyped<TqInt, TqFloat>* pTParam = static_cast<CqParameterTyped<TqInt, TqFloat>*>( pParameter );
-				TypedNaturalDice( uDiceSize, vDiceSize, pTParam, pData );
-				break;
-			}
-
-			case type_point:
-			case type_vector:
-			case type_normal:
-			{
-				CqParameterTyped<CqVector3D, CqVector3D>* pTParam = static_cast<CqParameterTyped<CqVector3D, CqVector3D>*>( pParameter );
-				TypedNaturalDice( uDiceSize, vDiceSize, pTParam, pData );
-				break;
-			}
-
-			case type_hpoint:
-			{
-				CqParameterTyped<CqVector4D, CqVector3D>* pTParam = static_cast<CqParameterTyped<CqVector4D, CqVector3D>*>( pParameter );
-				TypedNaturalDice( uDiceSize, vDiceSize, pTParam, pData );
-				break;
-			}
-
-			case type_color:
-			{
-				CqParameterTyped<CqColor, CqColor>* pTParam = static_cast<CqParameterTyped<CqColor, CqColor>*>( pParameter );
-				TypedNaturalDice( uDiceSize, vDiceSize, pTParam, pData );
-				break;
-			}
-
-			case type_string:
-			{
-				CqParameterTyped<CqString, CqString>* pTParam = static_cast<CqParameterTyped<CqString, CqString>*>( pParameter );
-				TypedNaturalDice( uDiceSize, vDiceSize, pTParam, pData );
-				break;
-			}
-
-			case type_matrix:
-			{
-				CqParameterTyped<CqMatrix, CqMatrix>* pTParam = static_cast<CqParameterTyped<CqMatrix, CqMatrix>*>( pParameter );
-				TypedNaturalDice( uDiceSize, vDiceSize, pTParam, pData );
-				break;
-			}
-
-			default:
-			{
-				// left blank to avoid compiler warnings about unhandled types
-				break;
-			}
+		case type_float:
+			bicubicPatchNatDice<TqFloat, TqFloat>(uDiceSize, vDiceSize, pParam, pData);
+			break;
+		case type_integer:
+			bicubicPatchNatDice<TqInt, TqFloat>( uDiceSize, vDiceSize, pParam, pData );
+			break;
+		case type_point:
+		case type_vector:
+		case type_normal:
+			bicubicPatchNatDice<CqVector3D, CqVector3D>( uDiceSize, vDiceSize, pParam, pData );
+			break;
+		case type_hpoint:
+			bicubicPatchNatDice<CqVector4D, CqVector3D>(uDiceSize, vDiceSize, pParam, pData);
+			break;
+		case type_color:
+			bicubicPatchNatDice<CqColor, CqColor>(uDiceSize, vDiceSize, pParam, pData);
+			break;
+		case type_string:
+			bicubicPatchNatDice<CqString, CqString>(uDiceSize, vDiceSize, pParam, pData);
+			break;
+		case type_matrix:
+			bicubicPatchNatDice<CqMatrix, CqMatrix>( uDiceSize, vDiceSize, pParam, pData );
+			break;
+		default:
+			// left blank to avoid compiler warnings about unhandled types
+			break;
 	}
 }
 
@@ -266,7 +284,7 @@ void CqSurfacePatchBicubic::NaturalDice( CqParameter* pParameter, TqInt uDiceSiz
 /** Split the patch into smaller patches.
  */
 
-TqInt CqSurfacePatchBicubic::PreSubdivide( std::vector<boost::shared_ptr<CqSurface> >& aSplits, TqBool u )
+TqInt CqSurfacePatchBicubic::PreSubdivide( std::vector<boost::shared_ptr<CqSurface> >& aSplits, bool u )
 {
 	// Create two new surface of the appropriate type
 	aSplits.push_back( boost::shared_ptr<CqSurface>( new CqSurfacePatchBicubic ) );
@@ -279,23 +297,22 @@ TqInt CqSurfacePatchBicubic::PreSubdivide( std::vector<boost::shared_ptr<CqSurfa
 /** Determine whether or not the patch is diceable
  */
 
-TqBool	CqSurfacePatchBicubic::Diceable()
+bool	CqSurfacePatchBicubic::Diceable()
 {
 	assert( NULL != P() );
 	// If the cull check showed that the primitive cannot be diced due to crossing the e and hither planes,
 	// then we can return immediately.
 	if ( !m_fDiceable )
-		return ( TqFalse );
+		return ( false );
 
 	// Otherwise we should continue to try to find the most advantageous split direction, OR the dice size.
-	const CqMatrix & matCtoR = QGetRenderContext() ->matSpaceToSpace( "camera", "raster", NULL, NULL, QGetRenderContext()->Time() );
+	CqMatrix matCtoR;
+	QGetRenderContext() ->matSpaceToSpace( "camera", "raster", NULL, NULL, QGetRenderContext()->Time(), matCtoR );
 
 	// Convert the control hull to raster space.
 	CqVector2D	avecHull[ 16 ];
 	TqInt i;
 
-	TqFloat ShadingRate = pAttributes() ->GetFloatAttribute( "System", "ShadingRate" ) [ 0 ];
-
 	for ( i = 0; i < 16; i++ )
 		avecHull[ i ] = matCtoR * P()->pValue( i )[0];
 
@@ -325,7 +342,7 @@ TqBool	CqSurfacePatchBicubic::Diceable()
 			vec.Unit();
 			vec %= vecL;
 			if ( vec.Magnitude() > 1 )
-				return ( TqFalse );
+				return ( false );
 		}
 	}
 	m_SplitDir = SplitDir_V;
@@ -352,7 +369,7 @@ TqBool	CqSurfacePatchBicubic::Diceable()
 			vec.Unit();
 			vec %= vecL;
 			if ( vec.Magnitude() > 1 )
-				return ( TqFalse );
+				return ( false );
 		}
 	}
 
@@ -385,28 +402,29 @@ TqBool	CqSurfacePatchBicubic::Diceable()
 			vLen = Vec3.Magnitude2();
 	}
 
-	uLen = sqrt( uLen  / ShadingRate);
-	vLen = sqrt( vLen  / ShadingRate);
+	TqFloat shadingRate = AdjustedShadingRate();
+	uLen = sqrt(uLen/shadingRate);
+	vLen = sqrt(vLen/shadingRate);
 
 	m_SplitDir = ( uLen > vLen ) ? SplitDir_U : SplitDir_V;
 	// TODO: Should ensure powers of half to prevent cracking.
 	uLen *= 3;
 	vLen *= 3;
-	m_uDiceSize = static_cast<TqInt>( MAX( ROUND( uLen ), 1 ) );
-	m_vDiceSize = static_cast<TqInt>( MAX( ROUND( vLen ), 1 ) );
+	m_uDiceSize = max<TqInt>(lround( uLen ), 1);
+	m_vDiceSize = max<TqInt>(lround( vLen ), 1);
 
 	// Ensure power of 2 to avoid cracking
 	const TqInt *binary = pAttributes() ->GetIntegerAttribute( "dice", "binary" );
 	if ( binary && *binary)
 	{
-		m_uDiceSize = CEIL_POW2( m_uDiceSize );
-		m_vDiceSize = CEIL_POW2( m_vDiceSize );
+		m_uDiceSize = ceilPow2( m_uDiceSize );
+		m_vDiceSize = ceilPow2( m_vDiceSize );
 	}
 
 	if ( uLen < FLT_EPSILON || vLen < FLT_EPSILON )
 	{
-		m_fDiscard = TqTrue;
-		return ( TqFalse );
+		m_fDiscard = true;
+		return ( false );
 	}
 
 	TqFloat gs = 16.0f;
@@ -414,10 +432,11 @@ TqBool	CqSurfacePatchBicubic::Diceable()
 	if( NULL != poptGridSize )
 		gs = poptGridSize[0];
 
-	if( m_uDiceSize * m_vDiceSize > gs * gs )
-		return TqFalse;
+	TqFloat gs2 = gs*gs;
+	if( m_uDiceSize > gs2 || m_vDiceSize > gs2 || (m_uDiceSize * m_vDiceSize) > gs2 )
+		return false;
 
-	return ( TqTrue );
+	return ( true );
 }
 
 
@@ -435,7 +454,7 @@ void CqSurfacePatchBicubic::ConvertToBezierBasis( CqMatrix& matuBasis, CqMatrix&
 		for ( i = 0; i < 4; i++ )
 			for ( j = 0; j < 4; j++ )
 				matMim1[ i ][ j ] = RiBezierBasis[ i ][ j ];
-		matMim1.SetfIdentity( TqFalse );
+		matMim1.SetfIdentity( false );
 		matMim1 = matMim1.Inverse();
 	}
 
@@ -473,10 +492,10 @@ void CqSurfacePatchBicubic::ConvertToBezierBasis( CqMatrix& matuBasis, CqMatrix&
 								matCPh[ i ][ j ] = 1.0f;
 							}
 						}
-						matCPx.SetfIdentity( TqFalse );
-						matCPy.SetfIdentity( TqFalse );
-						matCPz.SetfIdentity( TqFalse );
-						matCPh.SetfIdentity( TqFalse );
+						matCPx.SetfIdentity( false );
+						matCPy.SetfIdentity( false );
+						matCPz.SetfIdentity( false );
+						matCPh.SetfIdentity( false );
 
 						matCPx = matuConv.Transpose() * matCPx * matvConv;
 						matCPy = matuConv.Transpose() * matCPy * matvConv;
@@ -512,10 +531,10 @@ void CqSurfacePatchBicubic::ConvertToBezierBasis( CqMatrix& matuBasis, CqMatrix&
 								matCPh[ i ][ j ] = pParam->pValue( i*4 + j )[0][3];
 							}
 						}
-						matCPx.SetfIdentity( TqFalse );
-						matCPy.SetfIdentity( TqFalse );
-						matCPz.SetfIdentity( TqFalse );
-						matCPh.SetfIdentity( TqFalse );
+						matCPx.SetfIdentity( false );
+						matCPy.SetfIdentity( false );
+						matCPz.SetfIdentity( false );
+						matCPh.SetfIdentity( false );
 
 						matCPx = matuConv.Transpose() * matCPx * matvConv;
 						matCPy = matuConv.Transpose() * matCPy * matvConv;
@@ -551,9 +570,9 @@ void CqSurfacePatchBicubic::ConvertToBezierBasis( CqMatrix& matuBasis, CqMatrix&
 								matBlue[ i ][ j ] = pParam->pValue( i*4 + j )[0][2];
 							}
 						}
-						matRed.SetfIdentity( TqFalse );
-						matGreen.SetfIdentity( TqFalse );
-						matBlue.SetfIdentity( TqFalse );
+						matRed.SetfIdentity( false );
+						matGreen.SetfIdentity( false );
+						matBlue.SetfIdentity( false );
 
 						matRed = matuConv.Transpose() * matRed * matvConv;
 						matGreen = matuConv.Transpose() * matGreen * matvConv;
@@ -583,7 +602,7 @@ void CqSurfacePatchBicubic::ConvertToBezierBasis( CqMatrix& matuBasis, CqMatrix&
 							for ( j = 0; j < 4; j++ )
 								matCPx[ i ][ j ] = pParam->pValue( i*4 + j )[0];
 						}
-						matCPx.SetfIdentity( TqFalse );
+						matCPx.SetfIdentity( false );
 						matCPx = matuConv.Transpose() * matCPx * matvConv;
 
 						for ( i = 0; i < 4; i++ )
@@ -616,7 +635,7 @@ void CqSurfacePatchBicubic::ConvertToBezierBasis( CqMatrix& matuBasis, CqMatrix&
 /** Constructor.
  */
 
-CqSurfacePatchBilinear::CqSurfacePatchBilinear() : CqSurface(), m_fHasPhantomFourthVertex( TqFalse ), m_iInternalu( -1 ), m_iInternalv( -1 )
+CqSurfacePatchBilinear::CqSurfacePatchBilinear() : CqSurface(), m_fHasPhantomFourthVertex( false ), m_iInternalu( -1 ), m_iInternalv( -1 )
 {}
 
 
@@ -662,7 +681,7 @@ CqSurface* CqSurfacePatchBilinear::Clone() const
 /** Return the boundary extents in camera space of the surface patch
  */
 
-CqBound CqSurfacePatchBilinear::Bound() const
+void CqSurfacePatchBilinear::Bound(IqBound* bound) const
 {
 	assert( NULL != P() );
 
@@ -686,10 +705,9 @@ CqBound CqSurfacePatchBilinear::Bound() const
 		if ( vecV.z() > vecB.z() )
 			vecB.z( vecV.z() );
 	}
-	CqBound	B;
-	B.vecMin() = vecA;
-	B.vecMax() = vecB;
-	return ( AdjustBoundForTransformationMotion( B ) );
+	bound->vecMin() = vecA;
+	bound->vecMax() = vecB;
+	AdjustBoundForTransformationMotion( bound );
 }
 
 
@@ -698,7 +716,7 @@ CqBound CqSurfacePatchBilinear::Bound() const
 /** Split the patch into smaller patches.
  */
 
-TqInt CqSurfacePatchBilinear::PreSubdivide( std::vector<boost::shared_ptr<CqSurface> >& aSplits, TqBool u )
+TqInt CqSurfacePatchBilinear::PreSubdivide( std::vector<boost::shared_ptr<CqSurface> >& aSplits, bool u )
 {
 	aSplits.push_back( boost::shared_ptr<CqSurface>( new CqSurfacePatchBilinear ) );
 	aSplits.push_back( boost::shared_ptr<CqSurface>( new CqSurfacePatchBilinear ) );
@@ -711,24 +729,23 @@ TqInt CqSurfacePatchBilinear::PreSubdivide( std::vector<boost::shared_ptr<CqSurf
 /** Determine whether or not the patch is diceable
  */
 
-TqBool	CqSurfacePatchBilinear::Diceable()
+bool	CqSurfacePatchBilinear::Diceable()
 {
 	assert( NULL != P() );
 
 	// If the cull check showed that the primitive cannot be diced due to crossing the e and hither planes,
 	// then we can return immediately.
 	if ( !m_fDiceable )
-		return ( TqFalse );
+		return ( false );
 
 	// Otherwise we should continue to try to find the most advantageous split direction, OR the dice size.
-	const CqMatrix & matCtoR = QGetRenderContext() ->matSpaceToSpace( "camera", "raster", NULL, NULL, QGetRenderContext()->Time() );
+	CqMatrix matCtoR;
+	QGetRenderContext() ->matSpaceToSpace( "camera", "raster", NULL, NULL, QGetRenderContext()->Time(), matCtoR );
 
 	// Convert the control hull to raster space.
 	CqVector2D	avecHull[ 4 ];
 	TqInt i;
 
-	TqFloat ShadingRate = pAttributes() ->GetFloatAttribute( "System", "ShadingRate" ) [ 0 ];
-
 	for ( i = 0; i < 4; i++ )
 		avecHull[ i ] = matCtoR * P()->pValue( i )[0];
 
@@ -743,14 +760,15 @@ TqBool	CqSurfacePatchBilinear::Diceable()
 	Vec2 = avecHull[ 3 ] - avecHull[ 1 ];
 	vLen = ( Vec1.Magnitude2() > Vec2.Magnitude2() ) ? Vec1.Magnitude2() : Vec2.Magnitude2();
 
-	uLen = sqrt( uLen / ShadingRate);
-	vLen = sqrt( vLen / ShadingRate);
+	TqFloat shadingRate = AdjustedShadingRate();
+	uLen = sqrt(uLen/shadingRate);
+	vLen = sqrt(vLen/shadingRate);
 
 	m_SplitDir = ( uLen > vLen ) ? SplitDir_U : SplitDir_V;
 
 	// TODO: Should ensure powers of half to prevent cracking.
-	uLen = MAX( ROUND( uLen ), 1 );
-	vLen = MAX( ROUND( vLen ), 1 );
+	uLen = max<TqInt>(lround(uLen), 1);
+	vLen = max<TqInt>(lround(vLen), 1);
 
 	m_uDiceSize = static_cast<TqInt>( uLen );
 	m_vDiceSize = static_cast<TqInt>( vLen );
@@ -759,14 +777,14 @@ TqBool	CqSurfacePatchBilinear::Diceable()
 	const TqInt *binary = pAttributes() ->GetIntegerAttribute( "dice", "binary" );
 	if ( binary && *binary)
 	{
-		m_uDiceSize = CEIL_POW2( m_uDiceSize );
-		m_vDiceSize = CEIL_POW2( m_vDiceSize );
+		m_uDiceSize = ceilPow2( m_uDiceSize );
+		m_vDiceSize = ceilPow2( m_vDiceSize );
 	}
 
 	if ( uLen < FLT_EPSILON || vLen < FLT_EPSILON )
 	{
-		m_fDiscard = TqTrue;
-		return ( TqFalse );
+		m_fDiscard = true;
+		return ( false );
 	}
 
 	TqFloat gs = 16.0f;
@@ -776,9 +794,9 @@ TqBool	CqSurfacePatchBilinear::Diceable()
 
 	TqFloat gs2 = gs*gs;
 	if( m_uDiceSize > gs2 || m_vDiceSize > gs2 || (m_uDiceSize * m_vDiceSize) > gs2 )
-		return TqFalse;
+		return false;
 
-	return ( TqTrue );
+	return ( true );
 }
 
 
@@ -804,16 +822,16 @@ TqInt CqSurfacePatchBilinear::Split( std::vector<boost::shared_ptr<CqSurface> >&
 		aSplits.push_back( boost::shared_ptr<CqSurface>( new CqSurfacePatchBilinear ) );
 		aSplits.push_back( boost::shared_ptr<CqSurface>( new CqSurfacePatchBilinear ) );
 	}
-	TqBool direction = SplitDir() == SplitDir_U;
-	TqBool opposite = !direction;
+	bool direction = SplitDir() == SplitDir_U;
+	bool opposite = !direction;
 	TqInt i;
 	// Fill in the standard data from this donor to the new patches.
 	for ( i = 0; i < ( m_fHasPhantomFourthVertex ? 4 : 2 ); i++ )
 	{
 		aSplits[ i ] ->SetSurfaceParameters( *this );
 		aSplits[ i ] ->SetSplitDir( direction ? SplitDir_V : SplitDir_U );
-		aSplits[ i ] ->SetEyeSplitCount( EyeSplitCount() );
-		aSplits[ i ] ->m_fDiceable = TqTrue;
+		aSplits[ i ] ->SetSplitCount( SplitCount() + 1 );
+		aSplits[ i ] ->m_fDiceable = true;
 	}
 
 	// Iterate through any use parameters subdividing and storing the second value in the target surface.
@@ -858,17 +876,17 @@ TqInt CqSurfacePatchBilinear::Split( std::vector<boost::shared_ptr<CqSurface> >&
 		aSplits.pop_back();
 
 		// And set the phantom status of the remaining 3 patches.
-		static_cast<CqSurfacePatchBilinear*>( aSplits[ 0 ].get() ) ->m_fHasPhantomFourthVertex = TqFalse;
-		static_cast<CqSurfacePatchBilinear*>( aSplits[ 1 ].get() ) ->m_fHasPhantomFourthVertex = TqTrue;
-		static_cast<CqSurfacePatchBilinear*>( aSplits[ 2 ].get() ) ->m_fHasPhantomFourthVertex = TqTrue;
+		static_cast<CqSurfacePatchBilinear*>( aSplits[ 0 ].get() ) ->m_fHasPhantomFourthVertex = false;
+		static_cast<CqSurfacePatchBilinear*>( aSplits[ 1 ].get() ) ->m_fHasPhantomFourthVertex = true;
+		static_cast<CqSurfacePatchBilinear*>( aSplits[ 2 ].get() ) ->m_fHasPhantomFourthVertex = true;
 
 		return ( 3 );
 	}
 	else
 	{
 		// If not phantom, just return the two new halves.
-		static_cast<CqSurfacePatchBilinear*>( aSplits[ 0 ].get() ) ->m_fHasPhantomFourthVertex = TqFalse;
-		static_cast<CqSurfacePatchBilinear*>( aSplits[ 1 ].get() ) ->m_fHasPhantomFourthVertex = TqFalse;
+		static_cast<CqSurfacePatchBilinear*>( aSplits[ 0 ].get() ) ->m_fHasPhantomFourthVertex = false;
+		static_cast<CqSurfacePatchBilinear*>( aSplits[ 1 ].get() ) ->m_fHasPhantomFourthVertex = false;
 
 		return ( 2 );
 	}
@@ -908,7 +926,7 @@ CqSurface* CqSurfacePatchMeshBicubic::Clone() const
 /** Get the boundary extents in camera space of the surface patch mesh
  */
 
-CqBound CqSurfacePatchMeshBicubic::Bound() const
+void CqSurfacePatchMeshBicubic::Bound(IqBound* bound) const
 {
 	assert( NULL != P() );
 
@@ -932,10 +950,9 @@ CqBound CqSurfacePatchMeshBicubic::Bound() const
 		if ( vecV.z() > vecB.z() )
 			vecB.z( vecV.z() );
 	}
-	CqBound	B;
-	B.vecMin() = vecA;
-	B.vecMax() = vecB;
-	return ( AdjustBoundForTransformationMotion( B ) );
+	bound->vecMin() = vecA;
+	bound->vecMax() = vecB;
+	AdjustBoundForTransformationMotion( bound );
 }
 
 
@@ -1140,7 +1157,7 @@ CqSurface* CqSurfacePatchMeshBilinear::Clone() const
 /** Get the boundary extents in camera space of the surface patch mesh
  */
 
-CqBound CqSurfacePatchMeshBilinear::Bound() const
+void CqSurfacePatchMeshBilinear::Bound(IqBound* bound) const
 {
 	assert( NULL != P() );
 
@@ -1164,10 +1181,9 @@ CqBound CqSurfacePatchMeshBilinear::Bound() const
 		if ( vecV.z() > vecB.z() )
 			vecB.z( vecV.z() );
 	}
-	CqBound	B;
-	B.vecMin() = vecA;
-	B.vecMax() = vecB;
-	return ( AdjustBoundForTransformationMotion( B ) );
+	bound->vecMin() = vecA;
+	bound->vecMax() = vecB;
+	AdjustBoundForTransformationMotion( bound );
 }
 
 
@@ -1313,4 +1329,4 @@ TqInt CqSurfacePatchMeshBilinear::Split( std::vector<boost::shared_ptr<CqSurface
 	return ( cSplits );
 }
 
-END_NAMESPACE( Aqsis )
+} // namespace Aqsis
