@@ -29,7 +29,7 @@
  *
  */
 
-#include	<math.h>
+#include	<cmath>
 
 #include	"aqsis.h"
 #include	"points.h"
@@ -39,18 +39,16 @@
 
 #include	"ri.h"
 
-#include	"MultiTimer.h"
+#include	"multitimer.h"
 
 
-START_NAMESPACE( Aqsis )
+namespace Aqsis {
 
 CqObjectPool<CqMovingMicroPolygonKeyPoints>	CqMovingMicroPolygonKeyPoints::m_thePool;
 CqObjectPool<CqMicroPolygonPoints>	CqMicroPolygonPoints::m_thePool;
 CqObjectPool<CqMicroPolygonMotionPoints>	CqMicroPolygonMotionPoints::m_thePool;
 
 
-#define NBR_SEGMENTS 6
-
 void CqPointsKDTreeData::SetpPoints( const CqPoints* pPoints )
 {
 	m_pPointsSurface = pPoints;
@@ -132,8 +130,6 @@ CqMicroPolyGridBase* CqPoints::Dice()
 {
 	assert( pPoints() );
 
-	std::vector<CqMicroPolyGrid*> apGrids;
-
 	CqMicroPolyGridPoints* pGrid = new CqMicroPolyGridPoints();
 	pGrid->Initialise(nVertices(), 1, shared_from_this());
 
@@ -180,22 +176,19 @@ CqMicroPolyGridBase* CqPoints::Dice()
 	if ( USES( lUses, EnvVars_N ) && ( NULL != pGrid->pVar(EnvVars_N) ) && pPoints()->bHasVar(EnvVars_N) )
 	{
 		NaturalDice( pPoints()->N(), nVertices(), 1, pGrid->pVar(EnvVars_N) );
-		pGrid->SetbShadingNormals( TqTrue );
+		pGrid->SetbShadingNormals( true );
 	}
 
 	if ( USES( lUses, EnvVars_Ng ) )
 	{
-		CqVector3D	N(0,0,1);
+		CqVector3D	N(0,0,-1);
 		//N = QGetRenderContext() ->matSpaceToSpace( "camera", "object", NULL, pGrid->pTransform() ) * N;
 		TqUint u;
 		for ( u = 0; u < nVertices(); u++ )
 		{
-			TqBool CSO = pTransform()->GetHandedness(pTransform()->Time(0));
-			TqBool O = pAttributes() ->GetIntegerAttribute( "System", "Orientation" ) [ 0 ] != 0;
-			N = ( (O && CSO) || (!O && !CSO) ) ? N : -N;
 			pGrid->pVar(EnvVars_Ng)->SetNormal( N, u );
 		}
-		pGrid->SetbGeometricNormals( TqTrue );
+		pGrid->SetbGeometricNormals( true );
 	}
 
 	// Now we need to dice the user specified parameters as appropriate.
@@ -243,66 +236,65 @@ CqMicroPolyGridBase* CqPoints::Dice()
 }
 
 
-void CqPoints::NaturalDice( CqParameter* pParameter, TqInt uDiceSize, TqInt vDiceSize, IqShaderData* pData )
+namespace {
+
+/** \brief Implementation of dicing for points (helper function for CqPoints::NaturalDice)
+ *
+ * \param pParam - pointer to the parameter to take values from
+ * \param paramIdx - indices in the full parameter array (pParam) which
+ *                   correspond to the current gprim
+ * \param diceSize - number of points in the diced output
+ * \param pData - destination for diced shader data.
+ */
+template <class T, class SLT>
+void pointsNaturalDice(CqParameter* pParam, const std::vector<TqInt>& paramIdx,
+		TqInt diceSize, IqShaderData* pData)
 {
-	switch ( pParameter->Type() )
+	CqParameterTyped<T, SLT>* pTParam = static_cast<CqParameterTyped<T, SLT>*>(pParam);
+	for(TqInt i = 0; i < diceSize; i++ )
 	{
-			case type_float:
-			{
-				CqParameterTyped<TqFloat, TqFloat>* pTParam = static_cast<CqParameterTyped<TqFloat, TqFloat>*>( pParameter );
-				TypedNaturalDice( pTParam, pData );
-				break;
-			}
+		IqShaderData* arrayValue;
+		for(TqInt j = 0; j < pTParam->Count(); j++)
+		{
+			arrayValue = pData->ArrayEntry(j);
+			arrayValue->SetValue( static_cast<SLT>( pTParam->pValue() [ paramIdx[i] ] ), i );
+		}
+	}
+}
 
-			case type_integer:
-			{
-				CqParameterTyped<TqInt, TqFloat>* pTParam = static_cast<CqParameterTyped<TqInt, TqFloat>*>( pParameter );
-				TypedNaturalDice( pTParam, pData );
-				break;
-			}
+} // unnamed namespace
 
-			case type_point:
-			case type_vector:
-			case type_normal:
-			{
-				CqParameterTyped<CqVector3D, CqVector3D>* pTParam = static_cast<CqParameterTyped<CqVector3D, CqVector3D>*>( pParameter );
-				TypedNaturalDice( pTParam, pData );
-				break;
-			}
-
-			case type_hpoint:
-			{
-				CqParameterTyped<CqVector4D, CqVector3D>* pTParam = static_cast<CqParameterTyped<CqVector4D, CqVector3D>*>( pParameter );
-				TypedNaturalDice( pTParam, pData );
-				break;
-			}
-
-			case type_color:
-			{
-				CqParameterTyped<CqColor, CqColor>* pTParam = static_cast<CqParameterTyped<CqColor, CqColor>*>( pParameter );
-				TypedNaturalDice( pTParam, pData );
-				break;
-			}
-
-			case type_string:
-			{
-				CqParameterTyped<CqString, CqString>* pTParam = static_cast<CqParameterTyped<CqString, CqString>*>( pParameter );
-				TypedNaturalDice( pTParam, pData );
-				break;
-			}
-
-			case type_matrix:
-			{
-				CqParameterTyped<CqMatrix, CqMatrix>* pTParam = static_cast<CqParameterTyped<CqMatrix, CqMatrix>*>( pParameter );
-				TypedNaturalDice( pTParam, pData );
-				break;
-			}
-
-			default:
-			{
-				// left blank to avoid compiler warnings about unhandled types
-				break;
-			}
+void CqPoints::NaturalDice(CqParameter* pParam, TqInt uDiceSize, TqInt vDiceSize,
+		IqShaderData* pData )
+{
+	switch(pParam->Type())
+	{
+		case type_float:
+			pointsNaturalDice<TqFloat, TqFloat>(pParam, m_KDTree.aLeaves(), uDiceSize, pData);
+			break;
+		case type_integer:
+			pointsNaturalDice<TqInt, TqFloat>(pParam, m_KDTree.aLeaves(), uDiceSize, pData);
+			break;
+		case type_point:
+		case type_vector:
+		case type_normal:
+			pointsNaturalDice<CqVector3D, CqVector3D>(pParam, m_KDTree.aLeaves(), uDiceSize, pData);
+			break;
+		case type_hpoint:
+			pointsNaturalDice<CqVector4D, CqVector3D>(pParam, m_KDTree.aLeaves(), uDiceSize, pData);
+			break;
+		case type_color:
+			pointsNaturalDice<CqColor, CqColor>(pParam, m_KDTree.aLeaves(), uDiceSize, pData);
+			break;
+		case type_string:
+			pointsNaturalDice<CqString, CqString>(pParam, m_KDTree.aLeaves(), uDiceSize, pData);
+			break;
+		case type_matrix:
+			pointsNaturalDice<CqMatrix, CqMatrix>(pParam, m_KDTree.aLeaves(), uDiceSize, pData);
+			break;
+		default:
+			// left blank to avoid compiler warnings about unhandled types
+			break;
 	}
 }
 
@@ -310,29 +302,18 @@ void CqPoints::NaturalDice( CqParameter* pParameter, TqInt uDiceSize, TqInt vDic
 /** Determine whether the quadric is suitable for dicing.
  */
 
-TqBool	CqPoints::Diceable()
+bool	CqPoints::Diceable()
 {
 	TqUint gridsize = 256;
 
 	const TqInt* poptGridSize = QGetRenderContext() ->poptCurrent()->GetIntegerOption( "limits", "gridsize" );
-
-	TqInt m_XBucketSize = 16;
-	TqInt m_YBucketSize = 16;
-
-	const TqInt* poptBucketSize = QGetRenderContext() ->poptCurrent()->GetIntegerOption( "limits", "bucketsize" );
-	if ( poptBucketSize != 0 )
-	{
-		m_XBucketSize = poptBucketSize[ 0 ];
-		m_YBucketSize = poptBucketSize[ 1 ];
-	}
-
 	if ( poptGridSize )
 		gridsize = (TqUint) poptGridSize[ 0 ];
 
 	if( nVertices() > gridsize )
-		return ( TqFalse );
+		return ( false );
 	else
-		return ( TqTrue );
+		return ( true );
 }
 
 
@@ -340,24 +321,20 @@ TqBool	CqPoints::Diceable()
 /** Get the geometric bound of this GPrim in 'current' space.
  */
 
-CqBound	CqPoints::Bound() const
+void	CqPoints::Bound(IqBound* bound) const
 {
-	CqBound	B;
-
-	TqUint i;
-
 /*	for ( t = 0; t < cTimes(); t++ )
 	{
 		CqPolygonPoints* pTimePoints = pPoints( t ).get();*/
-		for( i = 0; i < nVertices(); i++ )
-			B.Encapsulate( (CqVector3D)m_pPoints->P()->pValue( m_KDTree.aLeaves()[ i ] )[0] );
+		for( TqUint i = 0; i < nVertices(); i++ )
+			bound->Encapsulate( (CqVector3D)m_pPoints->P()->pValue( m_KDTree.aLeaves()[ i ] )[0] );
 /*	}*/
 
 	// Expand the bound to take into account the width of the particles.
-	B.vecMax() += CqVector3D( m_MaxWidth, m_MaxWidth, m_MaxWidth );
-	B.vecMin() -= CqVector3D( m_MaxWidth, m_MaxWidth, m_MaxWidth );
+	bound->vecMax() += CqVector3D( m_MaxWidth, m_MaxWidth, m_MaxWidth );
+	bound->vecMin() -= CqVector3D( m_MaxWidth, m_MaxWidth, m_MaxWidth );
 
-	return ( AdjustBoundForTransformationMotion( B ) );
+	AdjustBoundForTransformationMotion( bound );
 }
 
 //---------------------------------------------------------------------
@@ -420,8 +397,7 @@ TqInt CqPoints::CopySplit( std::vector<boost::shared_ptr<CqSurface> >& aSplits,
 void CqPoints::InitialiseKDTree()
 {
 	m_KDTree.aLeaves().reserve( nVertices() );
-	TqUint i;
-	for( i = 0; i < nVertices(); i++ )
+	for( TqUint i = 0; i < nVertices(); i++ )
 		m_KDTree.aLeaves().push_back( i );
 }
 
@@ -430,19 +406,16 @@ void CqPoints::InitialiseMaxWidth()
 {
 	TqInt cu = nVertices();	// Only need cu, as we know cv is 1.
 
-	CqMatrix matObjectToCamera = QGetRenderContext() ->matSpaceToSpace( "object", "camera", NULL, pTransform().get(), QGetRenderContext()->Time() );
+	CqMatrix matObjectToCamera;
+	QGetRenderContext() ->matSpaceToSpace( "object", "camera", NULL, pTransform().get(), QGetRenderContext()->Time(), matObjectToCamera );
 	const CqParameterTypedConstant<TqFloat, type_float, TqFloat>* pConstantWidthParam = constantwidth( );
 
-	TqInt iu;
-	TqInt gsmin1;
-	gsmin1 = cu - 1;
-
 	CqVector3D Point0 = matObjectToCamera * CqVector3D(0,0,0);
 
 	TqFloat i_radius = 1.0f;
 	if( NULL != pConstantWidthParam )
 		i_radius = pConstantWidthParam->pValue( 0 )[ 0 ];
-	for ( iu = 0; iu < cu; iu++ )
+	for ( TqInt iu = 0; iu < cu; iu++ )
 	{
 		TqFloat radius;
 		// Find out if the "width" parameter was specified.
@@ -456,7 +429,7 @@ void CqPoints::InitialiseMaxWidth()
 		CqVector3D Point1 = matObjectToCamera * CqVector3D(radius,0,0);
 		radius = (Point1-Point0).Magnitude();
 
-		m_MaxWidth = MAX(m_MaxWidth, radius );
+		m_MaxWidth = max(m_MaxWidth, radius );
 	}
 }
 
@@ -479,9 +452,12 @@ void CqMicroPolyGridPoints::Split( CqImageBuffer* pImage, long xmin, long xmax,
 
 	ADDREF( this );
 
-	CqMatrix matCameraToObject0 = QGetRenderContext() ->matSpaceToSpace( "camera", "object", NULL, pSurface()->pTransform().get(), pSurface()->pTransform()->Time(0) );
-	CqMatrix matObjectToCamera = QGetRenderContext() ->matSpaceToSpace( "object", "camera", NULL, pSurface()->pTransform().get(), pSurface()->pTransform()->Time(0) );
-	CqMatrix matCameraToRaster = QGetRenderContext() ->matSpaceToSpace( "camera", "raster", NULL, NULL, pSurface()->pTransform()->Time(0) );
+	CqMatrix matCameraToObject0;
+	QGetRenderContext() ->matSpaceToSpace( "camera", "object", NULL, pSurface()->pTransform().get(), pSurface()->pTransform()->Time(0), matCameraToObject0 );
+	CqMatrix matObjectToCamera;
+	QGetRenderContext() ->matSpaceToSpace( "object", "camera", NULL, pSurface()->pTransform().get(), pSurface()->pTransform()->Time(0), matObjectToCamera );
+	CqMatrix matCameraToRaster;
+	QGetRenderContext() ->matSpaceToSpace( "camera", "raster", NULL, NULL, pSurface()->pTransform()->Time(0), matCameraToRaster );
 
 	CqVector3D vecdefOriginRaster = matCameraToRaster * CqVector3D( 0.0f,0.0f,0.0f );
 
@@ -515,15 +491,15 @@ void CqMicroPolyGridPoints::Split( CqImageBuffer* pImage, long xmin, long xmax,
 
 		CqMatrix matObjectToCameraT;
 		register TqInt i;
-		TqInt gsmin1;
-		gsmin1 = GridSize() - 1;
+		TqInt gsmin1 = GridSize() - 1;
 
 
 		for( iTime = 0; iTime < tTime; iTime++ )
 		{
-			CqMatrix matCameraToObjectT = QGetRenderContext() ->matSpaceToSpace( "camera", "object", NULL, pSurface()->pTransform().get(), pSurface()->pTransform()->Time( iTime ) );
-			amatObjectToCameraT[ iTime ] = QGetRenderContext() ->matSpaceToSpace( "object", "camera", NULL, pSurface()->pTransform().get(),  pSurface()->pTransform()->Time( iTime ) );
-			amatNObjectToCameraT[ iTime ] = QGetRenderContext() ->matNSpaceToSpace( "object", "camera", NULL, pSurface()->pTransform().get(), pSurface()->pTransform()->Time( iTime ) );
+			CqMatrix matCameraToObjectT;
+			QGetRenderContext() ->matSpaceToSpace( "camera", "object", NULL, pSurface()->pTransform().get(), pSurface()->pTransform()->Time( iTime ), matCameraToObjectT );
+			QGetRenderContext() ->matSpaceToSpace( "object", "camera", NULL, pSurface()->pTransform().get(),  pSurface()->pTransform()->Time( iTime ), amatObjectToCameraT[ iTime ] );
+			QGetRenderContext() ->matNSpaceToSpace( "object", "camera", NULL, pSurface()->pTransform().get(), pSurface()->pTransform()->Time( iTime ), amatNObjectToCameraT[ iTime ] );
 
 			aaPtimes[ iTime ].resize( gsmin1 + 1 );
 
@@ -594,9 +570,12 @@ void CqMicroPolyGridPoints::Split( CqImageBuffer* pImage, long xmin, long xmax,
 	else
 	{
 		iTime = 0;
-		CqMatrix matWorldToObjectT = QGetRenderContext() ->matSpaceToSpace( "world", "object", NULL, pSurface()->pTransform().get(), pSurface()->pTransform()->Time( iTime ) );
-		CqMatrix matObjectToCameraT = QGetRenderContext() ->matSpaceToSpace( "object", "camera", NULL, pSurface()->pTransform().get(), pSurface()->pTransform()->Time( iTime ) );
-		CqMatrix matNObjectToCameraT = QGetRenderContext() ->matNSpaceToSpace( "object", "camera", NULL, pSurface()->pTransform().get(), pSurface()->pTransform()->Time( iTime ) );
+		CqMatrix matWorldToObjectT;
+		QGetRenderContext() ->matSpaceToSpace( "world", "object", NULL, pSurface()->pTransform().get(), pSurface()->pTransform()->Time( iTime ), matWorldToObjectT );
+		CqMatrix matObjectToCameraT;
+		QGetRenderContext() ->matSpaceToSpace( "object", "camera", NULL, pSurface()->pTransform().get(), pSurface()->pTransform()->Time( iTime ), matObjectToCameraT );
+		CqMatrix matNObjectToCameraT;
+		QGetRenderContext() ->matNSpaceToSpace( "object", "camera", NULL, pSurface()->pTransform().get(), pSurface()->pTransform()->Time( iTime ), matNObjectToCameraT );
 
 		for ( iu = 0; iu < cu; iu++ )
 		{
@@ -655,7 +634,7 @@ void CqMicroPolyGridPoints::Split( CqImageBuffer* pImage, long xmin, long xmax,
 }
 
 
-TqBool	CqMicroPolygonPoints::Sample( const SqSampleData& sample, TqFloat& D, TqFloat time, TqBool UsingDof )
+bool	CqMicroPolygonPoints::Sample( const SqSampleData& sample, TqFloat& D, TqFloat time, bool UsingDof )
 {
 	const CqVector2D& vecSample = sample.m_Position;
 
@@ -664,9 +643,21 @@ TqBool	CqMicroPolygonPoints::Sample( const SqSampleData& sample, TqFloat& D, TqF
 	if( (CqVector2D( P.x(), P.y() ) - vecSample).Magnitude() < m_radius )
 	{
 		D = P.z();
-		return( TqTrue );
+		return( true );
 	}
-	return( TqFalse );
+	return( false );
+}
+
+void CqMicroPolygonPoints::CacheOutputInterpCoeffs(SqMpgSampleInfo& cache) const
+{
+	CacheOutputInterpCoeffsConstant(cache);
+}
+
+void CqMicroPolygonPoints::InterpolateOutputs(const SqMpgSampleInfo& cache,
+		const CqVector2D& pos, CqColor& outCol, CqColor& outOpac) const
+{
+	outCol = cache.color;
+	outOpac = cache.opacity;
 }
 
 
@@ -692,12 +683,17 @@ void CqMotionMicroPolyGridPoints::Split( CqImageBuffer* pImage, long xmin, long
 
 	TqInt cu = pGridA->uGridRes();	// Only need cu, as we know cv is 1.
 
-	CqMatrix matCameraToObject0 = QGetRenderContext() ->matSpaceToSpace( "camera", "object", NULL, pSurface()->pTransform().get(), pSurface()->pTransform()->Time(0) );
-	CqMatrix matObjectToCamera = QGetRenderContext() ->matSpaceToSpace( "object", "camera", NULL, pSurface()->pTransform().get(), pSurface()->pTransform()->Time(0) );
-	CqMatrix matCameraToRaster = QGetRenderContext() ->matSpaceToSpace( "camera", "raster", NULL, NULL, pSurface()->pTransform()->Time(0) );
+	CqMatrix matCameraToObject0;
+	QGetRenderContext() ->matSpaceToSpace( "camera", "object", NULL, pSurface()->pTransform().get(), pSurface()->pTransform()->Time(0), matCameraToObject0 );
+	CqMatrix matObjectToCamera;
+	QGetRenderContext() ->matSpaceToSpace( "object", "camera", NULL, pSurface()->pTransform().get(), pSurface()->pTransform()->Time(0), matObjectToCamera );
+	CqMatrix matCameraToRaster;
+	QGetRenderContext() ->matSpaceToSpace( "camera", "raster", NULL, NULL, pSurface()->pTransform()->Time(0), matCameraToRaster );
 
-	CqMatrix matTx = QGetRenderContext() ->matSpaceToSpace( "object", "camera", NULL, pSurface()->pTransform().get(), pSurface()->pTransform()->Time(0) );
-	CqMatrix matITTx = QGetRenderContext() ->matNSpaceToSpace( "object", "camera", NULL, pSurface()->pTransform().get(), pSurface()->pTransform()->Time(0) );
+	CqMatrix matTx;
+	QGetRenderContext() ->matSpaceToSpace( "object", "camera", NULL, pSurface()->pTransform().get(), pSurface()->pTransform()->Time(0), matTx );
+	CqMatrix matITTx;
+	QGetRenderContext() ->matNSpaceToSpace( "object", "camera", NULL, pSurface()->pTransform().get(), pSurface()->pTransform()->Time(0), matITTx );
 
 	CqVector3D vecdefOriginRaster = matCameraToRaster * CqVector3D( 0.0f,0.0f,0.0f );
 
@@ -715,14 +711,14 @@ void CqMotionMicroPolyGridPoints::Split( CqImageBuffer* pImage, long xmin, long
 
 	CqMatrix matObjectToCameraT;
 	register TqInt i;
-	TqInt gsmin1;
-	gsmin1 = pGridA->pShaderExecEnv()->shadingPointCount() - 1;
+	TqInt gsmin1 = pGridA->pShaderExecEnv()->shadingPointCount() - 1;
 
 	for( iTime = 0; iTime < NumTimes; iTime++ )
 	{
-		CqMatrix matCameraToObjectT = QGetRenderContext() ->matSpaceToSpace( "camera", "object", NULL, pSurface()->pTransform().get(), Time( iTime ) );
-		amatObjectToCameraT[ iTime ] = QGetRenderContext() ->matSpaceToSpace( "object", "camera", NULL, pSurface()->pTransform().get(),  Time( iTime ) );
-		amatNObjectToCameraT[ iTime ] = QGetRenderContext() ->matNSpaceToSpace( "object", "camera", NULL, pSurface()->pTransform().get(), Time( iTime ) );
+		CqMatrix matCameraToObjectT;
+		QGetRenderContext() ->matSpaceToSpace( "camera", "object", NULL, pSurface()->pTransform().get(), Time( iTime ), matCameraToObjectT );
+		QGetRenderContext() ->matSpaceToSpace( "object", "camera", NULL, pSurface()->pTransform().get(),  Time( iTime ), amatObjectToCameraT[ iTime ] );
+		QGetRenderContext() ->matNSpaceToSpace( "object", "camera", NULL, pSurface()->pTransform().get(), Time( iTime ), amatNObjectToCameraT[ iTime ] );
 
 		aaPtimes[ iTime ].resize( gsmin1 + 1 );
 
@@ -825,7 +821,10 @@ void CqMicroPolygonMotionPoints::CalculateTotalBound()
 	m_Bound = m_Keys[0]->GetTotalBound();
 	std::vector<CqMovingMicroPolygonKeyPoints*>::iterator i;
 	for ( i = m_Keys.begin(); i != m_Keys.end(); i++ )
-		m_Bound.Encapsulate( (*i)->GetTotalBound() );
+	{
+		CqBound B = (*i)->GetTotalBound();
+		m_Bound.Encapsulate( &B );
+	}
 }
 
 //---------------------------------------------------------------------
@@ -867,7 +866,8 @@ void CqMicroPolygonMotionPoints::BuildBoundList()
 		{
 			mid1.vecMin() = delta * ( end.vecMin() - start.vecMin() ) + start.vecMin();
 			mid1.vecMax() = delta * ( end.vecMax() - start.vecMax() ) + start.vecMax();
-			m_BoundList.Set( d-1, mid0.Combine( mid1 ), time );
+			mid0.Encapsulate(&mid1);
+			m_BoundList.Set( d-1, mid0, time );
 			time = delta * ( endTime - startTime ) + startTime;
 			mid0 = mid1;
 			delta += delta;
@@ -875,7 +875,7 @@ void CqMicroPolygonMotionPoints::BuildBoundList()
 		start = end;
 		startTime = endTime;
 	}
-	m_BoundReady = TqTrue;
+	m_BoundReady = true;
 }
 
 
@@ -887,12 +887,23 @@ void CqMicroPolygonMotionPoints::BuildBoundList()
  * \return Boolean indicating smaple hit.
  */
 
-TqBool CqMicroPolygonMotionPoints::Sample( const SqSampleData& sample, TqFloat& D, TqFloat time, TqBool UsingDof )
+bool CqMicroPolygonMotionPoints::Sample( const SqSampleData& sample, TqFloat& D, TqFloat time, bool UsingDof )
 {
 	const CqVector2D& vecSample = sample.m_Position;
 	return( fContains( vecSample, D, time ) );
 }
 
+void CqMicroPolygonMotionPoints::CacheOutputInterpCoeffs(SqMpgSampleInfo& cache) const
+{
+	CacheOutputInterpCoeffsConstant(cache);
+}
+
+void CqMicroPolygonMotionPoints::InterpolateOutputs(const SqMpgSampleInfo& cache,
+		const CqVector2D& pos, CqColor& outCol, CqColor& outOpac) const
+{
+	outCol = cache.color;
+	outOpac = cache.opacity;
+}
 
 //---------------------------------------------------------------------
 /** Store the vectors of the micropolygon at the specified shutter time.
@@ -914,7 +925,10 @@ void CqMicroPolygonMotionPoints::AppendKey( const CqVector3D& vA, TqFloat radius
 	if ( m_Times.size() == 1 )
 		m_Bound = pMP->GetTotalBound();
 	else
-		m_Bound.Encapsulate( pMP->GetTotalBound() );
+	{
+		CqBound B(pMP->GetTotalBound());
+		m_Bound.Encapsulate( &B );
+	}
 }
 
 
@@ -926,11 +940,11 @@ void CqMicroPolygonMotionPoints::AppendKey( const CqVector3D& vA, TqFloat radius
  * \return Boolean indicating sample hit.
  */
 
-TqBool CqMicroPolygonMotionPoints::fContains( const CqVector2D& vecP, TqFloat& Depth, TqFloat time ) const
+bool CqMicroPolygonMotionPoints::fContains( const CqVector2D& vecP, TqFloat& Depth, TqFloat time ) const
 {
 	TqInt iIndex = 0;
 	TqFloat Fraction = 0.0f;
-	TqBool Exact = TqTrue;
+	bool Exact = true;
 
 	if ( time > m_Times.front() )
 	{
@@ -962,15 +976,16 @@ TqBool CqMicroPolygonMotionPoints::fContains( const CqVector2D& vecP, TqFloat& D
 		if( (CqVector2D( MidPoint.x(), MidPoint.y() ) - vecP).Magnitude() < MidRadius )
 		{
 			Depth = MidPoint.z();
-			return( TqTrue );
+			return( true );
 		}
-		return ( TqFalse );
+		return ( false );
 	}
 }
 
 
 
-END_NAMESPACE( Aqsis )
+} // namespace Aqsis
 //---------------------------------------------------------------------
 
 
+
