@@ -1,5 +1,5 @@
 // Aqsis
-// Copyright  2001, Paul C. Gregory
+// Copyright (C) 2001, Paul C. Gregory
 //
 // Contact: pgregory@aqsis.org
 //
@@ -41,7 +41,7 @@
 #include	<algorithm>
 #include	<functional>
 
-START_NAMESPACE( Aqsis )
+namespace Aqsis {
 
 
 //----------------------------------------------------------------------
@@ -130,7 +130,7 @@ class CqPoints : public CqSurface
 		}
 		// Overrides from CqSurface
 		virtual	CqMicroPolyGridBase* Dice();
-		virtual TqBool	Diceable();
+		virtual bool	Diceable();
 
 		virtual void	Transform( const CqMatrix& matTx, const CqMatrix& matITTx, const CqMatrix& matRTx, TqInt iTime = 0 );
 
@@ -142,12 +142,12 @@ class CqPoints : public CqSurface
 		/** Determine whether the passed surface is valid to be used as a
 		 *  frame in motion blur for this surface.
 		 */
-		virtual TqBool	IsMotionBlurMatch( CqSurface* pSurf )
+		virtual bool	IsMotionBlurMatch( CqSurface* pSurf )
 		{
-			return( TqFalse );
+			return( false );
 		}
 
-		virtual	CqBound	Bound() const;
+		virtual	void	Bound(IqBound* bound) const;
 		virtual	TqInt	Split( std::vector<boost::shared_ptr<CqSurface> >& aSplits );
 
 		virtual CqSurface* Clone() const;
@@ -276,23 +276,6 @@ class CqPoints : public CqSurface
 			return ( A );
 		}
 
-	protected:
-		template <class T, class SLT>
-		void	TypedNaturalDice( CqParameterTyped<T, SLT>* pParam, IqShaderData* pData )
-		{
-			TqUint i;
-			for ( i = 0; i < nVertices(); i++ )
-			{
-				IqShaderData* arrayValue;
-				TqInt j;
-				for(j = 0; j<pParam->Count(); j++)
-				{
-					arrayValue = pData->ArrayEntry(j);
-					arrayValue->SetValue( static_cast<SLT>( pParam->pValue() [ m_KDTree.aLeaves()[ i ] ] ), i );
-				}
-			}
-		}
-
 	private:
 		boost::shared_ptr<CqPolygonPoints> m_pPoints;				///< Pointer to the surface storing the primtive variables.
 		TqInt	m_nVertices;					///< Number of points this surfaces represents.
@@ -323,6 +306,22 @@ class CqMicroPolyGridPoints : public CqMicroPolyGrid
 		{
 			return ( cu * cv );
 		}
+		virtual bool	hasValidDerivatives() const
+		{
+			return false;
+		}
+		/** \brief Set surface derivative in u.
+		 * 
+		 *  Set the value of du if needed, du is constant across the grid being shaded.
+		 */
+		virtual void setDu();
+		/** \brief Set surface derivative in v.
+		 * 
+		 *  Set the value of dv if needed, dv is constant across the grid being shaded.
+		 */
+		virtual void setDv();
+		virtual void CalcNormals();
+		virtual void CalcSurfaceDerivatives();
 };
 
 class CqMotionMicroPolyGridPoints : public CqMotionMicroPolyGrid
@@ -382,11 +381,14 @@ class CqMicroPolygonPoints : public CqMicroPolygon
 			b.vecMax() = Pmax;
 			return( b );
 		}
-		virtual	TqBool	Sample( const SqSampleData& sample, TqFloat& D, TqFloat time, TqBool UsingDof = TqFalse );
+		virtual	bool	Sample( const SqSampleData& sample, TqFloat& D, TqFloat time, bool UsingDof = false );
 		virtual void	CacheHitTestValues(CqHitTestCache* cache, CqVector3D* points) {}
 		virtual void	CacheHitTestValues(CqHitTestCache* cache) {}
 		virtual void	CacheHitTestValuesDof(CqHitTestCache* cache, const CqVector2D& DofOffset, CqVector2D* coc) {}
 
+		virtual void CacheOutputInterpCoeffs(SqMpgSampleInfo& cache) const;
+		virtual void InterpolateOutputs(const SqMpgSampleInfo& cache,
+				const CqVector2D& pos, CqColor& outCol, CqColor& outOpac) const;
 
 	private:
 		CqMicroPolygonPoints( const CqMicroPolygonPoints& From )
@@ -430,14 +432,14 @@ class CqMovingMicroPolygonKeyPoints
 		}
 
 	public:
-		TqBool	fContains( const CqVector2D& vecP, TqFloat& Depth, TqFloat time ) const
+		bool	fContains( const CqVector2D& vecP, TqFloat& Depth, TqFloat time ) const
 		{
 			if( (CqVector2D( m_Point0.x(), m_Point0.y() ) - vecP).Magnitude() < m_radius )
 			{
 				Depth = m_Point0.z();
-				return( TqTrue );
+				return( true );
 			}
-			return( TqFalse );
+			return( false );
 		}
 		virtual void	CacheHitTestValues(CqHitTestCache* cache, CqVector3D* points) {}
 		virtual void	CacheHitTestValues(CqHitTestCache* cache) {}
@@ -476,7 +478,7 @@ class CqMovingMicroPolygonKeyPoints
 class CqMicroPolygonMotionPoints : public CqMicroPolygon
 {
 	public:
-		CqMicroPolygonMotionPoints() : CqMicroPolygon(), m_BoundReady( TqFalse )
+		CqMicroPolygonMotionPoints() : CqMicroPolygon(), m_BoundReady( false )
 		{ }
 		virtual	~CqMicroPolygonMotionPoints()
 		{
@@ -501,16 +503,16 @@ class CqMicroPolygonMotionPoints : public CqMicroPolygon
 
 	public:
 		void	AppendKey( const CqVector3D& vA, TqFloat radius, TqFloat time );
-		void	DeleteVariables( TqBool all )
+		void	DeleteVariables( bool all )
 		{}
 
 		// Overrides from CqMicroPolygon
-		virtual TqBool	fContains( const CqVector2D& vecP, TqFloat& Depth, TqFloat time ) const;
+		virtual bool	fContains( const CqVector2D& vecP, TqFloat& Depth, TqFloat time ) const;
 		virtual void CalculateTotalBound();
-		virtual	CqBound&		GetTotalBound( /*TqBool fForce = TqFalse */);
+		virtual	CqBound&		GetTotalBound( /*bool fForce = false */);
 		virtual const CqBound&	GetTotalBound() const
 		{
-			return ( m_Bound );
+			return( m_Bound );
 		}
 		virtual	TqInt			cSubBounds()
 		{
@@ -528,15 +530,18 @@ class CqMicroPolygonMotionPoints : public CqMicroPolygon
 		}
 		virtual void	BuildBoundList();
 
-		virtual TqBool IsMoving()
+		virtual bool IsMoving()
 		{
-			return TqTrue;
+			return true;
 		}
-		virtual	TqBool	Sample( const SqSampleData& sample, TqFloat& D, TqFloat time, TqBool UsingDof = TqFalse );
+		virtual	bool	Sample( const SqSampleData& sample, TqFloat& D, TqFloat time, bool UsingDof = false );
+		virtual void CacheOutputInterpCoeffs(SqMpgSampleInfo& cache) const;
+		virtual void InterpolateOutputs(const SqMpgSampleInfo& cache,
+				const CqVector2D& pos, CqColor& outCol, CqColor& outOpac) const;
 	private:
 		CqBound	m_Bound;					///< Stored bound.
 		CqBoundList	m_BoundList;			///< List of bounds to get a tighter fit.
-		TqBool	m_BoundReady;				///< Flag indicating the boundary has been initialised.
+		bool	m_BoundReady;				///< Flag indicating the boundary has been initialised.
 		std::vector<TqFloat> m_Times;
 		std::vector<CqMovingMicroPolygonKeyPoints*>	m_Keys;
 
@@ -597,8 +602,8 @@ class CqDeformingPointsSurface : public CqDeformingSurface
 			for ( i = 0; i < cSplits; i++ )
 			{
 				boost::shared_ptr<CqDeformingPointsSurface> pNewMotion( new CqDeformingPointsSurface( boost::shared_ptr<CqSurface>() ) );
-				pNewMotion->m_fDiceable = TqTrue;
-				pNewMotion->m_EyeSplitCount = m_EyeSplitCount;
+				pNewMotion->m_fDiceable = true;
+				pNewMotion->m_SplitCount = m_SplitCount + 1;
 				TqInt j;
 				for ( j = 0; j < cTimes(); j++ )
 					pNewMotion->AddTimeSlot( Time( j ), aaMotionSplits[ j ][ i ] );
@@ -619,9 +624,31 @@ class CqDeformingPointsSurface : public CqDeformingSurface
 	protected:
 };
 
+//==============================================================================
+// Implementation details
+//==============================================================================
+
+inline void CqMicroPolyGridPoints::setDu()
+{
+	pVar(EnvVars_du)->SetFloat(1.0f);
+}
+
+inline void CqMicroPolyGridPoints::setDv()
+{
+	pVar(EnvVars_dv)->SetFloat(1.0f);
+}
+
+inline void	CqMicroPolyGridPoints::CalcNormals()
+{
+}
+
+inline void	CqMicroPolyGridPoints::CalcSurfaceDerivatives()
+{
+}
+
 
 //-----------------------------------------------------------------------
 
-END_NAMESPACE( Aqsis )
+} // namespace Aqsis
 
 #endif	// !POINTS_H_INCLUDED
