@@ -1,5 +1,5 @@
 // Aqsis
-// Copyright  1997 - 2001, Paul C. Gregory
+// Copyright (C) 1997 - 2001, Paul C. Gregory
 //
 // Contact: pgregory@aqsis.org
 //
@@ -28,14 +28,14 @@
 #include	"patch.h"
 #include	"imagebuffer.h"
 
-START_NAMESPACE( Aqsis )
+namespace Aqsis {
 
 
 //---------------------------------------------------------------------
 /** Return the boundary extents in camera space of the polygon
  */
 
-CqBound CqPolygonBase::Bound() const
+void CqPolygonBase::Bound(IqBound* bound) const
 {
 	CqVector3D	vecA( FLT_MAX, FLT_MAX, FLT_MAX );
 	CqVector3D	vecB( -FLT_MAX, -FLT_MAX, -FLT_MAX );
@@ -57,10 +57,8 @@ CqBound CqPolygonBase::Bound() const
 		if ( vecV.z() > vecB.z() )
 			vecB.z( vecV.z() );
 	}
-	CqBound	B;
-	B.vecMin() = vecA;
-	B.vecMax() = vecB;
-	return ( B );
+	bound->vecMin() = vecA;
+	bound->vecMax() = vecB;
 }
 
 
@@ -80,8 +78,8 @@ TqInt CqPolygonBase::Split( std::vector<boost::shared_ptr<CqSurface> >& aSplits
 	// We need to take into account Orientation here, even though most other
 	// primitives leave it up to the CalcNormals function on the MPGrid, because we
 	// are forcing N to be setup here, so clockwise nature is important.
-	TqBool CSO = pTransform()->GetHandedness(pTransform()->Time(0));
-	TqBool O = pAttributes() ->GetIntegerAttribute( "System", "Orientation" ) [ 0 ] != 0;
+	bool CSO = pTransform()->GetHandedness(pTransform()->Time(0));
+	bool O = pAttributes() ->GetIntegerAttribute( "System", "Orientation" ) [ 0 ] != 0;
 
 	indexA = 0;
 	indexB = 1;
@@ -214,7 +212,7 @@ TqInt CqPolygonBase::Split( std::vector<boost::shared_ptr<CqSurface> >& aSplits
 
 		// If this is a triangle, then mark the patch as a special case. See function header comment for more details.
 		if ( indexC == indexD )
-			pNew->SetfHasPhantomFourthVertex( TqTrue );
+			pNew->SetfHasPhantomFourthVertex( true );
 
 		// If there are no smooth normals specified, then fill in the facet normal at each vertex.
 		if ( !bHasVar(EnvVars_N) && USES( iUses, EnvVars_N ) )
@@ -234,7 +232,8 @@ TqInt CqPolygonBase::Split( std::vector<boost::shared_ptr<CqSurface> >& aSplits
 		if ( USES( iUses, EnvVars_s ) || USES( iUses, EnvVars_t ) || USES( iUses, EnvVars_u ) || USES( iUses, EnvVars_v ) )
 		{
 			CqVector3D PA, PB, PC, PD;
-			const CqMatrix& matCurrentToWorld = QGetRenderContext() ->matSpaceToSpace( "current", "object", NULL, Surface().pTransform().get(), Surface().pTransform() ->Time(0) );
+			CqMatrix matCurrentToWorld;
+			QGetRenderContext() ->matSpaceToSpace( "current", "object", NULL, Surface().pTransform().get(), Surface().pTransform() ->Time(0), matCurrentToWorld );
 			PA = matCurrentToWorld * pNew->P() ->pValue() [ 0 ];
 			PB = matCurrentToWorld * pNew->P() ->pValue() [ 1 ];
 			PC = matCurrentToWorld * pNew->P() ->pValue() [ 3 ];
@@ -271,10 +270,25 @@ TqInt CqPolygonBase::Split( std::vector<boost::shared_ptr<CqSurface> >& aSplits
 				CqParameterTypedVarying<TqFloat, type_float, TqFloat>* pNewUP = new CqParameterTypedVarying<TqFloat, type_float, TqFloat>( "u" );
 				pNewUP->SetSize( pNew->cVarying() );
 
-				pNewUP->pValue() [ 0 ] = PA.x();
-				pNewUP->pValue() [ 1 ] = PB.x();
-				pNewUP->pValue() [ 2 ] = PD.x();
-				pNewUP->pValue() [ 3 ] = PC.x();
+				// We choose to violate the RI standard here - the RISpec
+				// requires the the surface coordinate u takes the value of the
+				// x-coordinates of the corners of the patch in the same way
+				// that the texture coordinate "t" does.
+				//
+				// This really screws up derivative calcuations in the shading
+				// language since following the RISpec implies that u isn't
+				// aligned with the grid.  Worse, du can turn out to be
+				// technically equal to 0, which prevents the expression
+				//
+				//   dPu = Du(P)*du;
+				//
+				// from making sense.  This is really bad from the point of
+				// view of anyone trying to use something like dPu to antialias
+				// thier shaders.
+				pNewUP->pValue() [ 0 ] = 0;
+				pNewUP->pValue() [ 1 ] = 1;
+				pNewUP->pValue() [ 2 ] = 0;
+				pNewUP->pValue() [ 3 ] = 1;
 
 				pNew->AddPrimitiveVariable( pNewUP );
 			}
@@ -284,10 +298,12 @@ TqInt CqPolygonBase::Split( std::vector<boost::shared_ptr<CqSurface> >& aSplits
 				CqParameterTypedVarying<TqFloat, type_float, TqFloat>* pNewUP = new CqParameterTypedVarying<TqFloat, type_float, TqFloat>( "v" );
 				pNewUP->SetSize( pNew->cVarying() );
 
-				pNewUP->pValue() [ 0 ] = PA.y();
-				pNewUP->pValue() [ 1 ] = PB.y();
-				pNewUP->pValue() [ 2 ] = PD.y();
-				pNewUP->pValue() [ 3 ] = PC.y();
+				// We choose to violate the RI standard here - see the comments
+				// above about u.
+				pNewUP->pValue() [ 0 ] = 0;
+				pNewUP->pValue() [ 1 ] = 0;
+				pNewUP->pValue() [ 2 ] = 1;
+				pNewUP->pValue() [ 3 ] = 1;
 
 				pNew->AddPrimitiveVariable( pNewUP );
 			}
@@ -409,17 +425,17 @@ CqSurfacePolygon::~CqSurfacePolygon()
 /** Check if a polygon is degenerate, i.e. all points collapse to the same or almost the same place.
  */
 
-TqBool CqSurfacePolygon::CheckDegenerate() const
+bool CqSurfacePolygon::CheckDegenerate() const
 {
 	// Check if all points are within a minute distance of each other.
-	TqBool	fDegen = TqTrue;
+	bool	fDegen = true;
 	TqInt i, n;
 	n = NumVertices();
 	for ( i = 1; i < n; i++ )
 	{
 		if ( ( PolyP( i ) - PolyP( i - 1 ) ).Magnitude() > FLT_EPSILON )
 		{
-			fDegen = TqFalse;
+			fDegen = false;
 			break;
 		}
 	}
@@ -486,7 +502,7 @@ void	CqPolygonPoints::Transform( const CqMatrix& matTx, const CqMatrix& matITTx,
 
 	CqSurface::Transform( matTx, matITTx, matRTx, iTime );
 
-//	m_Transformed = TqTrue;
+//	m_Transformed = true;
 }
 
 
@@ -502,16 +518,15 @@ CqSurface* CqPolygonPoints::Clone() const
 }
 
 
-CqBound	CqSurfacePointsPolygons::Bound() const
+void	CqSurfacePointsPolygons::Bound(IqBound* bound) const
 {
-	CqBound B;
 	if( m_pPoints && m_pPoints->P() )
 	{
 		TqInt PointIndex;
 		for( PointIndex = m_pPoints->P()->Size()-1; PointIndex >= 0; PointIndex-- )
-			B.Encapsulate( (CqVector3D)m_pPoints->P()->pValue()[PointIndex] );
+			bound->Encapsulate( (CqVector3D)m_pPoints->P()->pValue()[PointIndex] );
 	}
-	return(AdjustBoundForTransformationMotion( B ));
+	AdjustBoundForTransformationMotion( bound );
 }
 
 TqInt CqSurfacePointsPolygons::Split( std::vector<boost::shared_ptr<CqSurface> >& aSplits )
@@ -566,5 +581,5 @@ CqSurface* CqSurfacePointsPolygons::Clone() const
 	return(clone);
 }
 
-END_NAMESPACE( Aqsis )
+} // namespace Aqsis
 //---------------------------------------------------------------------
