@@ -1,5 +1,5 @@
 // Aqsis
-// Copyright  1997 - 2001, Paul C. Gregory
+// Copyright (C) 1997 - 2001, Paul C. Gregory
 //
 // Contact: pgregory@aqsis.org
 //
@@ -26,6 +26,7 @@
 #include	"aqsis.h"
 
 #include	<time.h>
+#include	<boost/bind.hpp>
 
 #include	"imagebuffer.h"
 #include	"lights.h"
@@ -37,14 +38,14 @@
 #include	"render.h"
 #include	"transform.h"
 #include	"rifile.h"
-#include	"texturemap.h"
+#include	"texturemap_old.h"
 #include	"shadervm.h"
 #include	"inlineparse.h"
 #include	"tiffio.h"
 #include	"objectinstance.h"
 
 
-START_NAMESPACE( Aqsis )
+namespace Aqsis {
 
 extern IqDDManager* CreateDisplayDriverManager();
 extern IqRaytrace* CreateRaytracer();
@@ -74,11 +75,12 @@ static CqMatrix oldresult[2];
 CqRenderer::CqRenderer() :
 		m_pImageBuffer( 0 ),
 		m_Mode( RenderMode_Image ),
-		m_fSaveGPrims( TqFalse ),
+		m_textureCache(boost::bind(&CqRenderer::textureSearchPath, this)),
+		m_fSaveGPrims( false ),
 		m_OutputDataOffset(9),		// Cs, Os, z, coverage, a
 		m_OutputDataTotalSize(9),	// Cs, Os, z, coverage, a
 		m_FrameNo( 0 ),
-		m_bObjectOpen(TqFalse),
+		m_bObjectOpen(false),
 		m_pErrorHandler( &RiErrorPrint ),
 		m_pProgressHandler( NULL ),
 		m_pPreRenderFunction( NULL ),
@@ -93,7 +95,7 @@ CqRenderer::CqRenderer() :
 	m_pTransDefault = CqTransformPtr( new CqTransform );
 	m_pTransCamera  = CqTransformPtr( new CqTransform );
 	m_pTransDefObj  = CqTransformPtr( new CqTransform );
-	m_fWorldBegin = TqFalse;
+	m_fWorldBegin = false;
 
 	m_poptDefault = CqOptionsPtr(new CqOptions );
 
@@ -362,6 +364,27 @@ boost::shared_ptr<CqModeBlock>	CqRenderer::BeginMotionModeBlock( TqInt N, TqFloa
 		return boost::shared_ptr<CqModeBlock>( );
 }
 
+//---------------------------------------------------------------------
+/** Create a new resource context.
+ */
+
+boost::shared_ptr<CqModeBlock>	CqRenderer::BeginResourceModeBlock()
+{
+	// XXX: Error checking may eventually be unnecessary.  - ajb
+	if ( m_pconCurrent )
+	{
+		boost::shared_ptr<CqModeBlock> pconNew = m_pconCurrent->BeginResourceModeBlock();
+		if ( pconNew )
+		{
+			m_pconCurrent = pconNew;
+			return ( pconNew );
+		}
+		else
+			return boost::shared_ptr<CqModeBlock>( );
+	}
+	else
+		return boost::shared_ptr<CqModeBlock>( );
+}
 
 //----------------------------------------------------------------------
 /** Delete the current context presuming it is a main context.
@@ -480,6 +503,19 @@ void	CqRenderer::EndMotionModeBlock()
 	}
 }
 
+//
+//----------------------------------------------------------------------
+/** Delete the current context presuming it is a resource context.
+ */
+
+void	CqRenderer::EndResourceModeBlock()
+{
+	if ( m_pconCurrent && (m_pconCurrent->Type() == Resource))
+	{
+		m_pconCurrent->EndResourceModeBlock();
+		m_pconCurrent = m_pconCurrent->pconParent();
+	}
+}
 
 //----------------------------------------------------------------------
 /** Get the current shutter time, always returns 0.0 unless within a motion block,
@@ -661,9 +697,10 @@ void	CqRenderer::ptransConcatCurrentTime( const CqMatrix& matTrans )
 /** Render all surface in the current list to the image buffer.
  */
 
-void CqRenderer::RenderWorld(TqBool clone)
+void CqRenderer::RenderWorld(bool clone)
 {
-	// While rendering, all primitives should fasttrack straight into the pipeline, the easiest way to ensure this
+	// While rendering, all primitives should fasttrack straight into the pipeline, 
+	// and shaders should automatically initialise, the easiest way to ensure this
 	// is to switch into 'non' multipass mode.
 	TqInt multiPass = 0;
 	TqInt* pMultipass = GetIntegerOptionWrite("Render", "multipass");
@@ -766,7 +803,7 @@ void CqRenderer::RenderAutoShadows()
 				SetCameraTransform(lightTrans);
 				
 				// Render the world
-				RenderWorld(TqTrue);
+				RenderWorld(true);
 
 				popOptions();
 				SetCameraTransform(defaultCamera);
@@ -775,7 +812,8 @@ void CqRenderer::RenderAutoShadows()
 				delete(m_pDDManager);
 				m_pDDManager = realDDManager;
 
-				CqTextureMap::FlushCache();
+				CqTextureMapOld::FlushCache();
+				m_textureCache.flush();
 				CqOcclusionBox::DeleteHierarchy();
 				clippingVolume().clear();
 			}
@@ -824,9 +862,9 @@ void CqRenderer::Initialise()
  */
 
 
-CqMatrix	CqRenderer::matSpaceToSpace( const char* strFrom, const char* strTo, const IqTransform* transShaderToWorld, const IqTransform* transObjectToWorld, TqFloat time )
+bool	CqRenderer::matSpaceToSpace( const char* strFrom, const char* strTo, const IqTransform* transShaderToWorld, const IqTransform* transObjectToWorld, TqFloat time, CqMatrix& result )
 {
-	CqMatrix	matResult, matA, matB;
+	CqMatrix	matA, matB;
 	TqUlong fhash, thash;
 
 	// Get the hash keys for From,To spaces
@@ -850,7 +888,8 @@ CqMatrix	CqRenderer::matSpaceToSpace( const char* strFrom, const char* strTo, co
 	}
 	else
 	{
-		WhichMatToWorld( matA, fhash );
+		if(!WhichMatToWorld( matA, fhash ))
+			return(false);
 	}
 
 
@@ -868,12 +907,13 @@ CqMatrix	CqRenderer::matSpaceToSpace( const char* strFrom, const char* strTo, co
 			matB = m_pTransCamera->matObjectToWorld( time );
 	} else
 	{
-		WhichMatWorldTo( matB, thash );
+		if(!WhichMatWorldTo( matB, thash ))
+			return(false);
 	}
 
-	matResult = matB * matA;
+	result = matB * matA;
 
-	return ( matResult );
+	return ( true );
 }
 
 
@@ -882,9 +922,9 @@ CqMatrix	CqRenderer::matSpaceToSpace( const char* strFrom, const char* strTo, co
 /** Get the matrix to convert vectors between the specified coordinate systems.
  */
 
-CqMatrix	CqRenderer::matVSpaceToSpace( const char* strFrom, const char* strTo, const IqTransform* transShaderToWorld, const IqTransform* transObjectToWorld, TqFloat time )
+bool	CqRenderer::matVSpaceToSpace( const char* strFrom, const char* strTo, const IqTransform* transShaderToWorld, const IqTransform* transObjectToWorld, TqFloat time, CqMatrix& result )
 {
-	CqMatrix	matResult, matA, matB;
+	CqMatrix	matA, matB;
 
 	TqUlong fhash, thash;
 
@@ -908,7 +948,8 @@ CqMatrix	CqRenderer::matVSpaceToSpace( const char* strFrom, const char* strTo, c
 			matA = m_pTransCamera->matObjectToWorld( time ).Inverse();
 	} else
 	{
-		WhichMatToWorld ( matA, fhash );
+		if(!WhichMatToWorld ( matA, fhash ))
+			return(false);
 	}
 
 	if ( thash == ohash )
@@ -926,26 +967,27 @@ CqMatrix	CqRenderer::matVSpaceToSpace( const char* strFrom, const char* strTo, c
 			matB = m_pTransCamera->matObjectToWorld( time );
 	} else
 	{
-		WhichMatWorldTo ( matB, thash );
+		if(!WhichMatWorldTo ( matB, thash ))
+			return(false);
 	}
 
-	matResult = matB * matA;
+	result = matB * matA;
 
 
 
-	if (memcmp((void *) oldkey[0].pElements(), (void *) matResult.pElements(), sizeof(TqFloat) * 16) != 0)
+	if (memcmp((void *) oldkey[0].pElements(), (void *) result.pElements(), sizeof(TqFloat) * 16) != 0)
 	{
-		oldkey[0] = matResult;
-		matResult[ 3 ][ 0 ] = matResult[ 3 ][ 1 ] = matResult[ 3 ][ 2 ] = matResult[ 0 ][ 3 ] = matResult[ 1 ][ 3 ] = matResult[ 2 ][ 3 ] = 0.0;
-		matResult[ 3 ][ 3 ] = 1.0;
-		oldresult[0] = matResult;
+		oldkey[0] = result;
+		result[ 3 ][ 0 ] = result[ 3 ][ 1 ] = result[ 3 ][ 2 ] = result[ 0 ][ 3 ] = result[ 1 ][ 3 ] = result[ 2 ][ 3 ] = 0.0;
+		result[ 3 ][ 3 ] = 1.0;
+		oldresult[0] = result;
 
 	}
 	else
 	{
-		return oldresult[0];
+		result = oldresult[0];
 	}
-	return ( matResult );
+	return ( true );
 }
 
 
@@ -953,9 +995,9 @@ CqMatrix	CqRenderer::matVSpaceToSpace( const char* strFrom, const char* strTo, c
 /** Get the matrix to convert normals between the specified coordinate systems.
  */
 
-CqMatrix	CqRenderer::matNSpaceToSpace( const char* strFrom, const char* strTo, const IqTransform* transShaderToWorld, const IqTransform* transObjectToWorld, TqFloat time )
+bool	CqRenderer::matNSpaceToSpace( const char* strFrom, const char* strTo, const IqTransform* transShaderToWorld, const IqTransform* transObjectToWorld, TqFloat time, CqMatrix& result )
 {
-	CqMatrix	matResult, matA, matB;
+	CqMatrix	matA, matB;
 
 	TqUlong fhash, thash;
 
@@ -979,7 +1021,8 @@ CqMatrix	CqRenderer::matNSpaceToSpace( const char* strFrom, const char* strTo, c
 			matA = m_pTransCamera->matObjectToWorld( time ).Inverse();
 	} else
 	{
-		WhichMatToWorld ( matA, fhash );
+		if(!WhichMatToWorld ( matA, fhash ))
+			return(false);
 	}
 
 	if ( thash == ohash )
@@ -996,26 +1039,27 @@ CqMatrix	CqRenderer::matNSpaceToSpace( const char* strFrom, const char* strTo, c
 			matB = m_pTransCamera->matObjectToWorld( time );
 	} else
 	{
-		WhichMatWorldTo ( matB, thash );
+		if(!WhichMatWorldTo ( matB, thash ))
+			return(false);
 	}
 
 
-	matResult = matB * matA;
-	if (memcmp((void *) oldkey[1].pElements(), (void *) matResult.pElements(), sizeof(TqFloat) * 16) != 0)
+	result = matB * matA;
+	if (memcmp((void *) oldkey[1].pElements(), (void *) result.pElements(), sizeof(TqFloat) * 16) != 0)
 	{
-		oldkey[1] = matResult;
-		matResult[ 3 ][ 0 ] = matResult[ 3 ][ 1 ] = matResult[ 3 ][ 2 ] = matResult[ 0 ][ 3 ] = matResult[ 1 ][ 3 ] = matResult[ 2 ][ 3 ] = 0.0;
-		matResult[ 3 ][ 3 ] = 1.0;
-		matResult = matResult.Inverse().Transpose();
-		oldresult[1] = matResult;
+		oldkey[1] = result;
+		result[ 3 ][ 0 ] = result[ 3 ][ 1 ] = result[ 3 ][ 2 ] = result[ 0 ][ 3 ] = result[ 1 ][ 3 ] = result[ 2 ][ 3 ] = 0.0;
+		result[ 3 ][ 3 ] = 1.0;
+		result = result.Inverse().Transpose();
+		oldresult[1] = result;
 
 	}
 	else
 	{
-		return oldresult[1];
+		result = oldresult[1];
 	}
 
-	return ( matResult );
+	return ( true );
 }
 
 
@@ -1073,10 +1117,10 @@ CqColor*	CqRenderer::GetColorOptionWrite( const char* strName, const char* strPa
 
 //----------------------------------------------------------------------
 /** Store the named coordinate system in the array of named coordinate systems, overwrite any existing
- * with the same name. Returns TqTrue if system already exists.
+ * with the same name. Returns true if system already exists.
  */
 
-TqBool	CqRenderer::SetCoordSystem( const char* strName, const CqMatrix& matToWorld )
+bool	CqRenderer::SetCoordSystem( const char* strName, const CqMatrix& matToWorld )
 {
 	// Search for the same named system in the current list.
 	TqUlong hash = CqString::hash( strName );
@@ -1086,13 +1130,13 @@ TqBool	CqRenderer::SetCoordSystem( const char* strName, const CqMatrix& matToWor
 		{
 			m_aCoordSystems[ i ].m_matToWorld = matToWorld;
 			m_aCoordSystems[ i ].m_matWorldTo = matToWorld.Inverse();
-			return ( TqTrue );
+			return ( true );
 		}
 	}
 
 	// If we got here, it didn't exists.
 	m_aCoordSystems.push_back( SqCoordSys( strName, matToWorld, matToWorld.Inverse() ) );
-	return ( TqFalse );
+	return ( false );
 }
 
 
@@ -1218,7 +1262,7 @@ void CqRenderer::AddParameterDecl( const char* strName, const char* strType )
 	}
 	catch( XqException e )
 	{
-		Aqsis::log() << error << e.strReason().c_str() << std::endl;
+		Aqsis::log() << error << e.what () << std::endl;
 		return;
 	}
 
@@ -1227,43 +1271,6 @@ void CqRenderer::AddParameterDecl( const char* strName, const char* strType )
 }
 
 
-//---------------------------------------------------------------------
-/** Register a shader of the specified type with the specified name.
- */
-#if 0
-void CqRenderer::RegisterShader( const char* strName, EqShaderType type, IqShader* pShader )
-{
-	assert( pShader );
-	m_Shaders.LinkLast( new CqShaderRegister( strName, type, pShader ) );
-}
-#endif
-
-
-//---------------------------------------------------------------------
-/** Find a shader of the specified type with the specified name.
- */
-#if 0
-CqShaderRegister* CqRenderer::FindShader( const char* strName, EqShaderType type )
-{
-	// Search the register list.
-	CqShaderRegister * pShaderRegister = m_Shaders.pFirst();
-
-
-
-	while ( pShaderRegister )
-	{
-		if ( pShaderRegister->strName() == strName && pShaderRegister->Type() == type )
-		{
-			pOShaderRegister = pShaderRegister ;
-			return ( pShaderRegister );
-		}
-
-		pShaderRegister = pShaderRegister->pNext();
-	}
-	return ( 0 );
-}
-#endif
-
 //---------------------------------------------------------------------
 /** Returns a pointer to the default surface.
  */
@@ -1316,10 +1323,13 @@ boost::shared_ptr<IqShader> CqRenderer::CreateShader(
 
 	// first, look for the shader of the appropriate type and name in the
 	//  map of shader "templates"
-	if ( m_Shaders.find(key) != m_Shaders.end() )
+	TqShaderMap::const_iterator shadLocation = m_Shaders.find(key);
+	if(shadLocation != m_Shaders.end())
 	{
-		// the shader template is present, so return its clone
-		boost::shared_ptr<IqShader> newShader(m_Shaders[key]->Clone());
+		if(!shadLocation->second)
+			return boost::shared_ptr<IqShader>();
+		// The shader template is present, so return its clone
+		boost::shared_ptr<IqShader> newShader(shadLocation->second->Clone());
 		newShader->SetType( type );
 		m_InstancedShaders.push_back(newShader);
 		return (newShader);
@@ -1333,30 +1343,37 @@ boost::shared_ptr<IqShader> CqRenderer::CreateShader(
 	CqRiFile SLXFile( strFilename.c_str(), "shader" );
 	if ( SLXFile.IsValid() )
 	{
-		boost::shared_ptr<IqShader> pRet( new CqShaderVM(this) );
-
-		CqShaderVM* pShader = static_cast<CqShaderVM*>( pRet.get() );
-		const CqString* poptDSOPath = QGetRenderContext()->
-		                              poptCurrent()->GetStringOption( "searchpath", "shader" );
+		boost::shared_ptr<CqShaderVM> pRet(new CqShaderVM(this));
 
+		const CqString* poptDSOPath = QGetRenderContext()->poptCurrent()
+			->GetStringOption( "searchpath", "shader" );
 		if(poptDSOPath)
 		{
 			Aqsis::log() << info << "DSO lib path set to \"" << poptDSOPath->c_str()
-			<< "\"" << std::endl;
-
-			pShader->SetDSOPath( poptDSOPath->c_str() );
+				<< "\"" << std::endl;
+			pRet->SetDSOPath(poptDSOPath->c_str());
 		}
 
-		CqString strRealName( SLXFile.strRealName() );
 		Aqsis::log() << info << "Loading shader \"" << strName
-		<< "\" from file \"" << strRealName.c_str()
-		<< "\"" << std::endl;
+			<< "\" from file \"" << SLXFile.strRealName()
+			<< "\"" << std::endl;
 
-		pShader->SetstrName( strName );
-		pShader->LoadProgram( SLXFile );
+		pRet->SetstrName( strName );
+		try
+		{
+			pRet->LoadProgram( SLXFile );
+		}
+		catch(XqBadShader& e)
+		{
+			Aqsis::log() << error << "could not load shader \"" << strName << "\": "
+				<< e.what() << "\n";
+			// couldn't load the shader; put a null pointer in the map so we
+			// don't try again, and return null.
+			m_Shaders[key] = boost::shared_ptr<CqShaderVM>();
+			return boost::shared_ptr<CqShaderVM>();
+		}
 
-		// add the shader to the map as a template and return its
-		//  clone
+		// add the shader to the map as a template and return its clone
 		m_Shaders[key] = pRet;
 		boost::shared_ptr<IqShader> newShader(pRet->Clone());
 		newShader->SetType( type );
@@ -1365,10 +1382,7 @@ boost::shared_ptr<IqShader> CqRenderer::CreateShader(
 	}
 	else
 	{
-		if (
-		    (strcmp( strName, "null" )  != 0) &&
-		    (strcmp( strName, "_def_" ) != 0)
-		)
+		if( strcmp(strName, "null") != 0  &&  strcmp(strName, "_def_") != 0 )
 		{
 			CqString strError;
 			strError.Format( "Shader \"%s\" not found", strName ? strName : "" );
@@ -1398,7 +1412,6 @@ boost::shared_ptr<IqShader> CqRenderer::CreateShader(
 		}
 		else
 		{
-			// the boost::shared_ptr analogue of return NULL:
 			return boost::shared_ptr<IqShader>();
 		}
 	}
@@ -1418,9 +1431,11 @@ void CqRenderer::StorePrimitive( const boost::shared_ptr<CqSurface>& pSurface )
 		m_aWorld.push_back(pSurface);
 	else
 	{
-		pSurface->Transform( QGetRenderContext() ->matSpaceToSpace( "world", "camera", NULL, pSurface->pTransform().get(), 0 ),
-						 QGetRenderContext() ->matNSpaceToSpace( "world", "camera", NULL, pSurface->pTransform().get(), 0 ),
-						 QGetRenderContext() ->matVSpaceToSpace( "world", "camera", NULL, pSurface->pTransform().get(), 0 ) );
+		CqMatrix matWtoC, matNWtoC, matVWtoC;
+		QGetRenderContext() ->matSpaceToSpace( "world", "camera", NULL, pSurface->pTransform().get(), 0, matWtoC );
+		QGetRenderContext() ->matNSpaceToSpace( "world", "camera", NULL, pSurface->pTransform().get(), 0, matNWtoC );
+		QGetRenderContext() ->matVSpaceToSpace( "world", "camera", NULL, pSurface->pTransform().get(), 0, matVWtoC );
+		pSurface->Transform( matWtoC, matNWtoC, matVWtoC);
 		pSurface->PrepareTrimCurve();
 		PostSurface(pSurface);
 	}
@@ -1432,9 +1447,11 @@ void CqRenderer::PostWorld()
 	{
 		boost::shared_ptr<CqSurface> pSurface = m_aWorld.front();
 		
-		pSurface->Transform( QGetRenderContext() ->matSpaceToSpace( "world", "camera", NULL, pSurface->pTransform().get(), 0 ),
-						 QGetRenderContext() ->matNSpaceToSpace( "world", "camera", NULL, pSurface->pTransform().get(), 0 ),
-						 QGetRenderContext() ->matVSpaceToSpace( "world", "camera", NULL, pSurface->pTransform().get(), 0 ) );
+		CqMatrix matWtoC, matNWtoC, matVWtoC;
+		QGetRenderContext() ->matSpaceToSpace( "world", "camera", NULL, pSurface->pTransform().get(), 0, matWtoC );
+		QGetRenderContext() ->matNSpaceToSpace( "world", "camera", NULL, pSurface->pTransform().get(), 0, matNWtoC );
+		QGetRenderContext() ->matVSpaceToSpace( "world", "camera", NULL, pSurface->pTransform().get(), 0, matVWtoC );
+		pSurface->Transform( matWtoC, matNWtoC, matVWtoC);
 		pSurface->PrepareTrimCurve();
 		PostSurface(pSurface);
 		m_aWorld.pop_front();
@@ -1448,9 +1465,11 @@ void CqRenderer::PostCloneOfWorld()
 	for(i=m_aWorld.begin(); i!=m_aWorld.end(); i++)
 	{
 		boost::shared_ptr<CqSurface> pSurface((*i)->Clone());
-		pSurface->Transform( QGetRenderContext() ->matSpaceToSpace( "world", "camera", NULL, pSurface->pTransform().get(), 0 ),
-						 QGetRenderContext() ->matNSpaceToSpace( "world", "camera", NULL, pSurface->pTransform().get(), 0 ),
-						 QGetRenderContext() ->matVSpaceToSpace( "world", "camera", NULL, pSurface->pTransform().get(), 0 ) );
+		CqMatrix matWtoC, matNWtoC, matVWtoC;
+		QGetRenderContext() ->matSpaceToSpace( "world", "camera", NULL, pSurface->pTransform().get(), 0, matWtoC );
+		QGetRenderContext() ->matNSpaceToSpace( "world", "camera", NULL, pSurface->pTransform().get(), 0, matNWtoC );
+		QGetRenderContext() ->matVSpaceToSpace( "world", "camera", NULL, pSurface->pTransform().get(), 0, matVWtoC );
+		pSurface->Transform( matWtoC, matNWtoC, matVWtoC);
 		pSurface->PrepareTrimCurve();
 		PostSurface(pSurface);
 	}
@@ -1466,7 +1485,9 @@ void CqRenderer::PostSurface( const boost::shared_ptr<CqSurface>& pSurface )
 	CqBound bound(boundAttr);
 	if(bound.Volume2() > 0)
 	{
-		bound.Transform( QGetRenderContext() ->matSpaceToSpace( "object", "raster", NULL, pSurface->pTransform().get(), QGetRenderContext()->Time() ) );
+		CqMatrix mat;
+		QGetRenderContext() ->matSpaceToSpace( "object", "raster", NULL, pSurface->pTransform().get(), QGetRenderContext()->Time(), mat );
+		bound.Transform( mat );
 
 		TqFloat ruler = fabs( ( bound.vecMax().x() - bound.vecMin().x() ) * ( bound.vecMax().y() - bound.vecMin().y() ) );
 
@@ -1482,13 +1503,13 @@ void CqRenderer::PostSurface( const boost::shared_ptr<CqSurface>& pSurface )
 		if( rangeAttr[1] == rangeAttr[0] )
 			minImportance = ruler < rangeAttr[1] ? 1.0f : 0.0f;
 		else
-			minImportance = CLAMP( ( rangeAttr[1] - ruler ) / ( rangeAttr[1] - rangeAttr[0] ), 0, 1 );
+			minImportance = clamp(( rangeAttr[1] - ruler ) / ( rangeAttr[1] - rangeAttr[0] ), 0.0f, 1.0f);
 
 		TqFloat maxImportance;
 		if ( rangeAttr[2] == rangeAttr[3] )
 			maxImportance = ruler < rangeAttr[2] ? 1.0f : 0.0f;
 		else
-			maxImportance = CLAMP( ( rangeAttr[3] - ruler ) / ( rangeAttr[3] - rangeAttr[2] ), 0, 1 );
+			maxImportance = clamp((rangeAttr[3] - ruler)/(rangeAttr[3] - rangeAttr[2]), 0.0f, 1.0f);
 
 		if ( minImportance >= maxImportance )
 			// Geomtry must be culled.
@@ -1515,70 +1536,6 @@ void CqRenderer::PrepareShaders()
 }
 
 
-//---------------------------------------------------------------------
-/** Find a shader of the specified type with the specified name.
- * If not found, try and load one.
- */
-#if 0
-IqShader* CqRenderer::CreateShader( const char* strName, EqShaderType type )
-{
-	CqShaderRegister * pReg = NULL;
-
-	if ( pOShaderRegister && pOShaderRegister->strName() == strName && pOShaderRegister->Type() == type )
-		pReg = pOShaderRegister;
-	else
-		pReg = FindShader( strName, type );
-	if ( pReg != 0 )
-	{
-		IqShader * pShader = pReg->Create();
-		RegisterShader( strName, type, pShader );
-		return ( pShader );
-	}
-	else
-	{
-		// Search in the current directory first.
-		CqString strFilename( strName );
-		strFilename += RI_SHADER_EXTENSION;
-		CqRiFile SLXFile( strFilename.c_str(), "shader" );
-		if ( SLXFile.IsValid() )
-		{
-			CqShaderVM * pShader = new CqShaderVM(this);
-			const CqString *poptDSOPath = QGetRenderContext()->poptCurrent()->GetStringOption( "searchpath","shader" );
-			pShader->SetDSOPath( poptDSOPath );
-
-			CqString strRealName( SLXFile.strRealName() );
-			Aqsis::log() << info << "Loading shader \"" << strName << "\" from file \"" << strRealName.c_str() << "\"" << std::endl;
-
-			pShader->SetstrName( strName );
-			pShader->LoadProgram( SLXFile );
-			RegisterShader( strName, type, pShader );
-			return ( pShader );
-		}
-		else
-		{
-			if ( strcmp( strName, "null" ) != 0 )
-			{
-				CqString strError;
-				strError.Format( "Shader \"%s\" not found", strName ? strName : "" );
-				Aqsis::log() << error << strError.c_str() << std::endl;
-			}
-			if( type == Type_Surface )
-			{
-				CqShaderVM * pShader = new CqShaderVM(this);
-				pShader->SetstrName( "null" );
-				pShader->DefaultSurface();
-				RegisterShader( strName, type, pShader );
-				return ( pShader );
-			}
-			else
-				return ( NULL );
-		}
-	}
-}
-#endif
-
-
-
 //---------------------------------------------------------------------
 /** Add a new requested display driver to the list.
  */
@@ -1611,43 +1568,52 @@ IqRenderer* QGetRenderContextI()
 }
 
 
-IqTextureMap* CqRenderer::GetTextureMap( const CqString& strFileName )
+CqTextureCache& CqRenderer::textureCache()
 {
-	return ( CqTextureMap::GetTextureMap( strFileName ) );
+	return m_textureCache;
 }
 
-IqTextureMap* CqRenderer::GetEnvironmentMap( const CqString& strFileName )
+IqTextureMapOld* CqRenderer::GetEnvironmentMap( const CqString& strFileName )
 {
-	return ( CqTextureMap::GetEnvironmentMap( strFileName ) );
+	return ( CqTextureMapOld::GetEnvironmentMap( strFileName ) );
 }
 
-IqTextureMap* CqRenderer::GetShadowMap( const CqString& strFileName )
+IqTextureMapOld* CqRenderer::GetOcclusionMap( const CqString& strFileName )
 {
-	return ( CqTextureMap::GetShadowMap( strFileName ) );
+	return ( CqTextureMapOld::GetShadowMap( strFileName ) );
 }
 
-IqTextureMap* CqRenderer::GetLatLongMap( const CqString& strFileName )
+IqTextureMapOld* CqRenderer::GetLatLongMap( const CqString& strFileName )
 {
-	return ( CqTextureMap::GetLatLongMap( strFileName ) );
+	return ( CqTextureMapOld::GetLatLongMap( strFileName ) );
+}
+
+const char* CqRenderer::textureSearchPath()
+{
+	const CqString* pathPtr = poptCurrent()->GetStringOption("searchpath", "texture");
+	if(pathPtr)
+		return pathPtr->c_str();
+	else
+		return "";
 }
 
 
-TqBool	CqRenderer::GetBasisMatrix( CqMatrix& matBasis, const CqString& name )
+bool	CqRenderer::GetBasisMatrix( CqMatrix& matBasis, const CqString& name )
 {
 	RtBasis basis;
 	if ( BasisFromName( &basis, name.c_str() ) )
 	{
 		matBasis = basis;
-		return ( TqTrue );
+		return ( true );
 	}
 	else
-		return ( TqFalse );
+		return ( false );
 }
 
 //---------------------------------------------------------------------
 /** Which matrix will be used in ToWorld
  */
-void CqRenderer::WhichMatToWorld( CqMatrix &matA, TqUlong thash )
+bool CqRenderer::WhichMatToWorld( CqMatrix &matA, TqUlong thash )
 {
 	static TqInt awhich = 0;
 	TqInt tmp = awhich;
@@ -1658,7 +1624,7 @@ void CqRenderer::WhichMatToWorld( CqMatrix &matA, TqUlong thash )
 		if ( m_aCoordSystems[ awhich ].m_hash == thash )
 		{
 			matA = m_aCoordSystems[ awhich ].m_matToWorld;
-			return ;
+			return(true);
 		}
 	}
 
@@ -1668,16 +1634,17 @@ void CqRenderer::WhichMatToWorld( CqMatrix &matA, TqUlong thash )
 		if ( m_aCoordSystems[ awhich ].m_hash == thash )
 		{
 			matA = m_aCoordSystems[ awhich ].m_matToWorld;
-			break;
+			return(true);
 		}
 	}
+	return(false);
 }
 
 //---------------------------------------------------------------------
 /** Which matrix will be used in WorldTo
  */
 
-void CqRenderer::WhichMatWorldTo( CqMatrix &matB, TqUlong thash )
+bool CqRenderer::WhichMatWorldTo( CqMatrix &matB, TqUlong thash )
 {
 	static TqInt bwhich = 0;
 	TqInt tmp = bwhich;
@@ -1688,7 +1655,7 @@ void CqRenderer::WhichMatWorldTo( CqMatrix &matB, TqUlong thash )
 		if ( m_aCoordSystems[ bwhich ].m_hash == thash )
 		{
 			matB = m_aCoordSystems[ bwhich ].m_matWorldTo;
-			return ;
+			return(true);
 		}
 	}
 
@@ -1698,9 +1665,10 @@ void CqRenderer::WhichMatWorldTo( CqMatrix &matB, TqUlong thash )
 		if ( m_aCoordSystems[ bwhich ].m_hash == thash )
 		{
 			matB = m_aCoordSystems[ bwhich ].m_matWorldTo;
-			break;
+			return(true);
 		}
 	}
+	return(false);
 }
 
 
@@ -1717,7 +1685,7 @@ TqInt CqRenderer::RegisterOutputData( const char* name )
 	}
 	catch( XqException e )
 	{
-		Aqsis::log() << error << e.strReason().c_str() << std::endl;
+			Aqsis::log() << error << e.what() << std::endl;
 		return(-1);
 	}
 	if( Decl.m_Type != type_invalid )
@@ -1779,7 +1747,7 @@ TqInt CqRenderer::OutputDataIndex( const char* name )
 	}
 	catch( XqException e )
 	{
-		Aqsis::log() << error << e.strReason().c_str() << std::endl;
+		Aqsis::log() << error << e.what() << std::endl;
 		return(-1);
 	}
 	if( Decl.m_Type != type_invalid )
@@ -1800,7 +1768,7 @@ TqInt CqRenderer::OutputDataSamples( const char* name )
 	}
 	catch( XqException e )
 	{
-		Aqsis::log() << error << e.strReason().c_str() << std::endl;
+		Aqsis::log() << error << e.what() << std::endl;
 		return(-1);
 	}
 	if( Decl.m_Type != type_invalid )
@@ -1822,7 +1790,7 @@ TqInt CqRenderer::OutputDataType( const char* name )
 	}
 	catch( XqException e )
 	{
-		Aqsis::log() << error << e.strReason().c_str() << std::endl;
+		Aqsis::log() << error << e.what() << std::endl;
 		return(-1);
 	}
 	if( Decl.m_Type != type_invalid )
@@ -1838,7 +1806,7 @@ TqInt CqRenderer::OutputDataType( const char* name )
 CqObjectInstance* CqRenderer::OpenNewObjectInstance()
 {
 	assert( !m_bObjectOpen );
-	m_bObjectOpen = TqTrue;
+	m_bObjectOpen = true;
 	CqObjectInstance* pNew = new CqObjectInstance;
 	m_ObjectInstances.push_back(pNew);
 
@@ -1874,7 +1842,24 @@ void	CqRenderer::SetImage( CqImageBuffer* pImage )
 	m_pImageBuffer = pImage;
 }
 
+const TqFloat CqRenderer::MinCoCForBound(const CqBound& bound) const
+{
+	assert(m_UsingDepthOfField);
+	TqFloat z1 = bound.vecMin().z();
+	TqFloat z2 = bound.vecMax().z();
+	// First check whether the bound spans the focal plane; if so, return 0.
+	TqFloat focalDist = 1/m_OneOverFocalDistance;
+	if((z1 - focalDist)*(z2 - focalDist) < 0)
+		return 0;
+	// Otherwise, the minimum focal blur is achieved at one of the
+	// z-extents of the bound.
+	TqFloat minBlur = min(std::fabs(1/z1 - m_OneOverFocalDistance),
+				std::fabs(1/z2 - m_OneOverFocalDistance));
+	return m_DofMultiplier * min(m_DepthOfFieldScale.x(),
+			m_DepthOfFieldScale.y()) * minBlur;
+}
+
 //---------------------------------------------------------------------
 
-END_NAMESPACE( Aqsis )
+} // namespace Aqsis
 
