@@ -1,5 +1,5 @@
 // Aqsis
-// Copyright  1997 - 2001, Paul C. Gregory
+// Copyright (C) 1997 - 2001, Paul C. Gregory
 //
 // Contact: pgregory@aqsis.org
 //
@@ -45,10 +45,11 @@
 #include	"irenderer.h"
 #include	"iraytrace.h"
 #include	"iraytrace.h"
+#include	"texturecache.h"
 
 #include	"clippingvolume.h"
 
-START_NAMESPACE( Aqsis )
+namespace Aqsis {
 
 class CqImageBuffer;
 class CqObjectInstance;
@@ -117,6 +118,7 @@ class CqRenderer : public IqRenderer
 		virtual	boost::shared_ptr<CqModeBlock>	BeginSolidModeBlock( CqString& type );
 		virtual	boost::shared_ptr<CqModeBlock>	BeginObjectModeBlock();
 		virtual	boost::shared_ptr<CqModeBlock>	BeginMotionModeBlock( TqInt N, TqFloat times[] );
+		virtual	boost::shared_ptr<CqModeBlock>	BeginResourceModeBlock();
 
 		virtual	void	EndMainModeBlock();
 		virtual	void	EndFrameModeBlock();
@@ -126,6 +128,7 @@ class CqRenderer : public IqRenderer
 		virtual	void	EndSolidModeBlock();
 		virtual	void	EndObjectModeBlock();
 		virtual	void	EndMotionModeBlock();
+		virtual	void	EndResourceModeBlock();
 
 		virtual	const IqOptionsPtr	poptCurrent() const;
 		virtual	IqOptionsPtr	poptWriteCurrent();
@@ -178,9 +181,9 @@ class CqRenderer : public IqRenderer
 		virtual	void	SetImage( CqImageBuffer* pImage );
 
 		// Handle various coordinate system transformation requirements.
-		virtual	CqMatrix	matSpaceToSpace	( const char* strFrom, const char* strTo, const IqTransform* transShaderToWorld, const IqTransform* transObjectToWorld, TqFloat time );
-		virtual	CqMatrix	matVSpaceToSpace	( const char* strFrom, const char* strTo, const IqTransform* transShaderToWorld, const IqTransform* transObjectToWorld, TqFloat time );
-		virtual	CqMatrix	matNSpaceToSpace	( const char* strFrom, const char* strTo, const IqTransform* transShaderToWorld, const IqTransform* transObjectToWorld, TqFloat time );
+		virtual	bool	matSpaceToSpace	( const char* strFrom, const char* strTo, const IqTransform* transShaderToWorld, const IqTransform* transObjectToWorld, TqFloat time, CqMatrix& result );
+		virtual	bool	matVSpaceToSpace	( const char* strFrom, const char* strTo, const IqTransform* transShaderToWorld, const IqTransform* transObjectToWorld, TqFloat time, CqMatrix& result );
+		virtual	bool	matNSpaceToSpace	( const char* strFrom, const char* strTo, const IqTransform* transShaderToWorld, const IqTransform* transObjectToWorld, TqFloat time, CqMatrix& result );
 
 		virtual	const	TqFloat*	GetFloatOption( const char* strName, const char* strParam ) const;
 		virtual	const	TqInt*	GetIntegerOption( const char* strName, const char* strParam ) const;
@@ -199,12 +202,14 @@ class CqRenderer : public IqRenderer
 			std::cout << str;
 		}
 
-		virtual	IqTextureMap* GetTextureMap( const CqString& strFileName );
-		virtual	IqTextureMap* GetEnvironmentMap( const CqString& strFileName );
-		virtual	IqTextureMap* GetShadowMap( const CqString& strFileName );
-		virtual	IqTextureMap* GetLatLongMap( const CqString& strFileName );
+		virtual	CqTextureCache& textureCache();
+		virtual	IqTextureMapOld* GetEnvironmentMap( const CqString& strFileName );
+		virtual	IqTextureMapOld* GetOcclusionMap(const CqString& fileName);
+		virtual	IqTextureMapOld* GetLatLongMap( const CqString& strFileName );
 
-		virtual	TqBool	GetBasisMatrix( CqMatrix& matBasis, const CqString& name );
+		virtual const char* textureSearchPath();
+
+		virtual	bool	GetBasisMatrix( CqMatrix& matBasis, const CqString& name );
 
 
 		/** Get a read only reference to the current transformation matrix.
@@ -215,11 +220,11 @@ class CqRenderer : public IqRenderer
 			return ( pconCurrent() ->matCurrent( time ) );
 		}
 
-		virtual	TqBool	SetCoordSystem( const char* strName, const CqMatrix& matToWorld );
+		virtual	bool	SetCoordSystem( const char* strName, const CqMatrix& matToWorld );
 
 		// Function which can be overridden by the derived class.
 		virtual	void	Initialise();
-		virtual	void	RenderWorld(TqBool clone = TqFalse);
+		virtual	void	RenderWorld(bool clone = false);
 		virtual void	RenderAutoShadows();
 
 		virtual	void	AddDisplayRequest( const TqChar* name, const TqChar* type, const TqChar* mode, TqInt modeID, TqInt dataOffset, TqInt dataSize, std::map<std::string, void*>& mapOfArguments );
@@ -338,9 +343,9 @@ class CqRenderer : public IqRenderer
 		}
 
 		/** Find out if we are using depth of field or not.
-		 * \return TqTrue if depth of field is turned on.
+		 * \return true if depth of field is turned on.
 		 */
-		TqBool	UsingDepthOfField( ) const
+		bool	UsingDepthOfField( ) const
 		{
 			return m_UsingDepthOfField;
 		}
@@ -366,8 +371,13 @@ class CqRenderer : public IqRenderer
 			return CqVector2D(m_DepthOfFieldScale * c);
 		}
 
-		//void	RegisterShader( const char* strName, EqShaderType type, IqShader* pShader );
-		//CqShaderRegister* FindShader( const char* strName, EqShaderType type );
+		/** \brief Return minimum circle of confusion radius for objects inside a bound
+		 *
+		 * If the bound spans the focal plane 0 is returned.  In other cases
+		 * the minimum circle of confusion is nonzero and lies on the bound.
+		 */
+		const TqFloat MinCoCForBound(const CqBound& bound) const;
+
 		boost::shared_ptr<IqShader> getDefaultSurfaceShader();
 
 		struct SqOutputDataEntry
@@ -414,7 +424,7 @@ class CqRenderer : public IqRenderer
 		virtual void InstantiateObject( CqObjectInstance* handle );
 		virtual	void CloseObjectInstance()
 		{
-			m_bObjectOpen = TqFalse;
+			m_bObjectOpen = false;
 		}
 
 		/** Get a pointer to the error handler function.
@@ -478,12 +488,12 @@ class CqRenderer : public IqRenderer
 			return( m_pRaytracer );
 		}
 
-		TqBool	IsWorldBegin() const
+		bool	IsWorldBegin() const
 		{
 			return(m_fWorldBegin);
 		}
 
-		void SetWorldBegin(TqBool begin = TqTrue)
+		void SetWorldBegin(bool begin = true)
 		{
 			m_fWorldBegin = begin;
 		}
@@ -499,6 +509,9 @@ class CqRenderer : public IqRenderer
 
 
 	private:
+		/// Map type to hold loaded reference shaders.
+		typedef std::map< CqShaderKey, boost::shared_ptr<IqShader> > TqShaderMap;
+
 		boost::shared_ptr<CqModeBlock>	m_pconCurrent;					///< Pointer to the current context.
 		CqStats	m_Stats;						///< Global statistics.
 		CqAttributes*	m_pAttrDefault;					///< Default attributes.
@@ -509,22 +522,25 @@ class CqRenderer : public IqRenderer
 		IqDDManager*	m_pDDManager;
 
 		EqRenderMode	m_Mode;
-		std::map< CqShaderKey, boost::shared_ptr<IqShader> > m_Shaders;
+		TqShaderMap m_Shaders;
 		std::vector< boost::shared_ptr<IqShader> >  m_InstancedShaders;
 
-		TqBool	m_fSaveGPrims;
+		CqTextureCache m_textureCache; ///< Cache for aqsistex texture access.
+		 
+
+		bool	m_fSaveGPrims;
 		CqTransformPtr	m_pTransCamera;					///< The camera transform.
 		CqTransformPtr	m_pTransDefObj;				///< The initial transformation for objects.
-		TqBool			m_fWorldBegin;
+		bool			m_fWorldBegin;
 		std::vector<SqParameterDeclaration>	m_Symbols;	///< Symbol table.
 
 		TqFloat			m_DofMultiplier;
 		TqFloat			m_OneOverFocalDistance;
-		TqBool			m_UsingDepthOfField;
+		bool			m_UsingDepthOfField;
 		CqVector2D		m_DepthOfFieldScale;
 
-		void WhichMatWorldTo(CqMatrix &a, TqUlong thash);
-		void WhichMatToWorld(CqMatrix &b, TqUlong thash);
+		bool WhichMatWorldTo(CqMatrix &a, TqUlong thash);
+		bool WhichMatToWorld(CqMatrix &b, TqUlong thash);
 
 		std::map<std::string, SqOutputDataEntry>	m_OutputDataEntries;
 		TqInt	m_OutputDataOffset;
@@ -533,7 +549,7 @@ class CqRenderer : public IqRenderer
 
 		TqInt	m_FrameNo;
 		std::vector<CqObjectInstance*>	m_ObjectInstances;
-		TqBool	m_bObjectOpen;
+		bool	m_bObjectOpen;
 
 		RtErrorFunc	m_pErrorHandler;		///< A pointer to the error hadling function.
 		RtProgressFunc	m_pProgressHandler;		///< A pointer to the progress hadling function.
@@ -563,7 +579,7 @@ void	QSetRenderContext( CqRenderer* pRenderer );
 
 //-----------------------------------------------------------------------
 
-END_NAMESPACE( Aqsis )
+} // namespace Aqsis
 
 //}  // End of #ifdef RENDERER_H_INCLUDED
 #endif
