@@ -1,5 +1,5 @@
 // Aqsis
-// Copyright  1997 - 2001, Paul C. Gregory
+// Copyright (C) 1997 - 2001, Paul C. Gregory
 //
 // Contact: pgregory@aqsis.org
 //
@@ -30,7 +30,7 @@
 #include	"vector2d.h"
 #include	"imagebuffer.h"
 
-START_NAMESPACE( Aqsis )
+namespace Aqsis {
 
 TqFloat CqSurface::m_fGridSize = sqrt(256.0);
 
@@ -111,10 +111,8 @@ TqInt CqSurface::Uses() const
 /** Adjust the bound of the quadric taking into account transformation motion blur.
  */
 
-CqBound	CqSurface::AdjustBoundForTransformationMotion( const CqBound& B ) const
+void CqSurface::AdjustBoundForTransformationMotion( IqBound* B ) const
 {
-	CqBound Bm( B );
-
 	// Create a map of transformation keyframes, taking into account both object and camera motion.
 	TqInt iTime;
 	IqTransformPtr objectTransform = pTransform();
@@ -130,21 +128,23 @@ CqBound	CqSurface::AdjustBoundForTransformationMotion( const CqBound& B ) const
 
 	if( keyframeTimes.size() > 1 )
 	{
-		CqMatrix matCameraToObject0 = QGetRenderContext() ->matSpaceToSpace( "camera", "object", NULL, pTransform().get(), keyframeTimes.begin()->second );
-		CqBound B0( B );
+		CqMatrix matCameraToObject0;
+		QGetRenderContext() ->matSpaceToSpace( "camera", "object", NULL, pTransform().get(), keyframeTimes.begin()->second, matCameraToObject0 );
+		CqBound B0;
+		B0.vecMin() = B->vecMin();
+		B0.vecMax() = B->vecMax();
 		B0.Transform( matCameraToObject0 );
 
 		std::map<TqFloat, TqFloat>::iterator keyFrame;
 		for( keyFrame = keyframeTimes.begin(); keyFrame != keyframeTimes.end(); keyFrame++)
 		{
 			CqBound Btx( B0 );
-			CqMatrix matObjectToCameraT = QGetRenderContext() ->matSpaceToSpace( "object", "camera", NULL, pTransform().get(), keyFrame->second );
+			CqMatrix matObjectToCameraT;
+			QGetRenderContext() ->matSpaceToSpace( "object", "camera", NULL, pTransform().get(), keyFrame->second, matObjectToCameraT );
 			Btx.Transform( matObjectToCameraT );
-			Bm.Encapsulate( Btx );
+			B->Encapsulate( &Btx );
 		}
 	}
-
-	return ( Bm );
 }
 
 
@@ -152,17 +152,24 @@ CqBound	CqSurface::AdjustBoundForTransformationMotion( const CqBound& B ) const
 /** Default constructor
  */
 
-CqSurface::CqSurface() : m_fDiceable( TqTrue ), m_fDiscard( TqFalse ), m_EyeSplitCount( 0 ),
-		m_pAttributes( 0 ), m_SplitDir( SplitDir_U )
+CqSurface::CqSurface()
+	: m_fDiceable(true),
+	m_fDiscard(false),
+	m_SplitCount(0),
+	m_aUserParams(),
+	m_pAttributes(0),
+	m_pTransform(QGetRenderContext()->ptransCurrent()),
+	m_uDiceSize(1),
+	m_vDiceSize(1),
+	m_SplitDir(SplitDir_U),
+	m_CachedBound(false),
+	m_Bound(),
+	m_pCSGNode()
 {
 	// Set a refernce with the current attributes.
 	m_pAttributes = const_cast<CqAttributes*>( QGetRenderContext() ->pattrCurrent() );
 	ADDREF( m_pAttributes );
 
-	m_pTransform = QGetRenderContext() ->ptransCurrent();
-
-	m_CachedBound = TqFalse;
-
 	// If the current context is a solid node, and is a 'primitive', attatch this surface to the node.
 	if ( QGetRenderContext() ->pconCurrent() ->isSolid() )
 	{
@@ -181,7 +188,6 @@ CqSurface::CqSurface() : m_fDiceable( TqTrue ), m_fDiscard( TqFalse ), m_EyeSpli
 		}
 	}
 
-
 	// Nullify the standard primitive variables index table.
 	TqInt i;
 	for ( i = 0; i < EnvVars_Last; i++ )
@@ -203,7 +209,7 @@ CqSurface::CqSurface() : m_fDiceable( TqTrue ), m_fDiscard( TqFalse ), m_EyeSpli
 void CqSurface::CloneData( CqSurface* clone ) const
 {
 	clone->m_fDiceable = m_fDiceable;
-	clone->m_EyeSplitCount = m_EyeSplitCount;
+	clone->m_SplitCount = m_SplitCount;
 	clone->m_fDiscard = m_fDiscard;
 
 	clone->SetSurfaceParameters( *this );
@@ -240,7 +246,7 @@ void CqSurface::ClonePrimitiveVariables( const CqSurface& From )
  * primitive variables.
  */
 
-void CqSurface::SetDefaultPrimitiveVariables( TqBool bUseDef_st )
+void CqSurface::SetDefaultPrimitiveVariables( bool bUseDef_st )
 {
 	TqInt bUses = Uses();
 
@@ -284,148 +290,144 @@ void CqSurface::SetDefaultPrimitiveVariables( TqBool bUseDef_st )
 }
 
 
-void CqSurface::NaturalSubdivide( CqParameter* pParam, CqParameter* pParam1, CqParameter* pParam2, TqBool u )
+
+namespace {
+
+/** \brief Helper function for generic natural subdivision of surface parameters.
+ *
+ * Uses linear interpolation to split the parameters along either the u or v
+ * directions.
+ *
+ * \param pParam - pointer to source parameter data
+ * \param pResult1 - pointer to output parameter for first result curve
+ * \param pResult1 - pointer to output parameter for second result curve
+ * \param u - if true, split along the u-direction, otherwise split along v.
+ */
+template <class T, class SLT>
+void surfaceNaturalSubdivide(CqParameter* pParam, CqParameter* pResult1,
+		CqParameter* pResult2, bool u )
 {
-	switch ( pParam->Type() )
+	CqParameterTyped<T, SLT>* pTParam = static_cast<CqParameterTyped<T, SLT>*>( pParam );
+	CqParameterTyped<T, SLT>* pTResult1 = static_cast<CqParameterTyped<T, SLT>*>( pResult1 );
+	CqParameterTyped<T, SLT>* pTResult2 = static_cast<CqParameterTyped<T, SLT>*>( pResult2 );
+
+	for(TqInt i = 0; i < pParam->Count(); i++)
 	{
-			case type_float:
-			{
-				CqParameterTyped<TqFloat, TqFloat>* pTParam = static_cast<CqParameterTyped<TqFloat, TqFloat>*>( pParam );
-				CqParameterTyped<TqFloat, TqFloat>* pTResult1 = static_cast<CqParameterTyped<TqFloat, TqFloat>*>( pParam1 );
-				CqParameterTyped<TqFloat, TqFloat>* pTResult2 = static_cast<CqParameterTyped<TqFloat, TqFloat>*>( pParam2 );
-				TypedNaturalSubdivide( pTParam, pTResult1, pTResult2, u );
-				break;
-			}
+		if ( u )
+		{
+			pTResult2->pValue( 1 ) [ i ] = pTParam->pValue( 1 ) [ i ];
+			pTResult2->pValue( 3 ) [ i ] = pTParam->pValue( 3 ) [ i ];
+			pTResult1->pValue( 1 ) [ i ] = pTResult2->pValue( 0 ) [ i ] = static_cast<T>( ( pTParam->pValue( 0 ) [ i ] + pTParam->pValue( 1 ) [ i ] ) * 0.5 );
+			pTResult1->pValue( 3 ) [ i ] = pTResult2->pValue( 2 ) [ i ] = static_cast<T>( ( pTParam->pValue( 2 ) [ i ] + pTParam->pValue( 3 ) [ i ] ) * 0.5 );
+		}
+		else
+		{
+			pTResult2->pValue( 2 ) [ i ] = pTParam->pValue( 2 ) [ i ];
+			pTResult2->pValue( 3 ) [ i ] = pTParam->pValue( 3 ) [ i ];
+			pTResult1->pValue( 2 ) [ i ] = pTResult2->pValue( 0 ) [ i ] = static_cast<T>( ( pTParam->pValue( 0 ) [ i ] + pTParam->pValue( 2 ) [ i ] ) * 0.5 );
+			pTResult1->pValue( 3 ) [ i ] = pTResult2->pValue( 1 ) [ i ] = static_cast<T>( ( pTParam->pValue( 1 ) [ i ] + pTParam->pValue( 3 ) [ i ] ) * 0.5 );
+		}
+	}
+}
 
-			case type_integer:
-			{
-				CqParameterTyped<TqInt, TqFloat>* pTParam = static_cast<CqParameterTyped<TqInt, TqFloat>*>( pParam );
-				CqParameterTyped<TqInt, TqFloat>* pTResult1 = static_cast<CqParameterTyped<TqInt, TqFloat>*>( pParam1 );
-				CqParameterTyped<TqInt, TqFloat>* pTResult2 = static_cast<CqParameterTyped<TqInt, TqFloat>*>( pParam2 );
-				TypedNaturalSubdivide( pTParam, pTResult1, pTResult2, u );
-				break;
-			}
+} // unnamed namespace
 
-			case type_point:
-			case type_vector:
-			case type_normal:
-			{
-				CqParameterTyped<CqVector3D, CqVector3D>* pTParam = static_cast<CqParameterTyped<CqVector3D, CqVector3D>*>( pParam );
-				CqParameterTyped<CqVector3D, CqVector3D>* pTResult1 = static_cast<CqParameterTyped<CqVector3D, CqVector3D>*>( pParam1 );
-				CqParameterTyped<CqVector3D, CqVector3D>* pTResult2 = static_cast<CqParameterTyped<CqVector3D, CqVector3D>*>( pParam2 );
-				TypedNaturalSubdivide( pTParam, pTResult1, pTResult2, u );
-				break;
-			}
-
-			case type_hpoint:
-			{
-				CqParameterTyped<CqVector4D, CqVector3D>* pTParam = static_cast<CqParameterTyped<CqVector4D, CqVector3D>*>( pParam );
-				CqParameterTyped<CqVector4D, CqVector3D>* pTResult1 = static_cast<CqParameterTyped<CqVector4D, CqVector3D>*>( pParam1 );
-				CqParameterTyped<CqVector4D, CqVector3D>* pTResult2 = static_cast<CqParameterTyped<CqVector4D, CqVector3D>*>( pParam2 );
-				TypedNaturalSubdivide( pTParam, pTResult1, pTResult2, u );
-				break;
-			}
-
-
-			case type_color:
-			{
-				CqParameterTyped<CqColor, CqColor>* pTParam = static_cast<CqParameterTyped<CqColor, CqColor>*>( pParam );
-				CqParameterTyped<CqColor, CqColor>* pTResult1 = static_cast<CqParameterTyped<CqColor, CqColor>*>( pParam1 );
-				CqParameterTyped<CqColor, CqColor>* pTResult2 = static_cast<CqParameterTyped<CqColor, CqColor>*>( pParam2 );
-				TypedNaturalSubdivide( pTParam, pTResult1, pTResult2, u );
-				break;
-			}
-
-			case type_string:
-			{
-				CqParameterTyped<CqString, CqString>* pTParam = static_cast<CqParameterTyped<CqString, CqString>*>( pParam );
-				CqParameterTyped<CqString, CqString>* pTResult1 = static_cast<CqParameterTyped<CqString, CqString>*>( pParam1 );
-				CqParameterTyped<CqString, CqString>* pTResult2 = static_cast<CqParameterTyped<CqString, CqString>*>( pParam2 );
-				TypedNaturalSubdivide( pTParam, pTResult1, pTResult2, u );
-				break;
-			}
-
-			case type_matrix:
-			{
-				CqParameterTyped<CqMatrix, CqMatrix>* pTParam = static_cast<CqParameterTyped<CqMatrix, CqMatrix>*>( pParam );
-				CqParameterTyped<CqMatrix, CqMatrix>* pTResult1 = static_cast<CqParameterTyped<CqMatrix, CqMatrix>*>( pParam1 );
-				CqParameterTyped<CqMatrix, CqMatrix>* pTResult2 = static_cast<CqParameterTyped<CqMatrix, CqMatrix>*>( pParam2 );
-				TypedNaturalSubdivide( pTParam, pTResult1, pTResult2, u );
-				break;
-			}
-
-			default:
-			{
-				// blank to avoid compiler warnings about unhandled cases
-				break;
-			}
+void CqSurface::NaturalSubdivide( CqParameter* pParam, CqParameter* pParam1, CqParameter* pParam2, bool u )
+{
+	switch(pParam->Type())
+	{
+		case type_float:
+			surfaceNaturalSubdivide<TqFloat, TqFloat>(pParam, pParam1, pParam2, u);
+			break;
+		case type_integer:
+			surfaceNaturalSubdivide<TqInt, TqFloat>(pParam, pParam1, pParam2, u);
+			break;
+		case type_point:
+		case type_vector:
+		case type_normal:
+			surfaceNaturalSubdivide<CqVector3D, CqVector3D>(pParam, pParam1, pParam2, u);
+			break;
+		case type_hpoint:
+			surfaceNaturalSubdivide<CqVector4D, CqVector3D>(pParam, pParam1, pParam2, u);
+			break;
+		case type_color:
+			surfaceNaturalSubdivide<CqColor, CqColor>(pParam, pParam1, pParam2, u);
+			break;
+		case type_string:
+			surfaceNaturalSubdivide<CqString, CqString>(pParam, pParam1, pParam2, u);
+			break;
+		case type_matrix:
+			surfaceNaturalSubdivide<CqMatrix, CqMatrix>(pParam, pParam1, pParam2, u);
+			break;
+		default:
+			// blank to avoid compiler warnings about unhandled cases
+			break;
 	}
 }
 
 
-void CqSurface::NaturalDice( CqParameter* pParameter, TqInt uDiceSize, TqInt vDiceSize, IqShaderData* pData )
+namespace {
+
+template <class T, class SLT>
+void surfaceNaturalDice(TqFloat uSize, TqFloat vSize, CqParameter* pParam,
+		IqShaderData* pData)
 {
-	switch ( pParameter->Type() )
+	CqParameterTyped<T, SLT>* pTParam = static_cast<CqParameterTyped<T, SLT>*>(pParam);
+	TqInt iv, iu;
+	for ( iv = 0; iv <= vSize; iv++ )
 	{
-			case type_float:
+		TqFloat v = ( 1.0f / vSize ) * iv;
+		for ( iu = 0; iu <= uSize; iu++ )
+		{
+			TqFloat u = ( 1.0f / uSize ) * iu;
+			IqShaderData* arrayValue;
+			TqInt i;
+			for(i = 0; i<pTParam->Count(); i++)
 			{
-				CqParameterTyped<TqFloat, TqFloat>* pTParam = static_cast<CqParameterTyped<TqFloat, TqFloat>*>( pParameter );
-				TypedNaturalDice( uDiceSize, vDiceSize, pTParam, pData );
-				break;
-			}
-
-			case type_integer:
-			{
-				CqParameterTyped<TqInt, TqFloat>* pTParam = static_cast<CqParameterTyped<TqInt, TqFloat>*>( pParameter );
-				TypedNaturalDice( uDiceSize, vDiceSize, pTParam, pData );
-				break;
-			}
-
-			case type_point:
-			case type_vector:
-			case type_normal:
-			{
-				CqParameterTyped<CqVector3D, CqVector3D>* pTParam = static_cast<CqParameterTyped<CqVector3D, CqVector3D>*>( pParameter );
-				TypedNaturalDice( uDiceSize, vDiceSize, pTParam, pData );
-				break;
-			}
-
-			case type_hpoint:
-			{
-				CqParameterTyped<CqVector4D, CqVector3D>* pTParam = static_cast<CqParameterTyped<CqVector4D, CqVector3D>*>( pParameter );
-				TypedNaturalDice( uDiceSize, vDiceSize, pTParam, pData );
-				break;
-			}
-
-			case type_color:
-			{
-				CqParameterTyped<CqColor, CqColor>* pTParam = static_cast<CqParameterTyped<CqColor, CqColor>*>( pParameter );
-				TypedNaturalDice( uDiceSize, vDiceSize, pTParam, pData );
-				break;
-			}
-
-			case type_string:
-			{
-				CqParameterTyped<CqString, CqString>* pTParam = static_cast<CqParameterTyped<CqString, CqString>*>( pParameter );
-				TypedNaturalDice( uDiceSize, vDiceSize, pTParam, pData );
-				break;
-			}
-
-			case type_matrix:
-			{
-				CqParameterTyped<CqMatrix, CqMatrix>* pTParam = static_cast<CqParameterTyped<CqMatrix, CqMatrix>*>( pParameter );
-				TypedNaturalDice( uDiceSize, vDiceSize, pTParam, pData );
-				break;
-			}
-
-			default:
-			{
-				// left blank to avoid compiler warnings about unhandled types
-				break;
+				arrayValue = pData->ArrayEntry(i);
+				T vec = BilinearEvaluate( pTParam->pValue(0) [ i ], pTParam->pValue(1) [ i ], pTParam->pValue(2) [ i ], pTParam->pValue(3) [ i ], u, v );
+				TqInt igrid = static_cast<TqInt>( ( iv * ( uSize + 1 ) ) + iu );
+				arrayValue->SetValue( static_cast<SLT>( vec ), igrid );
 			}
+		}
 	}
 }
 
+} // unnamed namespace
+
+void CqSurface::NaturalDice( CqParameter* pParam, TqInt uDiceSize,
+		TqInt vDiceSize, IqShaderData* pData )
+{
+	switch(pParam->Type())
+	{
+		case type_float:
+			surfaceNaturalDice<TqFloat, TqFloat>(uDiceSize, vDiceSize, pParam, pData);
+			break;
+		case type_integer:
+			surfaceNaturalDice<TqInt, TqFloat>(uDiceSize, vDiceSize, pParam, pData);
+			break;
+		case type_point:
+		case type_vector:
+		case type_normal:
+			surfaceNaturalDice<CqVector3D, CqVector3D>(uDiceSize, vDiceSize, pParam, pData);
+			break;
+		case type_hpoint:
+			surfaceNaturalDice<CqVector4D, CqVector3D>(uDiceSize, vDiceSize, pParam, pData);
+			break;
+		case type_color:
+			surfaceNaturalDice<CqColor, CqColor>(uDiceSize, vDiceSize, pParam, pData);
+			break;
+		case type_string:
+			surfaceNaturalDice<CqString, CqString>(uDiceSize, vDiceSize, pParam, pData);
+			break;
+		case type_matrix:
+			surfaceNaturalDice<CqMatrix, CqMatrix>(uDiceSize, vDiceSize, pParam, pData);
+			break;
+		default:
+			// left blank to avoid compiler warnings about unhandled types
+			break;
+	}
+}
 
 
 //---------------------------------------------------------------------
@@ -503,13 +505,10 @@ CqMicroPolyGridBase* CqSurface::Dice()
 
 	// If the shaders need N and they have been explicitly specified, then bilinearly interpolate them.
 	if ( isDONE( lDone, EnvVars_N ) )
-		pGrid->SetbShadingNormals( TqTrue );
+		pGrid->SetbShadingNormals( true );
 
-	if ( !isDONE( lDone, EnvVars_Ng ) && CanGenerateNormals() && USES( lUses, EnvVars_Ng ) )
-	{
-		GenerateGeometricNormals( m_uDiceSize, m_vDiceSize, pGrid->pVar(EnvVars_Ng) );
-		pGrid->SetbGeometricNormals( TqTrue );
-	}
+	if ( isDONE( lDone, EnvVars_Ng ) )
+		pGrid->SetbGeometricNormals( true );
 
 	// Now we need to dice the user specified parameters as appropriate.
 	std::vector<CqParameter*>::iterator iUP;
@@ -541,20 +540,20 @@ TqInt CqSurface::Split( std::vector<boost::shared_ptr<CqSurface> >& aSplits )
 
 	aSplits[ 0 ] ->SetSurfaceParameters( *this );
 	aSplits[ 0 ] ->SetSplitDir( ( SplitDir() == SplitDir_U ) ? SplitDir_V : SplitDir_U );
-	aSplits[ 0 ] ->SetEyeSplitCount( EyeSplitCount() );
-	aSplits[ 0 ] ->m_fDiceable = TqTrue;
+	aSplits[ 0 ] ->SetSplitCount( SplitCount() + 1 );
+	aSplits[ 0 ] ->m_fDiceable = true;
 	//ADDREF( aSplits[ 0 ] );
 
 	aSplits[ 1 ] ->SetSurfaceParameters( *this );
 	aSplits[ 1 ] ->SetSplitDir( ( SplitDir() == SplitDir_U ) ? SplitDir_V : SplitDir_U );
-	aSplits[ 1 ] ->SetEyeSplitCount( EyeSplitCount() );
-	aSplits[ 1 ] ->m_fDiceable = TqTrue;
+	aSplits[ 1 ] ->SetSplitCount( SplitCount() + 1 );
+	aSplits[ 1 ] ->m_fDiceable = true;
 	//ADDREF( aSplits[ 1 ] );
 
 	// Iterate through any use parameters subdividing and storing the second value in the target surface.
 	std::vector<CqParameter*>::iterator iUP;
 	std::vector<CqParameter*>::iterator end = m_aUserParams.end();
-	TqBool direction = SplitDir() == SplitDir_U;
+	bool direction = SplitDir() == SplitDir_U;
 
 	for ( iUP = m_aUserParams.begin(); iUP != end; iUP++ )
 	{
@@ -597,7 +596,7 @@ void CqSurface::uSubdivideUserParameters( CqSurface* pA, CqSurface* pB )
 	{
 		CqParameter* pNewA = ( *iUP ) ->Clone();
 		CqParameter* pNewB = ( *iUP ) ->Clone();
-		( *iUP ) ->Subdivide( pNewA, pNewB, TqTrue, this );
+		( *iUP ) ->Subdivide( pNewA, pNewB, true, this );
 		pA->AddPrimitiveVariable( pNewA );
 		pB->AddPrimitiveVariable( pNewB );
 	}
@@ -617,7 +616,7 @@ void CqSurface::vSubdivideUserParameters( CqSurface* pA, CqSurface* pB )
 	{
 		CqParameter* pNewA = ( *iUP ) ->Clone();
 		CqParameter* pNewB = ( *iUP ) ->Clone();
-		( *iUP ) ->Subdivide( pNewA, pNewB, TqFalse, this );
+		( *iUP ) ->Subdivide( pNewA, pNewB, false, this );
 		pA->AddPrimitiveVariable( pNewA );
 		pB->AddPrimitiveVariable( pNewB );
 	}
@@ -685,9 +684,30 @@ CqParameter* CqSurface::FindUserParam( const char* name ) const
 	return( NULL );
 }
 
+TqFloat CqSurface::AdjustedShadingRate() const
+{
+	TqFloat shadingRate =
+		m_pAttributes->GetFloatAttribute("System", "ShadingRate")[0];
+	CqRenderer* context = QGetRenderContext();
+	if(context->UsingDepthOfField())
+	{
+		// Adjust dice size with the CoC area.  This ensures that very blurry
+		// parts of a scene only have relatively few micropolys and allows the
+		// number of sample-in-micropolygon tests per pixel to be constant as
+		// the image size is increased.
+		//
+		// If this isn't included then render time increases roughly
+		// quadratically with number of pixels which makes things very slow.
+		const TqFloat* focusFactor =
+			m_pAttributes->GetFloatAttribute("System", "GeometricFocusFactor");
+		const TqFloat minCoC = context->MinCoCForBound(m_Bound);
+		shadingRate *= max(1.0, 0.25*minCoC*minCoC*focusFactor[0]);
+	}
+	return shadingRate;
+}
 
 //---------------------------------------------------------------------
 
-END_NAMESPACE( Aqsis )
+} // namespace Aqsis
 
 
