@@ -1,5 +1,5 @@
 // Aqsis
-// Copyright  1997 - 2001, Paul C. Gregory
+// Copyright (C) 1997 - 2001, Paul C. Gregory
 //
 // Contact: pgregory@aqsis.org
 //
@@ -47,7 +47,7 @@
 #include	"logging.h"
 #include	"stats.h"
 
-START_NAMESPACE( Aqsis )
+namespace Aqsis {
 
 
 
@@ -92,11 +92,14 @@ class CqSurface : public IqSurface, private boost::noncopyable, public boost::en
 		virtual	void	Reset()
 		{}
 
-		virtual TqBool	IsMotionBlurMatch( CqSurface* pSurf ) = 0;
+		virtual bool	IsMotionBlurMatch( CqSurface* pSurf ) = 0;
 
 		virtual CqString	strName() const;
 		virtual	TqInt	Uses() const;
 
+		/**
+		* \todo Review: Unused parameter pGrid
+		*/
 		virtual TqInt	DiceAll( CqMicroPolyGrid* pGrid )
 		{
 			return(0);
@@ -121,6 +124,7 @@ class CqSurface : public IqSurface, private boost::noncopyable, public boost::en
 
 		/** Get the surface paramter values for the given vertex index. Used when constructing a surface
 		 * using "Pz" point specification.
+		 * \todo Review: Unused parameter index
 		 */
 		virtual CqVector3D	SurfaceParametersAtVertex( TqInt index )
 		{
@@ -146,12 +150,11 @@ class CqSurface : public IqSurface, private boost::noncopyable, public boost::en
 		 */
 		virtual	void	ForceUndiceable()
 		{
-			m_fDiceable = TqFalse;
-			m_EyeSplitCount++;
+			m_fDiceable = false;
 		}
 		/** Query if this primitive has been marked as undiceable by the eyesplit check.
 		 */
-		virtual TqBool	IsUndiceable() const
+		virtual bool	IsUndiceable() const
 		{
 			return ( !m_fDiceable );
 		}
@@ -159,7 +162,7 @@ class CqSurface : public IqSurface, private boost::noncopyable, public boost::en
 		 */
 		virtual	void	Discard()
 		{
-			m_fDiscard = TqTrue;
+			m_fDiscard = true;
 		}
 		/** Copy the information about splitting and dicing from the specified GPrim.
 		 * \param From A CqSurface reference to copy the information from.
@@ -173,21 +176,19 @@ class CqSurface : public IqSurface, private boost::noncopyable, public boost::en
 
 		/** Determine whether this GPrim is to be discardrd.
 		 */
-		TqBool	fDiscard() const
+		bool	fDiscard() const
 		{
 			return ( m_fDiscard );
 		}
 		/** Get the number of times this GPrim has been split because if crossing the epsilon and eye planes.
 		 */
-		TqInt	EyeSplitCount() const
+		TqInt	SplitCount() const
 		{
-			return ( m_EyeSplitCount );
+			return ( m_SplitCount );
 		}
-		/** Set the number of times this GPrim has been split because if crossing the epsilon and eye planes.
-		 */
-		void	SetEyeSplitCount( TqInt EyeSplitCount )
+		void	SetSplitCount( TqInt SplitCount )
 		{
-			m_EyeSplitCount = EyeSplitCount;
+			m_SplitCount = SplitCount;
 		}
 		/** Get the precalculated split direction.
 		 */
@@ -208,7 +209,7 @@ class CqSurface : public IqSurface, private boost::noncopyable, public boost::en
 		void CacheRasterBound( CqBound& pBound )
 		{
 			m_Bound = pBound;
-			m_CachedBound = TqTrue;
+			m_CachedBound = true;
 		}
 		/** Retrieve the cached bound. If it has never been cached then we
 		 * throw an error as this is probably a bug.
@@ -216,7 +217,7 @@ class CqSurface : public IqSurface, private boost::noncopyable, public boost::en
 		 */
 		CqBound	GetCachedRasterBound()
 		{
-			if ( m_CachedBound == TqFalse && m_fDiceable )
+			if ( m_CachedBound == false && m_fDiceable )
 			{
 				Aqsis::log() << critical << "No cached bound available" << std::endl;
 			}
@@ -224,11 +225,11 @@ class CqSurface : public IqSurface, private boost::noncopyable, public boost::en
 			return m_Bound;
 		}
 
-		TqBool	fCachedBound() const
+		bool	fCachedBound() const
 		{
 			return ( m_CachedBound );
 		}
-		CqBound	AdjustBoundForTransformationMotion( const CqBound& B ) const;
+		void	AdjustBoundForTransformationMotion( IqBound* B ) const;
 
 		boost::shared_ptr<CqCSGTreeNode>& pCSGNode()
 		{
@@ -237,7 +238,7 @@ class CqSurface : public IqSurface, private boost::noncopyable, public boost::en
  		virtual CqSurface* Clone() const = 0;
 
 
-		virtual	void	SetDefaultPrimitiveVariables( TqBool bUseDef_st = TqTrue );
+		virtual	void	SetDefaultPrimitiveVariables( bool bUseDef_st = true );
 
 		void ClonePrimitiveVariables( const CqSurface& From );
 
@@ -409,7 +410,7 @@ class CqSurface : public IqSurface, private boost::noncopyable, public boost::en
 
 		/** Determine whether this surface has a specified primitive variable based on index
 		 */
-		virtual const	TqBool bHasVar(TqInt index) const
+		virtual const	bool bHasVar(TqInt index) const
 		{
 			assert( index >= EnvVars_Cs && index < EnvVars_Last );
 			// Special case for s & t if "st" is specified.
@@ -504,25 +505,28 @@ class CqSurface : public IqSurface, private boost::noncopyable, public boost::en
 
 		/** Determine whether this surface can be trimmed
 		 */
-		virtual const	TqBool	bCanBeTrimmed() const
+		virtual const	bool	bCanBeTrimmed() const
 		{
-			return ( TqFalse );
+			return ( false );
 		}
 		/** Determine if the specified point is trimmed.
+		 * \todo Review: Unused parameter p
 		 */
-		virtual	const	TqBool	bIsPointTrimmed( const CqVector2D& p ) const
+		virtual	const	bool	bIsPointTrimmed( const CqVector2D& p ) const
 		{
-			return ( TqFalse );
+			return ( false );
 		}
 		/** Determine if the specified edge crosses the trimming curves.
+		 * \todo Review: Unused parameter v1, v2
 		 */
-		virtual	const	TqBool	bIsLineIntersecting( const CqVector2D& v1, const CqVector2D& v2 ) const
+		virtual	const	bool	bIsLineIntersecting( const CqVector2D& v1, const CqVector2D& v2 ) const
 		{
-			return ( TqFalse );
+			return ( false );
 		}
 		/** Determine the level at which to split a trim curve according
 		 * to its screen size after application to the surface paramters of this
 		 * surface.
+		 * \todo Review: Unused parameter Curve. Shouldn't it be curve?
 		 */
 		virtual	TqInt	TrimDecimation( const CqTrimCurve& Curve )
 		{
@@ -536,47 +540,55 @@ class CqSurface : public IqSurface, private boost::noncopyable, public boost::en
 		void	uSubdivideUserParameters( CqSurface* pA, CqSurface* pB );
 		void	vSubdivideUserParameters( CqSurface* pA, CqSurface* pB );
 
-		virtual void	PreDice( TqInt uDiceSize, TqInt vDiceSize )
+		virtual void	PreDice( TqInt /* uDiceSize */, TqInt /* vDiceSize */ )
 		{}
-		virtual void	NaturalDice( CqParameter* pParameter, TqInt uDiceSize, TqInt vDiceSize, IqShaderData* pData );
-		virtual void	PostDice(CqMicroPolyGrid * pGrid)
+		virtual void	NaturalDice( CqParameter* pParameter, TqInt uDiceSize,
+			TqInt vDiceSize, IqShaderData* pData);
+		virtual void	PostDice(CqMicroPolyGrid * /* pGrid */)
 		{}
 
-		virtual TqInt	PreSubdivide( std::vector<boost::shared_ptr<CqSurface> >& aSplits, TqBool u )
+		/**
+		 * \todo Review: Unused parameter aSplits, u
+		 */
+		virtual TqInt	PreSubdivide( std::vector<boost::shared_ptr<CqSurface> >& aSplits, bool u )
 		{
 			return ( 0 );
 		}
-		virtual void	NaturalSubdivide( CqParameter* pParam, CqParameter* pParam1, CqParameter* pParam2, TqBool u );
-		virtual void	PostSubdivide(std::vector<boost::shared_ptr<CqSurface> >& aSplits)
-		{}
-
-		/** Virtual function to indicate whether a particular surface is able
-		 *  to generate geometric normals itself.
-		 */
-		virtual TqBool	CanGenerateNormals() const
-		{
-			return ( TqFalse );
-		}
-
-		/** Virtual function to genrate and fill in geomtric normals if a surface is able to do so.
-		 */
-		virtual	void	GenerateGeometricNormals( TqInt uDiceSize, TqInt vDiceSize, IqShaderData* pNormals )
+		virtual void	NaturalSubdivide( CqParameter* pParam, CqParameter* pParam1, CqParameter* pParam2, bool u );
+		virtual void	PostSubdivide(std::vector<boost::shared_ptr<CqSurface> >& /* aSplits */)
 		{}
 
 		virtual	CqMicroPolyGridBase* Dice();
 		virtual	TqInt	Split( std::vector<boost::shared_ptr<CqSurface> >& aSplits );
-		virtual TqBool	Diceable()
+		
+		/**
+		 * \todo Review: Could this be const?
+		 */
+		virtual bool	Diceable()
 		{
 			return(false);
 		}
 
+		/** Get the shading rate, adjusted for DoF (and MB in the future)
+		 *
+		 * Without depth of field or motion blur, this function just returns
+		 * the shading rate as given by the current attribute set.
+		 *
+		 * When depth of field is turned on, the shading rate is increased
+		 * according to value of the system "GeometricFocusFactor" attribute
+		 * (as specified with RiGeometricApproximation("focusfactor", ...) )
+		 *
+		 * \todo No adjustment is yet implemented for motion blur, but the
+		 * intention is to allow RiGeometricApproximation("motionfactor", ...)
+		 * to have an effect on the returned shading rate as well.
+		 */
+		TqFloat AdjustedShadingRate() const;
 
-
-
-		TqBool	m_fDiceable;		///< Flag to indicate that this GPrim is diceable.
-		TqBool	m_fDiscard;			///< Flag to indicate that this GPrim is to be discarded.
-		TqInt	m_EyeSplitCount;	///< The number of times this GPrim has been split because if crossing the epsilon and eye planes.
+		bool	m_fDiceable;		///< Flag to indicate that this GPrim is diceable.
+		bool	m_fDiscard;			///< Flag to indicate that this GPrim is to be discarded.
+		TqInt	m_SplitCount;		///< The number of times this GPrim has been split
 	protected:
+
 		/** Protected member function to clone the data, used by the Clone() functions
 		 *  on the derived classes.
 		 */
@@ -584,63 +596,13 @@ class CqSurface : public IqSurface, private boost::noncopyable, public boost::en
 		std::vector<CqParameter*>	m_aUserParams;			///< Storage for user defined paramter variables.
 		TqInt	m_aiStdPrimitiveVars[ EnvVars_Last ];		///< Quick lookup index into the primitive variables table for standard variables.
 
-		template <class T, class SLT>
-		void	TypedNaturalDice( TqFloat uSize, TqFloat vSize, CqParameterTyped<T, SLT>* pParam, IqShaderData* pData )
-		{
-			TqInt iv, iu;
-			for ( iv = 0; iv <= vSize; iv++ )
-			{
-				TqFloat v = ( 1.0f / vSize ) * iv;
-				for ( iu = 0; iu <= uSize; iu++ )
-				{
-					TqFloat u = ( 1.0f / uSize ) * iu;
-					IqShaderData* arrayValue;
-					TqInt i;
-					for(i = 0; i<pParam->Count(); i++)
-					{
-						arrayValue = pData->ArrayEntry(i);
-						T vec = BilinearEvaluate( pParam->pValue(0) [ i ], pParam->pValue(1) [ i ], pParam->pValue(2) [ i ], pParam->pValue(3) [ i ], u, v );
-						TqInt igrid = static_cast<TqInt>( ( iv * ( uSize + 1 ) ) + iu );
-						arrayValue->SetValue( static_cast<SLT>( vec ), igrid );
-					}
-				}
-			}
-		}
-
-		template <class T, class SLT>
-		void	TypedNaturalSubdivide( CqParameterTyped<T, SLT>* pParam, CqParameterTyped<T, SLT>* pResult1, CqParameterTyped<T, SLT>* pResult2, TqBool u )
-		{
-			CqParameterTyped<T, SLT>* pTParam = static_cast<CqParameterTyped<T, SLT>*>( pParam );
-			CqParameterTyped<T, SLT>* pTResult1 = static_cast<CqParameterTyped<T, SLT>*>( pResult1 );
-			CqParameterTyped<T, SLT>* pTResult2 = static_cast<CqParameterTyped<T, SLT>*>( pResult2 );
-
-			TqInt i;
-			for(i = 0; i<pParam->Count(); i++)
-			{
-				if ( u )
-				{
-					pTResult2->pValue( 1 ) [ i ] = pTParam->pValue( 1 ) [ i ];
-					pTResult2->pValue( 3 ) [ i ] = pTParam->pValue( 3 ) [ i ];
-					pTResult1->pValue( 1 ) [ i ] = pTResult2->pValue( 0 ) [ i ] = static_cast<T>( ( pTParam->pValue( 0 ) [ i ] + pTParam->pValue( 1 ) [ i ] ) * 0.5 );
-					pTResult1->pValue( 3 ) [ i ] = pTResult2->pValue( 2 ) [ i ] = static_cast<T>( ( pTParam->pValue( 2 ) [ i ] + pTParam->pValue( 3 ) [ i ] ) * 0.5 );
-				}
-				else
-				{
-					pTResult2->pValue( 2 ) [ i ] = pTParam->pValue( 2 ) [ i ];
-					pTResult2->pValue( 3 ) [ i ] = pTParam->pValue( 3 ) [ i ];
-					pTResult1->pValue( 2 ) [ i ] = pTResult2->pValue( 0 ) [ i ] = static_cast<T>( ( pTParam->pValue( 0 ) [ i ] + pTParam->pValue( 2 ) [ i ] ) * 0.5 );
-					pTResult1->pValue( 3 ) [ i ] = pTResult2->pValue( 1 ) [ i ] = static_cast<T>( ( pTParam->pValue( 1 ) [ i ] + pTParam->pValue( 3 ) [ i ] ) * 0.5 );
-				}
-			}
-		}
-
 		CqAttributes* m_pAttributes;	///< Pointer to the attributes state associated with this GPrim.
 		CqTransformPtr m_pTransform;		///< Pointer to the transformation state associated with this GPrim.
 
 		TqInt	m_uDiceSize;		///< Calculated dice size to achieve an appropriate shading rate.
 		TqInt	m_vDiceSize;		///< Calculated dice size to achieve an appropriate shading rate.
 		EqSplitDir	m_SplitDir;			///< The direction to split this GPrim to achieve best results.
-		TqBool	m_CachedBound;		///< Whether or not the bound has been cached
+		bool	m_CachedBound;		///< Whether or not the bound has been cached
 		CqBound	m_Bound;			///< The cached object bound
 		boost::shared_ptr<CqCSGTreeNode>	m_pCSGNode;		///< Pointer to the 'primitive' CSG node this surface belongs to, NULL if not part of a solid.
 		static TqFloat     m_fGridSize;   ///< standard sqrt(gridsize);
@@ -665,14 +627,17 @@ class CqDeformingSurface : public CqSurface, public CqMotionSpec<boost::shared_p
 		/** Get combnied bound for all times
 		 * \return CqBound representing the geometric boundary of this GPrim over all time slots.
 		 */
-		virtual	CqBound	Bound() const
+		virtual	void	Bound(IqBound* bound) const
 		{
-			CqBound B( FLT_MAX, FLT_MAX, FLT_MAX, -FLT_MAX, -FLT_MAX, -FLT_MAX );
+			bound->vecMin() = CqVector3D(FLT_MAX, FLT_MAX, FLT_MAX);
+			bound->vecMax() = CqVector3D(-FLT_MAX, -FLT_MAX, -FLT_MAX);
+			CqBound B;
 			TqInt i;
 			for ( i = 0; i < cTimes(); i++ )
-				B.Encapsulate( GetMotionObject( Time( i ) ) ->Bound() );
-
-			return ( B );
+			{
+				GetMotionObject( Time( i ) )->Bound(&B);
+				bound->Encapsulate( &B );
+			}
 		}
 		/** Dice this GPrim, creating a CqMotionMicroPolyGrid with all times in.
 		 */
@@ -687,6 +652,7 @@ class CqDeformingSurface : public CqSurface, public CqMotionSpec<boost::shared_p
 				ADDREF(pGrid2);
 				pGrid->SetfTriangular( pGrid2->fTriangular() );
 			}
+			pGrid->Initialise(m_uDiceSize, m_vDiceSize, shared_from_this());
 			return ( pGrid );
 		}
 		/** Split this GPrim, creating a series of CqDeformingSurface with all times in.
@@ -715,8 +681,8 @@ class CqDeformingSurface : public CqSurface, public CqMotionSpec<boost::shared_p
 			for ( i = 0; i < cSplits; i++ )
 			{
 				boost::shared_ptr<CqDeformingSurface> pNewMotion( new CqDeformingSurface( boost::shared_ptr<CqSurface>() ) );
-				pNewMotion->m_fDiceable = TqTrue;
-				pNewMotion->m_EyeSplitCount = m_EyeSplitCount;
+				pNewMotion->m_fDiceable = true;
+				pNewMotion->m_SplitCount = m_SplitCount + 1;
 				TqInt j;
 				for ( j = 0; j < cTimes(); j++ )
 					pNewMotion->AddTimeSlot( Time( j ), aaMotionSplits[ j ][ i ] );
@@ -728,9 +694,9 @@ class CqDeformingSurface : public CqSurface, public CqMotionSpec<boost::shared_p
 		 * determines the dicing rate, which is then copied to the other times.
 		 * \return Boolean indicating GPrim is diceable.
 		 */
-		virtual TqBool	Diceable()
+		virtual bool	Diceable()
 		{
-			TqBool f = GetMotionObject( Time( 0 ) ) ->Diceable();
+			bool f = GetMotionObject( Time( 0 ) ) ->Diceable();
 			// Copy the split info so that at each time slot, the gprims split the same.
 			TqInt i;
 			for ( i = 1; i < cTimes(); i++ )
@@ -751,9 +717,9 @@ class CqDeformingSurface : public CqSurface, public CqMotionSpec<boost::shared_p
 		/** Determine whether the passed surface is valid to be used as a
 		 *  frame in motion blur for this surface.
 		 */
-		virtual TqBool	IsMotionBlurMatch( CqSurface* pSurf )
+		virtual bool	IsMotionBlurMatch( CqSurface* pSurf )
 		{
-			return( TqFalse );
+			return( false );
 		}
 
 		/** Transform all GPrims by the specified transformation matrices.
@@ -840,7 +806,7 @@ class CqDeformingSurface : public CqSurface, public CqMotionSpec<boost::shared_p
 
 //-----------------------------------------------------------------------
 
-END_NAMESPACE( Aqsis )
+} // namespace Aqsis
 
 //}  // End of #ifdef SURFACE_H_INCLUDED
 #endif
