@@ -1,5 +1,5 @@
 // Aqsis
-// Copyright  1997 - 2001, Paul C. Gregory
+// Copyright (C) 1997 - 2001, Paul C. Gregory
 //
 // Contact: pgregory@aqsis.org
 //
@@ -29,8 +29,8 @@
 #include	"vector3d.h"
 #include	"parameters.h"
 
-START_NAMESPACE( Aqsis )
+namespace Aqsis {
 
 
-END_NAMESPACE( Aqsis )
+} // namespace Aqsis
 //---------------------------------------------------------------------
