@@ -42,7 +42,7 @@
 
 #include	"ri.h"
 
-START_NAMESPACE( Aqsis )
+namespace Aqsis {
 
 static CqVector3D Patch01[ 13*10 ] = { /*u=13 v=10 */
                                          CqVector3D( 1.5, 0.0, 0.0 ),
@@ -426,7 +426,7 @@ static CqVector3D Patch07[ 13*2 ] = { /*u=13 v=2 */
 /** Constructor.
  */
 
-CqTeapot::CqTeapot( TqBool addCrowBase ) : m_CrowBase( addCrowBase )
+CqTeapot::CqTeapot( bool addCrowBase ) : m_CrowBase( addCrowBase )
 {
 	int i;
 	TqInt lUses;
@@ -644,14 +644,15 @@ void	CqTeapot::Transform( const CqMatrix& matTx, const CqMatrix& matITTx, const
 /** Get the geometric bound of this GPrim.
  */
 
-CqBound	CqTeapot::Bound() const
+void	CqTeapot::Bound(IqBound* bound) const
 {
 	CqVector3D vecMin( -3.000, -2.0, 0.0 );
 	CqVector3D vecMax( 3.525, 2.0, ( m_CrowBase ? 3.15 : 3.15 - 0.15 /* remove bottom */ ) );
 
-	CqBound	B( vecMin, vecMax );
-	B.Transform( m_matTx );
-	return ( AdjustBoundForTransformationMotion( B ) );
+	bound->vecMin() = vecMin;
+	bound->vecMax() = vecMax;
+	bound->Transform( m_matTx );
+	AdjustBoundForTransformationMotion( bound );
 }
 
 //---------------------------------------------------------------------
@@ -666,7 +667,7 @@ TqInt CqTeapot::Split( std::vector<boost::shared_ptr<CqSurface> >& aSplits )
 }
 
 
-END_NAMESPACE( Aqsis )
+} // namespace Aqsis
 //---------------------------------------------------------------------
 
 
