@@ -22,7 +22,7 @@
  *  \author Lionel J. Lacour (intuition01@online.fr)
  */
 
-#ifdef	WIN32
+#if _MSC_VER
 #pragma warning(disable : 4786)
 #endif
 
@@ -33,7 +33,9 @@
 
 #include "stddef.h"
 
-USING_NAMESPACE( libri2rib );
+#include <cstring>
+
+namespace libri2rib {
 
 #define PR(x,y)  printRequest(x,y)
 #ifdef	PI
@@ -48,15 +50,11 @@ USING_NAMESPACE( libri2rib );
 
 
 CqOutput::CqOutput( const char *name, int fdesc,
-                    SqOptions::EqCompression comp,
-                    SqOptions::EqIndentation i, TqInt isize )
+                    SqOptions::EqCompression comp)
 		:
 		m_ColorNComps( 3 ),
 		m_ObjectHandle( 1 ),
-		m_LightHandle( 1 ),
-		m_Indentation( i ),
-		m_IndentSize( isize ),
-		m_IndentLevel( 0 )
+		m_LightHandle( 1 )
 {
 	switch ( comp )
 	{
@@ -115,23 +113,29 @@ void CqOutput::pop()
 // **************************************************************************
 // ******* ******* ******** CONTEXT NESTING FUNCTIONS ******* ******* *******
 // **************************************************************************
-const char* blockNames[] =
+const char* const CqOutput::m_blockNames[] =
     {
         "RiBegin/RiEnd", "Frame", "World", "Attribute", "Transform",
-        "Solid", "Object", "Motion"
+        "Solid", "Object", "Motion", "Resource"
     };
-const RtInt blockErrors[] =
+const RtInt CqOutput::m_blockErrors[] =
     {
         RIE_NESTING, RIE_NESTING, RIE_NESTING, RIE_NESTING, RIE_NESTING,
-        RIE_BADSOLID, RIE_NESTING, RIE_BADMOTION
+        RIE_BADSOLID, RIE_NESTING, RIE_BADMOTION, RIE_NESTING
+    };
+const CqOutput::EqFunctions CqOutput::m_blockFunctions[] =
+    {
+		LAST_Function, LAST_Function,  // < There's no function type for RiBegin/End
+		FrameBegin, FrameEnd, WorldBegin, WorldEnd, AttributeBegin, AttributeEnd,
+		TransformBegin, TransformEnd, SolidBegin, SolidEnd, ObjectBegin, ObjectEnd,
+		MotionBegin, MotionEnd, ResourceBegin, ResourceEnd
     };
-
 
 
 /**
  * Begins a block nesting.
  */
-bool CqOutput::beginNesting(EqBlocks type)
+void CqOutput::beginNesting(EqBlocks type)
 {
 	// we cannot have more than one Ri, Frame or World block, so check that
 	//  these are not present already
@@ -141,15 +145,19 @@ bool CqOutput::beginNesting(EqBlocks type)
 		{
 			throw CqError(RIE_NESTING, RIE_SEVERE,
 			              "Attempt to open another ",
-			              blockNames[type],
-			              " block when one is already open.", TqFalse);
-			return false;
+			              m_blockNames[type],
+			              " block when one is already open.", false);
 		}
 	}
 
+	if(type != B_Ri)
+	{
+		std::string requestName = std::string(m_blockNames[type]) + "Begin";
+		printRequest(requestName.c_str(), m_blockFunctions[type*2]);
+	}
+
 	// everything else is fine, so open the block
 	m_nesting.push_back(type);
-	return true;
 }
 
 
@@ -170,45 +178,44 @@ bool CqOutput::beginNesting(EqBlocks type)
  *      RiTransformEnd();               // Note: block is *not* Transform
  *              RiAttributeEnd();
  */
-bool CqOutput::endNesting(EqBlocks type)
+void CqOutput::endNesting(EqBlocks type)
 {
 	// check for an empty block stack.  this is not valid, since we
 	//  can't close a block that hasn't been opened.
 	if (m_nesting.empty())
 	{
 		// invalid
-		throw CqError(blockErrors[type], RIE_SEVERE,
-		              "Cannot close block of type ", blockNames[type],
-		              " when no blocks have yet been opened.", TqFalse);
-		return false;
+		throw CqError(m_blockErrors[type], RIE_SEVERE,
+		              "Cannot close block of type ", m_blockNames[type],
+		              " when no blocks have yet been opened.", false);
 	}
 
 	// check that the block we're closing is of the correct type
 	EqBlocks curBlock = m_nesting.back();
-	if (curBlock == type)
-	{
-		// valid
-		m_nesting.pop_back();
-		return true;
-	}
-	else
+	if (curBlock != type)
 	{
 		// invalid - we raise the error based upon the last entry on
 		//  the stack
 		std::stringstream strBuf;
 		strBuf << "Bad nesting: Attempting to close block of type "
-		<< blockNames[type]
+		<< m_blockNames[type]
 		<< " within a "
-		<< blockNames[curBlock]
+		<< m_blockNames[curBlock]
 		<< " block." << std::ends;
-		throw CqError(blockErrors[curBlock], RIE_SEVERE,
-		              &strBuf.str()[0], TqFalse);
-		return false;
+		throw CqError(m_blockErrors[curBlock], RIE_SEVERE,
+		              &strBuf.str()[0], false);
+	}
+
+	m_nesting.pop_back();
+
+	if(type != B_Ri)
+	{
+		std::string requestName = std::string(m_blockNames[type]) + "End";
+		printRequest(requestName.c_str(), m_blockFunctions[type*2+1]);
 	}
 }
 
 
-
 /**
  * Checks whether or not the current nesting contains a particular block type.
  *
@@ -236,7 +243,8 @@ bool CqOutput::nestingContains(EqBlocks type) const
 // ******* ******* ******* PRINTING TOOLS ******* ******* *******
 // **************************************************************
 void CqOutput::printPL( RtInt n, RtToken tokens[], RtPointer parms[],
-                        RtInt vertex, RtInt varying, RtInt uniform, RtInt facevarying )
+                        RtInt vertex, RtInt varying, RtInt uniform,
+						RtInt facevarying, RtInt facevertex)
 {
 	RtFloat * flt;
 	RtInt *nt;
@@ -263,7 +271,8 @@ void CqOutput::printPL( RtInt n, RtToken tokens[], RtPointer parms[],
 		printToken( tokens[ i ] );
 		S;
 		tt = m_Dictionary.getType( id );
-		sz = m_Dictionary.allocSize( id, vertex, varying, uniform, facevarying);
+		sz = m_Dictionary.allocSize( id, vertex, varying, uniform,
+				facevarying, facevertex);
 
 		switch ( tt )
 		{
@@ -323,7 +332,7 @@ std::string CqOutput::getFilterFuncName( RtFilterFunc filterfunc, const char *na
 	{
 		throw CqError( RIE_CONSISTENCY, RIE_WARNING,
 		               "Unknown RiFilterFunc. ",
-		               name, " function skipped.", TqTrue );
+		               name, " function skipped.", true );
 		return "";
 	}
 }
@@ -343,7 +352,7 @@ RtToken CqOutput::RiDeclare( const char *name, const char *declaration )
 	b += " ";
 	b += a;
 	ip.parse( b );
-	m_Dictionary.addToken( ip.getIdentifier(), ip.getClass(), ip.getType(), ip.getQuantity(), TqFalse );
+	m_Dictionary.addToken( ip.getIdentifier(), ip.getClass(), ip.getType(), ip.getQuantity(), false );
 
 	PR( "Declare", Declare );
 	S;
@@ -355,10 +364,14 @@ RtToken CqOutput::RiDeclare( const char *name, const char *declaration )
 	return const_cast<RtToken> ( name );
 }
 
+// *****************************************************************
+// ******* ******* *******   NESTED BLOCKS   ******* ******* *******
+// *****************************************************************
+
 RtVoid CqOutput::RiBegin( RtToken name )
 {
-	if (beginNesting(B_Ri))
-		printHeader();
+	beginNesting(B_Ri);
+	printHeader();
 }
 
 RtVoid CqOutput::RiEnd( )
@@ -369,89 +382,50 @@ RtVoid CqOutput::RiEnd( )
 
 RtVoid CqOutput::RiFrameBegin( RtInt frame )
 {
-	if (beginNesting(B_Frame))
-	{
-		PR( "FrameBegin", FrameBegin );
-		S;
-		PI( frame );
-		EOL;
-
-		m_IndentLevel++;
-		push();
-	}
+	beginNesting(B_Frame);
+	S;
+	PI( frame );
+	EOL;
+	push();
 }
 
 RtVoid CqOutput::RiFrameEnd( )
 {
-	if (endNesting(B_Frame))
-	{
-		m_IndentLevel--;
-		if ( m_IndentLevel < 0 )
-			m_IndentLevel = 0;
-		pop();
-
-		PR( "FrameEnd", FrameEnd );
-		EOL;
-	}
+	endNesting(B_Frame);
+	EOL;
+	pop();
 }
 
 RtVoid CqOutput::RiWorldBegin( )
 {
-	if (beginNesting(B_World))
-	{
-		PR( "WorldBegin", WorldBegin );
-		EOL;
-
-		m_IndentLevel++;
-		push();
-	}
+	beginNesting(B_World);
+	EOL;
+	push();
 }
 
 RtVoid CqOutput::RiWorldEnd( )
 {
-	if (endNesting(B_World))
-	{
-		m_IndentLevel--;
-		if ( m_IndentLevel < 0 )
-			m_IndentLevel = 0;
-		pop();
-
-		PR( "WorldEnd", WorldEnd );
-		EOL;
-	}
+	endNesting(B_World);
+	EOL;
+	pop();
 }
 
 RtObjectHandle CqOutput::RiObjectBegin( )
 {
-	if (beginNesting(B_Object))
-	{
-		PR( "ObjectBegin", ObjectBegin );
-		S;
-		PI( ( RtInt ) m_ObjectHandle );
-		EOL;
+	beginNesting(B_Object);
+	S;
+	PI( ( RtInt ) m_ObjectHandle );
+	EOL;
 
-		m_IndentLevel++;
-		push();
-		return reinterpret_cast<RtObjectHandle>(static_cast<ptrdiff_t>(m_ObjectHandle++));
-	}
-	else
-	{
-		return ( RtObjectHandle )NULL;
-	}
+	push();
+	return reinterpret_cast<RtObjectHandle>(static_cast<ptrdiff_t>(m_ObjectHandle++));
 }
 
 RtVoid CqOutput::RiObjectEnd( )
 {
-	if (endNesting(B_Object))
-	{
-		m_IndentLevel--;
-		if ( m_IndentLevel < 0 )
-			m_IndentLevel = 0;
-		pop();
-
-		PR( "ObjectEnd", ObjectEnd );
-		EOL;
-	}
+	endNesting(B_Object);
+	EOL;
+	pop();
 }
 
 RtVoid CqOutput::RiObjectInstance( RtObjectHandle handle )
@@ -464,108 +438,74 @@ RtVoid CqOutput::RiObjectInstance( RtObjectHandle handle )
 
 RtVoid CqOutput::RiAttributeBegin( )
 {
-	if (beginNesting(B_Attribute))
-	{
-		PR( "AttributeBegin", AttributeBegin );
-		EOL;
-
-		m_IndentLevel++;
-		push();
-	}
+	beginNesting(B_Attribute);
+	EOL;
+	push();
 }
 
 RtVoid CqOutput::RiAttributeEnd( )
 {
-	if (endNesting(B_Attribute))
-	{
-		m_IndentLevel--;
-		if ( m_IndentLevel < 0 )
-			m_IndentLevel = 0;
-		pop();
-
-		PR( "AttributeEnd", AttributeEnd );
-		EOL;
-	}
+	endNesting(B_Attribute);
+	EOL;
+	pop();
 }
 
 RtVoid CqOutput::RiTransformBegin( )
 {
-	if (beginNesting(B_Transform))
-	{
-		PR( "TransformBegin", TransformBegin );
-		EOL;
-
-		m_IndentLevel++;
-	}
+	beginNesting(B_Transform);
+	EOL;
 }
 
 RtVoid CqOutput::RiTransformEnd( )
 {
-	if (endNesting(B_Transform))
-	{
-		m_IndentLevel--;
-		if ( m_IndentLevel < 0 )
-			m_IndentLevel = 0;
-
-		PR( "TransformEnd", TransformEnd );
-		EOL;
-	}
+	endNesting(B_Transform);
+	EOL;
 }
 
 RtVoid CqOutput::RiSolidBegin( RtToken operation )
 {
-	if (beginNesting(B_Solid))
-	{
-		PR( "SolidBegin", SolidBegin );
-		S;
-		printToken( operation );
-		EOL;
+	beginNesting(B_Solid);
+	S;
+	printToken( operation );
+	EOL;
 
-		m_IndentLevel++;
-		push();
-	}
+	push();
 }
 
 RtVoid CqOutput::RiSolidEnd( )
 {
-	if (endNesting(B_Solid))
-	{
-		m_IndentLevel--;
-		if ( m_IndentLevel < 0 )
-			m_IndentLevel = 0;
-		pop();
-
-		PR( "SolidEnd", SolidEnd );
-		EOL;
-	}
+	endNesting(B_Solid);
+	EOL;
+	pop();
 }
 
 RtVoid CqOutput::RiMotionBeginV( RtInt n, RtFloat times[] )
 {
-	if (beginNesting(B_Motion))
-	{
-		PR( "MotionBegin", MotionBegin );
-		S;
-		printArray( n, times );
-		EOL;
-
-		m_IndentLevel++;
-	}
+	beginNesting(B_Motion);
+	S;
+	printArray( n, times );
+	EOL;
 }
 
 RtVoid CqOutput::RiMotionEnd( )
 {
-	if (endNesting(B_Motion))
-	{
-		m_IndentLevel--;
-		if ( m_IndentLevel < 0 )
-			m_IndentLevel = 0;
-
-		PR( "MotionEnd", MotionEnd );
-		EOL;
-	}
+	endNesting(B_Motion);
+	EOL;
 }
 
+RtVoid CqOutput::RiResourceBegin( )
+{
+	beginNesting(B_Resource);
+	EOL;
+	push();
+}
+
+RtVoid CqOutput::RiResourceEnd( )
+{
+	endNesting(B_Resource);
+	EOL;
+	pop();
+}
 
 
 
@@ -1006,6 +946,25 @@ RtVoid CqOutput::RiGeometricApproximation( RtToken type, RtFloat value )
 	EOL;
 }
 
+namespace {
+
+const char* getBasisName(RtBasis basis)
+{
+	if(basis == RiBezierBasis)
+		return "bezier";
+	if(basis == RiBSplineBasis)
+		return "b-spline";
+	if(basis == RiCatmullRomBasis)
+		return "catmull-rom";
+	if(basis == RiHermiteBasis)
+		return "hermite";
+	if(basis == RiPowerBasis)
+		return "power";
+	return 0;
+}
+
+} // unnamed namespace
+
 RtVoid CqOutput::RiBasis( RtBasis ubasis, RtInt ustep, RtBasis vbasis, RtInt vstep )
 {
 	RtInt i;
@@ -1013,20 +972,30 @@ RtVoid CqOutput::RiBasis( RtBasis ubasis, RtInt ustep, RtBasis vbasis, RtInt vst
 
 	PR( "Basis", Basis );
 	S;
-	for ( i = 0; i < 16; i++ )
+	if(const char* basisName = getBasisName(ubasis))
 	{
-		m[ i ] = ubasis[ i / 4 ][ i % 4 ];
+		PS(basisName);
+	}
+	else
+	{
+		for ( i = 0; i < 16; i++ )
+			m[ i ] = ubasis[ i / 4 ][ i % 4 ];
+		printArray( 16, &( m[ 0 ] ) );
 	}
-	printArray( 16, &( m[ 0 ] ) );
 	S;
 	PI( ustep );
 	S;
 
-	for ( i = 0; i < 16; i++ )
+	if(const char* basisName = getBasisName(vbasis))
 	{
-		m[ i ] = vbasis[ i / 4 ][ i % 4 ];
+		PS(basisName);
+	}
+	else
+	{
+		for ( i = 0; i < 16; i++ )
+			m[ i ] = vbasis[ i / 4 ][ i % 4 ];
+		printArray( 16, &( m[ 0 ] ) );
 	}
-	printArray( 16, m );
 	S;
 	PI( vstep );
 	EOL;
@@ -1179,6 +1148,17 @@ RtVoid CqOutput::RiRotate( RtFloat angle, RtFloat dx, RtFloat dy, RtFloat dz )
 	EOL;
 }
 
+RtVoid CqOutput::RiResourceV ( RtToken handle, RtToken type, RtInt n, RtToken tokens[], RtPointer parms[] )
+{
+	PR( "Resource", Projection );
+	S;
+	printToken( handle );
+	S;
+	printToken( type );
+	S;
+	printPL( n, tokens, parms );
+}
+
 RtVoid CqOutput::RiScale( RtFloat sx, RtFloat sy, RtFloat sz )
 {
 	PR( "Scale", Scale );
@@ -1256,7 +1236,7 @@ RtVoid CqOutput::RiPolygonV( RtInt nverts, RtInt n, RtToken tokens[], RtPointer
 {
 	PR( "Polygon", Polygon );
 	S;
-	printPL( n, tokens, parms, nverts, nverts, nverts );
+	printPL( n, tokens, parms, nverts, nverts, 1, nverts, nverts );
 }
 
 RtVoid CqOutput::RiGeneralPolygonV( RtInt nloops, RtInt nverts[],
@@ -1272,7 +1252,7 @@ RtVoid CqOutput::RiGeneralPolygonV( RtInt nloops, RtInt nverts[],
 	S;
 	printArray( nloops, nverts );
 	S;
-	printPL( n, tokens, parms, nbpts, nbpts, nbpts );
+	printPL( n, tokens, parms, nbpts, nbpts, 1, nbpts, nbpts );
 }
 
 RtVoid CqOutput::RiPointsPolygonsV( RtInt npolys, RtInt nverts[], RtInt verts[],
@@ -1298,7 +1278,7 @@ RtVoid CqOutput::RiPointsPolygonsV( RtInt npolys, RtInt nverts[], RtInt verts[],
 		if ( psize < verts[ i ] )
 			psize = verts[ i ];
 	}
-	printPL( n, tokens, parms, psize + 1, psize + 1, npolys, nbpts );
+	printPL( n, tokens, parms, psize + 1, psize + 1, npolys, nbpts, nbpts );
 }
 
 RtVoid CqOutput::RiPointsGeneralPolygonsV( RtInt npolys, RtInt nloops[], RtInt nverts[],
@@ -1333,8 +1313,7 @@ RtVoid CqOutput::RiPointsGeneralPolygonsV( RtInt npolys, RtInt nloops[], RtInt n
 		if ( psize < verts[ i ] )
 			psize = verts[ i ];
 	}
-	RtInt facevarying = nv;
-	printPL( n, tokens, parms, psize + 1, psize + 1, npolys, facevarying );
+	printPL( n, tokens, parms, psize + 1, psize + 1, npolys, nv, nv );
 }
 
 RtVoid CqOutput::RiPatchV( RtToken type, RtInt n, RtToken tokens[], RtPointer parms[] )
@@ -1348,7 +1327,7 @@ RtVoid CqOutput::RiPatchV( RtToken type, RtInt n, RtToken tokens[], RtPointer pa
 	{
 		throw CqError( RIE_BADTOKEN, RIE_ERROR,
 		               "Unknown RiPatch type: ", type,
-		               "  RiPatch() instruction skipped", TqTrue );
+		               "  RiPatch() instruction skipped", true );
 	}
 
 	PR( "Patch", Patch );
@@ -1363,42 +1342,43 @@ RtVoid CqOutput::RiPatchMeshV( RtToken type, RtInt nu, RtToken uwrap,
                                RtPointer parms[] )
 {
 	RtInt nuptch, nvptch;
-	RtInt ii = 0;
+	RtInt varyingDataLen = 1;
+	/// \todo Review: Consider removing redundent direct pointer comparisons to RI_BILINEAR.  Surely this isn't a worthwhile optimisation?
 	if ( type == RI_BILINEAR || (strcmp(type, RI_BILINEAR)==0) )
 	{
+		/// \todo Review: Factor out checks for RI_PERIODIC into a small helper function.
 		if ( uwrap == RI_PERIODIC || (strcmp(uwrap, RI_PERIODIC) ==0) )
 		{
 			nuptch = nu;
+			varyingDataLen *= nuptch;
 		}
 		else if ( uwrap == RI_NONPERIODIC || (strcmp(uwrap, RI_NONPERIODIC)==0) )
 		{
 			nuptch = nu - 1;
-			ii += 1;
+			varyingDataLen *= nuptch + 1;
 		}
 		else
 		{
 			throw CqError( RIE_BADTOKEN, RIE_ERROR,
 			               "Unknown RiPatchMesh uwrap token:", uwrap,
-			               "  RiPatchMesh instruction skipped", TqTrue );
+			               "  RiPatchMesh instruction skipped", true );
 		}
 		if ( vwrap == RI_PERIODIC || (strcmp(vwrap, RI_PERIODIC)==0) )
 		{
 			nvptch = nv;
+			varyingDataLen *= nvptch;
 		}
 		else if ( vwrap == RI_NONPERIODIC || (strcmp(vwrap, RI_NONPERIODIC)==0) )
 		{
 			nvptch = nv - 1;
-			ii += 1;
+			varyingDataLen *= nvptch + 1;
 		}
 		else
 		{
 			throw CqError( RIE_BADTOKEN, RIE_ERROR,
 			               "Unknown RiPatchMesh vwrap token:", vwrap,
-			               "  RiPatchMesh instruction skipped", TqTrue );
+			               "  RiPatchMesh instruction skipped", true );
 		}
-		ii += nuptch + nvptch;
-
-
 	}
 	else if ( type == RI_BICUBIC || (strcmp(type, RI_BICUBIC)==0) )
 	{
@@ -1408,42 +1388,41 @@ RtVoid CqOutput::RiPatchMeshV( RtToken type, RtInt nu, RtToken uwrap,
 		if ( uwrap == RI_PERIODIC || (strcmp(uwrap, RI_PERIODIC) ==0) )
 		{
 			nuptch = nu / nustep;
+			varyingDataLen *= nuptch;
 		}
 		else if ( uwrap == RI_NONPERIODIC || (strcmp(uwrap, RI_NONPERIODIC)==0) )
 		{
 			nuptch = ( nu - 4 ) / nustep + 1;
-			ii += 1;
+			varyingDataLen *= nuptch + 1;
 		}
 		else
 		{
 			throw CqError( RIE_BADTOKEN, RIE_ERROR,
 			               "Unknown RiPatchMesh uwrap token:", uwrap,
-			               "  RiPatchMesh instruction skipped", TqTrue );
+			               "  RiPatchMesh instruction skipped", true );
 		}
 		if ( vwrap == RI_PERIODIC || (strcmp(vwrap, RI_PERIODIC)==0) )
 		{
 			nvptch = nv / nvstep;
+			varyingDataLen *= nvptch;
 		}
 		else if ( vwrap == RI_NONPERIODIC || (strcmp(vwrap, RI_NONPERIODIC)==0) )
 		{
 			nvptch = ( nv - 4 ) / nvstep + 1;
-			ii += 1;
+			varyingDataLen *= nvptch + 1;
 		}
 		else
 		{
 			throw CqError( RIE_BADTOKEN, RIE_ERROR,
 			               "Unknown RiPatchMesh vwrap token:", vwrap,
-			               "  RiPatchMesh instruction skipped", TqTrue );
+			               "  RiPatchMesh instruction skipped", true );
 		}
-		ii += nuptch + nvptch;
-
-
 	}
 	else
 	{
 		throw CqError( RIE_BADTOKEN, RIE_ERROR,
 		               "Unknown RiPatchMesh type:", type,
-		               "  RiPatchMesh instruction skipped", TqTrue );
+		               "  RiPatchMesh instruction skipped", true );
 	}
 
 	PR( "PatchMesh", PatchMesh );
@@ -1458,7 +1437,7 @@ RtVoid CqOutput::RiPatchMeshV( RtToken type, RtInt nu, RtToken uwrap,
 	S;
 	printToken( vwrap );
 	S;
-	printPL( n, tokens, parms, nu * nv, ii, nuptch * nvptch );
+	printPL( n, tokens, parms, nu * nv, varyingDataLen, nuptch * nvptch );
 }
 
 RtVoid CqOutput::RiNuPatchV( RtInt nu, RtInt uorder, RtFloat uknot[],
@@ -1615,6 +1594,11 @@ RtVoid CqOutput::RiBlobbyV( RtInt nleaf, RtInt ncode, RtInt code[],
                             RtInt n, RtToken tokens[], RtPointer parms[] )
 {
 	PR( "Blobby", Blobby );
+	std::ostringstream tmp;
+	tmp << nleaf;
+	std::string tmp2 (tmp.str());
+	S;
+	print(tmp2.c_str());
 	S;
 	printArray( ncode, code );
 	S;
@@ -1666,7 +1650,7 @@ RtVoid CqOutput::RiCurvesV( RtToken type, RtInt ncurves,
 		{
 			throw CqError( RIE_BADTOKEN, RIE_ERROR,
 			               "Unknown RiCurves wrap token:", wrap,
-			               "  RiCurves instruction skipped", TqTrue );
+			               "  RiCurves instruction skipped", true );
 		}
 	}
 	else if ( type == RI_CUBIC || (strcmp(type, RI_CUBIC)==0) )
@@ -1689,14 +1673,14 @@ RtVoid CqOutput::RiCurvesV( RtToken type, RtInt ncurves,
 		{
 			throw CqError( RIE_BADTOKEN, RIE_ERROR,
 			               "Unknown RiCurves wrap token:", wrap,
-			               "  RiCurves instruction skipped", TqTrue );
+			               "  RiCurves instruction skipped", true );
 		}
 	}
 	else
 	{
 		throw CqError( RIE_BADTOKEN, RIE_ERROR,
 		               "Unknown RiCurves type:", type,
-		               "  RiCurves instruction skipped", TqTrue );
+		               "  RiCurves instruction skipped", true );
 	}
 
 	PR( "Curves", Curves );
@@ -1762,7 +1746,7 @@ RtVoid CqOutput::RiSubdivisionMeshV( RtToken mask, RtInt nf, RtInt nverts[],
 		if ( psize < verts[ i ] )
 			psize = verts[ i ];
 	}
-	printPL( n, tokens, parms, psize + 1, psize + 1, nf, vsize );
+	printPL( n, tokens, parms, psize + 1, psize + 1, nf, vsize, vsize );
 }
 
 RtVoid CqOutput::RiProcedural( RtPointer data, RtBound bound,
@@ -1778,7 +1762,7 @@ RtVoid CqOutput::RiProcedural( RtPointer data, RtBound bound,
 	}
 	else if ( subdivfunc == RiProcRunProgram )
 	{
-		sf = "ReadProgram";
+		sf = "RunProgram";
 		a = 2;
 	}
 	else if ( subdivfunc == RiProcDynamicLoad )
@@ -1788,7 +1772,7 @@ RtVoid CqOutput::RiProcedural( RtPointer data, RtBound bound,
 	}
 	else
 	{
-		throw CqError( RIE_SYNTAX, RIE_ERROR, "Unknown procedural function.", TqTrue );
+		throw CqError( RIE_SYNTAX, RIE_ERROR, "Unknown procedural function.", true );
 	}
 
 	PR( "Procedural", Procedural );
@@ -2008,7 +1992,7 @@ RtVoid CqOutput::RiArchiveRecord( RtToken type, std::string txt )
 	else
 	{
 		throw CqError( RIE_BADTOKEN, RIE_ERROR,
-		               "Unknown ArchiveRecord type: ", type, "", TqTrue );
+		               "Unknown ArchiveRecord type: ", type, "", true );
 	}
 	// EOL must be forced for binary
 	print( ( tmp + txt + "\n" ).c_str() );
@@ -2065,7 +2049,7 @@ RtVoid CqOutput::RiErrorHandler( RtErrorFunc handler )
 		ch = "abort";
 	else
 	{
-		throw CqError( RIE_CONSISTENCY, RIE_ERROR, "Unknown Error handler", TqTrue );
+		throw CqError( RIE_CONSISTENCY, RIE_ERROR, "Unknown Error handler", true );
 	}
 	PR( "ErrorHandler", ErrorHandler );
 	S;
@@ -2105,3 +2089,5 @@ RtVoid CqOutput::RiConnectShaderLayers( RtToken type, RtToken layer1, RtToken va
 	S;
 	printToken( variable2 );
 }
+
+} // namespace libri2rib
