@@ -1,5 +1,5 @@
 // Aqsis
-// Copyright  1997 - 2001, Paul C. Gregory
+// Copyright (C) 1997 - 2001, Paul C. Gregory
 //
 // Contact: pgregory@aqsis.org
 //
@@ -28,7 +28,7 @@
     \author Lionel J. Lacour (intuition01@online.fr)
 */
 
-#ifdef	WIN32
+#if _MSC_VER
 #pragma warning(disable : 4786)
 #endif
 
@@ -37,7 +37,7 @@
 #include <iostream>
 #include <iomanip>
 
-#include <stdio.h>
+#include <cstdlib>
 #include "bdec.h"
 
 #ifdef AQSIS_SYSTEM_WIN32
@@ -166,7 +166,7 @@ CqRibBinaryDecoder::initZlib( int buffersize )
 
 	/* Check the gzip magic number */
 	// Start by only buffering 2 bytes
-	zbuffersize = 2;
+	zbuffersize = 1;
 	for (len = 0; len < 2; len++)
 	{
 		try
@@ -179,10 +179,9 @@ CqRibBinaryDecoder::initZlib( int buffersize )
 		}
 		if (c != gz_magic[len])
 		{
-			if (len != 0)
-				zavailable++, zcurrent--;
-			if (c != EOF)
-				zavailable++, zcurrent--;
+			// Roll back any characters we peeked from the buffer.
+			zavailable += len+1;
+			zcurrent -= len+1;
 			is_gzip = 0;
 			zbuffersize = buffersize;
 
@@ -256,13 +255,13 @@ CqRibBinaryDecoder::CqRibBinaryDecoder( std::string filename, int buffersize )
 
 	if ( file == NULL )
 	{
-		fail_flag = TqTrue;
-		eof_flag = TqTrue;
+		fail_flag = true;
+		eof_flag = true;
 	}
 	else
 	{
-		fail_flag = TqFalse;
-		eof_flag = TqFalse;
+		fail_flag = false;
+		eof_flag = false;
 		initZlib( buffersize );
 
 	}
@@ -278,18 +277,23 @@ CqRibBinaryDecoder::CqRibBinaryDecoder( FILE *filehandle, int buffersize )
 {
 	//We dup here because we don't want to close the original
 	//filehandle.
+	// \todo Should find a way of abstacting this out so we don't have
+	//       system specific code in here if we can.
+#ifdef	AQSIS_SYSTEM_WIN32
+	file = fdopen( _dup( _fileno( filehandle ) ), "rb");
+#else
 	file = fdopen( dup( fileno( filehandle ) ), "rb");
-
+#endif // AQSIS_SYSTEM_WIN32
 
 	if ( file == NULL )
 	{
-		fail_flag = TqTrue;
-		eof_flag = TqTrue;
+		fail_flag = true;
+		eof_flag = true;
 	}
 	else
 	{
-		fail_flag = TqFalse;
-		eof_flag = TqFalse;
+		fail_flag = false;
+		eof_flag = false;
 		initZlib( buffersize );
 	}
 }
@@ -446,13 +450,13 @@ void CqRibBinaryDecoder::gc( TqChar &c )
 
 				if( zerr == Z_STREAM_END && (int) zstrm.avail_out == zbuffersize)
 				{
-					eof_flag = TqTrue;
+					eof_flag = true;
 					throw std::string( "" );
 				};
 
 				if( zerr != Z_OK && zerr != Z_STREAM_END )
 				{
-					fail_flag = eof_flag = TqTrue;
+					fail_flag = eof_flag = true;
 					throw std::string( "" );
 				};
 			};
@@ -467,7 +471,7 @@ void CqRibBinaryDecoder::gc( TqChar &c )
 			zavailable = zstrm.avail_in = fread( zin, 1, zbuffersize, file );
 			if( zavailable == 0 )
 			{
-				eof_flag = TqTrue;
+				eof_flag = true;
 				throw std::string( "" );
 				return;
 			};
@@ -674,6 +678,7 @@ void CqRibBinaryDecoder::getNext ()
 	TqChar b1, b2, b3, b4;
 	TqUint ui;
 	TqFloat f;
+	/// \todo <b>Code Review</b>: use a stream for cv directly rather than using the temporary string ostr.
 	std::string ostr;
 	std::string str;
 	std::string tmpstr;
@@ -1177,7 +1182,7 @@ TqInt CqRibBinaryDecoder::read( TqPchar buffer, TqUint size )
 
 				if( !cv.empty() )
 				{
-					if( cv.back() == '\n' )
+					if( cv.back() == '\n' || cv.back() == '\377' )
 					{
 						size = cv.size() + 1;
 						break;
@@ -1201,3 +1206,17 @@ TqInt CqRibBinaryDecoder::read( TqPchar buffer, TqUint size )
 	}
 	return n;
 }
+
+
+/*--------------------------------------------------------------------*/
+void CqRibBinaryDecoder::dumpToStream(std::ostream& out)
+{
+	const TqInt bufSize = 1024;
+	TqPchar buf = new TqChar[bufSize];
+	while(!eof())
+	{
+		TqInt readSize = read(buf, bufSize-1);
+		buf[readSize] = 0; // add null termination char.
+		out << buf;
+	}
+}
