@@ -3,6 +3,8 @@ using namespace librib;
 
 #include "ri.h"
 
+#include <cstring>
+
 namespace librib2ri
 {
 
@@ -69,6 +71,10 @@ RendermanInterface::RtVoid Engine::RiClipping( RtFloat cnear, RtFloat cfar )
 {
 	::RiClipping( cnear, cfar );
 }
+RendermanInterface::RtVoid Engine::RiClippingPlane( RtFloat x, RtFloat y, RtFloat z, RtFloat nx, RtFloat ny, RtFloat nz )
+{
+	::RiClippingPlane(x, y, z, nx, ny, nz);
+}
 RendermanInterface::RtVoid Engine::RiColor( RtColor Cq )
 {
 	::RiColor( Cq );
@@ -382,6 +388,18 @@ RendermanInterface::RtVoid Engine::RiRotate( RtFloat angle, RtFloat dx, RtFloat
 {
 	::RiRotate( angle, dx, dy, dz );
 }
+RendermanInterface::RtVoid Engine::RiResourceV(RtToken handle, RtToken type, RtInt count, RtToken tokens[], RtPointer values[])
+{
+	::RiResource(handle, type, count, tokens, values);
+}
+RendermanInterface::RtVoid Engine::RiResourceBegin()
+{
+	::RiResourceBegin();
+}
+RendermanInterface::RtVoid Engine::RiResourceEnd()
+{
+	::RiResourceEnd();
+}
 RendermanInterface::RtVoid Engine::RiScale( RtFloat sx, RtFloat sy, RtFloat sz )
 {
 	::RiScale( sx, sy, sz );
@@ -494,6 +512,8 @@ RendermanInterface::RtFilterFunc Engine::GetFilterFunction( RtToken type )
 		return ( &::RiGaussianFilter );
 	else if ( strcmp( type, "triangle" ) == 0 )
 		return ( &::RiTriangleFilter );
+	else if ( strcmp( type, "mitchell" ) == 0 )
+		return ( &::RiMitchellFilter );
 	else if ( strcmp( type, "catmull-rom" ) == 0 )
 		return ( &::RiCatmullRomFilter );
 	else if ( strcmp( type, "sinc" ) == 0 )
