@@ -1,5 +1,5 @@
 // Aqsis
-// Copyright  1997 - 2001, Paul C. Gregory
+// Copyright (C) 1997 - 2001, Paul C. Gregory
 //
 // Contact: pgregory@aqsis.org
 //
@@ -38,24 +38,17 @@
 #include	"ifuncdef.h"
 #include	"icodegen.h"
 
-START_NAMESPACE( Aqsis )
+namespace Aqsis {
 
 
-class CqCodeGenVM : private IqCodeGen
+class CqCodeGenVM : public IqCodeGen
 {
-	protected:
 	public:
-		CqCodeGenVM()
-		{
-		};
 		virtual void OutputTree( IqParseNode* pNode, std::string strOutName );
-		virtual ~CqCodeGenVM()
-		{
-		};
 };
 
 //-----------------------------------------------------------------------
 
-END_NAMESPACE( Aqsis )
+} // namespace Aqsis
 
 #endif	// !CODEGENVM_H_INCLUDED
