@@ -1,5 +1,5 @@
 // Aqsis
-// Copyright  1997 - 2001, Paul C. Gregory
+// Copyright (C) 1997 - 2001, Paul C. Gregory
 //
 // Contact: pgregory@aqsis.org
 //
@@ -32,14 +32,17 @@
 #include	<deque>
 #include	<string>
 #include	<map>
-
+#include	<numeric>
+#include	<cstdlib>
 
 #include	"version.h"
 #include	"vmoutput.h"
 
 #include	"parsenode.h"
+#include	"aqsismath.h"
+#include	"logging.h"
 
-START_NAMESPACE( Aqsis )
+namespace Aqsis {
 
 
 std::string* FindTemporaryVariable( std::string strName, std::deque<std::map<std::string, std::string> >& Stack );
@@ -75,6 +78,12 @@ void CqCodeGenOutput::Visit( IqParseNodeShader& S )
 	m_StackVarMap.push_back( temp );
 
 	m_slxFile.open( strOutName().c_str() );
+	if (m_slxFile.fail( ) )
+	{
+		std::cout << "Warning: Cannot open file \"" << strOutName().c_str() << "\"" << std::endl;
+		exit( 1 );
+	}
+
 	std::cout << "... " << strOutName().c_str() << std::endl;
 
 	m_slxFile << S.strShaderType() << std::endl;
@@ -84,6 +93,7 @@ void CqCodeGenOutput::Visit( IqParseNodeShader& S )
 	m_slxFile << std::endl << std::endl << "segment Data" << std::endl;
 
 	// Now that we have this information, work out which standard vars are used.
+	/// \todo Code Review: The use of gStandardVars here breaks the data encapsulation of the parser, and should be removed to a better location.
 	TqInt Use = m_pDataGather->VariableUsage();
 	TqUint i;
 	for ( i = 0; i < EnvVars_Last; i++ )
@@ -396,10 +406,10 @@ void CqCodeGenOutput::Visit( IqParseNodeOperator& OP )
 	IqParseNode* pOperandA = pNode->pChild();
 	IqParseNode* pOperandB = pOperandA->pNextSibling();
 
-	char* pstrAType = "";
+	const char* pstrAType = "";
 	if ( pOperandA )
 		pstrAType = gVariableTypeIdentifiers[ pOperandA->ResType() & Type_Mask ];
-	char* pstrBType = "";
+	const char* pstrBType = "";
 	if ( pOperandB )
 		pstrBType = gVariableTypeIdentifiers[ pOperandB->ResType() & Type_Mask ];
 
@@ -411,9 +421,9 @@ void CqCodeGenOutput::Visit( IqParseNodeOperator& OP )
 	if ( pNode->NodeType() != ParseNode_LogicalOp )
 	{
 		if ( pOperandA )
-			m_slxFile << pstrBType;
-		if ( pOperandB )
 			m_slxFile << pstrAType;
+		if ( pOperandB )
+			m_slxFile << pstrBType;
 	}
 	m_slxFile << std::endl;
 }
@@ -484,19 +494,58 @@ void CqCodeGenOutput::Visit( IqParseNodeWhileConstruct& WC )
 	assert( pStmt != 0 );
 	IqParseNode* pStmtInc = pStmt->pNextSibling();
 
+	rsPush();										// push running state
+	// Mark current RS stack depth so that break/continue will know where to
+	// jump to.
+	m_breakDepthStack.push_back(0);
 	m_slxFile << ":" << iLabelA << std::endl;		// loop back label
 	m_slxFile << "\tS_CLEAR" << std::endl;			// clear current state
 	pArg->Accept( *this );							// relation
 	m_slxFile << "\tS_GET" << std::endl;			// Get the current state by popping the t[ value off the stack
 	m_slxFile << "\tS_JZ " << iLabelB << std::endl;	// exit if false
-	m_slxFile << "\tRS_PUSH" << std::endl;			// push running state
 	m_slxFile << "\tRS_GET" << std::endl;			// get current state to running state
+	// The following push and associated pop are needed to implement continue statements correctly.
+	rsPush();										// push running state
 	pStmt->Accept( *this );							// statement
-	if ( pStmtInc )
+	rsPop();										// pop the running state
+	if( pStmtInc )
 		pStmtInc->Accept( *this );					// incrementor
-	m_slxFile << "\tRS_POP" << std::endl;			// Pop the running state
 	m_slxFile << "\tjmp " << iLabelA << std::endl;	// loop back jump
 	m_slxFile << ":" << iLabelB << std::endl;		// completion label
+	m_breakDepthStack.pop_back();
+	rsPop();										// pop the running state
+}
+
+void CqCodeGenOutput::Visit( IqParseNodeLoopMod& LM )
+{
+	IqParseNode* pChild = static_cast<IqParseNode*>(LM.GetInterface( ParseNode_Base ))->pChild();
+	TqInt breakDepth = 1;
+	if(pChild)
+	{
+		breakDepth = lround( static_cast<IqParseNodeConstantFloat*>(
+					pChild->GetInterface(ParseNode_ConstantFloat))->Value() );
+	}
+	if(breakDepth > static_cast<TqInt>(m_breakDepthStack.size()))
+	{
+		/// \todo This should have been done by the semantic analysis
+		/// stage, but here is better than doing nothing (for now).
+		Aqsis::log() << critical << "Break depth too deep\n";
+		// kludge: break output so it can't be accidentally used...
+		m_slxFile << "\tCANNOT_BREAK - COMPILE ERROR";
+		return;
+	}
+	TqInt rsPopNum = std::accumulate(m_breakDepthStack.rbegin(),
+				m_breakDepthStack.rbegin() + breakDepth, 0);
+	switch(LM.modType())
+	{
+		case LoopMod_Break:
+			// Need one extra pop for the break.
+			m_slxFile << "\tRS_BREAK " << rsPopNum << "\n";
+			break;
+		case LoopMod_Continue:
+			m_slxFile << "\tRS_BREAK " << (rsPopNum-1) << "\n";
+			break;
+	}
 }
 
 void CqCodeGenOutput::Visit( IqParseNodeIlluminateConstruct& IC )
@@ -519,11 +568,11 @@ void CqCodeGenOutput::Visit( IqParseNodeIlluminateConstruct& IC )
 	else
 		m_slxFile << "\tilluminate" << std::endl;
 	m_slxFile << "\tS_JZ " << iLabelB << std::endl;	// exit loop if false
-	m_slxFile << "\tRS_PUSH" << std::endl;			// Push running state
+	rsPush();										// push running state
 	m_slxFile << "\tRS_GET" << std::endl;			// Get state
 	pStmt->Accept( *this );							// statement
-	m_slxFile << "\tRS_POP" << std::endl;			// Pop the running state
-	m_slxFile << "\tjmp " << iLabelA << std::endl; // loop back jump
+	rsPop();										// pop the running state
+	m_slxFile << "\tjmp " << iLabelA << std::endl;	// loop back jump
 	m_slxFile << ":" << iLabelB << std::endl;		// completion label
 }
 
@@ -569,10 +618,10 @@ void CqCodeGenOutput::Visit( IqParseNodeIlluminanceConstruct& IC )
 	else
 		m_slxFile << "\tilluminance" << std::endl;
 	m_slxFile << "\tS_JZ " << iLabelC << std::endl;	// skip processing of statement if light has no influence
-	m_slxFile << "\tRS_PUSH" << std::endl;			// Push running state
+	rsPush();										// push running state
 	m_slxFile << "\tRS_GET" << std::endl;			// Get state
 	pStmt->Accept( *this );							// statement
-	m_slxFile << "\tRS_POP" << std::endl;			// Pop the running state
+	rsPop();										// pop the running state
 	m_slxFile << ":" << iLabelC << std::endl;		// continuation label
 	m_slxFile << "\tadvance_illuminance" << std::endl;
 	m_slxFile << "\tjnz " << iLabelA << std::endl; // loop back jump
@@ -598,11 +647,11 @@ void CqCodeGenOutput::Visit( IqParseNodeSolarConstruct& SC )
 		pArg->Accept( *this );
 		m_slxFile << "\tsolar2" << std::endl;
 		m_slxFile << "\tS_JZ " << iLabelB << std::endl;	// exit loop if false
-		m_slxFile << "\tRS_PUSH" << std::endl;			// Push running state
+		rsPush();										// push running state
 		m_slxFile << "\tRS_GET" << std::endl;			// set running state
 		if ( pStmt )
 			pStmt->Accept( *this );			// statement
-		m_slxFile << "\tRS_POP" << std::endl;			// Pop the running state
+		rsPop();										// pop the running state
 		m_slxFile << "\tjmp " << iLabelA << std::endl;	// loop back jump
 		m_slxFile << ":" << iLabelB << std::endl;		// completion label
 	}
@@ -613,16 +662,84 @@ void CqCodeGenOutput::Visit( IqParseNodeSolarConstruct& SC )
 		m_slxFile << "\tS_CLEAR" << std::endl;			// clear current state
 		m_slxFile << "\tsolar" << std::endl;
 		m_slxFile << "\tS_JZ " << iLabelB << std::endl;	// exit loop if false
-		m_slxFile << "\tRS_PUSH" << std::endl;			// Push running state
+		rsPush();										// push running state
 		m_slxFile << "\tRS_GET" << std::endl;			// set running state
 		if ( pStmt )
 			pStmt->Accept( *this );			// statement
-		m_slxFile << "\tRS_POP" << std::endl;			// Pop the running state
+		rsPop();										// pop the running state
 		m_slxFile << "\tjmp " << iLabelA << std::endl;	// loop back jump
 		m_slxFile << ":" << iLabelB << std::endl;		// completion label
 	}
 }
 
+void CqCodeGenOutput::Visit( IqParseNodeGatherConstruct& IC )
+{
+	IqParseNode * pNode;
+	pNode = static_cast<IqParseNode*>(IC.GetInterface( ParseNode_Base ));
+
+	TqInt iLabelA = m_gcLabels++;
+	TqInt iLabelB = m_gcLabels++;
+	TqInt iLabelC = m_gcLabels++;
+	TqInt iLabelD = m_gcLabels++;
+
+	IqParseNode* pArgs = pNode->pChild();
+	assert( pArgs != 0 );
+	IqParseNode* pHitStmt = pArgs->pNextSibling();
+	assert( pHitStmt != 0 );
+	IqParseNode* pNoHitStmt = pHitStmt->pNextSibling();
+
+	// samples is the 5th argument to the gather function, we need that to 
+	// initialise the init_gather call.
+	IqParseNode* pSamples = pArgs->pChild();
+	TqInt iArg = 5;
+	while(--iArg > 0)	
+	{
+		pSamples = pSamples->pNextSibling();
+		assert(pSamples);
+	}
+	pSamples->Accept(*this);
+
+	m_slxFile << "\tinit_gather" << std::endl;
+
+	m_slxFile << ":" << iLabelA << std::endl;		// loop back label
+	m_slxFile << "\tS_CLEAR" << std::endl;			// clear current state
+
+	// Output the arguments in reverse order so that the gather() function
+	// can pop them in the expected order.
+	IqParseNode* pArg = pArgs->pChild(); 
+	while ( pArg->pNextSibling() != 0 )
+		pArg = pArg->pNextSibling();
+	iArg = 0;
+	while ( pArg != 0 )
+	{
+		// Push the argument...
+		iArg++;
+		pArg->Accept( *this );
+		pArg = pArg->pPrevSibling();
+	}
+	// Now output the count of 'additional' arguments.
+	iArg -= 5;
+	CqParseNodeFloatConst C( static_cast<TqFloat>( abs( iArg ) ) );
+	C.Accept( *this );
+	m_slxFile << "\tgather" << std::endl;
+	rsPush();										// push running state
+	m_slxFile << "\tRS_GET" << std::endl;			// Get state
+	m_slxFile << "\tRS_JZ " << iLabelD << std::endl;	// Jump to no hit statement
+	pHitStmt->Accept( *this );						// ray hit statement
+	m_slxFile << ":" << iLabelD << std::endl;
+	if(pNoHitStmt)
+	{
+		m_slxFile << "\tRS_INVERSE" << std::endl;		// Inverse running state
+		m_slxFile << "\tRS_JZ " << iLabelC << std::endl;	// exit if not running
+		pNoHitStmt->Accept( *this );						// ray hit statement
+	}
+	m_slxFile << ":" << iLabelC << std::endl;		// continuation label
+	rsPop();										// pop the running state
+	m_slxFile << "\tadvance_gather" << std::endl;
+	m_slxFile << "\tjnz " << iLabelA << std::endl; // loop back jump
+	m_slxFile << ":" << iLabelB << std::endl;		// completion label
+}
+
 void CqCodeGenOutput::Visit( IqParseNodeConditional& C )
 {
 	IqParseNode * pNode;
@@ -640,7 +757,7 @@ void CqCodeGenOutput::Visit( IqParseNodeConditional& C )
 	m_slxFile << "\tS_CLEAR" << std::endl;			// clear current state
 	pArg->Accept( *this );							// relation
 	m_slxFile << "\tS_GET" << std::endl;			// Get the current state by popping the top value off the stack
-	m_slxFile << "\tRS_PUSH" << std::endl;			// push the running state
+	rsPush();										// push running state
 	m_slxFile << "\tRS_GET" << std::endl;			// get current state to running state
 	if ( pFalseStmt )
 	{
@@ -653,12 +770,12 @@ void CqCodeGenOutput::Visit( IqParseNodeConditional& C )
 	if ( pFalseStmt )
 	{
 		m_slxFile << ":" << iLabelB << std::endl;	// false part label
-		m_slxFile << "\tRS_JNZ " << iLabelA << std::endl;	// exit if all true
 		m_slxFile << "\tRS_INVERSE" << std::endl;	// Invert result
+		m_slxFile << "\tRS_JZ " << iLabelA << std::endl;	// exit if not running
 		pFalseStmt->Accept( *this );				// false statement
 	}
 	m_slxFile << ":" << iLabelA << std::endl;		// conditional exit point
-	m_slxFile << "\tRS_POP" << std::endl;			// pop running state
+	rsPop();										// pop the running state
 }
 
 void CqCodeGenOutput::Visit( IqParseNodeConditionalExpression& CE )
@@ -666,18 +783,31 @@ void CqCodeGenOutput::Visit( IqParseNodeConditionalExpression& CE )
 	IqParseNode * pNode;
 	pNode = static_cast<IqParseNode*>(CE.GetInterface( ParseNode_Base ));
 
-	IqParseNode * pArg = pNode->pChild();
-	assert( pArg != 0 );
-	IqParseNode* pTrueStmt = pArg->pNextSibling();
+	// Extract the statement trees from the AST
+	IqParseNode * pCondition = pNode->pChild();
+	assert( pCondition != 0 );
+	IqParseNode* pTrueStmt = pCondition->pNextSibling();
 	assert( pTrueStmt != 0 );
 	IqParseNode* pFalseStmt = pTrueStmt->pNextSibling();
+	assert( pFalseStmt != 0 );
 
+	// Write out VM code to evaluate each branch with the correct running state
+	m_slxFile << "\tS_CLEAR\n";		// clear current tmp state
+	pCondition->Accept( *this );	// evaluate conditional
+	m_slxFile << "\tdup\n";			// create a copy of the conditional for the merge step
+	m_slxFile << "\tS_GET\n";		// Pop main stack into current tmp state
+	rsPush();						// push current running state onto state stack
+	m_slxFile << "\tRS_GET\n";		// copy current tmp state to running state
+	pTrueStmt->Accept( *this );		// evaluate true statement
+	m_slxFile << "\tRS_INVERSE\n";	// Invert running state
+	pFalseStmt->Accept( *this );	// evaluate false statement
+	rsPop();						// pop the running state
+
+	// The stack now contains:
+	//  ... conditional true_stmt_result false_stmt_result
+	// Merge the results together
 	TqInt typeT = static_cast<TqInt>( pTrueStmt->ResType() & Type_Mask );
-	char* pstrTType = gVariableTypeIdentifiers[ typeT ];
-
-	pTrueStmt->Accept( *this );					// true statement
-	pFalseStmt->Accept( *this );				// false statement
-	pArg->Accept( *this );						// relation
+	const char* pstrTType = gVariableTypeIdentifiers[ typeT ];
 	m_slxFile << "\tmerge" << pstrTType << std::endl;
 }
 
@@ -696,8 +826,8 @@ void CqCodeGenOutput::Visit( IqParseNodeTypeCast& TC )
 	if ( !( ( typeA == Type_Point || typeA == Type_Normal || typeA == Type_Vector ) &&
 	        ( typeB == Type_Point || typeB == Type_Normal || typeB == Type_Vector ) ) )
 	{
-		char * pstrToType = gVariableTypeIdentifiers[ TC.CastTo() & Type_Mask ];
-		char* pstrFromType = gVariableTypeIdentifiers[ pOperand->ResType() & Type_Mask ];
+		const char* pstrToType = gVariableTypeIdentifiers[ TC.CastTo() & Type_Mask ];
+		const char* pstrFromType = gVariableTypeIdentifiers[ pOperand->ResType() & Type_Mask ];
 		m_slxFile << "\tset" << pstrFromType << pstrToType << std::endl;
 	}
 }
@@ -828,6 +958,9 @@ void CqCodeGenOutput::Visit( IqParseNodeMessagePassingFunction& MPF )
 			strCommType = "textureinfo";
 			break;
 
+			case CommTypeRayInfo:
+			strCommType = "rayinfo";
+			break;
 	}
 	// Output the comm function.
 	SqVarRef temp( MPF.VarRef() );
@@ -889,104 +1022,51 @@ const char* CqCodeGenOutput::MathOpName( TqInt op )
 	// Output this nodes operand name.
 	switch ( op )
 	{
-			case Op_Add:
-			return ( "add" );
-			break;
-
-			case Op_Sub:
-			return ( "sub" );
-			break;
-
-			case Op_Mul:
-			return ( "mul" );
-			break;
-
-			case Op_Div:
-			return ( "div" );
-			break;
-
-			case Op_Dot:
-			return ( "dot" );
-			break;
-
-			case Op_Crs:
-			return ( "crs" );
-			break;
-
-			case Op_Mod:
-			return ( "mod" );
-			break;
-
-			case Op_Lft:
-			return ( "left" );
-			break;
-
-			case Op_Rgt:
-			return ( "right" );
-			break;
-
-			case Op_And:
-			return ( "and" );
-			break;
-
-			case Op_Xor:
-			return ( "xor" );
-			break;
-
-			case Op_Or:
-			return ( "or" );
-			break;
-
-			case Op_L:
-			return ( "ls" );
-			break;
-
-			case Op_G:
-			return ( "gt" );
-			break;
-
-			case Op_GE:
-			return ( "ge" );
-			break;
-
-			case Op_LE:
-			return ( "le" );
-			break;
-
-			case Op_EQ:
-			return ( "eq" );
-			break;
-
-			case Op_NE:
-			return ( "ne" );
-			break;
-
-			case Op_Plus:
-			break;
-
-			case Op_Neg:
-			return ( "neg" );
-			break;
-
-			case Op_BitwiseComplement:
-			return ( "cmpl" );
-			break;
-
-			case Op_LogicalNot:
-			return ( "not" );
-			break;
-
-			case Op_LogAnd:
-			return ( "land" );
-			break;
-
-			case Op_LogOr:
-			return ( "lor" );
-			break;
+		case Op_Add: return "add";
+		case Op_Sub: return "sub";
+		case Op_Mul: return "mul";
+		case Op_Div: return "div";
+		case Op_Dot: return "dot";
+		case Op_Crs: return "crs";
+		case Op_Mod: return "mod";
+		case Op_Lft: return "left";
+		case Op_Rgt: return "right";
+		case Op_And: return "and";
+		case Op_Xor: return "xor";
+		case Op_Or: return "or";
+		case Op_L: return "ls";
+		case Op_G: return "gt";
+		case Op_GE: return "ge";
+		case Op_LE: return "le";
+		case Op_EQ: return "eq";
+		case Op_NE: return "ne";
+		case Op_Neg: return "neg";
+		case Op_BitwiseComplement: return "cmpl";
+		case Op_LogicalNot: return "not";
+		case Op_LogAnd: return "land";
+		case Op_LogOr: return "lor";
+		case Op_Plus: 
+		default:
+			return "error";
 	}
-	return ( "error" );
+}
+
+void CqCodeGenOutput::rsPush()
+{
+	// push the running state
+	if(!m_breakDepthStack.empty())
+		m_breakDepthStack.back()++;
+	m_slxFile << "\tRS_PUSH" << std::endl;
+}
+
+void CqCodeGenOutput::rsPop()
+{
+	// pop the running state
+	m_slxFile << "\tRS_POP" << std::endl;
+	if(!m_breakDepthStack.empty())
+		m_breakDepthStack.back()--;
 }
 
 //-----------------------------------------------------------------------
 
-END_NAMESPACE( Aqsis )
+} // namespace Aqsis
