@@ -1,5 +1,5 @@
 // Aqsis
-// Copyright  1997 - 2001, Paul C. Gregory
+// Copyright (C) 1997 - 2001, Paul C. Gregory
 //
 // Contact: pgregory@aqsis.org
 //
@@ -38,12 +38,39 @@
 #include	"version.h"
 #include	"logging.h"
 
-START_NAMESPACE(    Aqsis )
+namespace Aqsis {
+
+namespace {
+
+/// Extract additional named filter parameters from an array of stack entries.
+void	GetFilterParams( int cParams, IqShaderData** apParams, float& _pswidth, float& _ptwidth )
+{
+	CqString strParam;
+	TqFloat f;
+
+	int i = 0;
+	while ( cParams > 0 )
+	{
+		apParams[ i ] ->GetString( strParam, 0 );
+		apParams[ i + 1 ] ->GetFloat( f, 0 );
+
+		if ( strParam.compare( "width" ) == 0 )
+			_pswidth = _ptwidth = f;
+		else if ( strParam.compare( "swidth" ) == 0 )
+			_pswidth = f;
+		else if ( strParam.compare( "twidth" ) == 0 )
+			_ptwidth = f;
+		i += 2;
+		cParams -= 2;
+	}
+}
+
+} // unnamed namespace
 
 
 void	CqShaderExecEnv::SO_step( IqShaderData* _min, IqShaderData* value, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__fVarying=(_min)->Class()==class_varying;
@@ -51,7 +78,7 @@ void	CqShaderExecEnv::SO_step( IqShaderData* _min, IqShaderData* value, IqShader
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -71,7 +98,7 @@ void	CqShaderExecEnv::SO_step( IqShaderData* _min, IqShaderData* value, IqShader
 // smoothstep(_min,_max,value)
 void	CqShaderExecEnv::SO_smoothstep( IqShaderData* _min, IqShaderData* _max, IqShaderData* value, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__fVarying=(value)->Class()==class_varying;
@@ -80,7 +107,7 @@ void	CqShaderExecEnv::SO_smoothstep( IqShaderData* _min, IqShaderData* _max, IqS
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -110,11 +137,10 @@ void	CqShaderExecEnv::SO_smoothstep( IqShaderData* _min, IqShaderData* _max, IqS
 // spline(value, f1,f2,...,fn)
 void	CqShaderExecEnv::SO_fspline( IqShaderData* value, IqShaderData* Result, IqShader* pShader, int cParams, IqShaderData** apParams )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
-	CqSplineCubic spline( cParams );
-	spline.SetBasis("catmull-rom");
+	CqCubicSpline<TqFloat> spline( SplineBasis_CatmullRom, cParams );
 
 	__fVarying=(value)->Class()==class_varying;
 	TqInt v;
@@ -125,7 +151,7 @@ void	CqShaderExecEnv::SO_fspline( IqShaderData* value, IqShaderData* Result, IqS
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -151,11 +177,11 @@ void	CqShaderExecEnv::SO_fspline( IqShaderData* value, IqShaderData* Result, IqS
 				{
 					TqFloat fn;
 					apParams[ j ] ->GetFloat( fn, __iGrid );
-					spline[ j ] = CqVector4D( fn, 0.0f, 0.0f, 1.0f );
+					spline.pushBack( fn );
 				}
 
-				CqVector4D	res = spline.Evaluate( _aq_value );
-				(Result)->SetFloat(res.x(),__iGrid);
+				(Result)->SetFloat( spline.evaluate( _aq_value ), __iGrid );
+				spline.clear();
 			}
 		}
 	}
@@ -167,10 +193,10 @@ void	CqShaderExecEnv::SO_fspline( IqShaderData* value, IqShaderData* Result, IqS
 // spline(value, f1,f2,...,fn)
 void	CqShaderExecEnv::SO_cspline( IqShaderData* value, IqShaderData* Result, IqShader* pShader, int cParams, IqShaderData** apParams )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
-	CqSplineCubic spline( cParams );
+	CqCubicSpline<CqColor> spline( SplineBasis_CatmullRom, cParams );
 
 	__fVarying=(value)->Class()==class_varying;
 	TqInt v;
@@ -181,24 +207,24 @@ void	CqShaderExecEnv::SO_cspline( IqShaderData* value, IqShaderData* Result, IqS
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
 		{
 			TqFloat _aq_value;
-			(value)->GetFloat(_aq_value,__iGrid);
+			(value)->GetFloat( _aq_value, __iGrid );
 			if ( _aq_value >= 1.0f )
 			{
 				CqColor cl;
 				apParams[ cParams - 2 ] ->GetColor( cl, __iGrid );
-				(Result)->SetColor(cl,__iGrid);
+				(Result)->SetColor( cl, __iGrid );
 			}
 			else if ( _aq_value <= 0.0f )
 			{
 				CqColor cf;
 				apParams[ 1 ] ->GetColor( cf, __iGrid );
-				(Result)->SetColor(cf,__iGrid);
+				(Result)->SetColor( cf, __iGrid );
 			}
 			else
 			{
@@ -207,11 +233,11 @@ void	CqShaderExecEnv::SO_cspline( IqShaderData* value, IqShaderData* Result, IqS
 				{
 					CqColor cn;
 					apParams[ j ] ->GetColor( cn, __iGrid );
-					spline[ j ] = CqVector4D( cn.fRed(), cn.fGreen(), cn.fBlue(), 1.0f );
+					spline.pushBack( cn );
 				}
 
-				CqVector4D	res = spline.Evaluate( _aq_value );
-				(Result)->SetColor(CqColor( res.x(), res.y(), res.z() ),__iGrid);
+				(Result)->SetColor( spline.evaluate( _aq_value ), __iGrid);
+				spline.clear();
 			}
 		}
 	}
@@ -223,10 +249,10 @@ void	CqShaderExecEnv::SO_cspline( IqShaderData* value, IqShaderData* Result, IqS
 // spline(value, f1,f2,...,fn)
 void	CqShaderExecEnv::SO_pspline( IqShaderData* value, IqShaderData* Result, IqShader* pShader, int cParams, IqShaderData** apParams )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
-	CqSplineCubic spline( cParams );
+	CqCubicSpline<CqVector3D> spline( SplineBasis_CatmullRom, cParams );
 
 	__fVarying=(value)->Class()==class_varying;
 	TqInt v;
@@ -237,7 +263,7 @@ void	CqShaderExecEnv::SO_pspline( IqShaderData* value, IqShaderData* Result, IqS
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -263,11 +289,11 @@ void	CqShaderExecEnv::SO_pspline( IqShaderData* value, IqShaderData* Result, IqS
 				{
 					CqVector3D pn;
 					apParams[ j ] ->GetPoint( pn, __iGrid );
-					spline[ j ] = pn;
+					spline.pushBack( pn );
 				}
-
-				CqVector3D	res = spline.Evaluate( _aq_value );
-				(Result)->SetPoint(res,__iGrid);
+				
+				(Result)->SetPoint( spline.evaluate( _aq_value ), __iGrid );
+				spline.clear();
 			}
 		}
 	}
@@ -279,11 +305,9 @@ void	CqShaderExecEnv::SO_pspline( IqShaderData* value, IqShaderData* Result, IqS
 // spline(value, f1,f2,...,fn)
 void	CqShaderExecEnv::SO_sfspline( IqShaderData* basis, IqShaderData* value, IqShaderData* Result, IqShader* pShader, int cParams, IqShaderData** apParams )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
-	CqSplineCubic spline( cParams );
-
 	__fVarying=(value)->Class()==class_varying;
 	TqInt v;
 	for ( v = 0; v < cParams; v++ )
@@ -294,12 +318,11 @@ void	CqShaderExecEnv::SO_sfspline( IqShaderData* basis, IqShaderData* value, IqS
 
 	__iGrid = 0;
 	CqString _aq_basis;
-	(basis)->GetString(_aq_basis,__iGrid);
-	spline.SetBasis( _aq_basis );
-
+	(basis)->GetString( _aq_basis, __iGrid );
+	CqCubicSpline<TqFloat> spline( _aq_basis, cParams );
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -325,11 +348,11 @@ void	CqShaderExecEnv::SO_sfspline( IqShaderData* basis, IqShaderData* value, IqS
 				{
 					TqFloat fn;
 					apParams[ j ] ->GetFloat( fn, __iGrid );
-					spline[ j ] = CqVector4D( fn, 0.0f, 0.0f, 1.0f );
+					spline.pushBack( fn );
 				}
 
-				CqVector4D	res = spline.Evaluate( _aq_value );
-				(Result)->SetFloat(res.x(),__iGrid);
+				(Result)->SetFloat( spline.evaluate( _aq_value ), __iGrid );
+				spline.clear();
 			}
 		}
 	}
@@ -341,11 +364,9 @@ void	CqShaderExecEnv::SO_sfspline( IqShaderData* basis, IqShaderData* value, IqS
 // spline(value, f1,f2,...,fn)
 void	CqShaderExecEnv::SO_scspline( IqShaderData* basis, IqShaderData* value, IqShaderData* Result, IqShader* pShader, int cParams, IqShaderData** apParams )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
-	CqSplineCubic spline( cParams );
-
 	__fVarying=(value)->Class()==class_varying;
 	TqInt v;
 	for ( v = 0; v < cParams; v++ )
@@ -357,11 +378,10 @@ void	CqShaderExecEnv::SO_scspline( IqShaderData* basis, IqShaderData* value, IqS
 	__iGrid = 0;
 	CqString _aq_basis;
 	(basis)->GetString(_aq_basis,__iGrid);
-	spline.SetBasis( _aq_basis );
-
+	CqCubicSpline<CqColor> spline( _aq_basis, cParams );
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -387,11 +407,11 @@ void	CqShaderExecEnv::SO_scspline( IqShaderData* basis, IqShaderData* value, IqS
 				{
 					CqColor cn;
 					apParams[ j ] ->GetColor( cn, __iGrid );
-					spline[ j ] = CqVector4D( cn.fRed(), cn.fGreen(), cn.fBlue(), 1.0f );
+					spline.pushBack( cn );
 				}
 
-				CqVector4D	res = spline.Evaluate( _aq_value );
-				(Result)->SetColor(CqColor( res.x(), res.y(), res.z() ),__iGrid);
+				(Result)->SetColor( spline.evaluate( _aq_value ), __iGrid );
+				spline.clear();
 			}
 		}
 	}
@@ -403,11 +423,9 @@ void	CqShaderExecEnv::SO_scspline( IqShaderData* basis, IqShaderData* value, IqS
 // spline(value, f1,f2,...,fn)
 void	CqShaderExecEnv::SO_spspline( IqShaderData* basis, IqShaderData* value, IqShaderData* Result, IqShader* pShader, int cParams, IqShaderData** apParams )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
-	CqSplineCubic spline( cParams );
-
 	__fVarying=(value)->Class()==class_varying;
 	TqInt v;
 	for ( v = 0; v < cParams; v++ )
@@ -419,11 +437,10 @@ void	CqShaderExecEnv::SO_spspline( IqShaderData* basis, IqShaderData* value, IqS
 	__iGrid = 0;
 	CqString _aq_basis;
 	(basis)->GetString(_aq_basis,__iGrid);
-	spline.SetBasis( _aq_basis );
-
+	CqCubicSpline<CqVector3D> spline( _aq_basis, cParams );
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -449,11 +466,11 @@ void	CqShaderExecEnv::SO_spspline( IqShaderData* basis, IqShaderData* value, IqS
 				{
 					CqVector3D pn;
 					apParams[ j ] ->GetPoint( pn, __iGrid );
-					spline[ j ] = pn;
+					spline.pushBack( pn );
 				}
 
-				CqVector3D	res = spline.Evaluate( _aq_value );
-				(Result)->SetPoint(res,__iGrid);
+				(Result)->SetPoint( spline.evaluate( _aq_value ), __iGrid );
+				spline.clear();
 			}
 		}
 	}
@@ -463,20 +480,19 @@ void	CqShaderExecEnv::SO_spspline( IqShaderData* basis, IqShaderData* value, IqS
 
 void	CqShaderExecEnv::SO_fDu( IqShaderData* p, IqShaderData* Result, IqShader* pShader )
 {
-	TqFloat Deffloat = 0.0f;
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__fVarying=(p)->Class()==class_varying;
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
 		{
-			(Result)->SetFloat(SO_DuType<TqFloat>( p, __iGrid, this, Deffloat ),__iGrid);
+			Result->SetFloat(derivU<TqFloat>(p, __iGrid), __iGrid);
 		}
 	}
 	while( ( ++__iGrid < shadingPointCount() ) && __fVarying);
@@ -485,20 +501,19 @@ void	CqShaderExecEnv::SO_fDu( IqShaderData* p, IqShaderData* Result, IqShader* p
 
 void	CqShaderExecEnv::SO_fDv( IqShaderData* p, IqShaderData* Result, IqShader* pShader )
 {
-	TqFloat Deffloat = 0.0f;
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__fVarying=(p)->Class()==class_varying;
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
 		{
-			(Result)->SetFloat(SO_DvType<TqFloat>( p, __iGrid, this, Deffloat ),__iGrid);
+			(Result)->SetFloat(derivV<TqFloat>(p, __iGrid), __iGrid);
 		}
 	}
 	while( ( ++__iGrid < shadingPointCount() ) && __fVarying);
@@ -507,7 +522,7 @@ void	CqShaderExecEnv::SO_fDv( IqShaderData* p, IqShaderData* Result, IqShader* p
 
 void	CqShaderExecEnv::SO_fDeriv( IqShaderData* p, IqShaderData* den, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__fVarying=(p)->Class()==class_varying;
@@ -515,12 +530,12 @@ void	CqShaderExecEnv::SO_fDeriv( IqShaderData* p, IqShaderData* den, IqShaderDat
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
 		{
-			(Result)->SetFloat(SO_DerivType<TqFloat>( p, den, __iGrid, this ),__iGrid);
+			(Result)->SetFloat(deriv<TqFloat>(p, den, __iGrid), __iGrid);
 		}
 	}
 	while( ( ++__iGrid < shadingPointCount() ) && __fVarying);
@@ -529,20 +544,19 @@ void	CqShaderExecEnv::SO_fDeriv( IqShaderData* p, IqShaderData* den, IqShaderDat
 
 void	CqShaderExecEnv::SO_cDu( IqShaderData* p, IqShaderData* Result, IqShader* pShader )
 {
-	CqColor Defcol( 0.0f, 0.0f, 0.0f );
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__fVarying=(p)->Class()==class_varying;
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
 		{
-			(Result)->SetColor(SO_DuType<CqColor>( p, __iGrid, this, Defcol ),__iGrid);
+			Result->SetColor(derivU<CqColor>(p, __iGrid), __iGrid);
 		}
 	}
 	while( ( ++__iGrid < shadingPointCount() ) && __fVarying);
@@ -551,20 +565,19 @@ void	CqShaderExecEnv::SO_cDu( IqShaderData* p, IqShaderData* Result, IqShader* p
 
 void	CqShaderExecEnv::SO_cDv( IqShaderData* p, IqShaderData* Result, IqShader* pShader )
 {
-	CqColor Defcol( 0.0f, 0.0f, 0.0f );
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__fVarying=(p)->Class()==class_varying;
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
 		{
-			(Result)->SetColor(SO_DvType<CqColor>( p, __iGrid, this, Defcol ),__iGrid);
+			Result->SetColor(derivV<CqColor>(p, __iGrid), __iGrid);
 		}
 	}
 	while( ( ++__iGrid < shadingPointCount() ) && __fVarying);
@@ -573,7 +586,7 @@ void	CqShaderExecEnv::SO_cDv( IqShaderData* p, IqShaderData* Result, IqShader* p
 
 void	CqShaderExecEnv::SO_cDeriv( IqShaderData* p, IqShaderData* den, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__fVarying=(p)->Class()==class_varying;
@@ -581,12 +594,12 @@ void	CqShaderExecEnv::SO_cDeriv( IqShaderData* p, IqShaderData* den, IqShaderDat
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
 		{
-			(Result)->SetColor(SO_DerivType<CqColor>( p, den, __iGrid, this ),__iGrid);
+			(Result)->SetColor(deriv<CqColor>(p, den, __iGrid), __iGrid);
 		}
 	}
 	while( ( ++__iGrid < shadingPointCount() ) && __fVarying);
@@ -595,20 +608,19 @@ void	CqShaderExecEnv::SO_cDeriv( IqShaderData* p, IqShaderData* den, IqShaderDat
 
 void	CqShaderExecEnv::SO_pDu( IqShaderData* p, IqShaderData* Result, IqShader* pShader )
 {
-	CqVector3D Defvec( 0.0f, 0.0f, 0.0f );
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__fVarying=(p)->Class()==class_varying;
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
 		{
-			(Result)->SetPoint(SO_DuType<CqVector3D>( p, __iGrid, this, Defvec ),__iGrid);
+			Result->SetPoint(derivU<CqVector3D>(p, __iGrid), __iGrid);
 		}
 	}
 	while( ( ++__iGrid < shadingPointCount() ) && __fVarying);
@@ -617,20 +629,19 @@ void	CqShaderExecEnv::SO_pDu( IqShaderData* p, IqShaderData* Result, IqShader* p
 
 void	CqShaderExecEnv::SO_pDv( IqShaderData* p, IqShaderData* Result, IqShader* pShader )
 {
-	CqVector3D Defvec( 0.0f, 0.0f, 0.0f );
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__fVarying=(p)->Class()==class_varying;
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
 		{
-			(Result)->SetPoint(SO_DvType<CqVector3D>( p, __iGrid, this, Defvec ),__iGrid);
+			Result->SetPoint(derivV<CqVector3D>(p, __iGrid),__iGrid);
 		}
 	}
 	while( ( ++__iGrid < shadingPointCount() ) && __fVarying);
@@ -639,7 +650,7 @@ void	CqShaderExecEnv::SO_pDv( IqShaderData* p, IqShaderData* Result, IqShader* p
 
 void	CqShaderExecEnv::SO_pDeriv( IqShaderData* p, IqShaderData* den, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__fVarying=(p)->Class()==class_varying;
@@ -647,12 +658,12 @@ void	CqShaderExecEnv::SO_pDeriv( IqShaderData* p, IqShaderData* den, IqShaderDat
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
 		{
-			(Result)->SetPoint(SO_DerivType<CqVector3D>( p, den, __iGrid, this ),__iGrid);
+			(Result)->SetPoint(deriv<CqVector3D>(p, den, __iGrid), __iGrid);
 		}
 	}
 	while( ( ++__iGrid < shadingPointCount() ) && __fVarying);
@@ -662,31 +673,20 @@ void	CqShaderExecEnv::SO_pDeriv( IqShaderData* p, IqShaderData* den, IqShaderDat
 // area(P)
 void CqShaderExecEnv::SO_area( IqShaderData* p, IqShaderData* Result, IqShader* pShader )
 {
-	CqVector3D Defvec( 0.0f, 0.0f, 0.0f );
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
-	CqVector3D	vecR;
-
 	__fVarying=(p)->Class()==class_varying;
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
 		{
-			if ( m_pAttributes )
-			{
-				TqFloat fdu, fdv;
-				du() ->GetFloat( fdu, __iGrid );
-				dv() ->GetFloat( fdv, __iGrid );
-				vecR = ( SO_DuType<CqVector3D>( p, __iGrid, this, Defvec ) * fdu ) %
-				       ( SO_DvType<CqVector3D>( p, __iGrid, this, Defvec ) * fdv );
-				(Result)->SetFloat(vecR.Magnitude(),__iGrid);
-			}
-
+			(Result)->SetFloat( (diffU<CqVector3D>(p, __iGrid)
+					% diffV<CqVector3D>(p, __iGrid)).Magnitude() ,__iGrid);
 		}
 	}
 	while( ( ++__iGrid < shadingPointCount() ) && __fVarying);
@@ -695,14 +695,14 @@ void CqShaderExecEnv::SO_area( IqShaderData* p, IqShaderData* Result, IqShader*
 
 void	CqShaderExecEnv::SO_normalize( IqShaderData* V, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__fVarying=(V)->Class()==class_varying;
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	CqVector3D _old(1,0,0);
 	CqVector3D _unit(1,0,0);
 	do
@@ -731,7 +731,7 @@ void	CqShaderExecEnv::SO_normalize( IqShaderData* V, IqShaderData* Result, IqSha
 // faceforward(N,I)
 void CqShaderExecEnv::SO_faceforward( IqShaderData* N, IqShaderData* I, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__fVarying=(N)->Class()==class_varying;
@@ -739,7 +739,7 @@ void CqShaderExecEnv::SO_faceforward( IqShaderData* N, IqShaderData* I, IqShader
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -751,7 +751,8 @@ void CqShaderExecEnv::SO_faceforward( IqShaderData* N, IqShaderData* I, IqShader
 			CqVector3D Nref;
 			Ng() ->GetNormal( Nref, __iGrid );
 			TqFloat s = ( ( ( -_aq_I ) * Nref ) < 0.0f ) ? -1.0f : 1.0f;
-			(Result)->SetNormal(_aq_N * s,__iGrid);
+			TqFloat s2 = ( ( _aq_N * Nref ) < 0.0f ) ? -1.0f : 1.0f;
+			(Result)->SetNormal(s * s2 * _aq_N,__iGrid);
 		}
 	}
 	while( ( ++__iGrid < shadingPointCount() ) && __fVarying);
@@ -762,7 +763,7 @@ void CqShaderExecEnv::SO_faceforward( IqShaderData* N, IqShaderData* I, IqShader
 // faceforward(N,I,Nref)
 void CqShaderExecEnv::SO_faceforward2( IqShaderData* N, IqShaderData* I, IqShaderData* Nref, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__fVarying=(N)->Class()==class_varying;
@@ -771,7 +772,7 @@ void CqShaderExecEnv::SO_faceforward2( IqShaderData* N, IqShaderData* I, IqShade
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -794,15 +795,15 @@ void CqShaderExecEnv::SO_faceforward2( IqShaderData* N, IqShaderData* I, IqShade
 // spline(value, f1,f2,...,fn)
 void    CqShaderExecEnv::SO_fsplinea( IqShaderData* value, IqShaderData* a, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
-	TqBool __fVaryingA=TqFalse;
+	bool __fVarying;
+	bool __fVaryingA=false;
 	TqUint __iGrid;
 
 	assert( a->ArrayLength() > 0 );
 	assert( a->Type() == type_float );
-
+	
 	TqInt    cParams = a->ArrayLength();
-	CqSplineCubic spline( cParams );
+	CqCubicSpline<TqFloat> spline( SplineBasis_CatmullRom, cParams );
 
 	__fVarying=(value)->Class()==class_varying;
 	__fVaryingA=(a)->Class()==class_varying;
@@ -811,7 +812,6 @@ void    CqShaderExecEnv::SO_fsplinea( IqShaderData* value, IqShaderData* a, IqSh
 
 	__iGrid = 0;
 
-
 	if (!__fVaryingA)
 	{
 		TqInt j;
@@ -819,11 +819,11 @@ void    CqShaderExecEnv::SO_fsplinea( IqShaderData* value, IqShaderData* a, IqSh
 		for ( j = 0; j < cParams; j++ )
 		{
 			a->ArrayEntry( j ) ->GetFloat( fTemp, __iGrid );
-			spline[ j ] = CqVector4D( fTemp, 0.0f, 0.0f, 1.0f );
+			spline.pushBack( fTemp );
 		}
 	}
 
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 
 	do
 	{
@@ -845,15 +845,17 @@ void    CqShaderExecEnv::SO_fsplinea( IqShaderData* value, IqShaderData* a, IqSh
 			}
 			else
 			{
-				TqInt j;
-				for ( j = 0; __fVaryingA && j < cParams; j++ )
+				if(__fVaryingA)
 				{
-					a->ArrayEntry( j ) ->GetFloat( fTemp, __iGrid );
-					spline[ j ] = CqVector4D( fTemp, 0.0f, 0.0f, 1.0f );
+					spline.clear();
+					for(TqInt j = 0; j < cParams; j++)
+					{
+						a->ArrayEntry( j ) ->GetFloat( fTemp, __iGrid );
+						spline.pushBack( fTemp);
+					}
 				}
 
-				CqVector4D    res = spline.Evaluate( _aq_value );
-				(Result)->SetFloat(res.x(),__iGrid);
+				(Result)->SetFloat( spline.evaluate( _aq_value ), __iGrid );
 			}
 		}
 	}
@@ -865,15 +867,15 @@ void    CqShaderExecEnv::SO_fsplinea( IqShaderData* value, IqShaderData* a, IqSh
 // spline(value, f1,f2,...,fn)
 void    CqShaderExecEnv::SO_csplinea( IqShaderData* value, IqShaderData* a, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
-	TqBool __fVaryingA=TqFalse;
+	bool __fVarying;
+	bool __fVaryingA=false;
 	TqUint __iGrid;
 
 	assert( a->ArrayLength() > 0 );
 	assert( a->Type() == type_color );
-
+	
 	TqInt	cParams = a->ArrayLength();
-	CqSplineCubic spline( cParams );
+	CqCubicSpline<CqColor> spline( SplineBasis_CatmullRom, cParams );
 	CqColor colTemp;
 
 	__fVarying=(value)->Class()==class_varying;
@@ -889,11 +891,11 @@ void    CqShaderExecEnv::SO_csplinea( IqShaderData* value, IqShaderData* a, IqSh
 		for ( j = 0; j < cParams; j++ )
 		{
 			a->ArrayEntry( j ) ->GetColor( colTemp, __iGrid );
-			spline[ j ] = CqVector4D( colTemp.fRed(), colTemp.fGreen(), colTemp.fBlue(), 1.0f );
+			spline.pushBack( colTemp );
 		}
 	}
 
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 
 	do
 	{
@@ -915,41 +917,44 @@ void    CqShaderExecEnv::SO_csplinea( IqShaderData* value, IqShaderData* a, IqSh
 			}
 			else
 			{
-				TqInt j;
-				for ( j = 0; __fVaryingA && j < cParams; j++ )
+				if(__fVaryingA)
 				{
-					a->ArrayEntry( j ) ->GetColor( colTemp, __iGrid );
-					spline[ j ] = CqVector4D( colTemp.fRed(), colTemp.fGreen(), colTemp.fBlue(), 1.0f );
+					spline.clear();
+					for(TqInt j = 0; j < cParams; j++)
+					{
+						a->ArrayEntry( j ) ->GetColor( colTemp, __iGrid );
+						spline.pushBack( colTemp );
+					}
 				}
 
-				CqVector4D	res = spline.Evaluate( _aq_value );
-				(Result)->SetColor(CqColor( res.x(), res.y(), res.z() ),__iGrid);
+				(Result)->SetColor( spline.evaluate( _aq_value ), __iGrid );
 			}
 		}
 	}
 	while( ( ++__iGrid < shadingPointCount() ) && __fVarying);
 }
 
+
 //----------------------------------------------------------------------
 // spline(value, f1,f2,...,fn)
 void    CqShaderExecEnv::SO_psplinea( IqShaderData* value, IqShaderData* a, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
-	TqBool __fVaryingA=TqFalse;
+	bool __fVarying;
+	bool __fVaryingA=false;
 	TqUint __iGrid;
 
 	assert( a->ArrayLength() > 0 );
 	assert( a->Type() == type_point );
-
+	
 	TqInt    cParams = a->ArrayLength();
-	CqSplineCubic spline( cParams );
+	CqCubicSpline<CqVector3D> spline( SplineBasis_CatmullRom, cParams );
 	CqVector3D vecTemp;
 
 	__fVarying=(value)->Class()==class_varying;
 	__fVaryingA=(a)->Class()==class_varying;
 	__fVarying = __fVaryingA || __fVarying;
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
-
+ 
 	__iGrid = 0;
 
 	if (!__fVaryingA)
@@ -958,11 +963,11 @@ void    CqShaderExecEnv::SO_psplinea( IqShaderData* value, IqShaderData* a, IqSh
 		for ( j = 0; j < cParams; j++ )
 		{
 			a->ArrayEntry( j ) ->GetPoint( vecTemp, __iGrid );
-			spline[ j ] = vecTemp;
+			spline.pushBack( vecTemp );
 		}
 	}
 
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 
 	do
 	{
@@ -984,15 +989,17 @@ void    CqShaderExecEnv::SO_psplinea( IqShaderData* value, IqShaderData* a, IqSh
 			}
 			else
 			{
-				TqInt j;
-				for ( j = 0; __fVaryingA && j < cParams; j++ )
+				if(__fVaryingA)
 				{
-					a->ArrayEntry( j ) ->GetPoint( vecTemp, __iGrid );
-					spline[ j ] = vecTemp;
+					spline.clear();
+					for(TqInt j = 0; j < cParams; j++)
+					{
+						a->ArrayEntry( j ) ->GetPoint( vecTemp, __iGrid );
+						spline.pushBack( vecTemp );
+					}
 				}
 
-				CqVector3D    res = spline.Evaluate( _aq_value );
-				(Result)->SetPoint(res,__iGrid);
+				(Result)->SetPoint( spline.evaluate( _aq_value ), __iGrid );
 			}
 		}
 	}
@@ -1004,15 +1011,14 @@ void    CqShaderExecEnv::SO_psplinea( IqShaderData* value, IqShaderData* a, IqSh
 // spline(value, f1,f2,...,fn)
 void    CqShaderExecEnv::SO_sfsplinea( IqShaderData* basis, IqShaderData* value, IqShaderData* a, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
-	TqBool __fVaryingA=TqFalse;
+	bool __fVarying;
+	bool __fVaryingA=false;
 	TqUint __iGrid;
 
 	assert( a->ArrayLength() > 0 );
 	assert( a->Type() == type_float );
 
 	TqInt    cParams = a->ArrayLength();
-	CqSplineCubic spline( cParams );
 
 	__fVarying=(value)->Class()==class_varying;
 	__fVaryingA=(a)->Class()==class_varying;
@@ -1020,10 +1026,10 @@ void    CqShaderExecEnv::SO_sfsplinea( IqShaderData* basis, IqShaderData* value,
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
+	
 	CqString _aq_basis;
-	(basis)->GetString(_aq_basis,__iGrid);
-	spline.SetBasis( _aq_basis );
-
+	(basis)->GetString( _aq_basis, __iGrid );
+	CqCubicSpline<TqFloat> spline( _aq_basis, cParams );
 
 	__iGrid = 0;
 
@@ -1034,11 +1040,11 @@ void    CqShaderExecEnv::SO_sfsplinea( IqShaderData* basis, IqShaderData* value,
 		for ( j = 0; j < cParams; j++ )
 		{
 			a->ArrayEntry( j ) ->GetFloat( fTemp, __iGrid );
-			spline[ j ] = CqVector4D( fTemp, 0.0f, 0.0f, 1.0f );
+			spline.pushBack( fTemp );
 		}
 	}
 
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 
 	do
 	{
@@ -1060,15 +1066,17 @@ void    CqShaderExecEnv::SO_sfsplinea( IqShaderData* basis, IqShaderData* value,
 			}
 			else
 			{
-				TqInt j;
-				for ( j = 0; __fVaryingA && j < cParams; j++ )
+				if(__fVaryingA)
 				{
-					a->ArrayEntry( j ) ->GetFloat( fTemp, __iGrid );
-					spline[ j ] = CqVector4D( fTemp, 0.0f, 0.0f, 1.0f );
+					spline.clear();
+					for(TqInt j = 0; j < cParams; j++)
+					{
+						a->ArrayEntry( j ) ->GetFloat( fTemp, __iGrid );
+						spline.pushBack( fTemp );
+					}
 				}
 
-				CqVector4D    res = spline.Evaluate( _aq_value );
-				(Result)->SetFloat(res.x(),__iGrid);
+				(Result)->SetFloat( spline.evaluate( _aq_value ), __iGrid );
 			}
 		}
 	}
@@ -1080,15 +1088,14 @@ void    CqShaderExecEnv::SO_sfsplinea( IqShaderData* basis, IqShaderData* value,
 // spline(value, f1,f2,...,fn)
 void    CqShaderExecEnv::SO_scsplinea( IqShaderData* basis, IqShaderData* value, IqShaderData* a, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
-	TqBool __fVaryingA=TqFalse;
+	bool __fVarying;
+	bool __fVaryingA=false;
 	TqUint __iGrid;
 
 	assert( a->ArrayLength() > 0 );
 	assert( a->Type() == type_color );
 
 	TqInt    cParams = a->ArrayLength();
-	CqSplineCubic spline( cParams );
 	CqColor colTemp;
 
 	__fVarying=(value)->Class()==class_varying;
@@ -1099,8 +1106,7 @@ void    CqShaderExecEnv::SO_scsplinea( IqShaderData* basis, IqShaderData* value,
 	__iGrid = 0;
 	CqString _aq_basis;
 	(basis)->GetString(_aq_basis,__iGrid);
-	spline.SetBasis( _aq_basis );
-
+	CqCubicSpline<CqColor> spline( _aq_basis, cParams );
 
 	__iGrid = 0;
 
@@ -1110,11 +1116,11 @@ void    CqShaderExecEnv::SO_scsplinea( IqShaderData* basis, IqShaderData* value,
 		for ( j = 0; j < cParams; j++ )
 		{
 			a->ArrayEntry( j ) ->GetColor( colTemp, __iGrid );
-			spline[ j ] = CqVector4D( colTemp.fRed(), colTemp.fGreen(), colTemp.fBlue(), 1.0f );
+			spline.pushBack( colTemp );
 		}
 	}
 
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 
 	do
 	{
@@ -1136,15 +1142,17 @@ void    CqShaderExecEnv::SO_scsplinea( IqShaderData* basis, IqShaderData* value,
 			}
 			else
 			{
-				TqInt j;
-				for ( j = 0; __fVaryingA && j < cParams; j++ )
+				if(__fVaryingA)
 				{
-					a->ArrayEntry( j ) ->GetColor( colTemp, __iGrid );
-					spline[ j ] = CqVector4D( colTemp.fRed(), colTemp.fGreen(), colTemp.fBlue(), 1.0f );
+					spline.clear();
+					for (TqInt j = 0; j < cParams; j++ )
+					{
+						a->ArrayEntry( j ) ->GetColor( colTemp, __iGrid );
+						spline.pushBack( colTemp );
+					}
 				}
 
-				CqVector4D    res = spline.Evaluate( _aq_value );
-				(Result)->SetColor(CqColor( res.x(), res.y(), res.z() ),__iGrid);
+				(Result)->SetColor( spline.evaluate( _aq_value ), __iGrid );
 			}
 		}
 	}
@@ -1156,15 +1164,14 @@ void    CqShaderExecEnv::SO_scsplinea( IqShaderData* basis, IqShaderData* value,
 // spline(value, f1,f2,...,fn)
 void    CqShaderExecEnv::SO_spsplinea( IqShaderData* basis, IqShaderData* value, IqShaderData* a, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
-	TqBool __fVaryingA=TqFalse;
+	bool __fVarying;
+	bool __fVaryingA=false;
 	TqUint __iGrid;
 
 	assert( a->ArrayLength() > 0 );
 	assert( a->Type() == type_point );
 
 	TqInt    cParams = a->ArrayLength();
-	CqSplineCubic spline( cParams );
 	CqVector3D vecTemp;
 
 	__fVarying=(value)->Class()==class_varying;
@@ -1175,22 +1182,20 @@ void    CqShaderExecEnv::SO_spsplinea( IqShaderData* basis, IqShaderData* value,
 	__iGrid = 0;
 	CqString _aq_basis;
 	(basis)->GetString(_aq_basis,__iGrid);
-	spline.SetBasis( _aq_basis );
-
+	CqCubicSpline<CqVector3D> spline( _aq_basis, cParams );
 
 	__iGrid = 0;
-
 	if (!__fVaryingA)
 	{
 		TqInt j;
 		for ( j = 0; j < cParams; j++ )
 		{
 			a->ArrayEntry( j ) ->GetPoint( vecTemp, __iGrid );
-			spline[ j ] = vecTemp;
+			spline.pushBack( vecTemp );
 		}
 	}
 
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 
 	do
 	{
@@ -1212,15 +1217,17 @@ void    CqShaderExecEnv::SO_spsplinea( IqShaderData* basis, IqShaderData* value,
 			}
 			else
 			{
-				TqInt j;
-				for ( j = 0; __fVaryingA && j < cParams; j++ )
+				if(__fVaryingA)
 				{
-					a->ArrayEntry( j ) ->GetPoint( vecTemp, __iGrid );
-					spline[ j ] = vecTemp;
+					spline.clear();
+					for(TqInt j = 0; j < cParams; j++)
+					{
+						a->ArrayEntry( j ) ->GetPoint( vecTemp, __iGrid );
+						spline.pushBack( vecTemp );
+					}
 				}
 
-				CqVector3D    res = spline.Evaluate( _aq_value );
-				(Result)->SetPoint(res,__iGrid);
+				(Result)->SetPoint( spline.evaluate( _aq_value ), __iGrid );
 			}
 		}
 	}
@@ -1231,8 +1238,7 @@ void    CqShaderExecEnv::SO_spsplinea( IqShaderData* basis, IqShaderData* value,
 // filterstep(edge,s1)
 void CqShaderExecEnv::SO_filterstep( IqShaderData* edge, IqShaderData* s1, IqShaderData* Result, IqShader* pShader, int cParams, IqShaderData** apParams )
 {
-	TqFloat Deffloat = 0.0f;
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	float _pswidth=1.0f,_ptwidth=1.0f;
@@ -1243,13 +1249,7 @@ void CqShaderExecEnv::SO_filterstep( IqShaderData* edge, IqShaderData* s1, IqSha
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	TqFloat fdu, fdv;
-	du() ->GetFloat( fdu );
-	dv() ->GetFloat( fdv );
-
-
-	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -1258,24 +1258,14 @@ void CqShaderExecEnv::SO_filterstep( IqShaderData* edge, IqShaderData* s1, IqSha
 			(s1)->GetFloat(_aq_s1,__iGrid);
 			TqFloat _aq_edge;
 			(edge)->GetFloat(_aq_edge,__iGrid);
-			TqFloat dsdu = SO_DuType<TqFloat>( s1, __iGrid, this, Deffloat );
-			TqFloat dsdv = SO_DvType<TqFloat>( s1, __iGrid, this, Deffloat );
 
-			TqFloat uwidth = fabs( dsdu * fdu );
-			TqFloat vwidth = fabs( dsdv * fdv );
+			TqFloat uwidth = fabs( diffU<TqFloat>(s1, __iGrid) );
+			TqFloat vwidth = fabs( diffV<TqFloat>(s1, __iGrid) );
 
 			TqFloat w = uwidth + vwidth;
 			w *= _pswidth;
 
-			(Result)->SetFloat(CLAMP( ( _aq_s1 + w / 2.0f - _aq_edge ) / w, 0, 1 ),__iGrid);
-
-			//	TqFloat res  = RiCatmullRomFilter( FLOAT( s1 ) - FLOAT( edge ), 0, w, 0);
-			//	SETFLOAT( Result, res );
-
-			//	std::cout << res << std::endl;
-			//	TqFloat res = 1.0f - CLAMP( ( FLOAT( s1 ) + w / 2.0f - FLOAT( edge ) ) / w, 0, 1 );
-			//	if( res > 0.0f )
-			//		std::cout << "Aqsis angle/dangle: " << FLOAT(s1) << ", edge: " << FLOAT(edge) << ", dsdu: " << dsdu << ", dsdv: " << dsdv << ", w: " << w << ", res: " << res << std::endl;
+			(Result)->SetFloat(clamp(( _aq_s1 + w / 2.0f - _aq_edge ) / w, 0.0f, 1.0f), __iGrid);
 		}
 	}
 	while( ( ++__iGrid < shadingPointCount() ) && __fVarying);
@@ -1288,7 +1278,7 @@ void CqShaderExecEnv::SO_filterstep2( IqShaderData* edge, IqShaderData* s1, IqSh
 	float _pswidth=1.0f,_ptwidth=1.0f;
 	GetFilterParams(cParams, apParams, _pswidth,_ptwidth);
 
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__fVarying=(edge)->Class()==class_varying;
@@ -1297,7 +1287,7 @@ void CqShaderExecEnv::SO_filterstep2( IqShaderData* edge, IqShaderData* s1, IqSh
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -1310,10 +1300,13 @@ void CqShaderExecEnv::SO_filterstep2( IqShaderData* edge, IqShaderData* s1, IqSh
 			(s2)->GetFloat(_aq_s2,__iGrid);
 			TqFloat w = _aq_s2 - _aq_s1;
 			w *= _pswidth;
-			(Result)->SetFloat(CLAMP( ( _aq_s1 + w / 2.0f - _aq_edge ) / w, 0, 1 ),__iGrid);
+			(Result)->SetFloat(clamp( (_aq_s1 + w/2.0f - _aq_edge)/w, 0.0f, 1.0f), __iGrid);
 		}
 	}
 	while( ( ++__iGrid < shadingPointCount() ) && __fVarying);
 }
-END_NAMESPACE(    Aqsis )
+
+
+} // namespace Aqsis
+
 //---------------------------------------------------------------------
