@@ -1,5 +1,5 @@
 // Aqsis
-// Copyright  1997 - 2001, Paul C. Gregory
+// Copyright (C) 1997 - 2001, Paul C. Gregory
 //
 // Contact: pgregory@aqsis.org
 //
@@ -25,12 +25,12 @@
 
 #include	"aqsis.h"
 
-#include	<math.h>
 #include	<map>
 #include	<vector>
 #include	<string>
 #include	<stdio.h>
 
+#include	"aqsismath.h"
 #include	"shaderexecenv.h"
 #include	"shadervm.h"
 #include	"irenderer.h"
@@ -38,21 +38,21 @@
 #include	"version.h"
 #include	"logging.h"
 
-START_NAMESPACE(    Aqsis )
+namespace Aqsis {
 
 //----------------------------------------------------------------------
 // init_illuminance()
 // NOTE: There is duplication here between SO_init_illuminance and 
 // SO_advance_illuminance. This is required to ensure that the 
 // first light is not skipped.
-TqBool CqShaderExecEnv::SO_init_illuminance()
+bool CqShaderExecEnv::SO_init_illuminance()
 {
 	// Check if lighting is turned off.
 	if(getRenderContext())
 	{
 		const TqInt* enableLightingOpt = getRenderContext()->GetIntegerOption("EnableShaders", "lighting");
 		if(NULL != enableLightingOpt && enableLightingOpt[0] == 0)
-			return(TqFalse);
+			return(false);
 	}
 
 	m_li = 0;
@@ -62,22 +62,22 @@ TqBool CqShaderExecEnv::SO_init_illuminance()
 		m_li++;
 	}
 	if ( m_li < m_pAttributes ->cLights() )
-		return ( TqTrue );
+		return ( true );
 	else
-		return ( TqFalse );
+		return ( false );
 }
 
 
 //----------------------------------------------------------------------
 // advance_illuminance()
-TqBool CqShaderExecEnv::SO_advance_illuminance()
+bool CqShaderExecEnv::SO_advance_illuminance()
 {
 	// Check if lighting is turned off, should never need this check as SO_init_illuminance will catch first.
 	if(getRenderContext())
 	{
 		const TqInt* enableLightingOpt = getRenderContext()->GetIntegerOption("EnableShaders", "lighting");
 		if(NULL != enableLightingOpt && enableLightingOpt[0] == 0)
-			return(TqFalse);
+			return(false);
 	}
 
 	m_li++;
@@ -87,9 +87,49 @@ TqBool CqShaderExecEnv::SO_advance_illuminance()
 		m_li++;
 	}
 	if ( m_li < m_pAttributes ->cLights() )
-		return ( TqTrue );
+		return ( true );
 	else
-		return ( TqFalse );
+		return ( false );
+}
+
+
+//----------------------------------------------------------------------
+// init_gather()
+void CqShaderExecEnv::SO_init_gather(IqShaderData* samples, IqShader* pShader)
+{
+	bool __fVarying;
+	TqUint __iGrid = 0;
+
+	__fVarying=(samples)->Class()==class_varying;
+
+	TqFloat _aq_samples;
+	(samples)->GetFloat(_aq_samples,__iGrid);
+
+	// Check if lighting is turned off.
+	if(getRenderContext())
+	{
+		const TqInt* enableLightingOpt = getRenderContext()->GetIntegerOption("EnableShaders", "lighting");
+		if(NULL != enableLightingOpt && enableLightingOpt[0] == 0)
+			return;
+	}
+
+	m_gatherSample = static_cast<TqUint> (_aq_samples);
+}
+
+
+//----------------------------------------------------------------------
+// advance_illuminance()
+bool CqShaderExecEnv::SO_advance_gather()
+{
+	// Check if lighting is turned off, should never need this check as SO_init_illuminance will catch first.
+	if(getRenderContext())
+	{
+		const TqInt* enableLightingOpt = getRenderContext()->GetIntegerOption("EnableShaders", "lighting");
+		if(NULL != enableLightingOpt && enableLightingOpt[0] == 0)
+			return(false);
+	}
+
+	return((--m_gatherSample) > 0);
 }
 
 
@@ -104,7 +144,7 @@ void CqShaderExecEnv::ValidateIlluminanceCache( IqShaderData* pP, IqShaderData*
 			const TqInt* enableLightingOpt = getRenderContext()->GetIntegerOption("EnableShaders", "lighting");
 			if(NULL != enableLightingOpt && enableLightingOpt[0] == 0)
 			{
-				m_IlluminanceCacheValid = TqTrue;
+				m_IlluminanceCacheValid = true;
 				return;
 			}
 		}
@@ -116,13 +156,13 @@ void CqShaderExecEnv::ValidateIlluminanceCache( IqShaderData* pP, IqShaderData*
 		{
 			IqLightsource * lp = m_pAttributes ->pLight( li );
 			// Initialise the lightsource
-			lp->Initialise( uGridRes(), vGridRes(), microPolygonCount(), shadingPointCount() );
+			lp->Initialise( uGridRes(), vGridRes(), microPolygonCount(), shadingPointCount(), m_hasValidDerivatives );
 			m_Illuminate = 0;
 			// Evaluate the lightsource
 			lp->Evaluate( Ps, Ns, m_pCurrentSurface );
 			li++;
 		}
-		m_IlluminanceCacheValid = TqTrue;
+		m_IlluminanceCacheValid = true;
 	}
 }
 
@@ -130,7 +170,7 @@ void CqShaderExecEnv::ValidateIlluminanceCache( IqShaderData* pP, IqShaderData*
 // reflect(I,N)
 void CqShaderExecEnv::SO_reflect( IqShaderData* I, IqShaderData* N, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__fVarying=(I)->Class()==class_varying;
@@ -138,7 +178,7 @@ void CqShaderExecEnv::SO_reflect( IqShaderData* I, IqShaderData* N, IqShaderData
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -160,7 +200,7 @@ void CqShaderExecEnv::SO_reflect( IqShaderData* I, IqShaderData* N, IqShaderData
 // reftact(I,N,eta)
 void CqShaderExecEnv::SO_refract( IqShaderData* I, IqShaderData* N, IqShaderData* eta, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__fVarying=(I)->Class()==class_varying;
@@ -169,7 +209,7 @@ void CqShaderExecEnv::SO_refract( IqShaderData* I, IqShaderData* N, IqShaderData
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -195,7 +235,7 @@ void CqShaderExecEnv::SO_refract( IqShaderData* I, IqShaderData* N, IqShaderData
 
 void CqShaderExecEnv::SO_fresnel( IqShaderData* I, IqShaderData* N, IqShaderData* eta, IqShaderData* Kr, IqShaderData* Kt, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__fVarying=(I)->Class()==class_varying;
@@ -205,7 +245,7 @@ void CqShaderExecEnv::SO_fresnel( IqShaderData* I, IqShaderData* N, IqShaderData
 	__fVarying=(Kt)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -244,7 +284,7 @@ void CqShaderExecEnv::SO_fresnel( IqShaderData* I, IqShaderData* N, IqShaderData
 // fresnel(I,N,eta,Kr,Kt,R,T)
 void CqShaderExecEnv::SO_fresnel( IqShaderData* I, IqShaderData* N, IqShaderData* eta, IqShaderData* Kr, IqShaderData* Kt, IqShaderData* R, IqShaderData* T, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__fVarying=(I)->Class()==class_varying;
@@ -256,7 +296,7 @@ void CqShaderExecEnv::SO_fresnel( IqShaderData* I, IqShaderData* N, IqShaderData
 	__fVarying=(T)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -302,7 +342,7 @@ void CqShaderExecEnv::SO_fresnel( IqShaderData* I, IqShaderData* N, IqShaderData
 // depth(P)
 void CqShaderExecEnv::SO_depth( IqShaderData* p, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	if (!getRenderContext() )
@@ -312,7 +352,7 @@ void CqShaderExecEnv::SO_depth( IqShaderData* p, IqShaderData* Result, IqShader*
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -334,7 +374,7 @@ void CqShaderExecEnv::SO_depth( IqShaderData* p, IqShaderData* Result, IqShader*
 
 void CqShaderExecEnv::SO_ambient( IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	// Check if lighting is turned off.
@@ -358,13 +398,13 @@ void CqShaderExecEnv::SO_ambient( IqShaderData* Result, IqShader* pShader )
 
 		for ( TqUint light_index = 0; light_index < m_pAttributes ->cLights(); light_index++ )
 		{
-			__fVarying = TqTrue;
+			__fVarying = true;
 
 			IqLightsource* lp = m_pAttributes ->pLight( light_index );
 			if ( lp->pShader() ->fAmbient() )
 			{
 				__iGrid = 0;
-				CqBitVector& RS = RunningState();
+				const CqBitVector& RS = RunningState();
 				do
 				{
 					if(!__fVarying || RS.Value( __iGrid ) )
@@ -390,7 +430,7 @@ void CqShaderExecEnv::SO_ambient( IqShaderData* Result, IqShader* pShader )
 // diffuse(N)
 void CqShaderExecEnv::SO_diffuse( IqShaderData* N, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	// If the illuminance cache is already OK, then we don't need to bother filling in the illuminance parameters.
@@ -403,11 +443,11 @@ void CqShaderExecEnv::SO_diffuse( IqShaderData* N, IqShaderData* Result, IqShade
 	if ( NULL == pDefAngle )
 		return ;
 
-	pDefAngle->SetFloat( PIO2 );
+	pDefAngle->SetFloat( M_PI_2 );
 
 	Result->SetColor( gColBlack );
 
-	__fVarying = TqTrue;
+	__fVarying = true;
 	IqShaderData* __nondiffuse = NULL;
 	__nondiffuse = pShader->CreateTemporaryStorage( type_float, class_varying );
 
@@ -441,7 +481,7 @@ void CqShaderExecEnv::SO_diffuse( IqShaderData* N, IqShaderData* Result, IqShade
 			GetCurrentState();
 
 			__iGrid = 0;
-			CqBitVector& RS = RunningState();
+			const CqBitVector& RS = RunningState();
 			do
 			{
 				if(!__fVarying || RS.Value( __iGrid ) )
@@ -478,7 +518,7 @@ void CqShaderExecEnv::SO_diffuse( IqShaderData* N, IqShaderData* Result, IqShade
 // specular(N,V,roughness)
 void CqShaderExecEnv::SO_specular( IqShaderData* N, IqShaderData* V, IqShaderData* roughness, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	// If the illuminance cache is already OK, then we don't need to bother filling in the illuminance parameters.
@@ -491,10 +531,10 @@ void CqShaderExecEnv::SO_specular( IqShaderData* N, IqShaderData* V, IqShaderDat
 	if ( NULL == pDefAngle )
 		return ;
 
-	pDefAngle->SetFloat( PIO2 );
+	pDefAngle->SetFloat( M_PI_2 );
 
 	Result->SetColor( gColBlack );
-	__fVarying = TqTrue;
+	__fVarying = true;
 
 	IqShaderData* __nonspecular = NULL;
 	__nonspecular = pShader->CreateTemporaryStorage( type_float, class_varying );
@@ -528,7 +568,7 @@ void CqShaderExecEnv::SO_specular( IqShaderData* N, IqShaderData* V, IqShaderDat
 			PushState();
 			GetCurrentState();
 			__iGrid = 0;
-			CqBitVector& RS = RunningState();
+			const CqBitVector& RS = RunningState();
 			do
 			{
 				if(!__fVarying || RS.Value( __iGrid ) )
@@ -553,7 +593,7 @@ void CqShaderExecEnv::SO_specular( IqShaderData* N, IqShaderData* V, IqShaderDat
 					(roughness)->GetFloat(_aq_roughness,__iGrid);
 					CqColor colCl;
 					Cl() ->GetColor( colCl, __iGrid );
-					(Result)->SetColor(_aq_Result + colCl * pow( MAX( 0.0f, _aq_N * H ), 1.0f / ( _aq_roughness / 8.0f ) ),__iGrid);
+					(Result)->SetColor(_aq_Result + colCl * pow( max( 0.0f, _aq_N * H ), 1.0f / ( _aq_roughness / 8.0f ) ),__iGrid);
 
 				}
 			}
@@ -572,7 +612,7 @@ void CqShaderExecEnv::SO_specular( IqShaderData* N, IqShaderData* V, IqShaderDat
 // phong(N,V,size)
 void CqShaderExecEnv::SO_phong( IqShaderData* N, IqShaderData* V, IqShaderData* size, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	IqShaderData * pnV = pShader ->CreateTemporaryStorage( type_vector, class_varying );
@@ -588,9 +628,9 @@ void CqShaderExecEnv::SO_phong( IqShaderData* N, IqShaderData* V, IqShaderData*
 	SO_normalize( V, pnV );
 	SO_normalize( N, pnN );
 
-	__fVarying = TqTrue;
+	__fVarying = true;
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -617,7 +657,7 @@ void CqShaderExecEnv::SO_phong( IqShaderData* N, IqShaderData* V, IqShaderData*
 	if ( NULL == pDefAngle )
 		return ;
 
-	pDefAngle->SetFloat( PIO2 );
+	pDefAngle->SetFloat( M_PI_2 );
 
 	// Initialise the return value
 	Result->SetColor( gColBlack );
@@ -634,7 +674,7 @@ void CqShaderExecEnv::SO_phong( IqShaderData* N, IqShaderData* V, IqShaderData*
 			GetCurrentState();
 
 			__iGrid = 0;
-			CqBitVector& RS = RunningState();
+			const CqBitVector& RS = RunningState();
 			do
 			{
 				if(!__fVarying || RS.Value( __iGrid ) )
@@ -654,7 +694,7 @@ void CqShaderExecEnv::SO_phong( IqShaderData* N, IqShaderData* V, IqShaderData*
 					(size)->GetFloat(_aq_size,__iGrid);
 					CqColor colCl;
 					Cl() ->GetColor( colCl, __iGrid );
-					(Result)->SetColor(_aq_Result + colCl * pow( MAX( 0.0f, vecR * Ln ), _aq_size ),__iGrid);
+					(Result)->SetColor(_aq_Result + colCl * pow( max( 0.0f, vecR * Ln ), _aq_size ),__iGrid);
 
 				}
 			}
@@ -674,7 +714,7 @@ void CqShaderExecEnv::SO_phong( IqShaderData* N, IqShaderData* V, IqShaderData*
 // trace(P,R)
 void CqShaderExecEnv::SO_trace( IqShaderData* P, IqShaderData* R, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__fVarying=(P)->Class()==class_varying;
@@ -682,7 +722,7 @@ void CqShaderExecEnv::SO_trace( IqShaderData* P, IqShaderData* R, IqShaderData*
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -698,7 +738,7 @@ void CqShaderExecEnv::SO_trace( IqShaderData* P, IqShaderData* R, IqShaderData*
 // illuminance(P,nsamples)
 void CqShaderExecEnv::SO_illuminance( IqShaderData* Category, IqShaderData* P, IqShaderData* Axis, IqShaderData* Angle, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__iGrid = 0;
@@ -707,7 +747,7 @@ void CqShaderExecEnv::SO_illuminance( IqShaderData* Category, IqShaderData* P, I
 		Category->GetString( cat );
 
 
-	__fVarying = TqTrue;
+	__fVarying = true;
 
 	// Fill in the lightsource information, and transfer the results to the shader variables,
 	if ( m_pAttributes != 0 )
@@ -719,12 +759,12 @@ void CqShaderExecEnv::SO_illuminance( IqShaderData* Category, IqShaderData* P, I
 		if ( NULL != Angle )
 			__fVarying=(Angle)->Class()==class_varying||__fVarying;
 
-		TqBool exec = TqTrue;
+		bool exec = true;
 
 		if( cat.size() )
 		{
 
-			TqBool exclude = TqFalse;
+			bool exclude = false;
 			CqString lightcategories;
 			CqString catname;
 
@@ -744,7 +784,7 @@ void CqShaderExecEnv::SO_illuminance( IqShaderData* Category, IqShaderData* P, I
 			{
 				pcats->GetString( lightcategories );
 
-				exec = TqFalse;
+				exec = false;
 				// While no matching category has been found...
 				CqString::size_type tokenpos = 0, tokenend;
 				while( 1 )
@@ -755,7 +795,7 @@ void CqShaderExecEnv::SO_illuminance( IqShaderData* Category, IqShaderData* P, I
 					{
 						if( !exclude )
 						{
-							exec = TqTrue;
+							exec = true;
 							break;
 						}
 					}
@@ -770,7 +810,7 @@ void CqShaderExecEnv::SO_illuminance( IqShaderData* Category, IqShaderData* P, I
 		if( exec )
 		{
 			__iGrid = 0;
-			CqBitVector& RS = RunningState();
+			const CqBitVector& RS = RunningState();
 			do
 			{
 				if(!__fVarying || RS.Value( __iGrid ) )
@@ -791,16 +831,16 @@ void CqShaderExecEnv::SO_illuminance( IqShaderData* Category, IqShaderData* P, I
 					CqVector3D vecAxis( 0, 1, 0 );
 					if ( NULL != Axis )
 						Axis->GetVector( vecAxis, __iGrid );
-					TqFloat fAngle = PI;
+					TqFloat fAngle = M_PI;
 					if ( NULL != Angle )
 						Angle->GetFloat( fAngle, __iGrid );
 
 					TqFloat cosangle = Ln * vecAxis;
-					cosangle = CLAMP( cosangle, -1, 1 );
+					cosangle = clamp(cosangle, -1.0f, 1.0f);
 					if ( acos( cosangle ) > fAngle )
-						m_CurrentState.SetValue( __iGrid, TqFalse );
+						m_CurrentState.SetValue( __iGrid, false );
 					else
-						m_CurrentState.SetValue( __iGrid, TqTrue );
+						m_CurrentState.SetValue( __iGrid, true );
 				}
 			}
 			while( ( ++__iGrid < shadingPointCount() ) && __fVarying);
@@ -819,18 +859,18 @@ void	CqShaderExecEnv::SO_illuminance( IqShaderData* Category, IqShaderData* P, I
 // illuminate(P)
 void CqShaderExecEnv::SO_illuminate( IqShaderData* P, IqShaderData* Axis, IqShaderData* Angle, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
-	TqBool res = TqTrue;
+	bool res = true;
 	if ( m_Illuminate > 0 )
-		res = TqFalse;
+		res = false;
 
-	__fVarying = TqTrue;
+	__fVarying = true;
 	if ( res )
 	{
 		__iGrid = 0;
-		CqBitVector& RS = RunningState();
+		const CqBitVector& RS = RunningState();
 		do
 		{
 			if(!__fVarying || RS.Value( __iGrid ) )
@@ -850,19 +890,19 @@ void CqShaderExecEnv::SO_illuminate( IqShaderData* P, IqShaderData* Axis, IqShad
 				CqVector3D vecAxis( 0.0f, 1.0f, 0.0f );
 				if ( NULL != Axis )
 					Axis->GetVector( vecAxis, __iGrid );
-				TqFloat fAngle = PI;
+				TqFloat fAngle = M_PI;
 				if ( NULL != Angle )
 					Angle->GetFloat( fAngle, __iGrid );
 				TqFloat cosangle = Ln * vecAxis;
-				cosangle = CLAMP( cosangle, -1, 1 );
+				cosangle = clamp(cosangle, -1.0f, 1.0f);
 				if ( acos( cosangle ) > fAngle )
 				{
 					// Make sure we set the light color to zero in the areas that won't be lit.
 					Cl() ->SetColor( CqColor( 0, 0, 0 ), __iGrid );
-					m_CurrentState.SetValue( __iGrid, TqFalse );
+					m_CurrentState.SetValue( __iGrid, false );
 				}
 				else
-					m_CurrentState.SetValue( __iGrid, TqTrue );
+					m_CurrentState.SetValue( __iGrid, true );
 			}
 		}
 		while( ( ++__iGrid < shadingPointCount() ) && __fVarying);
@@ -883,16 +923,16 @@ void	CqShaderExecEnv::SO_illuminate( IqShaderData* P, IqShader* pShader )
 void CqShaderExecEnv::SO_solar( IqShaderData* Axis, IqShaderData* Angle, IqShader* pShader )
 {
 	// TODO: Check light cone, and exclude points outside.
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
-	TqBool res = TqTrue;
+	bool res = true;
 	if ( m_Illuminate > 0 )
-		res = TqFalse;
+		res = false;
 
-	__fVarying = TqTrue;
+	__fVarying = true;
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -905,7 +945,7 @@ void CqShaderExecEnv::SO_solar( IqShaderData* Axis, IqShaderData* Angle, IqShade
 				if ( NULL != Axis )
 					Axis->GetVector( vecAxis, __iGrid );
 				L() ->SetVector( vecAxis, __iGrid );
-				m_CurrentState.SetValue( __iGrid, TqTrue );
+				m_CurrentState.SetValue( __iGrid, true );
 			}
 		}
 	}
@@ -920,6 +960,28 @@ void	CqShaderExecEnv::SO_solar( IqShader* pShader )
 	SO_solar( NULL, NULL, pShader );
 }
 
+//----------------------------------------------------------------------
+// gather(category,P,N,angle,nsamples)
+void CqShaderExecEnv::SO_gather( IqShaderData* category, IqShaderData* P, IqShaderData* N, IqShaderData* angle, IqShaderData* samples, IqShader* pShader, int cParams, IqShaderData** apParams)
+{
+	bool __fVarying;
+	TqUint __iGrid;
+
+	__iGrid = 0;
+	__fVarying = true;
+	const CqBitVector& RS = RunningState();
+	do
+	{
+		if(!__fVarying || RS.Value( __iGrid ) )
+		{
+			m_CurrentState.SetValue( __iGrid, false );
+		}
+		else
+			m_CurrentState.SetValue( __iGrid, false );
+	}
+	while( ( ++__iGrid < shadingPointCount() ) && __fVarying);
+}
+
 //----------------------------------------------------------------------
 // incident
 
@@ -949,7 +1011,7 @@ void CqShaderExecEnv::SO_opposite( IqShaderData* name, IqShaderData* pV, IqShade
 // specularbrdf(L,N,V,rough)
 void CqShaderExecEnv::SO_specularbrdf( IqShaderData* L, IqShaderData* N, IqShaderData* V, IqShaderData* rough, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__fVarying=(L)->Class()==class_varying;
@@ -959,7 +1021,7 @@ void CqShaderExecEnv::SO_specularbrdf( IqShaderData* L, IqShaderData* N, IqShade
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -979,7 +1041,7 @@ void CqShaderExecEnv::SO_specularbrdf( IqShaderData* L, IqShaderData* N, IqShade
 			(rough)->GetFloat(_aq_rough,__iGrid);
 			CqColor colCl;
 			Cl() ->GetColor( colCl, __iGrid );
-			(Result)->SetColor(colCl * pow( MAX( 0.0f, _aq_N * H ), 1.0f / ( _aq_rough / 8.0f ) ),__iGrid);
+			(Result)->SetColor(colCl * pow( max( 0.0f, _aq_N * H ), 1.0f / ( _aq_rough / 8.0f ) ),__iGrid);
 		}
 	}
 	while( ( ++__iGrid < shadingPointCount() ) && __fVarying);
@@ -989,13 +1051,12 @@ void CqShaderExecEnv::SO_specularbrdf( IqShaderData* L, IqShaderData* N, IqShade
 // calculatenormal(P)
 void CqShaderExecEnv::SO_calculatenormal( IqShaderData* p, IqShaderData* Result, IqShader* pShader )
 {
-	CqVector3D Defvec( 0.0f, 0.0f, 0.0f );
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	// Find out if the orientation is inverted.
-	TqBool CSO = pTransform()->GetHandedness(getRenderContext()->Time());
-	TqBool O = TqFalse;
+	bool CSO = pTransform()->GetHandedness(getRenderContext()->Time());
+	bool O = false;
 	if( pAttributes() )
 		O = pAttributes() ->GetIntegerAttribute( "System", "Orientation" ) [ 0 ] != 0;
 	TqFloat neg = 1;
@@ -1006,49 +1067,13 @@ void CqShaderExecEnv::SO_calculatenormal( IqShaderData* p, IqShaderData* Result,
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
 		{
-			//CqVector3D	dPdu = SO_DuType<CqVector3D>( p, __iGrid, this, Defvec );
-			//CqVector3D	dPdv = SO_DvType<CqVector3D>( p, __iGrid, this, Defvec );
-			//CqVector3D	N = dPdu % dPdv;
-
-			CqVector3D Ret, Ret2;
-			TqInt uRes = uGridRes();
-			TqInt GridX = __iGrid % ( uRes + 1 );
-
-			CqVector3D v1, v2;
-			if ( GridX < uRes )
-			{
-				p->GetValue( v1, __iGrid + 1 );
-				p->GetValue( v2, __iGrid );
-				Ret = ( v1 - v2 );
-			}
-			else
-			{
-				p->GetValue( v1, __iGrid );
-				p->GetValue( v2, __iGrid - 1 );
-				Ret = ( v1 - v2 );
-			}
-			TqInt vRes = vGridRes();
-			TqInt GridY = ( __iGrid / ( uRes + 1 ) );
-
-			if ( GridY < vRes )
-			{
-				p->GetValue( v1, __iGrid + uRes + 1 );
-				p->GetValue( v2, __iGrid );
-				Ret2 = ( v1 - v2 );
-			}
-			else
-			{
-				p->GetValue( v1, __iGrid );
-				p->GetValue( v2, __iGrid - ( uRes + 1 ) );
-				Ret2 = ( v1 - v2 );
-			}
-
-			CqVector3D N = Ret % Ret2;
+			CqVector3D N = diffU<CqVector3D>(p, __iGrid)
+				% diffV<CqVector3D>(p, __iGrid);
 			N.Unit();
 			N *= neg;
 			(Result)->SetNormal(N,__iGrid);
@@ -1058,5 +1083,46 @@ void CqShaderExecEnv::SO_calculatenormal( IqShaderData* p, IqShaderData* Result,
 }
 
 
-END_NAMESPACE(    Aqsis )
+//----------------------------------------------------------------------
+// occlusion(P,N,samples)
+void CqShaderExecEnv::SO_occlusion_rt( IqShaderData* P, IqShaderData* N, IqShaderData* samples, IqShaderData* Result, IqShader* pShader, int cParams, IqShaderData** apParams )
+{
+	bool __fVarying;
+	TqUint __iGrid;
+
+	if ( !getRenderContext() )
+		return ;
+
+	__fVarying = true;
+	__iGrid = 0;
+	const CqBitVector& RS = RunningState();
+	do
+	{
+		if(!__fVarying || RS.Value( __iGrid ) )
+		{
+			(Result)->SetFloat(0.0f,__iGrid);	// Default, completely lit
+		}
+	}
+	while( ( ++__iGrid < shadingPointCount() ) && __fVarying);
+}
+
+//----------------------------------------------------------------------
+// rayinfo
+//
+
+void CqShaderExecEnv::SO_rayinfo( IqShaderData* dataname, IqShaderData* pV, IqShaderData* Result, IqShader* pShader )
+{
+	TqUint __iGrid;
+
+	if ( !getRenderContext() )
+		return ;
+
+	TqFloat Ret = 0.0f;
+
+	__iGrid = 0;
+
+	(Result)->SetFloat(Ret,__iGrid);
+}
+
+} // namespace Aqsis
 //---------------------------------------------------------------------
