@@ -1,5 +1,5 @@
 // Aqsis
-// Copyright  1997 - 2001, Paul C. Gregory
+// Copyright (C) 1997 - 2001, Paul C. Gregory
 //
 // Contact: pgregory@aqsis.org
 //
@@ -37,14 +37,14 @@
 #include	"version.h"
 #include	"logging.h"
 
-START_NAMESPACE(    Aqsis )
+namespace Aqsis {
 
 
 //----------------------------------------------------------------------
 // transform(s,s,P)
 void CqShaderExecEnv::SO_transform( IqShaderData* fromspace, IqShaderData* tospace, IqShaderData* p, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	assert( pShader != 0 );
@@ -59,11 +59,12 @@ void CqShaderExecEnv::SO_transform( IqShaderData* fromspace, IqShaderData* tospa
 		(fromspace)->GetString(_aq_fromspace,__iGrid);
 		CqString _aq_tospace;
 		(tospace)->GetString(_aq_tospace,__iGrid);
-		const CqMatrix& mat = getRenderContext() ->matSpaceToSpace( _aq_fromspace.c_str(), _aq_tospace.c_str(), pShader->getTransform(), pTransform().get(), getRenderContext()->Time() );
+		CqMatrix mat; 
+		getRenderContext() ->matSpaceToSpace( _aq_fromspace.c_str(), _aq_tospace.c_str(), pShader->getTransform(), pTransform().get(), getRenderContext()->Time(), mat );
 
 
 		__iGrid = 0;
-		CqBitVector& RS = RunningState();
+		const CqBitVector& RS = RunningState();
 		do
 		{
 			if(!__fVarying || RS.Value( __iGrid ) )
@@ -78,7 +79,7 @@ void CqShaderExecEnv::SO_transform( IqShaderData* fromspace, IqShaderData* tospa
 	else
 	{
 		__iGrid = 0;
-		CqBitVector& RS = RunningState();
+		const CqBitVector& RS = RunningState();
 		do
 		{
 			if(!__fVarying || RS.Value( __iGrid ) )
@@ -97,7 +98,7 @@ void CqShaderExecEnv::SO_transform( IqShaderData* fromspace, IqShaderData* tospa
 // transform(s,P)
 void CqShaderExecEnv::SO_transform( IqShaderData* tospace, IqShaderData* p, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	assert( pShader != 0 );
@@ -110,11 +111,12 @@ void CqShaderExecEnv::SO_transform( IqShaderData* tospace, IqShaderData* p, IqSh
 		__iGrid = 0;
 		CqString _aq_tospace;
 		(tospace)->GetString(_aq_tospace,__iGrid);
-		const CqMatrix& mat = getRenderContext() ->matSpaceToSpace( "current", _aq_tospace.c_str(), pShader->getTransform(), pTransform().get(), getRenderContext()->Time() );
+		CqMatrix mat;
+		getRenderContext() ->matSpaceToSpace( "current", _aq_tospace.c_str(), pShader->getTransform(), pTransform().get(), getRenderContext()->Time(), mat );
 
 
 		__iGrid = 0;
-		CqBitVector& RS = RunningState();
+		const CqBitVector& RS = RunningState();
 		do
 		{
 			if(!__fVarying || RS.Value( __iGrid ) )
@@ -129,7 +131,7 @@ void CqShaderExecEnv::SO_transform( IqShaderData* tospace, IqShaderData* p, IqSh
 	else
 	{
 		__iGrid = 0;
-		CqBitVector& RS = RunningState();
+		const CqBitVector& RS = RunningState();
 		do
 		{
 			if(!__fVarying || RS.Value( __iGrid ) )
@@ -148,7 +150,7 @@ void CqShaderExecEnv::SO_transform( IqShaderData* tospace, IqShaderData* p, IqSh
 // transform(m,P)
 void CqShaderExecEnv::SO_transformm( IqShaderData* tospace, IqShaderData* p, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	assert( pShader != 0 );
@@ -157,7 +159,7 @@ void CqShaderExecEnv::SO_transformm( IqShaderData* tospace, IqShaderData* p, IqS
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -177,7 +179,7 @@ void CqShaderExecEnv::SO_transformm( IqShaderData* tospace, IqShaderData* p, IqS
 // vtransform(s,s,P)
 void CqShaderExecEnv::SO_vtransform( IqShaderData* fromspace, IqShaderData* tospace, IqShaderData* p, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	assert( pShader != 0 );
@@ -192,11 +194,12 @@ void CqShaderExecEnv::SO_vtransform( IqShaderData* fromspace, IqShaderData* tosp
 		(fromspace)->GetString(_aq_fromspace,__iGrid);
 		CqString _aq_tospace;
 		(tospace)->GetString(_aq_tospace,__iGrid);
-		const CqMatrix& mat = getRenderContext() ->matVSpaceToSpace( _aq_fromspace.c_str(), _aq_tospace.c_str(), pShader->getTransform(), pTransform().get(), getRenderContext()->Time() );
+		CqMatrix mat;
+		getRenderContext() ->matVSpaceToSpace( _aq_fromspace.c_str(), _aq_tospace.c_str(), pShader->getTransform(), pTransform().get(), getRenderContext()->Time(), mat );
 
 
 		__iGrid = 0;
-		CqBitVector& RS = RunningState();
+		const CqBitVector& RS = RunningState();
 		do
 		{
 			if(!__fVarying || RS.Value( __iGrid ) )
@@ -211,7 +214,7 @@ void CqShaderExecEnv::SO_vtransform( IqShaderData* fromspace, IqShaderData* tosp
 	else
 	{
 		__iGrid = 0;
-		CqBitVector& RS = RunningState();
+		const CqBitVector& RS = RunningState();
 		do
 		{
 			if(!__fVarying || RS.Value( __iGrid ) )
@@ -230,7 +233,7 @@ void CqShaderExecEnv::SO_vtransform( IqShaderData* fromspace, IqShaderData* tosp
 // vtransform(s,P)
 void CqShaderExecEnv::SO_vtransform( IqShaderData* tospace, IqShaderData* p, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	assert( pShader != 0 );
@@ -243,11 +246,12 @@ void CqShaderExecEnv::SO_vtransform( IqShaderData* tospace, IqShaderData* p, IqS
 		__iGrid = 0;
 		CqString _aq_tospace;
 		(tospace)->GetString(_aq_tospace,__iGrid);
-		const CqMatrix& mat = getRenderContext() ->matVSpaceToSpace( "current", _aq_tospace.c_str(), pShader->getTransform(), pTransform().get(), getRenderContext()->Time() );
+		CqMatrix mat;
+		getRenderContext() ->matVSpaceToSpace( "current", _aq_tospace.c_str(), pShader->getTransform(), pTransform().get(), getRenderContext()->Time(), mat );
 
 
 		__iGrid = 0;
-		CqBitVector& RS = RunningState();
+		const CqBitVector& RS = RunningState();
 		do
 		{
 			if(!__fVarying || RS.Value( __iGrid ) )
@@ -262,7 +266,7 @@ void CqShaderExecEnv::SO_vtransform( IqShaderData* tospace, IqShaderData* p, IqS
 	else
 	{
 		__iGrid = 0;
-		CqBitVector& RS = RunningState();
+		const CqBitVector& RS = RunningState();
 		do
 		{
 			if(!__fVarying || RS.Value( __iGrid ) )
@@ -281,7 +285,7 @@ void CqShaderExecEnv::SO_vtransform( IqShaderData* tospace, IqShaderData* p, IqS
 // vtransform(m,P)
 void CqShaderExecEnv::SO_vtransformm( IqShaderData* tospace, IqShaderData* p, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	assert( pShader != 0 );
@@ -290,7 +294,7 @@ void CqShaderExecEnv::SO_vtransformm( IqShaderData* tospace, IqShaderData* p, Iq
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -310,7 +314,7 @@ void CqShaderExecEnv::SO_vtransformm( IqShaderData* tospace, IqShaderData* p, Iq
 // ntransform(s,s,P)
 void CqShaderExecEnv::SO_ntransform( IqShaderData* fromspace, IqShaderData* tospace, IqShaderData* p, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	assert( pShader != 0 );
@@ -325,11 +329,12 @@ void CqShaderExecEnv::SO_ntransform( IqShaderData* fromspace, IqShaderData* tosp
 		(fromspace)->GetString(_aq_fromspace,__iGrid);
 		CqString _aq_tospace;
 		(tospace)->GetString(_aq_tospace,__iGrid);
-		const CqMatrix& mat = getRenderContext() ->matNSpaceToSpace( _aq_fromspace.c_str(), _aq_tospace.c_str(), pShader->getTransform(), pTransform().get(), getRenderContext()->Time() );
+		CqMatrix mat;
+		getRenderContext() ->matNSpaceToSpace( _aq_fromspace.c_str(), _aq_tospace.c_str(), pShader->getTransform(), pTransform().get(), getRenderContext()->Time(), mat );
 		__iGrid = 0;
 
 		__iGrid = 0;
-		CqBitVector& RS = RunningState();
+		const CqBitVector& RS = RunningState();
 		do
 		{
 			if(!__fVarying || RS.Value( __iGrid ) )
@@ -344,7 +349,7 @@ void CqShaderExecEnv::SO_ntransform( IqShaderData* fromspace, IqShaderData* tosp
 	else
 	{
 		__iGrid = 0;
-		CqBitVector& RS = RunningState();
+		const CqBitVector& RS = RunningState();
 		do
 		{
 			if(!__fVarying || RS.Value( __iGrid ) )
@@ -363,7 +368,7 @@ void CqShaderExecEnv::SO_ntransform( IqShaderData* fromspace, IqShaderData* tosp
 // ntransform(s,P)
 void CqShaderExecEnv::SO_ntransform( IqShaderData* tospace, IqShaderData* p, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	assert( pShader != 0 );
@@ -376,11 +381,12 @@ void CqShaderExecEnv::SO_ntransform( IqShaderData* tospace, IqShaderData* p, IqS
 		__iGrid = 0;
 		CqString _aq_tospace;
 		(tospace)->GetString(_aq_tospace,__iGrid);
-		const CqMatrix& mat = getRenderContext() ->matNSpaceToSpace( "current", _aq_tospace.c_str(), pShader->getTransform(), pTransform().get(), getRenderContext()->Time() );
+		CqMatrix mat;
+		getRenderContext() ->matNSpaceToSpace( "current", _aq_tospace.c_str(), pShader->getTransform(), pTransform().get(), getRenderContext()->Time(), mat );
 		__iGrid = 0;
 
 		__iGrid = 0;
-		CqBitVector& RS = RunningState();
+		const CqBitVector& RS = RunningState();
 		do
 		{
 			if(!__fVarying || RS.Value( __iGrid ) )
@@ -395,7 +401,7 @@ void CqShaderExecEnv::SO_ntransform( IqShaderData* tospace, IqShaderData* p, IqS
 	else
 	{
 		__iGrid = 0;
-		CqBitVector& RS = RunningState();
+		const CqBitVector& RS = RunningState();
 		do
 		{
 			if(!__fVarying || RS.Value( __iGrid ) )
@@ -414,7 +420,7 @@ void CqShaderExecEnv::SO_ntransform( IqShaderData* tospace, IqShaderData* p, IqS
 // ntransform(m,P)
 void CqShaderExecEnv::SO_ntransformm( IqShaderData* tospace, IqShaderData* p, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	assert( pShader != 0 );
@@ -423,7 +429,7 @@ void CqShaderExecEnv::SO_ntransformm( IqShaderData* tospace, IqShaderData* p, Iq
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -440,7 +446,7 @@ void CqShaderExecEnv::SO_ntransformm( IqShaderData* tospace, IqShaderData* p, Iq
 
 void CqShaderExecEnv::SO_cmix( IqShaderData* color0, IqShaderData* color1, IqShaderData* value, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__fVarying=(color0)->Class()==class_varying;
@@ -449,7 +455,7 @@ void CqShaderExecEnv::SO_cmix( IqShaderData* color0, IqShaderData* color1, IqSha
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -467,9 +473,40 @@ void CqShaderExecEnv::SO_cmix( IqShaderData* color0, IqShaderData* color1, IqSha
 	while( ( ++__iGrid < shadingPointCount() ) && __fVarying);
 }
 
+void CqShaderExecEnv::SO_cmixc( IqShaderData* color0, IqShaderData* color1, IqShaderData* value, IqShaderData* Result, IqShader* pShader )
+{
+	bool __fVarying;
+	TqUint __iGrid;
+
+	__fVarying=(color0)->Class()==class_varying;
+	__fVarying=(color1)->Class()==class_varying||__fVarying;
+	__fVarying=(value)->Class()==class_varying||__fVarying;
+	__fVarying=(Result)->Class()==class_varying||__fVarying;
+
+	__iGrid = 0;
+	const CqBitVector& RS = RunningState();
+	do
+	{
+		if(!__fVarying || RS.Value( __iGrid ) )
+		{
+			CqColor _aq_color0;
+			(color0)->GetColor(_aq_color0,__iGrid);
+			CqColor _aq_color1;
+			(color1)->GetColor(_aq_color1,__iGrid);
+			CqColor _aq_value;
+			(value)->GetColor(_aq_value,__iGrid);
+			TqFloat c1 = ( 1.0f - _aq_value[0] ) * _aq_color0[0] + _aq_value[0] * _aq_color1[0] ;
+			TqFloat c2 = ( 1.0f - _aq_value[1] ) * _aq_color0[1] + _aq_value[1] * _aq_color1[1] ;
+			TqFloat c3 = ( 1.0f - _aq_value[2] ) * _aq_color0[2] + _aq_value[2] * _aq_color1[2] ;
+			(Result)->SetColor(CqColor(c1,c2,c3),__iGrid);
+		}
+	}
+	while( ( ++__iGrid < shadingPointCount() ) && __fVarying);
+}
+
 void	CqShaderExecEnv::SO_fmix( IqShaderData* f0, IqShaderData* f1, IqShaderData* value, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__fVarying=(f0)->Class()==class_varying;
@@ -477,7 +514,7 @@ void	CqShaderExecEnv::SO_fmix( IqShaderData* f0, IqShaderData* f1, IqShaderData*
 	__fVarying=(value)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -495,9 +532,9 @@ void	CqShaderExecEnv::SO_fmix( IqShaderData* f0, IqShaderData* f1, IqShaderData*
 	while( ( ++__iGrid < shadingPointCount() ) && __fVarying);
 }
 
-void	CqShaderExecEnv::SO_pmix( IqShaderData* p0, IqShaderData* p1, IqShaderData* value, IqShaderData* Result, IqShader* pShader )
+void    CqShaderExecEnv::SO_pmix( IqShaderData* p0, IqShaderData* p1, IqShaderData* value, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__fVarying=(p0)->Class()==class_varying;
@@ -505,7 +542,7 @@ void	CqShaderExecEnv::SO_pmix( IqShaderData* p0, IqShaderData* p1, IqShaderData*
 	__fVarying=(value)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -523,9 +560,39 @@ void	CqShaderExecEnv::SO_pmix( IqShaderData* p0, IqShaderData* p1, IqShaderData*
 	while( ( ++__iGrid < shadingPointCount() ) && __fVarying);
 }
 
-void	CqShaderExecEnv::SO_vmix( IqShaderData* v0, IqShaderData* v1, IqShaderData* value, IqShaderData* Result, IqShader* pShader )
+void	CqShaderExecEnv::SO_pmixc( IqShaderData* p0, IqShaderData* p1, IqShaderData* value, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
+	TqUint __iGrid;
+
+	__fVarying=(p0)->Class()==class_varying;
+	__fVarying=(p1)->Class()==class_varying||__fVarying;
+	__fVarying=(value)->Class()==class_varying||__fVarying;
+
+	__iGrid = 0;
+	const CqBitVector& RS = RunningState();
+	do
+	{
+		if(!__fVarying || RS.Value( __iGrid ) )
+		{
+			CqVector3D _aq_p0;
+			(p0)->GetPoint(_aq_p0,__iGrid);
+			CqVector3D _aq_p1;
+			(p1)->GetPoint(_aq_p1,__iGrid);
+			CqColor _aq_value;
+			(value)->GetColor(_aq_value,__iGrid);
+			TqFloat p1 = ( 1.0f - _aq_value[0] ) * _aq_p0[0] + _aq_value[0] * _aq_p1[0] ;
+			TqFloat p2 = ( 1.0f - _aq_value[1] ) * _aq_p0[1] + _aq_value[1] * _aq_p1[1] ;
+			TqFloat p3 = ( 1.0f - _aq_value[2] ) * _aq_p0[2] + _aq_value[2] * _aq_p1[2] ;
+			(Result)->SetPoint(CqVector3D(p1,p2,p3),__iGrid);
+		}
+	}
+	while( ( ++__iGrid < shadingPointCount() ) && __fVarying);
+}
+
+void    CqShaderExecEnv::SO_vmix( IqShaderData* v0, IqShaderData* v1, IqShaderData* value, IqShaderData* Result, IqShader* pShader )
+{
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__fVarying=(v0)->Class()==class_varying;
@@ -533,7 +600,7 @@ void	CqShaderExecEnv::SO_vmix( IqShaderData* v0, IqShaderData* v1, IqShaderData*
 	__fVarying=(value)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -551,9 +618,39 @@ void	CqShaderExecEnv::SO_vmix( IqShaderData* v0, IqShaderData* v1, IqShaderData*
 	while( ( ++__iGrid < shadingPointCount() ) && __fVarying);
 }
 
+void	CqShaderExecEnv::SO_vmixc( IqShaderData* v0, IqShaderData* v1, IqShaderData* value, IqShaderData* Result, IqShader* pShader )
+{
+	bool __fVarying;
+	TqUint __iGrid;
+
+	__fVarying=(v0)->Class()==class_varying;
+	__fVarying=(v1)->Class()==class_varying||__fVarying;
+	__fVarying=(value)->Class()==class_varying||__fVarying;
+
+	__iGrid = 0;
+	const CqBitVector& RS = RunningState();
+	do
+	{
+		if(!__fVarying || RS.Value( __iGrid ) )
+		{
+			CqVector3D _aq_v0;
+			(v0)->GetVector(_aq_v0,__iGrid);
+			CqVector3D _aq_v1;
+			(v1)->GetVector(_aq_v1,__iGrid);
+			CqColor _aq_value;
+			(value)->GetColor(_aq_value,__iGrid);
+                        TqFloat v1 = ( 1.0f - _aq_value[0] ) * _aq_v0[0] + _aq_value[0] * _aq_v1[0] ;
+                        TqFloat v2 = ( 1.0f - _aq_value[1] ) * _aq_v0[1] + _aq_value[1] * _aq_v1[1] ;
+                        TqFloat v3 = ( 1.0f - _aq_value[2] ) * _aq_v0[2] + _aq_value[2] * _aq_v1[2] ;
+                        (Result)->SetVector(CqVector3D(v1,v2,v3),__iGrid);
+		}
+	}
+	while( ( ++__iGrid < shadingPointCount() ) && __fVarying);
+}
+
 void	CqShaderExecEnv::SO_nmix( IqShaderData* n0, IqShaderData* n1, IqShaderData* value, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__fVarying=(n0)->Class()==class_varying;
@@ -561,7 +658,7 @@ void	CqShaderExecEnv::SO_nmix( IqShaderData* n0, IqShaderData* n1, IqShaderData*
 	__fVarying=(value)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -579,12 +676,42 @@ void	CqShaderExecEnv::SO_nmix( IqShaderData* n0, IqShaderData* n1, IqShaderData*
 	while( ( ++__iGrid < shadingPointCount() ) && __fVarying);
 }
 
+void	CqShaderExecEnv::SO_nmixc( IqShaderData* n0, IqShaderData* n1, IqShaderData* value, IqShaderData* Result, IqShader* pShader )
+{
+	bool __fVarying;
+	TqUint __iGrid;
+
+	__fVarying=(n0)->Class()==class_varying;
+	__fVarying=(n1)->Class()==class_varying||__fVarying;
+	__fVarying=(value)->Class()==class_varying||__fVarying;
+
+	__iGrid = 0;
+	const CqBitVector& RS = RunningState();
+	do
+	{
+		if(!__fVarying || RS.Value( __iGrid ) )
+		{
+			CqVector3D _aq_n0;
+			(n0)->GetNormal(_aq_n0,__iGrid);
+			CqVector3D _aq_n1;
+			(n1)->GetNormal(_aq_n1,__iGrid);
+                        CqColor _aq_value;
+                        (value)->GetColor(_aq_value,__iGrid);
+                        TqFloat n1 = ( 1.0f - _aq_value[0] ) * _aq_n0[0] + _aq_value[0] * _aq_n1[0] ;
+                        TqFloat n2 = ( 1.0f - _aq_value[1] ) * _aq_n0[1] + _aq_value[1] * _aq_n1[1] ;
+                        TqFloat n3 = ( 1.0f - _aq_value[2] ) * _aq_n0[2] + _aq_value[2] * _aq_n1[2] ;
+                        (Result)->SetNormal(CqVector3D(n1,n2,n3),__iGrid);
+		}
+	}
+	while( ( ++__iGrid < shadingPointCount() ) && __fVarying);
+}
+
 
 //----------------------------------------------------------------------
 // ctransform(s,s,c)
 void CqShaderExecEnv::SO_ctransform( IqShaderData* fromspace, IqShaderData* tospace, IqShaderData* c, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__fVarying=(c)->Class()==class_varying;
@@ -599,35 +726,34 @@ void CqShaderExecEnv::SO_ctransform( IqShaderData* fromspace, IqShaderData* tosp
 
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
 		{
-			CqColor _aq_c;
-			(c)->GetColor(_aq_c,__iGrid);
-			CqColor res( _aq_c );
+			CqColor res;
+			(c)->GetColor(res,__iGrid);
 			if ( strfromspace.compare( "hsv" ) == 0 )
-				res = _aq_c.hsvtorgb();
+				res = res.hsvtorgb();
 			else if ( strfromspace.compare( "hsl" ) == 0 )
-				res = _aq_c.hsltorgb();
+				res = res.hsltorgb();
 			else if ( strfromspace.compare( "XYZ" ) == 0 )
-				res = _aq_c.XYZtorgb();
+				res = res.XYZtorgb();
 			else if ( strfromspace.compare( "xyY" ) == 0 )
-				res = _aq_c.xyYtorgb();
+				res = res.xyYtorgb();
 			else if ( strfromspace.compare( "YIQ" ) == 0 )
-				res = _aq_c.YIQtorgb();
+				res = res.YIQtorgb();
 
 			if ( _aq_tospace.compare( "hsv" ) == 0 )
-				res = _aq_c.rgbtohsv();
+				res = res.rgbtohsv();
 			else if ( _aq_tospace.compare( "hsl" ) == 0 )
-				res = _aq_c.rgbtohsl();
+				res = res.rgbtohsl();
 			else if ( _aq_tospace.compare( "XYZ" ) == 0 )
-				res = _aq_c.rgbtoXYZ();
+				res = res.rgbtoXYZ();
 			else if ( _aq_tospace.compare( "xyY" ) == 0 )
-				res = _aq_c.rgbtoxyY();
+				res = res.rgbtoxyY();
 			else if ( _aq_tospace.compare( "YIQ" ) == 0 )
-				res = _aq_c.rgbtoYIQ();
+				res = res.rgbtoYIQ();
 
 			(Result)->SetColor(res,__iGrid);
 		}
@@ -649,7 +775,7 @@ void CqShaderExecEnv::SO_ctransform( IqShaderData* tospace, IqShaderData* c, IqS
 // ctransform(s,c)
 void CqShaderExecEnv::SO_ptlined( IqShaderData* P0, IqShaderData* P1, IqShaderData* Q, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__fVarying=(P0)->Class()==class_varying;
@@ -658,7 +784,7 @@ void CqShaderExecEnv::SO_ptlined( IqShaderData* P0, IqShaderData* P1, IqShaderDa
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -699,7 +825,7 @@ void CqShaderExecEnv::SO_ptlined( IqShaderData* P0, IqShaderData* P1, IqShaderDa
 // mtransform(s,s,M)
 void CqShaderExecEnv::SO_mtransform( IqShaderData* fromspace, IqShaderData* tospace, IqShaderData* m, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	assert( pShader != 0 );
@@ -714,11 +840,12 @@ void CqShaderExecEnv::SO_mtransform( IqShaderData* fromspace, IqShaderData* tosp
 		(fromspace)->GetString(_aq_fromspace,__iGrid);
 		CqString _aq_tospace;
 		(tospace)->GetString(_aq_tospace,__iGrid);
-		const CqMatrix& mat = getRenderContext() ->matNSpaceToSpace( _aq_fromspace.c_str(), _aq_tospace.c_str(), pShader->getTransform(), pTransform().get(), getRenderContext()->Time() );
+		CqMatrix mat;
+		getRenderContext() ->matNSpaceToSpace( _aq_fromspace.c_str(), _aq_tospace.c_str(), pShader->getTransform(), pTransform().get(), getRenderContext()->Time(), mat );
 		__iGrid = 0;
 
 		__iGrid = 0;
-		CqBitVector& RS = RunningState();
+		const CqBitVector& RS = RunningState();
 		do
 		{
 			if(!__fVarying || RS.Value( __iGrid ) )
@@ -733,7 +860,7 @@ void CqShaderExecEnv::SO_mtransform( IqShaderData* fromspace, IqShaderData* tosp
 	else
 	{
 		__iGrid = 0;
-		CqBitVector& RS = RunningState();
+		const CqBitVector& RS = RunningState();
 		do
 		{
 			if(!__fVarying || RS.Value( __iGrid ) )
@@ -752,7 +879,7 @@ void CqShaderExecEnv::SO_mtransform( IqShaderData* fromspace, IqShaderData* tosp
 // mtransform(s,M)
 void CqShaderExecEnv::SO_mtransform( IqShaderData* tospace, IqShaderData* m, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	assert( pShader != 0 );
@@ -765,11 +892,12 @@ void CqShaderExecEnv::SO_mtransform( IqShaderData* tospace, IqShaderData* m, IqS
 		__iGrid = 0;
 		CqString _aq_tospace;
 		(tospace)->GetString(_aq_tospace,__iGrid);
-		const CqMatrix& mat = getRenderContext() ->matNSpaceToSpace( "current", _aq_tospace.c_str(), pShader->getTransform(), pTransform().get(), getRenderContext()->Time() );
+		CqMatrix mat;
+		getRenderContext() ->matNSpaceToSpace( "current", _aq_tospace.c_str(), pShader->getTransform(), pTransform().get(), getRenderContext()->Time(), mat );
 		__iGrid = 0;
 
 		__iGrid = 0;
-		CqBitVector& RS = RunningState();
+		const CqBitVector& RS = RunningState();
 		do
 		{
 			if(!__fVarying || RS.Value( __iGrid ) )
@@ -784,7 +912,7 @@ void CqShaderExecEnv::SO_mtransform( IqShaderData* tospace, IqShaderData* m, IqS
 	else
 	{
 		__iGrid = 0;
-		CqBitVector& RS = RunningState();
+		const CqBitVector& RS = RunningState();
 		do
 		{
 			if(!__fVarying || RS.Value( __iGrid ) )
@@ -804,14 +932,14 @@ void CqShaderExecEnv::SO_mtransform( IqShaderData* tospace, IqShaderData* m, IqS
 // determinant(m)
 void CqShaderExecEnv::SO_determinant( IqShaderData* M, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__fVarying=(M)->Class()==class_varying;
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -829,7 +957,7 @@ void CqShaderExecEnv::SO_determinant( IqShaderData* M, IqShaderData* Result, IqS
 // translate(m,v)
 void CqShaderExecEnv::SO_mtranslate( IqShaderData* M, IqShaderData* V, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__fVarying=(M)->Class()==class_varying;
@@ -837,7 +965,7 @@ void CqShaderExecEnv::SO_mtranslate( IqShaderData* M, IqShaderData* V, IqShaderD
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -857,7 +985,7 @@ void CqShaderExecEnv::SO_mtranslate( IqShaderData* M, IqShaderData* V, IqShaderD
 // rotate(m,v)
 void CqShaderExecEnv::SO_mrotate( IqShaderData* M, IqShaderData* angle, IqShaderData* axis, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__fVarying=(M)->Class()==class_varying;
@@ -866,7 +994,7 @@ void CqShaderExecEnv::SO_mrotate( IqShaderData* M, IqShaderData* angle, IqShader
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -888,7 +1016,7 @@ void CqShaderExecEnv::SO_mrotate( IqShaderData* M, IqShaderData* angle, IqShader
 // scale(m,p)
 void CqShaderExecEnv::SO_mscale( IqShaderData* M, IqShaderData* S, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__fVarying=(M)->Class()==class_varying;
@@ -896,7 +1024,7 @@ void CqShaderExecEnv::SO_mscale( IqShaderData* M, IqShaderData* S, IqShaderData*
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -917,7 +1045,7 @@ void CqShaderExecEnv::SO_mscale( IqShaderData* M, IqShaderData* S, IqShaderData*
 // setmcomp(p,v)
 void	CqShaderExecEnv::SO_setmcomp( IqShaderData* M, IqShaderData* r, IqShaderData* c, IqShaderData* v, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__fVarying=(M)->Class()==class_varying;
@@ -926,7 +1054,7 @@ void	CqShaderExecEnv::SO_setmcomp( IqShaderData* M, IqShaderData* r, IqShaderDat
 	__fVarying=(v)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -940,7 +1068,7 @@ void	CqShaderExecEnv::SO_setmcomp( IqShaderData* M, IqShaderData* r, IqShaderDat
 			TqFloat _aq_v;
 			(v)->GetFloat(_aq_v,__iGrid);
 			_aq_M [ static_cast<TqInt>( _aq_r ) ][ static_cast<TqInt>( _aq_c ) ] = _aq_v;
-			_aq_M.SetfIdentity( TqFalse );
+			_aq_M.SetfIdentity( false );
 			M->SetValue( _aq_M, __iGrid );
 		}
 	}
@@ -952,7 +1080,7 @@ void	CqShaderExecEnv::SO_setmcomp( IqShaderData* M, IqShaderData* r, IqShaderDat
 // rotate(Q,angle,P0,P1)
 void CqShaderExecEnv::SO_rotate( IqShaderData* Q, IqShaderData* angle, IqShaderData* P0, IqShaderData* P1, IqShaderData* Result, IqShader* pShader )
 {
-	TqBool __fVarying;
+	bool __fVarying;
 	TqUint __iGrid;
 
 	__fVarying=(Q)->Class()==class_varying;
@@ -962,7 +1090,7 @@ void CqShaderExecEnv::SO_rotate( IqShaderData* Q, IqShaderData* angle, IqShaderD
 	__fVarying=(Result)->Class()==class_varying||__fVarying;
 
 	__iGrid = 0;
-	CqBitVector& RS = RunningState();
+	const CqBitVector& RS = RunningState();
 	do
 	{
 		if(!__fVarying || RS.Value( __iGrid ) )
@@ -986,5 +1114,5 @@ void CqShaderExecEnv::SO_rotate( IqShaderData* Q, IqShaderData* angle, IqShaderD
 	while( ( ++__iGrid < shadingPointCount() ) && __fVarying);
 }
 
-END_NAMESPACE(    Aqsis )
+} // namespace Aqsis
 //---------------------------------------------------------------------
