@@ -1,5 +1,5 @@
 // Aqsis
-// Copyright  1997 - 2001, Paul C. Gregory
+// Copyright (C) 1997 - 2001, Paul C. Gregory
 //
 // Contact: pgregory@aqsis.org
 //
@@ -43,7 +43,7 @@
 #include	"shadervariable.h"
 #include	"shadervm_common.h"
 
-START_NAMESPACE( Aqsis )
+namespace Aqsis {
 
 #define DECLARE_SHADERSTACK_TEMPS \
 static TqFloat temp_float; \
@@ -144,7 +144,7 @@ static CqMatrix temp_matrix;
 
 #define OpABRS(OP, NAME) \
 		template <class A, class B, class R>	\
-		inline void	Op##NAME( A& a, B&b, R& r, IqShaderData* pA, IqShaderData* pB, IqShaderData* pRes, CqBitVector& RunningState ) \
+		inline void	Op##NAME( A& a, B&b, R& r, IqShaderData* pA, IqShaderData* pB, IqShaderData* pRes, const CqBitVector& RunningState ) \
 		{ \
 			A vA; \
 			B vB; \
@@ -153,8 +153,8 @@ static CqMatrix temp_matrix;
 			R* pdR; \
 			TqInt i, ii; \
 			\
-			TqBool fAVar = pA->Size() > 1; \
-			TqBool fBVar = pB->Size() > 1; \
+			bool fAVar = pA->Size() > 1; \
+			bool fBVar = pB->Size() > 1; \
 			\
 			if( fAVar && fBVar )\
 			{ \
@@ -218,7 +218,7 @@ static CqMatrix temp_matrix;
 
 struct	SqStackEntry
 {
-	TqBool			m_IsTemp;
+	bool			m_IsTemp;
 	IqShaderData*	m_Data;
 };
 
@@ -227,7 +227,7 @@ class SHADERVM_SHARE CqShaderStack
 	public:
 		CqShaderStack() : m_iTop( 0 )
 		{
-			m_maxsamples = MAX(m_maxsamples, m_samples);
+			m_maxsamples = max(m_maxsamples, m_samples);
 			m_Stack.resize( m_maxsamples);
 		}
 		virtual ~CqShaderStack()
@@ -251,9 +251,9 @@ class SHADERVM_SHARE CqShaderStack
 			}
 
 			m_Stack[ m_iTop ].m_Data = pv;
-			m_Stack[ m_iTop ].m_IsTemp = TqTrue;
+			m_Stack[ m_iTop ].m_IsTemp = true;
 			m_iTop ++;
-			m_maxsamples = MAX(m_iTop, m_maxsamples);
+			m_maxsamples = max(m_iTop, m_maxsamples);
 
 
 		}
@@ -271,9 +271,9 @@ class SHADERVM_SHARE CqShaderStack
 			}
 
 			m_Stack[ m_iTop ].m_Data = pv;
-			m_Stack[ m_iTop ].m_IsTemp = TqFalse;
+			m_Stack[ m_iTop ].m_IsTemp = false;
 			m_iTop ++;
-			m_maxsamples = MAX(m_iTop, m_maxsamples);
+			m_maxsamples = max(m_iTop, m_maxsamples);
 
 		}
 
@@ -282,7 +282,7 @@ class SHADERVM_SHARE CqShaderStack
 		 * \param f Boolean value to update if this is varying. If not varying, leaves f unaltered.
 		 * \return Reference to the top stack entry.
 		 */
-		SqStackEntry Pop( TqBool& f )
+		SqStackEntry Pop( bool& f )
 		{
 			if ( m_iTop )
 				m_iTop--;
@@ -302,21 +302,19 @@ class SHADERVM_SHARE CqShaderStack
 		{
 			TqInt iTop = m_iTop-1;
 
-			IqShaderData* s = GetNextTemp( m_Stack[ iTop ].m_Data ->Type(), m_Stack[ iTop ].m_Data ->Class() );
-			s->SetValueFromVariable( m_Stack[ iTop ].m_Data );
+			IqShaderData* top = m_Stack[ iTop ].m_Data;
+			IqShaderData* s = GetNextTemp( top->Type(), top->Class() );
+			s->SetSize(top->Size());
+			s->SetValueFromVariable( top );
 			Push( s );
-
-
 		}
 
 		/** Drop the top stack entry.
 		 */
 		void	Drop()
 		{
-			TqBool f = TqFalse;
-			Pop( f );
-
-
+			bool f = false;
+			Release(Pop(f));
 		}
 
 		/**
@@ -324,7 +322,6 @@ class SHADERVM_SHARE CqShaderStack
 		 */
 		static void Statistics();
 
-
 		/** set the more efficient number of samples per type of variable at run-time.
 		 */
 		static void	SetSamples(TqInt n)
@@ -332,8 +329,6 @@ class SHADERVM_SHARE CqShaderStack
 			m_samples = (TqUint) n;
 		}
 
-
-
 	protected:
 		std::vector<SqStackEntry>	m_Stack;
 		TqUint	m_iTop;										///< Index of the top entry.
@@ -402,15 +397,16 @@ OpABRS( != , NE )
 OpABRS( *, MUL )
 /** Special case vector multiplication operator.
  */
-inline void	OpMULV( IqShaderData* pA, IqShaderData* pB, IqShaderData* pRes, CqBitVector& RunningState )
+inline void	OpMULV( IqShaderData* pA, IqShaderData* pB, IqShaderData* pRes,
+		const CqBitVector& RunningState )
 {
 	CqVector3D vA, vB;
 	CqVector3D* pdA;
 	CqVector3D* pdB;
 	TqInt i, ii;
 
-	TqBool fAVar = pA->Size() > 1;
-	TqBool fBVar = pB->Size() > 1;
+	bool fAVar = pA->Size() > 1;
+	bool fBVar = pB->Size() > 1;
 
 	if ( fAVar && fBVar )
 	{
@@ -479,7 +475,8 @@ OpABRS( / , DIV )
  * \param pRes The shader data to store the results in.
  * \param RunningState The current SIMD state.
  */
-inline void	OpDIVMM( IqShaderData* pA, IqShaderData* pB, IqShaderData* pRes, CqBitVector& RunningState )
+inline void	OpDIVMM( IqShaderData* pA, IqShaderData* pB, IqShaderData* pRes,
+		const CqBitVector& RunningState )
 {
 	CqMatrix vA;
 	CqMatrix vB;
@@ -487,8 +484,8 @@ inline void	OpDIVMM( IqShaderData* pA, IqShaderData* pB, IqShaderData* pRes, CqB
 	CqMatrix* pdB;
 	TqInt i, ii;
 	
-	TqBool fAVar = pA->Size() > 1;
-	TqBool fBVar = pB->Size() > 1;
+	bool fAVar = pA->Size() > 1;
+	bool fBVar = pB->Size() > 1;
 	
 	if( fAVar && fBVar )
 	{
@@ -571,13 +568,14 @@ OpABRS( || , LOR )
  * \param RunningState The current SIMD state.
  */
 template <class A>
-inline void	OpNEG( A& a, IqShaderData* pA, IqShaderData* pRes, CqBitVector& RunningState )
+inline void	OpNEG( A& a, IqShaderData* pA, IqShaderData* pRes,
+		const CqBitVector& RunningState )
 {
 	A vA;
 	A* pdA;
 	TqInt i, ii;
 
-	TqBool fAVar = pA->Size() > 1;
+	bool fAVar = pA->Size() > 1;
 
 	if ( fAVar )
 	{
@@ -607,13 +605,14 @@ inline void	OpNEG( A& a, IqShaderData* pA, IqShaderData* pRes, CqBitVector& Runn
  * \param RunningState The current SIMD state.
  */
 template <class A, class B>
-inline void	OpCAST( A& a, B& b, IqShaderData* pA, IqShaderData* pRes, CqBitVector& RunningState )
+inline void	OpCAST( A& a, B& b, IqShaderData* pA, IqShaderData* pRes,
+		const CqBitVector& RunningState )
 {
 	A vA;
 	A* pdA;
 	TqInt i, ii;
 
-	TqBool fAVar = pA->Size() > 1;
+	bool fAVar = pA->Size() > 1;
 
 	if ( fAVar )
 	{
@@ -643,12 +642,13 @@ inline void	OpCAST( A& a, B& b, IqShaderData* pA, IqShaderData* pRes, CqBitVecto
  * \param RunningState The current SIMD state.
  */
 template <class A>
-inline void	OpTRIPLE( A&a, IqShaderData* pRes, IqShaderData* pA, IqShaderData* pB, IqShaderData* pC, CqBitVector& RunningState )
+inline void	OpTRIPLE( A&a, IqShaderData* pRes, IqShaderData* pA, IqShaderData* pB,
+		IqShaderData* pC, const CqBitVector& RunningState )
 {
 	TqFloat x, y, z;
 
-	TqInt i = MAX( MAX( pA->Size(), pB->Size() ), pC->Size() ) - 1;
-	TqBool __fVarying = i > 0;
+	TqInt i = max( max( pA->Size(), pB->Size() ), pC->Size() ) - 1;
+	bool __fVarying = i > 0;
 	for ( ; i >= 0; i-- )
 		if ( !__fVarying || RunningState.Value( i ) )
 		{
@@ -684,19 +684,20 @@ inline void	OpHEXTUPLE( A& z, IqShaderData* pRes,
                         IqShaderData* pA, IqShaderData* pB, IqShaderData* pC, IqShaderData* pD,
                         IqShaderData* pE, IqShaderData* pF, IqShaderData* pG, IqShaderData* pH,
                         IqShaderData* pI, IqShaderData* pJ, IqShaderData* pK, IqShaderData* pL,
-                        IqShaderData* pM, IqShaderData* pN, IqShaderData* pO, IqShaderData* pP, CqBitVector& RunningState )
+                        IqShaderData* pM, IqShaderData* pN, IqShaderData* pO, IqShaderData* pP,
+						const CqBitVector& RunningState )
 {
 	TqFloat a1, a2, a3, a4;
 	TqFloat b1, b2, b3, b4;
 	TqFloat c1, c2, c3, c4;
 	TqFloat d1, d2, d3, d4;
 
-	TqInt ii1 = MAX( MAX( MAX( pA->Size(), pB->Size() ), pC->Size() ), pD->Size() );
-	TqInt ii2 = MAX( MAX( MAX( pE->Size(), pF->Size() ), pG->Size() ), pH->Size() );
-	TqInt ii3 = MAX( MAX( MAX( pI->Size(), pJ->Size() ), pK->Size() ), pL->Size() );
-	TqInt ii4 = MAX( MAX( MAX( pM->Size(), pN->Size() ), pO->Size() ), pP->Size() );
-	TqInt ii = MAX( MAX( MAX( ii1, ii2 ), ii3 ), ii4 );
-	TqBool __fVarying = ii > 0;
+	TqInt ii1 = max( max( max( pA->Size(), pB->Size() ), pC->Size() ), pD->Size() );
+	TqInt ii2 = max( max( max( pE->Size(), pF->Size() ), pG->Size() ), pH->Size() );
+	TqInt ii3 = max( max( max( pI->Size(), pJ->Size() ), pK->Size() ), pL->Size() );
+	TqInt ii4 = max( max( max( pM->Size(), pN->Size() ), pO->Size() ), pP->Size() );
+	TqInt ii = max( max( max( ii1, ii2 ), ii3 ), ii4 ) - 1;
+	bool __fVarying = ii > 0;
 	for ( ; ii >= 0; ii-- )
 	{
 		if ( !__fVarying || RunningState.Value( ii ) )
@@ -721,7 +722,7 @@ inline void	OpHEXTUPLE( A& z, IqShaderData* pRes,
 			      b1, b2, b3, b4,
 			      c1, c2, c3, c4,
 			      d1, d2, d3, d4 );
-			tt.SetfIdentity( TqFalse );
+			tt.SetfIdentity( false );
 			pRes->SetValue( tt, ii );
 		}
 	}
@@ -734,13 +735,14 @@ inline void	OpHEXTUPLE( A& z, IqShaderData* pRes,
  * \param RunningState The current SIMD state.
  */
 template <class A>
-inline void	OpCOMP( A& z, IqShaderData* pA, int index, IqShaderData* pRes, CqBitVector& RunningState )
+inline void	OpCOMP( A& z, IqShaderData* pA, int index, IqShaderData* pRes,
+		const CqBitVector& RunningState )
 {
 	A vA;
 	A* pdA;
 	TqInt i, ii;
 
-	TqBool fAVar = pA->Size() > 1;
+	bool fAVar = pA->Size() > 1;
 
 	if ( fAVar )
 	{
@@ -769,7 +771,8 @@ inline void	OpCOMP( A& z, IqShaderData* pA, int index, IqShaderData* pRes, CqBit
  * \param RunningState The current SIMD state.
  */
 template <class A>
-inline void	OpCOMP( A& z, IqShaderData* pA, IqShaderData* pB, IqShaderData* pRes, CqBitVector& RunningState )
+inline void	OpCOMP( A& z, IqShaderData* pA, IqShaderData* pB, IqShaderData* pRes,
+		const CqBitVector& RunningState )
 {
 	A vA;
 	TqFloat vB;
@@ -777,8 +780,8 @@ inline void	OpCOMP( A& z, IqShaderData* pA, IqShaderData* pB, IqShaderData* pRes
 	TqFloat* pdB;
 	TqInt i, ii;
 
-	TqBool fAVar = pA->Size() > 1;
-	TqBool fBVar = pB->Size() > 1;
+	bool fAVar = pA->Size() > 1;
+	bool fBVar = pB->Size() > 1;
 
 	if ( fAVar && fBVar )
 	{
@@ -839,13 +842,14 @@ inline void	OpCOMP( A& z, IqShaderData* pA, IqShaderData* pB, IqShaderData* pRes
  * \param RunningState The current SIMD state.
  */
 template <class A>
-inline void	OpSETCOMP( A& z, IqShaderData* pRes, int index, IqShaderData* pA, CqBitVector& RunningState )
+inline void	OpSETCOMP( A& z, IqShaderData* pRes, int index, IqShaderData* pA,
+		const CqBitVector& RunningState )
 {
 	A vA;
 	TqFloat val;
 
-	TqInt i = MAX( pRes->Size(), pA->Size() ) - 1;
-	TqBool __fVarying = i > 0;
+	TqInt i = max( pRes->Size(), pA->Size() ) - 1;
+	bool __fVarying = i > 0;
 	for ( ; i >= 0; i-- )
 	{
 		if ( !__fVarying || RunningState.Value( i ) )
@@ -865,13 +869,14 @@ inline void	OpSETCOMP( A& z, IqShaderData* pRes, int index, IqShaderData* pA, Cq
  * \param RunningState The current SIMD state.
  */
 template <class A>
-inline void	OpSETCOMP( A& z, IqShaderData* pRes, IqShaderData* index, IqShaderData* pA, CqBitVector& RunningState )
+inline void	OpSETCOMP( A& z, IqShaderData* pRes, IqShaderData* index, IqShaderData* pA,
+		const CqBitVector& RunningState )
 {
 	A vA;
 	TqFloat val, fi;
 
-	TqInt i = MAX( MAX( pRes->Size(), pA->Size() ), index->Size() ) - 1;
-	TqBool __fVarying = i > 0;
+	TqInt i = max( max( pRes->Size(), pA->Size() ), index->Size() ) - 1;
+	bool __fVarying = i > 0;
 	for ( ; i >= 0; i-- )
 	{
 		if ( !__fVarying || RunningState.Value( i ) )
@@ -892,13 +897,14 @@ inline void	OpSETCOMP( A& z, IqShaderData* pRes, IqShaderData* index, IqShaderDa
  * \param pRes The shader data to store the result in.
  * \param RunningState The current SIMD state.
  */
-inline void	OpCOMPM( IqShaderData* pA, IqShaderData* pR, IqShaderData* pC, IqShaderData* pRes, CqBitVector& RunningState )
+inline void	OpCOMPM( IqShaderData* pA, IqShaderData* pR, IqShaderData* pC,
+		IqShaderData* pRes, const CqBitVector& RunningState )
 {
 	CqMatrix m;
 	TqFloat fr, fc;
 
-	TqInt i = MAX( pA->Size(), pRes->Size() ) - 1;
-	TqBool __fVarying = i > 0;
+	TqInt i = max( pA->Size(), pRes->Size() ) - 1;
+	bool __fVarying = i > 0;
 	for ( ; i >= 0; i-- )
 		if ( !__fVarying || RunningState.Value( i ) )
 		{
@@ -916,13 +922,14 @@ inline void	OpCOMPM( IqShaderData* pA, IqShaderData* pR, IqShaderData* pC, IqSha
  * \param pV The shader data which holds the value to place in the appropriate row/column.
  * \param RunningState The current SIMD state.
  */
-inline void	OpSETCOMPM( IqShaderData* pA, IqShaderData* pR, IqShaderData* pC, IqShaderData* pV, CqBitVector& RunningState )
+inline void	OpSETCOMPM( IqShaderData* pA, IqShaderData* pR, IqShaderData* pC,
+		IqShaderData* pV, const CqBitVector& RunningState )
 {
 	CqMatrix m;
 	TqFloat fr, fc, val;
 
-	TqInt i = MAX( pA->Size(), pV->Size() ) - 1;
-	TqBool __fVarying = i > 0;
+	TqInt i = max( pA->Size(), pV->Size() ) - 1;
+	bool __fVarying = i > 0;
 	for ( ; i >= 0; i-- )
 	{
 		if ( !__fVarying || RunningState.Value( i ) )
@@ -940,6 +947,6 @@ inline void	OpSETCOMPM( IqShaderData* pA, IqShaderData* pR, IqShaderData* pC, Iq
 
 //-----------------------------------------------------------------------
 
-END_NAMESPACE( Aqsis )
+} // namespace Aqsis
 
 #endif	// !SHADERSTACK_H_INCLUDED
