@@ -1,5 +1,5 @@
 // aQSIS
-// Copyright  1997 - 2001, Paul C. Gregory
+// Copyright (C) 1997 - 2001, Paul C. Gregory
 //
 // Contact: pgregory@aqsis.org
 //
@@ -23,34 +23,32 @@
 		\author Paul C. Gregory (pgregory@aqsis.org)
 */
 
-#include	"aqsis.h"
+#include "shadervm.h"
 
-#include	"MultiTimer.h"
+#include <cstring>
+#include <ctype.h>
+#include <iostream>
+#include <sstream>
+#include <stddef.h>
 
-#include	<iostream>
+#include "iparameter.h"
+#include "irenderer.h"
+#include "isurface.h"
+#include "logging.h"
+#include "multitimer.h"
+#include "shadervariable.h"
+#include "sstring.h"
+#include "version.h"
 
-#include	<sstream>
-#include	<ctype.h>
-#include	<stddef.h>
 
-#include	"shadervm.h"
-#include	"symbols.h"
-#include	"version.h"
-#include	"sstring.h"
-
-#include	"irenderer.h"
-#include	"isurface.h"
-#include	"shadervariable.h"
-#include	"logging.h"
-
-START_NAMESPACE( Aqsis )
+namespace Aqsis {
 
 /*
  * Type, name and private hash key for the name
  */
 static struct shader_types
 {
-	char *name;
+	const TqChar* name;
 	EqShaderType type;
 	TqUlong hash;
 }
@@ -78,9 +76,8 @@ SqOpCodeTrans CqShaderVM::m_TransTable[] =
         {"RS_GET", 0, &CqShaderVM::SO_RS_GET, 0, {0}},
         {"RS_INVERSE", 0, &CqShaderVM::SO_RS_INVERSE, 0, {0}},
         {"RS_JZ", 0, &CqShaderVM::SO_RS_JZ, 1, {type_float}},
-        {"RS_JNZ", 0, &CqShaderVM::SO_RS_JNZ, 1, {type_float}},
+        {"RS_BREAK", 0, &CqShaderVM::SO_RS_BREAK, 1, {type_integer}},
         {"S_JZ", 0, &CqShaderVM::SO_S_JZ, 1, {type_float}},
-        {"S_JNZ", 0, &CqShaderVM::SO_S_JNZ, 1, {type_float}},
         {"S_GET", 0, &CqShaderVM::SO_S_GET, 0, {0}},
         {"S_CLEAR", 0, &CqShaderVM::SO_S_CLEAR, 0, {0}},
 
@@ -323,6 +320,10 @@ SqOpCodeTrans CqShaderVM::m_TransTable[] =
         {"vmix", 0, &CqShaderVM::SO_vmix, 0, {0}},
         {"nmix", 0, &CqShaderVM::SO_nmix, 0, {0}},
         {"comp", 0, &CqShaderVM::SO_comp, 0, {0}},
+        {"cmixc", 0, &CqShaderVM::SO_cmixc, 0, {0}},
+        {"pmixc", 0, &CqShaderVM::SO_pmixc, 0, {0}},
+        {"vmixc", 0, &CqShaderVM::SO_vmixc, 0, {0}},
+        {"nmixc", 0, &CqShaderVM::SO_nmixc, 0, {0}},
         {"setcomp", 0, &CqShaderVM::SO_setcomp, 0, {0}},
         {"ambient", 0, &CqShaderVM::SO_ambient, 0, {0}},
         {"diffuse", 0, &CqShaderVM::SO_diffuse, 0, {0}},
@@ -354,6 +355,9 @@ SqOpCodeTrans CqShaderVM::m_TransTable[] =
         {"advance_illuminance", 0, &CqShaderVM::SO_advance_illuminance, 0, {0}},
         {"solar2", 0, &CqShaderVM::SO_solar2, 0, {0}},
         {"solar", 0, &CqShaderVM::SO_solar, 0, {0}},
+        {"init_gather", 0, &CqShaderVM::SO_init_gather, 0, {0}},
+        {"advance_gather", 0, &CqShaderVM::SO_advance_gather, 0, {0}},
+        {"gather", 0, &CqShaderVM::SO_gather, 0, {0}},
         {"printf", 0, &CqShaderVM::SO_printf, 0, {0}},
 
         {"fcellnoise1", 0, &CqShaderVM::SO_fcellnoise1, 0, {0}},
@@ -435,6 +439,9 @@ SqOpCodeTrans CqShaderVM::m_TransTable[] =
         {"external", 0, &CqShaderVM::SO_external, 1, {type_invalid}},
 
         {"occlusion", 0, &CqShaderVM::SO_occlusion, 0, {0}},
+        {"occlusion_rt", 0, &CqShaderVM::SO_occlusion_rt, 0, {0}},
+
+        {"rayinfo", 0, &CqShaderVM::SO_rayinfo, 1, {type_invalid}},
     };
 
 /*
@@ -446,25 +453,113 @@ TqInt CqShaderVM::m_cTransSize = sizeof( m_TransTable ) / sizeof( m_TransTable[
  * Private hash keys for "Data", "Init", "Code", "segment", "param", 
  *          "varying", "uniform", "USES"
  */
-static TqUlong dhash = CqString::hash("Data");
+static const TqUlong dhash = CqString::hash("Data");
 ;
-static TqUlong ihash = CqString::hash("Init");
-static TqUlong chash = CqString::hash("Code");
-static TqUlong shash = CqString::hash("segment");
-static TqUlong phash = CqString::hash("param");
-static TqUlong vhash = CqString::hash("varying");
-static TqUlong uhash = CqString::hash("uniform");
-static TqUlong ushash = CqString::hash("USES");
-static TqUlong ehash = CqString::hash("external");
-static TqUlong ohash = CqString::hash("output");
+static const TqUlong ihash = CqString::hash("Init");
+static const TqUlong chash = CqString::hash("Code");
+static const TqUlong shash = CqString::hash("segment");
+static const TqUlong phash = CqString::hash("param");
+static const TqUlong vhash = CqString::hash("varying");
+static const TqUlong uhash = CqString::hash("uniform");
+static const TqUlong ushash = CqString::hash("USES");
+static const TqUlong ehash = CqString::hash("external");
+static const TqUlong ohash = CqString::hash("output");
 
 
+CqShaderVM::CqShaderVM(IqRenderer* pRenderContext)
+	: CqShaderStack(),
+	m_Uses(0xFFFFFFFF),
+	m_strName(),
+	m_Type(Type_Surface),
+	m_LocalIndex(0),
+	m_pEnv(),
+	m_pTransform(),
+	m_LocalVars(),
+	m_StoredArguments(),
+	m_ProgramInit(),
+	m_Program(),
+	m_ProgramStrings(),
+	m_uGridRes(0),
+	m_vGridRes(0),
+	m_shadingPointCount(0),
+	m_PC(0),
+	m_PO(0),
+	m_PE(0),
+	m_fAmbient(true),
+	m_outsideWorld(false),
+	m_pRenderContext(pRenderContext)
+{
+	// Find out if this shader is being declared outside the world construct. If so
+	// if is effectively being defined in 'camera' space, which will affect the
+	// transformation of parameters. Should only affect lightsource shaders as these
+	// are the only ones valid outside the world.
+	if (NULL != m_pRenderContext)
+		m_outsideWorld = !m_pRenderContext->IsWorldBegin();
+	else
+		m_outsideWorld = false;
+}
+
+CqShaderVM::CqShaderVM(const CqShaderVM& From)
+	: CqShaderStack(),
+	m_Uses(0),
+	m_strName(),
+	m_Type(Type_Surface),
+	m_LocalIndex(0),
+	m_pEnv(),
+	m_pTransform(),
+	m_LocalVars(),
+	m_StoredArguments(),
+	m_ProgramInit(),
+	m_Program(),
+	m_ProgramStrings(),
+	m_uGridRes(0),
+	m_vGridRes(0),
+	m_shadingPointCount(0),
+	m_PC(0),
+	m_PO(0),
+	m_PE(0),
+	m_fAmbient(true),
+	m_outsideWorld(false),
+	m_pRenderContext(0)
+{
+	*this = From;
+	// Find out if this shader is being declared outside the world construct. If so
+	// if is effectively being defined in 'camera' space, which will affect the
+	// transformation of parameters. Should only affect lightsource shaders as these
+	// are the only ones valid outside the world.
+	if (NULL != m_pRenderContext)
+		m_outsideWorld = !m_pRenderContext->IsWorldBegin();
+	else
+		m_outsideWorld = false;
+}
+
+CqShaderVM::~CqShaderVM()
+{
+	// Delete the local variables.
+	for ( std::vector<IqShaderData*>::iterator i = m_LocalVars.begin(); i != m_LocalVars.end(); i++ )
+	{
+		delete *i;
+	}
+	// Delete strings used by the program
+	for ( std::list<CqString*>::iterator i = m_ProgramStrings.begin();
+			i != m_ProgramStrings.end(); i++ )
+	{
+		delete *i;
+	}
+	// Delete stored shader arguments
+	for(std::vector<SqArgumentRecord>::iterator i = m_StoredArguments.begin();
+			i != m_StoredArguments.end(); ++i)
+	{
+		delete i->m_Value;
+	}
+}
+
 //---------------------------------------------------------------------
 /**
  *  Function to create a local variable for a specific shader
  */
 
-IqShaderData* CqShaderVM::CreateVariable( EqVariableType Type, EqVariableClass Class, const CqString& name, TqBool fParameter, TqBool fOutput )
+IqShaderData* CqShaderVM::CreateVariable( EqVariableType Type, EqVariableClass Class, const CqString& name, bool fParameter, bool fOutput )
 {
 	// Create a VM specific shader variable, which implements the IqShaderData interface,
 	// based on the type and class specified.
@@ -483,7 +578,7 @@ IqShaderData* CqShaderVM::CreateVariable( EqVariableType Type, EqVariableClass C
 						default: // Clear up compiler warnings
 						break;
 				}
-				assert( TqFalse );	// If we get here, something is wrong with the request.
+				assert( false );	// If we get here, something is wrong with the request.
 				return ( NULL );
 			}
 
@@ -498,7 +593,7 @@ IqShaderData* CqShaderVM::CreateVariable( EqVariableType Type, EqVariableClass C
 						default: // Clear up compiler warnings
 						break;
 				}
-				assert( TqFalse );	// If we get here, something is wrong with the request.
+				assert( false );	// If we get here, something is wrong with the request.
 				return ( NULL );
 			}
 
@@ -513,7 +608,7 @@ IqShaderData* CqShaderVM::CreateVariable( EqVariableType Type, EqVariableClass C
 						default: // Clear up compiler warnings
 						break;
 				}
-				assert( TqFalse );	// If we get here, something is wrong with the request.
+				assert( false );	// If we get here, something is wrong with the request.
 				return ( NULL );
 			}
 
@@ -528,7 +623,7 @@ IqShaderData* CqShaderVM::CreateVariable( EqVariableType Type, EqVariableClass C
 						default: // Clear up compiler warnings
 						break;
 				}
-				assert( TqFalse );	// If we get here, something is wrong with the request.
+				assert( false );	// If we get here, something is wrong with the request.
 				return ( NULL );
 			}
 
@@ -543,7 +638,7 @@ IqShaderData* CqShaderVM::CreateVariable( EqVariableType Type, EqVariableClass C
 						default: // Clear up compiler warnings
 						break;
 				}
-				assert( TqFalse );	// If we get here, something is wrong with the request.
+				assert( false );	// If we get here, something is wrong with the request.
 				return ( NULL );
 			}
 
@@ -558,14 +653,14 @@ IqShaderData* CqShaderVM::CreateVariable( EqVariableType Type, EqVariableClass C
 						default: // Clear up compiler warnings
 						break;
 				}
-				assert( TqFalse );	// If we get here, something is wrong with the request.
+				assert( false );	// If we get here, something is wrong with the request.
 				return ( NULL );
 			}
 
 			case type_triple:
 			case type_hpoint:
 			case type_void:
-			assert( TqFalse );	// We don't support triples in the engine as variables.
+			assert( false );	// We don't support triples in the engine as variables.
 			return ( NULL );
 
 			case type_matrix:
@@ -579,13 +674,13 @@ IqShaderData* CqShaderVM::CreateVariable( EqVariableType Type, EqVariableClass C
 						default: // Clear up compiler warnings
 						break;
 				}
-				assert( TqFalse );	// If we get here, something is wrong with the request.
+				assert( false );	// If we get here, something is wrong with the request.
 				return ( NULL );
 			}
 			default: // Clear up compiler warnings
 			break;
 	}
-	assert( TqFalse );	// If we get here, something is wrong with the request.
+	assert( false );	// If we get here, something is wrong with the request.
 	return ( NULL );
 }
 
@@ -594,7 +689,7 @@ IqShaderData* CqShaderVM::CreateVariable( EqVariableType Type, EqVariableClass C
  *  Function to create a local variable array for a specific shader
  */
 
-IqShaderData* CqShaderVM::CreateVariableArray( EqVariableType VarType, EqVariableClass VarClass, const CqString& name, TqInt Count, TqBool fParameter, TqBool fOutput )
+IqShaderData* CqShaderVM::CreateVariableArray( EqVariableType VarType, EqVariableClass VarClass, const CqString& name, TqInt Count, bool fParameter, bool fOutput )
 {
 	IqShaderData * pVar = 0;
 	switch ( VarType )
@@ -739,13 +834,13 @@ void CqShaderVM::DefaultSurface()
 * Function to determine if not the character is ' '
 */
 
-static TqBool notspace(char C)
+static bool notspace(char C)
 {
-	bool retval = TqTrue;
+	bool retval = true;
 
 	if ((C == 0x20) ||
 	        (( C >= 0x09 ) && (C <= 0x0D)))
-		retval = TqFalse;
+		retval = false;
 
 	return retval;
 }
@@ -808,13 +903,10 @@ void CqShaderVM::LoadProgram( std::istream* pFile )
 	std::vector<TqUlong> itypes;
 
 	// Initialise the private hash keys.
-
-
 	for(i = 0; i< (TqUint) gcVariableTypeNames; i++)
 		itypes.push_back(CqString::hash(gVariableTypeNames[i]));
 
-
-	TqBool fShaderSpec = TqFalse;
+	bool fShaderSpec = false;
 	while ( !pFile->eof() )
 	{
 		GetToken( token, 255, pFile );
@@ -836,12 +928,12 @@ void CqShaderVM::LoadProgram( std::istream* pFile )
 				if ( gShaderTypeNames[i].hash == htoken )
 				{
 					m_Type = gShaderTypeNames[i].type;
-					fShaderSpec = TqTrue;
+					fShaderSpec = true;
 					tmp = i;
 					break;
 				}
 			}
-			if (fShaderSpec == TqFalse)
+			if (fShaderSpec == false)
 			{
 				for (i=0 ; i < tmp; i++ )
 				{
@@ -852,7 +944,7 @@ void CqShaderVM::LoadProgram( std::istream* pFile )
 					if ( gShaderTypeNames[i].hash == htoken )
 					{
 						m_Type = gShaderTypeNames[i].type;
-						fShaderSpec = TqTrue;
+						fShaderSpec = true;
 						tmp = i;
 						break;
 					}
@@ -864,18 +956,14 @@ void CqShaderVM::LoadProgram( std::istream* pFile )
 		if ( strcmp( token, "AQSIS_V" ) == 0 )
 		{
 			GetToken( token, 255, pFile );
-
+			// Check that the version string matches the current one.  If not,
+			// fail fatally.
+			if(std::string(token) != VERSION_STR)
+			{
+				AQSIS_THROW(XqBadShader, "Shader compiled with an old/different version ("
+						<< token << ") of aqsis.  Please recompile.");
+			}
 			continue;
-
-			// Get the version information.
-			CqString strVersion(token);
-			//TqInt vMaj, vMin, build;
-			//GET_VERSION_FROM_STRING(vMaj,vMin,build);
-			//if(CHECK_NEWER_VERSION(vMaj,vMin,build))
-			//{
-			//	CqBasicError(0,Severity_Fatal,"SLX built by more recent version of Aqsis");
-			//	return;
-			//}
 		}
 
 		if ( ushash == htoken) // == "USES"
@@ -908,20 +996,20 @@ void CqShaderVM::LoadProgram( std::istream* pFile )
 		{
 			EqVariableType VarType = type_invalid;
 			EqVariableClass VarClass = class_varying;
-			TqBool			fVarArray = TqFalse;
-			TqBool			fParameter = TqFalse;
-			TqBool			fOutput = TqFalse;
+			bool fVarArray = false;
+			bool fParameter = false;
+			bool fOutput = false;
 			switch ( Segment )
 			{
-					case Seg_Data:
+				case Seg_Data:
 					VarType = type_invalid;
 					VarClass = class_invalid;
 					while ( VarType == type_invalid )
 					{
 						if ( ohash == htoken) // == "output"
-							fOutput = TqTrue;
+							fOutput = true;
 						else if ( phash == htoken) // == "param"
-							fParameter = TqTrue;
+							fParameter = true;
 						else if ( vhash == htoken) // == "varying"
 							VarClass = class_varying;
 						else if ( uhash == htoken) // == "uniform"
@@ -946,15 +1034,13 @@ void CqShaderVM::LoadProgram( std::istream* pFile )
 						while ( i < strlen( token ) && token[ i ] != '[' ) i++;
 						if ( i == strlen( token ) )
 						{
-							//CqBasicError( 0, Severity_Fatal, "Invalid variable specification in slx file" );
-							Aqsis::log() << critical << "Invalid variable specification in slx file" << std::endl;
-							return ;
+							AQSIS_THROW(XqBadShader, "Invalid variable specification in slx file");
 						}
 						token[ strlen( token ) - 1 ] = '\0';
 						token[ i ] = '\0';
 						i++;
 						array_count = atoi( &token[ i ] );
-						fVarArray = TqTrue;
+						fVarArray = true;
 					}
 					// Check if there is a valid variable specifier
 					if ( VarType == type_invalid ||
@@ -967,8 +1053,8 @@ void CqShaderVM::LoadProgram( std::istream* pFile )
 						AddLocalVariable( CreateVariable( VarType, VarClass, token, fParameter, fOutput ) );
 					break;
 
-					case Seg_Init:
-					case Seg_Code:
+				case Seg_Init:
+				case Seg_Code:
 					// Check if it is a label
 					if ( strcmp( token, ":" ) == 0 )
 					{
@@ -1009,8 +1095,9 @@ void CqShaderVM::LoadProgram( std::istream* pFile )
 								candidates = getShadeOpMethods(&strFunc);
 								if( candidates == NULL )
 								{
-									Aqsis::log() << critical << "\"" << strName().c_str() << "\": No DSO found for external shadeop: \"" << strFunc.c_str() << "\"" << std::endl;
-									exit(1);
+									AQSIS_THROW(XqBadShader, "\"" << strName().c_str()
+										<< "\": No DSO found for external shadeop: \""
+										<< strFunc.c_str() << "\"\n");
 								}
 								m_ActiveDSOMap[strFunc]=candidates;
 							};
@@ -1020,15 +1107,15 @@ void CqShaderVM::LoadProgram( std::istream* pFile )
 							m_itTypeIdMap = m_TypeIdMap.find( strRetType[1] );
 							if (m_itTypeIdMap != m_TypeIdMap.end())
 							{
-								RetType = (*m_itTypeIdMap)
-								          .second;
+								RetType = (*m_itTypeIdMap).second;
 							}
 							else
 							{
 								//error, we dont know this return type
-								Aqsis::log() << critical << "\"" << strName().c_str() << "\": Invalid return type in call to external shadeop: \"" << strFunc.c_str() << "\" : \"" << strRetType.c_str() << "\"" << std::endl;
-								exit(1);
-							};
+								AQSIS_THROW(XqBadShader, "\"" << strName()
+									<< "\": Invalid return type in call to external shadeop: \""
+									<< strFunc << "\" : \"" << strRetType << "\"");
+							}
 
 							*pFile >> strArgTypes;
 							for ( TqUint x=1; x < strArgTypes.length()-1; x++ )
@@ -1042,11 +1129,12 @@ void CqShaderVM::LoadProgram( std::istream* pFile )
 								else
 								{
 									// Error, unknown arg type
-									Aqsis::log() << critical << "\"" << strName().c_str() << "\": Invalid argument type in call to external shadeop: \"" << strFunc.c_str() << "\" : \"" << strArgTypes[x] << "\"" << std::endl;
-									exit(1);
-								};
+									AQSIS_THROW(XqBadShader, "\"" << strName()
+										<< "\": Invalid argument type in call to external shadeop: \""
+										<< strFunc << "\" : \"" << strArgTypes[x] << "\"");
+								}
 
-							};
+							}
 
 							//Now we need to find a good candidate.
 							std::list<SqDSOExternalCall*>::iterator candidate;
@@ -1057,13 +1145,12 @@ void CqShaderVM::LoadProgram( std::istream* pFile )
 								if ((*candidate)->return_type == RetType &&
 								                        (*candidate)->arg_types == ArgTypes) break;
 								candidate++;
-							};
+							}
 
 							// If we are looking for a void return type but have not
 							// found an exact match, we will take the first match with
 							// suitable arguments and force the return value to be
 							// discarded.
-							bool forcedrop = false;
 							if(candidate == candidates->end() && RetType == type_void)
 							{
 								candidate = candidates->begin()
@@ -1077,28 +1164,27 @@ void CqShaderVM::LoadProgram( std::istream* pFile )
 										Aqsis::log() << info << "\"" << strName().c_str() << "\": Using non-void DSO shadeop:  \"" << strProto.c_str() << "\"" <<
 										"\"" << strName().c_str() << "\": In place of requested void shadeop: \"" << strFunc.c_str() << "\"" <<
 										"\"" << strName().c_str() << "\": If this is not the operation you intended you should force the correct shadeop in your shader source." << std::endl;
-										forcedrop = true;
 										break;
-									};
+									}
 									candidate++;
-								};
+								}
 							}
 
 							if(candidate == candidates->end())
 							{
-								Aqsis::log() << critical << "\"" << strName()
-								.c_str() << "\": No candidate found for call to external shadeop: \"" << strFunc.c_str() <<
-								"\"" << strName().c_str() << "\": Perhaps you need some casts?" <<
-								"\"" << strName().c_str() << "\": The following candidates are in you current DSO path:" << std::endl;
+								Aqsis::log() << error << "\"" << strName()
+									<< "\": No candidate found for call to external shadeop: \""
+									<< strFunc << "\"" << strName() << "\": Perhaps you need some casts?"
+									<< "\"" << strName() << "\": The following candidates are in you current DSO path:\n";
 								candidate = candidates->begin();
 								while (candidate !=candidates->end())
 								{
 									CqString strProto = strPrototype(&strFunc, (*candidate));
 									Aqsis::log() << info << "\"" << strName().c_str() << "\": \t" << strProto.c_str() << std::endl;
 									candidate++;
-								};
-								exit(1);
-							};
+								}
+								AQSIS_THROW(XqBadShader, "External shadeop not found");
+							}
 
 							if(!(*candidate)->initialised )
 							{
@@ -1112,16 +1198,13 @@ void CqShaderVM::LoadProgram( std::istream* pFile )
 									// possibly non-unique per thread.
 									(*candidate)->initData =
 									    ((*candidate)->init)(static_cast<int>(reinterpret_cast<ptrdiff_t>(this)),NULL);
-								};
+								}
 								(*candidate)->initialised = true;
-							};
+							}
 
 							AddCommand( &CqShaderVM::SO_external, pProgramArea );
 							AddDSOExternalCall( (*candidate),pProgramArea );
 
-							if( forcedrop )
-								AddCommand( &CqShaderVM::SO_drop, pProgramArea );
-
 							break;
 						}
 
@@ -1137,7 +1220,7 @@ void CqShaderVM::LoadProgram( std::istream* pFile )
 							        &CqShaderVM::SO_illuminate2 == m_TransTable[ i ].m_pCommand ||
 							        &CqShaderVM::SO_solar == m_TransTable[ i ].m_pCommand ||
 							        &CqShaderVM::SO_solar2 == m_TransTable[ i ].m_pCommand )
-								m_fAmbient = TqFalse;
+								m_fAmbient = false;
 
 							// Add this opcode to the program segment.
 							AddCommand( m_TransTable[ i ].m_pCommand, pProgramArea );
@@ -1162,38 +1245,47 @@ void CqShaderVM::LoadProgram( std::istream* pFile )
 								{
 									switch ( m_TransTable[ i ].m_aParamTypes[ p ] )
 									{
-											case type_float:
-											( *pFile ) >> std::ws;
-											TqFloat f;
-											( *pFile ) >> f;
-											AddFloat( f, pProgramArea );
+										case type_float:
+											{
+												( *pFile ) >> std::ws;
+												TqFloat f;
+												( *pFile ) >> f;
+												AddFloat( f, pProgramArea );
+											}
 											break;
-
-											case type_string:
-											( *pFile ) >> std::ws;
-											char c;
-											CqString s( "" );
-											pFile->get
-											();
-											while ( ( c = pFile->get
-											              () ) != '"' )
-												s += c;
-											AddString( s.c_str(), pProgramArea );
+										case type_integer:
+											{
+												( *pFile ) >> std::ws;
+												TqInt i;
+												( *pFile ) >> i;
+												AddInteger( i, pProgramArea );
+											}
 											break;
+										case type_string:
+											{
+												( *pFile ) >> std::ws;
+												char c;
+												CqString s( "" );
+												pFile->get
+												();
+												while ( ( c = pFile->get
+															() ) != '"' )
+													s += c;
+												AddString( s.c_str(), pProgramArea );
+											}
+											break;
+										default:
+											AQSIS_THROW(XqBadShader, "Unknown literal type");
 									}
 								}
 							}
 							break;
 						}
 					}
-					// If we have not found the opcode, throw an error.
 					if ( i == m_cTransSize )
 					{
-						CqString strErr( "Invalid opcode found : " )
-						;
-						strErr += token;
-						Aqsis::log() << critical << strErr.c_str() << std::endl;
-						return ;
+						// If we have not found the opcode, throw an error.
+						AQSIS_THROW(XqBadShader, "Invalid opcode found: " << token);
 					}
 					break;
 			}
@@ -1210,9 +1302,7 @@ void CqShaderVM::LoadProgram( std::istream* pFile )
 		        E.m_Command == &CqShaderVM::SO_jmp ||
 		        E.m_Command == &CqShaderVM::SO_jz ||
 		        E.m_Command == &CqShaderVM::SO_RS_JZ ||
-		        E.m_Command == &CqShaderVM::SO_RS_JNZ ||
-		        E.m_Command == &CqShaderVM::SO_S_JZ ||
-		        E.m_Command == &CqShaderVM::SO_S_JNZ )
+		        E.m_Command == &CqShaderVM::SO_S_JZ)
 		{
 			SqLabel lab;
 			lab.m_Offset = aLabels[ static_cast<unsigned int>( m_Program[ i ].m_FloatVal ) ];
@@ -1234,7 +1324,6 @@ void CqShaderVM::LoadProgram( std::istream* pFile )
 			}
 		}
 	}
-
 }
 
 
@@ -1333,7 +1422,7 @@ void CqShaderVM::ExecuteInit()
 	boost::shared_ptr<IqShaderExecEnv> pOldEnv = m_pEnv;
 
 	boost::shared_ptr<IqShaderExecEnv> Env(new CqShaderExecEnv(m_pRenderContext));
-	Env->Initialise( 1, 1, 1, 1, 0, boost::shared_ptr<IqTransform>(), this, m_Uses );
+	Env->Initialise( 1, 1, 1, 1, false, 0, boost::shared_ptr<IqTransform>(), this, m_Uses );
 	Initialise( 1, 1, 1, Env );
 
 	// Execute the init program.
@@ -1455,7 +1544,7 @@ void CqShaderVM::SetArgument( const CqString& strName, EqVariableType type, cons
 			rec.m_strSpace = strSpace;
 			rec.m_strName = strName;
 			m_StoredArguments.push_back(rec);
-			Aqsis::log() << debug << "Storing argument : " << strName.c_str() << " : on : " << m_strName.c_str() << std::endl;
+			Aqsis::log() << debug << "Storing argument on shader @" << this << " : " << strName.c_str() << " : on : " << m_strName.c_str() << std::endl;
 		}
 		else
 		{
@@ -1501,7 +1590,7 @@ void CqShaderVM::PrepareShaderForUse( )
 			Aqsis::log() << debug << "imager shader " << strName().c_str() << std::endl;
 			break;
 		default:
-			Aqsis::log() << debug << "unknown shader type " << strName().c_str() << std::endl;
+			Aqsis::log() << error << "unknown shader type " << strName().c_str() << std::endl;
 			break;
 	}	
 
@@ -1509,7 +1598,7 @@ void CqShaderVM::PrepareShaderForUse( )
 
 void CqShaderVM::InitialiseParameters( )
 {
-	Aqsis::log() << debug << "Preparing shader : " << strName().c_str() << " [" << m_StoredArguments.size() << " args]"  << std::endl;
+	Aqsis::log() << debug << "Preparing shader @" << this << " : " << strName().c_str() << " [" << m_StoredArguments.size() << " args]"  << std::endl;
 
 	// Reinitialise the local variables to their defaults.
 	PrepareDefArgs();
@@ -1537,38 +1626,45 @@ void CqShaderVM::InitialiseParameters( )
 		CqMatrix matTrans;
 
 		if (getTransform())
-			matTrans = m_pRenderContext ->matSpaceToSpace( _strSpace.c_str(), "current", getTransform(), getTransform(), m_pRenderContext->Time() );
+			m_pRenderContext ->matSpaceToSpace( _strSpace.c_str(), "current", getTransform(), getTransform(), m_pRenderContext->Time(), matTrans );
 
 		while ( count-- > 0 )
 		{
 			// Get the specified value out.
-			IqShaderData* pVMVal = m_StoredArguments[i].m_Value;
+			IqShaderData* pVMVal = CreateTemporaryStorage(m_StoredArguments[i].m_Value->Type(), m_StoredArguments[i].m_Value->Class());
 
 			// If it is a color or a point, ensure it is the correct 'space'
 			if ( pVMVal->Type() == type_point || pVMVal->Type() == type_hpoint )
 			{
 				CqVector3D p;
-				pVMVal->GetPoint( p, 0 );
+				m_StoredArguments[i].m_Value->GetPoint( p, 0 );
 				pVMVal->SetPoint( matTrans * p );
 			}
 			else if ( pVMVal->Type() == type_normal )
 			{
 				CqVector3D p;
-				pVMVal->GetNormal( p, 0 );
+				m_StoredArguments[i].m_Value->GetNormal( p, 0 );
 				pVMVal->SetNormal( matTrans * p );
 			}
 			else if ( pVMVal->Type() == type_vector )
 			{
 				CqVector3D p;
-				pVMVal->GetVector( p, 0 );
+				m_StoredArguments[i].m_Value->GetVector( p, 0 );
 				pVMVal->SetVector( matTrans * p );
 			}
 			else if ( pVMVal->Type() == type_matrix )
 			{
 				CqMatrix m;
-				pVMVal->GetMatrix( m, 0 );
+				m_StoredArguments[i].m_Value->GetMatrix( m, 0 );
 				pVMVal->SetMatrix( matTrans * m );
 			}
+			else
+			{
+				if ( pArray )
+					pVMVal->ArrayEntry(arrayindex)->SetValueFromVariable( m_StoredArguments[i].m_Value->ArrayEntry(arrayindex));
+				else
+					pVMVal->SetValueFromVariable(m_StoredArguments[i].m_Value);
+			}
 
 			if ( pArray )
 			{
@@ -1577,6 +1673,7 @@ void CqShaderVM::InitialiseParameters( )
 			}
 			else
 				m_LocalVars[ varindex ] ->SetValueFromVariable( pVMVal );
+			DeleteTemporaryStorage(pVMVal);
 		}
 	}
 }
@@ -1586,7 +1683,7 @@ void CqShaderVM::InitialiseParameters( )
 /** Set the instance variables on this shader, used for varying variables which will be set from a parameter in the surface.
 */
 
-void CqShaderVM::SetArgument( CqParameter* pParam, IqSurface* pSurface )
+void CqShaderVM::SetArgument( IqParameter* pParam, IqSurface* pSurface )
 {
 	// Find the relevant variable.
 	TqInt i = FindLocalVarIndex( pParam->strName().c_str() );
@@ -1619,16 +1716,16 @@ IqShaderData* CqShaderVM::FindArgument( const CqString& name )
 /** Get a value from an instance variable on this shader, and fill in the passed variable reference.
 */
 
-TqBool CqShaderVM::GetVariableValue( const char* name, IqShaderData* res )
+bool CqShaderVM::GetVariableValue( const char* name, IqShaderData* res )
 {
 	// Find the relevant variable.
 	TqInt i = FindLocalVarIndex( name );
 	if ( i >= 0 )
 	{
 		res->SetValueFromVariable( m_LocalVars[ i ]);
-		return ( TqTrue );
+		return ( true );
 	}
-	return ( TqFalse );
+	return ( false );
 }
 
 //---------------------------------------------------------------------
@@ -1748,5 +1845,5 @@ void CqShaderVM::ShutdownShaderEngine()
 }
 
 
-END_NAMESPACE( Aqsis )
+} // namespace Aqsis
 //---------------------------------------------------------------------
