@@ -1,5 +1,5 @@
 // Aqsis
-// Copyright  1997 - 2001, Paul C. Gregory
+// Copyright (C) 1997 - 2001, Paul C. Gregory
 //
 // Contact: pgregory@aqsis.org
 //
@@ -33,17 +33,17 @@
 #include	"sstring.h"
 
 
-START_NAMESPACE( Aqsis )
+namespace Aqsis {
 
 struct IqParseNode;
 
 /// Parses an input stream, using the supplied callback object and sending error data to the supplied output stream
-TqBool Parse( std::istream& InputStream, const CqString StreamName, std::ostream& ErrorStream );
+bool Parse( std::istream& InputStream, const CqString StreamName, std::ostream& ErrorStream );
 /// Resets the state of the parser, clearing any symbol tables, etc.
 void ResetParser();
 
 IqParseNode* GetParseTree();
 
-END_NAMESPACE( Aqsis )
+} // namespace Aqsis
 
 #endif //LIBSLPARSE_H_INCLUDED
