@@ -11,6 +11,7 @@
 #define PARSENODE_H_INCLUDED 1
 
 #include	<vector>
+#include	<list>
 
 #include	"aqsis.h"
 
@@ -20,9 +21,9 @@
 #include	"funcdef.h"
 #include	"list.h"
 
-START_NAMESPACE( Aqsis )
+namespace Aqsis {
 
-extern char* gShaderTypeNames[];
+extern const char* gShaderTypeNames[];
 extern TqInt gcShaderTypeNames;
 
 ///----------------------------------------------------------------------
@@ -41,7 +42,7 @@ class CqParseNode : public CqListEntry<CqParseNode>, public IqParseNode
 		};
 
 
-		CqParseNode() : m_pChild( 0 ), m_pParent( 0 ), m_fVarying( TqFalse ), m_LineNo( -1 )
+		CqParseNode() : m_pChild( 0 ), m_pParent( 0 ), m_fVarying( false ), m_LineNo( -1 )
 		{}
 		virtual	~CqParseNode()
 		{
@@ -75,9 +76,9 @@ class CqParseNode : public CqListEntry<CqParseNode>, public IqParseNode
 		{
 			return ( m_strFileName.c_str() );
 		}
-		virtual	TqBool	IsVariableRef() const
+		virtual	bool	IsVariableRef() const
 		{
-			return ( TqFalse );
+			return ( false );
 		}
 		virtual	TqInt	ResType() const
 		{
@@ -86,7 +87,7 @@ class CqParseNode : public CqListEntry<CqParseNode>, public IqParseNode
 			else
 				return ( m_pChild->ResType() );
 		}
-		virtual	TqBool	fVarying() const
+		virtual	bool	fVarying() const
 		{
 			return ( m_fVarying );
 		}
@@ -191,8 +192,8 @@ class CqParseNode : public CqListEntry<CqParseNode>, public IqParseNode
 			m_strFileName = pos.m_strFileName;
 		}
 
-		virtual	TqBool	Optimise();
-		virtual TqInt	TypeCheck( TqInt* pTypes, TqInt Count, TqBool& needsCast, TqBool CheckOnly )
+		virtual	bool	Optimise();
+		virtual TqInt	TypeCheck( TqInt* pTypes, TqInt Count, bool& needsCast, bool CheckOnly )
 		{
 			TqInt NewType = Type_Nil;
 			CqParseNode* pChild = pFirstChild();
@@ -205,6 +206,7 @@ class CqParseNode : public CqListEntry<CqParseNode>, public IqParseNode
 			}
 			return ( NewType );
 		}
+		virtual void	validTypes( std::list<std::pair<TqInt,TqInt> >& types );
 		virtual	void	NoDup()
 		{}
 		virtual	CqParseNode* Clone( CqParseNode* pParent = 0 )
@@ -215,14 +217,14 @@ class CqParseNode : public CqListEntry<CqParseNode>, public IqParseNode
 			pNew->m_pParent = pParent;
 			return ( pNew );
 		}
-		virtual	TqBool	UpdateStorageStatus()
+		virtual	bool	UpdateStorageStatus()
 		{
-			m_fVarying = TqFalse;
+			m_fVarying = false;
 			CqParseNode* pChild = m_pChild;
 			while ( pChild != 0 )
 			{
 				if ( pChild->UpdateStorageStatus() )
-					m_fVarying = TqTrue;
+					m_fVarying = true;
 				pChild = pChild->pNext();
 			}
 			return ( m_fVarying );
@@ -230,9 +232,9 @@ class CqParseNode : public CqListEntry<CqParseNode>, public IqParseNode
 
 		CqParseNodeShader* pShaderNode();
 
-		static	char*	TypeIdentifier( int Type );
+		static	const char*	TypeIdentifier( int Type );
 		static	TqInt	TypeFromIdentifier( char Id );
-		static	char*	TypeName( int Type );
+		static	const char*	TypeName( int Type );
 		static	TqInt	FindCast( TqInt CurrType, TqInt* pTypes, TqInt Count, TqInt& index );
 		static	TqInt*	pAllTypes()
 		{
@@ -242,7 +244,7 @@ class CqParseNode : public CqListEntry<CqParseNode>, public IqParseNode
 	protected:
 		CqParseNode*	m_pChild;
 		CqParseNode*	m_pParent;
-		TqBool	m_fVarying;
+		bool	m_fVarying;
 		TqInt	m_LineNo;
 		CqString	m_strFileName;
 
@@ -372,9 +374,10 @@ class CqParseNodeFunctionCall : public CqParseNode, public IqParseNodeFunctionCa
 		}
 
 
-		virtual	TqInt	TypeCheck( TqInt* pTypes, TqInt Count, TqBool& needsCast, TqBool CheckOnly );
-		virtual	TqBool	Optimise();
+		virtual	TqInt	TypeCheck( TqInt* pTypes, TqInt Count, bool& needsCast, bool CheckOnly );
+		virtual	bool	Optimise();
 		virtual	TqInt	ResType() const;
+		virtual void	validTypes( std::list<std::pair<TqInt,TqInt> >& types );
 		virtual	CqParseNode*	Clone( CqParseNode* pParent = 0 )
 		{
 			CqParseNodeFunctionCall * pNew = new CqParseNodeFunctionCall( *this );
@@ -435,8 +438,8 @@ class CqParseNodeUnresolvedCall : public CqParseNode, public IqParseNodeUnresolv
 		}
 
 
-		virtual	TqInt	TypeCheck( TqInt* pTypes, TqInt Count, TqBool& needsCast, TqBool CheckOnly );
-		virtual	TqBool	Optimise();
+		virtual	TqInt	TypeCheck( TqInt* pTypes, TqInt Count, bool& needsCast, bool CheckOnly );
+		virtual	bool	Optimise();
 		virtual	TqInt	ResType() const;
 		virtual	CqParseNode*	Clone( CqParseNode* pParent = 0 )
 		{
@@ -487,15 +490,15 @@ class CqParseNodeVariable : public CqParseNode, public IqParseNodeVariable
 		{
 			return ( m_Extra );
 		}
-		virtual	TqBool	IsLocal() const
+		virtual	bool	IsLocal() const
 		{
 			return ( m_VarRef.m_Type == VarTypeLocal );
 		}
 
 		// Overridden from IqParseNode
-		virtual	TqBool	IsVariableRef() const
+		virtual	bool	IsVariableRef() const
 		{
-			return ( TqTrue );
+			return ( true );
 		}
 		virtual	void*	GetInterface( EqParseNodeType type) const
 		{
@@ -515,8 +518,8 @@ class CqParseNodeVariable : public CqParseNode, public IqParseNodeVariable
 
 
 
-		virtual	TqBool	Optimise();
-		virtual	TqInt	TypeCheck( TqInt* pTypes, TqInt Count, TqBool& needsCast, TqBool CheckOnly );
+		virtual	bool	Optimise();
+		virtual	TqInt	TypeCheck( TqInt* pTypes, TqInt Count, bool& needsCast, bool CheckOnly );
 		virtual	TqInt	ResType() const;
 		virtual	CqParseNode*	Clone( CqParseNode* pParent = 0 )
 		{
@@ -545,6 +548,10 @@ class CqParseNodeVariable : public CqParseNode, public IqParseNodeVariable
 			if ( pVarDef != 0 )
 				pVarDef->SetDefaultStorage( Storage );
 		}
+		virtual	bool	UpdateStorageStatus()
+		{
+			return fVarying();
+		}
 
 	protected:
 		SqVarRef	m_VarRef;
@@ -588,7 +595,7 @@ class CqParseNodeVariableArray : public CqParseNodeVariable, public IqParseNodeA
 		}
 
 
-		virtual	TqInt	TypeCheck( TqInt* pTypes, TqInt Count, TqBool& needsCast, TqBool CheckOnly );
+		virtual	TqInt	TypeCheck( TqInt* pTypes, TqInt Count, bool& needsCast, bool CheckOnly );
 		virtual	CqParseNode*	Clone( CqParseNode* pParent = 0 )
 		{
 			CqParseNodeVariableArray * pNew = new CqParseNodeVariableArray( *this );
@@ -614,7 +621,7 @@ class CqParseNodeAssign : public CqParseNodeVariable, public IqParseNodeVariable
 		{}
 		CqParseNodeAssign( SqVarRef VarRef ) :
 				CqParseNodeVariable( VarRef ),
-				m_fNoDup( TqFalse )
+				m_fNoDup( false )
 		{}
 		CqParseNodeAssign( CqParseNodeVariable* pVar ) :
 				CqParseNodeVariable( pVar )
@@ -626,7 +633,7 @@ class CqParseNodeAssign : public CqParseNodeVariable, public IqParseNodeVariable
 
 
 
-		virtual	TqBool	fDiscardResult() const
+		virtual	bool	fDiscardResult() const
 		{
 			return ( m_fNoDup );
 		}
@@ -649,11 +656,11 @@ class CqParseNodeAssign : public CqParseNodeVariable, public IqParseNodeVariable
 
 
 
-		virtual	TqBool	Optimise();
-		virtual	TqInt	TypeCheck( TqInt* pTypes, TqInt Count, TqBool& needsCast, TqBool CheckOnly );
+		virtual	bool	Optimise();
+		virtual	TqInt	TypeCheck( TqInt* pTypes, TqInt Count, bool& needsCast, bool CheckOnly );
 		virtual	void	NoDup()
 		{
-			m_fNoDup = TqTrue;
+			m_fNoDup = true;
 		}
 		virtual	TqInt	ResType() const;
 		virtual	CqParseNode*	Clone( CqParseNode* pParent = 0 )
@@ -664,12 +671,12 @@ class CqParseNodeAssign : public CqParseNodeVariable, public IqParseNodeVariable
 			pNew->m_pParent = pParent;
 			return ( pNew );
 		}
-		virtual	TqBool	UpdateStorageStatus()
+		virtual	bool	UpdateStorageStatus()
 		{
 			// Varying status is a combination of the varying status of all chidren
 			// and the varying status of the variable to which we are assigning.
-			TqBool fVarying = CqParseNode::UpdateStorageStatus();
-			m_fVarying = TqFalse;
+			bool fVarying = CqParseNode::UpdateStorageStatus();
+			m_fVarying = false;
 			IqVarDef* pVarDef = CqVarDef::GetVariablePtr( m_VarRef );
 			if ( pVarDef != 0 )
 				m_fVarying = ( pVarDef->Type() & Type_Varying ) != 0;
@@ -680,7 +687,7 @@ class CqParseNodeAssign : public CqParseNodeVariable, public IqParseNodeVariable
 		}
 
 	protected:
-		TqBool	m_fNoDup;
+		bool	m_fNoDup;
 };
 
 
@@ -725,7 +732,7 @@ class CqParseNodeAssignArray : public CqParseNodeAssign, public IqParseNodeArray
 
 
 
-		virtual	TqInt	TypeCheck( TqInt* pTypes, TqInt Count, TqBool& needsCast, TqBool CheckOnly );
+		virtual	TqInt	TypeCheck( TqInt* pTypes, TqInt Count, bool& needsCast, bool CheckOnly );
 		virtual	CqParseNode*	Clone( CqParseNode* pParent = 0 )
 		{
 			CqParseNodeAssignArray * pNew = new CqParseNodeAssignArray( *this );
@@ -770,7 +777,7 @@ class CqParseNodeOp : public CqParseNode, public IqParseNodeOperator
 			V.Visit(static_cast<IqParseNodeOperator&>(*this));
 		}
 
-		virtual	TqInt	TypeCheck( TqInt* pTypes, TqInt Count, TqBool& needsCast, TqBool CheckOnly );
+		virtual	TqInt	TypeCheck( TqInt* pTypes, TqInt Count, bool& needsCast, bool CheckOnly );
 };
 
 
@@ -854,7 +861,7 @@ class CqParseNodeMathOpDot : public CqParseNodeMathOp
 		virtual	~CqParseNodeMathOpDot()
 		{}
 
-		virtual	TqInt	TypeCheck( TqInt* pTypes, TqInt Count, TqBool& needsCast, TqBool CheckOnly );
+		virtual	TqInt	TypeCheck( TqInt* pTypes, TqInt Count, bool& needsCast, bool CheckOnly );
 };
 
 
@@ -915,7 +922,7 @@ class CqParseNodeRelOp : public CqParseNodeOp, public IqParseNodeRelationalOp
 			pNew->m_pParent = pParent;
 			return ( pNew );
 		}
-		virtual	TqInt	TypeCheck( TqInt* pTypes, TqInt Count, TqBool& needsCast, TqBool CheckOnly );
+		virtual	TqInt	TypeCheck( TqInt* pTypes, TqInt Count, bool& needsCast, bool CheckOnly );
 
 	protected:
 		EqRelOp	m_Operator;
@@ -966,7 +973,7 @@ class CqParseNodeUnaryOp : public CqParseNodeOp, public IqParseNodeUnaryOp
 		}
 
 
-		virtual	TqInt	TypeCheck( TqInt* pTypes, TqInt Count, TqBool& needsCast, TqBool CheckOnly );
+		virtual	TqInt	TypeCheck( TqInt* pTypes, TqInt Count, bool& needsCast, bool CheckOnly );
 		virtual	CqParseNode*	Clone( CqParseNode* pParent = 0 )
 		{
 			CqParseNodeUnaryOp * pNew = new CqParseNodeUnaryOp( *this );
@@ -1110,7 +1117,7 @@ class CqParseNodeConst : public CqParseNode
 		virtual	~CqParseNodeConst()
 		{}
 
-		virtual	TqInt	TypeCheck( TqInt* pTypes, TqInt Count, TqBool& needsCast, TqBool CheckOnly );
+		virtual	TqInt	TypeCheck( TqInt* pTypes, TqInt Count, bool& needsCast, bool CheckOnly );
 	private:
 };
 
@@ -1290,6 +1297,59 @@ class CqParseNodeWhileConstruct : public CqParseNode, public IqParseNodeWhileCon
 };
 
 
+///----------------------------------------------------------------------
+/// CqParseNodeLoopMod
+/// Parsenode specifying a loop modification (break or continue)
+
+class CqParseNodeLoopMod : public CqParseNode, public IqParseNodeLoopMod
+{
+	public:
+		CqParseNodeLoopMod( const CqParseNodeLoopMod& from )
+			: CqParseNode( from ),
+			m_modType(from.m_modType)
+		{}
+		CqParseNodeLoopMod(EqLoopModType modType)
+			: CqParseNode(),
+			m_modType(modType)
+		{}
+		virtual	~CqParseNodeLoopMod() {}
+
+		virtual EqLoopModType modType()
+		{
+			return m_modType;
+		}
+
+		// Overridden fromIqParseNode
+
+		virtual	void*	GetInterface( EqParseNodeType type) const
+		{
+			void* pNode;
+			if ( ( pNode = ( void* ) QueryNodeType<IqParseNodeLoopMod>( this, type ) ) != 0 )
+				return ( pNode );
+			return ( CqParseNode::GetInterface( type ) );
+		}
+		virtual	TqInt	NodeType() const
+		{
+			return ( IqParseNodeLoopMod::m_ID );
+		}
+		virtual	void	Accept( IqParseNodeVisitor &V)
+		{
+			V.Visit(static_cast<IqParseNodeLoopMod&>(*this));
+		}
+
+		virtual	CqParseNode*	Clone( CqParseNode* pParent = 0 )
+		{
+			CqParseNodeLoopMod * pNew = new CqParseNodeLoopMod( *this );
+			if ( m_pChild )
+				pNew->m_pChild = m_pChild->Clone( pNew );
+			pNew->m_pParent = pParent;
+			return ( pNew );
+		}
+	private:
+		EqLoopModType m_modType;
+};
+
+
 ///----------------------------------------------------------------------
 /// CqParseNodeIlluminateConstruct
 /// Parsenode an illuminate construct.
@@ -1301,7 +1361,7 @@ class CqParseNodeIlluminateConstruct : public CqParseNode, public IqParseNodeIll
 				CqParseNode( from ),
 				m_fAxisAngle( from.m_fAxisAngle )
 		{}
-		CqParseNodeIlluminateConstruct( TqBool fAxisAngle = TqFalse ) :
+		CqParseNodeIlluminateConstruct( bool fAxisAngle = false ) :
 				CqParseNode(),
 				m_fAxisAngle( fAxisAngle )
 		{}
@@ -1312,7 +1372,7 @@ class CqParseNodeIlluminateConstruct : public CqParseNode, public IqParseNodeIll
 
 
 
-		virtual	TqBool	fHasAxisAngle() const
+		virtual	bool	fHasAxisAngle() const
 		{
 			return ( m_fAxisAngle );
 		}
@@ -1344,7 +1404,7 @@ class CqParseNodeIlluminateConstruct : public CqParseNode, public IqParseNodeIll
 		}
 
 	private:
-		TqBool	m_fAxisAngle;
+		bool	m_fAxisAngle;
 };
 
 
@@ -1359,7 +1419,7 @@ class CqParseNodeIlluminanceConstruct : public CqParseNode, public IqParseNodeIl
 				CqParseNode( from ),
 				m_fAxisAngle( from.m_fAxisAngle )
 		{}
-		CqParseNodeIlluminanceConstruct( TqBool fAxisAngle = TqFalse ) :
+		CqParseNodeIlluminanceConstruct( bool fAxisAngle = false ) :
 				CqParseNode(),
 				m_fAxisAngle( fAxisAngle )
 		{}
@@ -1370,7 +1430,7 @@ class CqParseNodeIlluminanceConstruct : public CqParseNode, public IqParseNodeIl
 
 
 
-		virtual	TqBool	fHasAxisAngle() const
+		virtual	bool	fHasAxisAngle() const
 		{
 			return ( m_fAxisAngle );
 		}
@@ -1402,7 +1462,7 @@ class CqParseNodeIlluminanceConstruct : public CqParseNode, public IqParseNodeIl
 		}
 
 	private:
-		TqBool	m_fAxisAngle;
+		bool	m_fAxisAngle;
 };
 
 
@@ -1417,7 +1477,7 @@ class CqParseNodeSolarConstruct : public CqParseNode, public IqParseNodeSolarCon
 				CqParseNode( from ),
 				m_fAxisAngle( from.m_fAxisAngle )
 		{}
-		CqParseNodeSolarConstruct( TqBool fAxisAngle = TqFalse ) :
+		CqParseNodeSolarConstruct( bool fAxisAngle = false ) :
 				CqParseNode(),
 				m_fAxisAngle( fAxisAngle )
 		{}
@@ -1428,7 +1488,7 @@ class CqParseNodeSolarConstruct : public CqParseNode, public IqParseNodeSolarCon
 
 
 
-		virtual	TqBool	fHasAxisAngle() const
+		virtual	bool	fHasAxisAngle() const
 		{
 			return ( m_fAxisAngle );
 		}
@@ -1460,7 +1520,55 @@ class CqParseNodeSolarConstruct : public CqParseNode, public IqParseNodeSolarCon
 		}
 
 	private:
-		TqBool	m_fAxisAngle;
+		bool	m_fAxisAngle;
+};
+
+
+///----------------------------------------------------------------------
+/// CqParseNodeGatherConstruct
+/// Parsenode a gather construct.
+
+class CqParseNodeGatherConstruct : public CqParseNode, public IqParseNodeGatherConstruct
+{
+	public:
+		CqParseNodeGatherConstruct( const CqParseNodeGatherConstruct& from ) :
+				CqParseNode( from )
+		{}
+		CqParseNodeGatherConstruct( bool fAxisAngle = false ) :
+				CqParseNode()
+		{}
+		virtual	~CqParseNodeGatherConstruct()
+		{}
+
+		// Overridden from IqParseNode
+
+		virtual	void*	GetInterface( EqParseNodeType type) const
+		{
+			void* pNode;
+			if ( ( pNode = ( void* ) QueryNodeType<IqParseNodeGatherConstruct>( this, type ) ) != 0 )
+				return ( pNode );
+			return ( CqParseNode::GetInterface( type ) );
+		}
+		virtual	TqInt	NodeType() const
+		{
+			return ( IqParseNodeGatherConstruct::m_ID );
+		}
+		virtual	void	Accept( IqParseNodeVisitor &V)
+		{
+			V.Visit(static_cast<IqParseNodeGatherConstruct&>(*this));
+		}
+
+
+		virtual	CqParseNode*	Clone( CqParseNode* pParent = 0 )
+		{
+			CqParseNodeGatherConstruct * pNew = new CqParseNodeGatherConstruct( *this );
+			if ( m_pChild )
+				pNew->m_pChild = m_pChild->Clone( pNew );
+			pNew->m_pParent = pParent;
+			return ( pNew );
+		}
+
+	private:
 };
 
 
@@ -1510,7 +1618,7 @@ class CqParseNodeConditional : public CqParseNode, public IqParseNodeConditional
 			pNew->m_pParent = pParent;
 			return ( pNew );
 		}
-		virtual	TqInt	TypeCheck( TqInt* pTypes, TqInt Count, TqBool& needsCast, TqBool CheckOnly )
+		virtual	TqInt	TypeCheck( TqInt* pTypes, TqInt Count, bool& needsCast, bool CheckOnly )
 		{
 			TqInt Types = Type_Float;
 			TqInt NewType = Type_Nil;
@@ -1571,7 +1679,7 @@ class CqParseNodeQCond : public CqParseNode, public IqParseNodeConditionalExpres
 		}
 
 
-		virtual	TqInt	TypeCheck( TqInt* pTypes, TqInt Count, TqBool& needsCast, TqBool CheckOnly );
+		virtual	TqInt	TypeCheck( TqInt* pTypes, TqInt Count, bool& needsCast, bool CheckOnly );
 		virtual	CqParseNode*	Clone( CqParseNode* pParent = 0 )
 		{
 			CqParseNodeQCond * pNew = new CqParseNodeQCond( *this );
@@ -1636,8 +1744,8 @@ class CqParseNodeCast : public CqParseNode, public IqParseNodeTypeCast
 
 
 
-		virtual	TqBool	Optimise();
-		virtual	TqInt	TypeCheck( TqInt* pTypes, TqInt Count, TqBool& needsCast, TqBool CheckOnly );
+		virtual	bool	Optimise();
+		virtual	TqInt	TypeCheck( TqInt* pTypes, TqInt Count, bool& needsCast, bool CheckOnly );
 		virtual	CqParseNode*	Clone( CqParseNode* pParent = 0 )
 		{
 			CqParseNodeCast * pNew = new CqParseNodeCast( *this );
@@ -1692,7 +1800,7 @@ class CqParseNodeTriple : public CqParseNode, public IqParseNodeTriple
 		}
 
 
-		virtual	TqInt	TypeCheck( TqInt* pTypes, TqInt Count, TqBool& needsCast, TqBool CheckOnly );
+		virtual	TqInt	TypeCheck( TqInt* pTypes, TqInt Count, bool& needsCast, bool CheckOnly );
 		virtual	CqParseNode*	Clone( CqParseNode* pParent = 0 )
 		{
 			CqParseNodeTriple * pNew = new CqParseNodeTriple( *this );
@@ -1747,7 +1855,7 @@ class CqParseNodeHexTuple : public CqParseNode, public IqParseNodeSixteenTuple
 		}
 
 
-		virtual	TqInt	TypeCheck( TqInt* pTypes, TqInt Count, TqBool& needsCast, TqBool CheckOnly );
+		virtual	TqInt	TypeCheck( TqInt* pTypes, TqInt Count, bool& needsCast, bool CheckOnly );
 		virtual	CqParseNode*	Clone( CqParseNode* pParent = 0 )
 		{
 			CqParseNodeHexTuple * pNew = new CqParseNodeHexTuple( *this );
@@ -1828,7 +1936,7 @@ class CqParseNodeCommFunction : public CqParseNode, public IqParseNodeMessagePas
 		}
 
 
-		virtual	TqInt	TypeCheck( TqInt* pTypes, TqInt Count, TqBool& needsCast, TqBool CheckOnly );
+		virtual	TqInt	TypeCheck( TqInt* pTypes, TqInt Count, bool& needsCast, bool CheckOnly );
 		virtual	CqParseNode*	Clone( CqParseNode* pParent = 0 )
 		{
 			CqParseNodeCommFunction * pNew = new CqParseNodeCommFunction( *this );
@@ -1862,7 +1970,7 @@ class CqParseNodeDeclaration : public CqParseNode
 				CqParseNode(),
 				m_strName( strName ),
 				m_Type( Type ),
-				m_Output( TqFalse )
+				m_Output( false )
 		{
 			m_fVarying = ( m_Type & Type_Varying ) != 0;
 		}
@@ -1880,11 +1988,11 @@ class CqParseNodeDeclaration : public CqParseNode
 		{
 			m_Type = Type;
 		}
-		TqBool	Output() const
+		bool	Output() const
 		{
 			return ( m_Output );
 		}
-		void	SetOutput( TqBool Output )
+		void	SetOutput( bool Output )
 		{
 			m_Output = Output;
 		}
@@ -1906,12 +2014,12 @@ class CqParseNodeDeclaration : public CqParseNode
 	protected:
 		CqString	m_strName;
 		TqInt	m_Type;
-		TqBool	m_Output;
+		bool	m_Output;
 };
 
 
 //-----------------------------------------------------------------------
 
-END_NAMESPACE( Aqsis )
+} // namespace Aqsis
 
 #endif	// !PARSENODE_H_INCLUDED
