@@ -1,4 +1,4 @@
-////---------------------------------------------------------------------
+///---------------------------------------------------------------------
 ////    Class definition file:  TYPECHECK.CPP
 ////    Associated header file: PARSENODE.H
 ////
@@ -8,10 +8,11 @@
 
 #include	"aqsis.h"
 #include	"parsenode.h"
+#include	"logging.h"
 
 #include	<queue>
 
-START_NAMESPACE( Aqsis )
+namespace Aqsis {
 
 ///---------------------------------------------------------------------
 /// CqParseNodeFunctionCall::TypeCheck
@@ -21,7 +22,7 @@ START_NAMESPACE( Aqsis )
 class CqFunctionSignature
 {
 	public:
-		CqFunctionSignature(SqFuncRef ref, CqFuncDef* func, TqInt weight, TqBool mustCast) : m_ref(ref), m_func(func), m_weight(weight), m_mustCast(mustCast)
+		CqFunctionSignature(SqFuncRef ref, CqFuncDef* func, TqInt weight, bool mustCast) : m_ref(ref), m_func(func), m_weight(weight), m_mustCast(mustCast)
 		{}
 		~CqFunctionSignature()
 		{}
@@ -46,7 +47,7 @@ class CqFunctionSignature
 			return(m_weight);
 		}
 
-		TqBool needsCast() const
+		bool needsCast() const
 		{
 			return(m_mustCast);
 		}
@@ -55,7 +56,7 @@ class CqFunctionSignature
 		SqFuncRef m_ref;
 		CqFuncDef* m_func;
 		TqInt m_weight;
-		TqBool m_mustCast;
+		bool m_mustCast;
 
 		friend bool operator<(const CqFunctionSignature& a, const CqFunctionSignature& b)
 		{
@@ -65,7 +66,7 @@ class CqFunctionSignature
 };
 
 
-TqInt	CqParseNodeFunctionCall::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast, TqBool CheckOnly )
+TqInt	CqParseNodeFunctionCall::TypeCheck( TqInt* pTypes, TqInt Count,  bool& needsCast, bool CheckOnly )
 {
 	TqInt cSpecifiedArgs = 0;
 
@@ -75,15 +76,6 @@ TqInt	CqParseNodeFunctionCall::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& n
 
 	CqString strMyName = strName();
 
-	// Prepare the error string.
-	CqString strErr( strFileName() );
-	strErr += " : ";
-	strErr += CqString( LineNo() );
-	strErr += " : ";
-	strErr += strMyName;
-	strErr += " : ";
-	CqString strErrDesc = "Valid function declaration not found : ";
-
 	// First discard any function possibilities which don't have the correct number of arguments.
 	// Find out how many arguments have been specified.
 	CqParseNode* pArg = m_pChild;
@@ -93,16 +85,32 @@ TqInt	CqParseNodeFunctionCall::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& n
 		pArg = pArg->pNext();
 	}
 
+	Aqsis::log() << info << "Typechecking function call \"" << strName() << "\", possible matches: " << m_aFuncRef.size() <<  "[Line: " << LineNo() << "]" << std::endl;
 
-	// Build the queue
+	// First build a list of valid types, ordered by preference, for each specified argument.
+	pArg = m_pChild;
+	TqUint iArg = 0;
+	std::vector<std::list<std::pair<TqInt, TqInt> > >	argTypes;
+	while(pArg != 0)
+	{
+		CqParseNode* pNext = pArg->pNext();
+		std::list<std::pair<TqInt, TqInt> > types;
+		pArg->validTypes(types);
+		argTypes.push_back(types);
+		++iArg;
+		pArg = pNext;
+	}
+
+	// Now for each candidate, check if the valid types can match, storing a weight depending on the suitability.
 	std::vector<SqFuncRef>::iterator i;
 	for ( i = m_aFuncRef.begin(); i != m_aFuncRef.end();  )
 	{
 		CqFuncDef* pfunc = CqFuncDef::GetFunctionPtr( (*i) );
 		if( pfunc != 0 )
 		{
+			Aqsis::log() << info << "Considering candidate (" << TypeName(pfunc->Type()) << ")" << pfunc->strName() << "[" << pfunc->strParams() << "]" << std::endl;
 			TqInt weight = 0;
-			TqBool mustCast = TqFalse;
+			bool mustCast = false;
 			TqInt cArgs = pfunc->cTypeSpecLength();
 			if ( ( cArgs > 0 ) &&
 			        ( ( !pfunc->fVarying() && cArgs != cSpecifiedArgs ) ||
@@ -120,7 +128,9 @@ TqInt	CqParseNodeFunctionCall::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& n
 				TqInt castType, index;
 				if( (castType = FindCast(pfunc->Type(), pTypes, Count, index)) == Type_Nil )
 				{
-					// This signature cannot return any of the required types.
+					// This signature cannot return any of the requested types,
+					// Can't delete it, as it is possible that TypeCheck may be later called
+					// with a different set of requested types.
 					++i;
 					continue;
 				}
@@ -129,58 +139,44 @@ TqInt	CqParseNodeFunctionCall::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& n
 					// A function signature that can return one of the requested types without
 					// casting has more weight.
 					TqInt retType;
-					mustCast = TqTrue;
+					mustCast = true;
 					for(retType = 0; retType < Count; retType++)
 					{
 						if(castType == pfunc->Type())
 						{
 							weight++;
-							mustCast = TqFalse;
+							mustCast = false;
 							break;
 						}
 					}
 				}
-				// Check if the args list matches the arguments passed, the weight
-				// is adjusted according to the number of casts required.
-				pArg = m_pChild;
-				TqUint iArg = 0;
-				TqBool fArgsFailed = TqFalse;
-				while(pArg != 0)
+				// Check the arguments in turn to see if they can be used for this candidate.
+				TqInt iArg = 0;
+				bool fArgsFailed = false;
+				for(std::vector<std::list<std::pair<TqInt, TqInt> > >::iterator arg = argTypes.begin(); arg != argTypes.end(); ++arg, ++iArg)
 				{
-					CqParseNode * pNext = pArg->pNext();
-
-					// Check if this is a varying length argument function.
-					if ( (TqUint) pfunc->cTypeSpecLength() <= iArg || ( pfunc->fVarying() && (TqUint) pfunc->cTypeSpecLength() == iArg ) )
-					{
-						// Not entirely happy about this, if the function takes a varying number
-						// of arguments, and this argument is amongst the varying list, then
-						// just accept it, no casting necessary.
+					// If the candidate signature is varying, and we've hit the '*' exit at this point.
+					if(pfunc->fVarying() && iArg >= cArgs)
 						break;
-					}
-
-					TqInt paramType = pfunc->aTypeSpec()[iArg];
-					TqInt castType;
-					TqBool castReturn = TqFalse;
-					if((castType = pArg->TypeCheck(&paramType, 1, castReturn, TqTrue)) == Type_Nil)
+					std::list<std::pair<TqInt, TqInt> >::const_iterator match;
+					for(match = arg->begin(); match != arg->end(); ++match)
 					{
-						// We've found and argument that can't be cast to the suitable type
-						// so this signature is no good, don't add it.
-						fArgsFailed = TqTrue;
-						break;
-					}
-					else
-					{
-						// If we have an upper case type in the function declaration, then we need an actual variable.
-						if( (paramType & Type_Variable) && !pArg->IsVariableRef() )
+						/// \todo: Need to check the 'usage' of the required argument, specifically
+						/// check if Type_Variable matches.
+						if((match->first & Type_Mask) == (pfunc->aTypeSpec()[iArg] & Type_Mask))
+						{
+							weight += match->second;
 							break;
-						else
-							// Did the argument have to cast to match? If not, then increase the weight.
-							if(!castReturn)
-								weight++;
+						}
+					}
+					if(match == arg->end())
+					{
+						Aqsis::log() << info << "Argument " << iArg << " doesn't match:";
+						if(!arg->empty())
+							Aqsis::log() << info << " got " << TypeName(arg->front().first);
+						Aqsis::log() << info << " expected " << TypeName(pfunc->aTypeSpec()[iArg]) << std::endl;
+						fArgsFailed = true;
 					}
-
-					pArg = pNext;
-					iArg++;
 				}
 
 				// If we got this far, the function signature matches, and we have a weight
@@ -209,13 +205,17 @@ TqInt	CqParseNodeFunctionCall::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& n
 	{
 		if(!CheckOnly)
 		{
-			strErr += "Arguments to function not valid : ";
+			CqString strErr = "Arguments to function not valid : ";
+			strErr += strName();
+			strErr += " - ";
+			strErr += LineNo();
 			throw( strErr );
 		}
 		return ( Type_Nil );
 	}
 
 	const CqFunctionSignature& best = functions.top();
+	Aqsis::log() << info << "Selected (" << TypeName(best.getFunc()->Type()) << ")" << best.getFunc()->strName() << "[" << best.getFunc()->strParams() << "]" << std::endl;
 	needsCast = best.needsCast();
 
 	// If we are not just checking, at this point, we have the best function candidate,
@@ -240,7 +240,7 @@ TqInt	CqParseNodeFunctionCall::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& n
 				argType = pfunc->aTypeSpec() [ iArg ];
 
 			// Now type check the argument, and cast if necessary
-			argType = pArg->TypeCheck( &argType, 1, needsCast, TqFalse );
+			argType = pArg->TypeCheck( &argType, 1, needsCast, false );
 
 			pArg = pNext;
 			iArg++;
@@ -298,12 +298,12 @@ void CqParseNodeFunctionCall::CheckArgCast( std::vector<TqInt>& aRes )
 		}
 
 		// Now check through to see if the specified arguments can be converted to the required ones.
-		TqBool	fValid = TqTrue;
+		bool	fValid = true;
 		TqUint j;
 		TqInt index;
 		for ( j = 0; j < aTypes.size(); j++ )
 			if ( FindCast( aArgTypes[ j ], &aTypes[ j ], 1, index ) == Type_Nil )
-				fValid = TqFalse;
+				fValid = false;
 
 		// If we have found a match, return it.
 		if ( !fValid )
@@ -338,8 +338,8 @@ void CqParseNodeFunctionCall::ArgCast( TqInt iIndex )
 	{
 		// Must get the next here incase the check inserts a cast.
 		CqParseNode * pNext = pArg->pNext();
-		TqBool needsCast;
-		pArg->TypeCheck( &aTypes[ j++ ], 1, needsCast, TqFalse );
+		bool needsCast;
+		pArg->TypeCheck( &aTypes[ j++ ], 1, needsCast, false );
 		pArg = pNext;
 	}
 }
@@ -351,7 +351,7 @@ void CqParseNodeFunctionCall::ArgCast( TqInt iIndex )
 /// to be, then later when the shader vm parses the .slx we can try and build
 /// a cast then, the same goes for the arguments.
 
-TqInt	CqParseNodeUnresolvedCall::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast, TqBool CheckOnly )
+TqInt	CqParseNodeUnresolvedCall::TypeCheck( TqInt* pTypes, TqInt Count,  bool& needsCast, bool CheckOnly )
 {
 	TqInt NewType = Type_Nil;
 
@@ -363,7 +363,7 @@ TqInt	CqParseNodeUnresolvedCall::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool&
 	while (pArg != NULL)
 	{
 		CqParseNode *pNext = pArg->pNext();
-		pArg->TypeCheck( pAllTypes(), Type_Last - 1, needsCast, TqFalse );
+		pArg->TypeCheck( pAllTypes(), Type_Last - 1, needsCast, false );
 		strArgTypes+=CqParseNode::TypeIdentifier(pArg->ResType());
 		pArg = pNext;
 	};
@@ -402,7 +402,7 @@ TqInt	CqParseNodeUnresolvedCall::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool&
 /// CqParseNodeVariable::TypeCheck
 /// Do a type check based on suitable types requested, and add a cast in required.
 
-TqInt	CqParseNodeVariable::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast, TqBool CheckOnly )
+TqInt	CqParseNodeVariable::TypeCheck( TqInt* pTypes, TqInt Count,  bool& needsCast, bool CheckOnly )
 {
 	// Check if the variable type matches any of the requested ones.
 	TqInt	MyType = ( ResType() & Type_Mask );
@@ -415,7 +415,7 @@ TqInt	CqParseNodeVariable::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needs
 	// If we got here, we are not of the correct type, so find a suitable cast.
 	TqInt index;
 	TqInt NewType = FindCast( MyType, pTypes, Count, index );
-	needsCast = TqTrue;
+	needsCast = true;
 	if(!CheckOnly)
 	{
 		CqParseNodeCast* pCast = new CqParseNodeCast( NewType );
@@ -442,11 +442,11 @@ TqInt	CqParseNodeVariable::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needs
 /// CqParseNodeVariableArray::TypeCheck
 /// Do a type check based on suitable types requested, and add a cast in required.
 
-TqInt	CqParseNodeVariableArray::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast, TqBool CheckOnly )
+TqInt	CqParseNodeVariableArray::TypeCheck( TqInt* pTypes, TqInt Count,  bool& needsCast, bool CheckOnly )
 {
 	// Check if the child can be made into a float.
 	TqInt aType = Type_Float;
-	TqBool temp;
+	bool temp;
 	if ( m_pChild && m_pChild->TypeCheck( &aType, 1, temp, CheckOnly ) == Type_Nil )
 	{
 		TqInt	IndexType = ( m_pChild->ResType() & Type_Mask );
@@ -486,7 +486,7 @@ TqInt	CqParseNodeVariableArray::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool&
 
 extern EqShaderType gShaderType;
 
-TqInt	CqParseNodeAssign::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast, TqBool CheckOnly )
+TqInt	CqParseNodeAssign::TypeCheck( TqInt* pTypes, TqInt Count,  bool& needsCast, bool CheckOnly )
 {
 	// First check if the assign is to a read only variable.
 	if( CqVarDef::GetVariablePtr( m_VarRef ) &&
@@ -506,6 +506,24 @@ TqInt	CqParseNodeAssign::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCa
 		return ( Type_Nil );
 	}
 
+	// First check if we are assigning a varying value to a uniform variable.
+	bool assigneeIsVarying = false;
+	IqVarDef* pVarDef = CqVarDef::GetVariablePtr( m_VarRef );
+	if ( pVarDef != 0 )
+		assigneeIsVarying = ( pVarDef->Type() & Type_Varying ) != 0;
+	if(!assigneeIsVarying && m_fVarying)
+	{
+		CqString strErr( strFileName() );
+		strErr += " : ";
+		strErr += LineNo();
+		strErr += " : ";
+		strErr += "Cannot assign a varying value to the uniform variable '";
+		strErr += CqVarDef::GetVariablePtr( m_VarRef )->strName();
+		strErr += "'";
+		throw( strErr );
+		return ( Type_Nil );
+	}
+	
 	TqInt	MyType = ( ResType() & Type_Mask );
 	// Type check the assignment expression first.
 	CqParseNode* pExpr = m_pChild;
@@ -522,7 +540,7 @@ TqInt	CqParseNodeAssign::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCa
 	// If we got here, we are not of the correct type, so find a suitable cast.
 	TqInt index;
 	TqInt NewType = FindCast( MyType, pTypes, Count, index );
-	needsCast = TqTrue;
+	needsCast = true;
 	if(!CheckOnly)
 	{
 		CqParseNodeCast* pCast = new CqParseNodeCast( NewType );
@@ -549,11 +567,11 @@ TqInt	CqParseNodeAssign::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCa
 /// CqParseNodeAssignArray::TypeCheck
 /// Do a type check based on suitable types requested, and add a cast in required.
 
-TqInt	CqParseNodeAssignArray::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast, TqBool CheckOnly )
+TqInt	CqParseNodeAssignArray::TypeCheck( TqInt* pTypes, TqInt Count,  bool& needsCast, bool CheckOnly )
 {
 	// Check if the child can be made into a float.
 	TqInt aType = Type_Float;
-	TqBool temp;
+	bool temp;
 	if ( m_pChild->pNext() ->TypeCheck( &aType, 1, temp, CheckOnly ) == Type_Nil )
 	{
 		TqInt	IndexType = ( m_pChild->pNext() ->ResType() & Type_Mask );
@@ -575,7 +593,7 @@ TqInt	CqParseNodeAssignArray::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& ne
 /// CqParseNodeOp::TypeCheck
 /// Do a type check based on suitable types requested, and add a cast in required.
 
-TqInt	CqParseNodeOp::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast, TqBool CheckOnly )
+TqInt	CqParseNodeOp::TypeCheck( TqInt* pTypes, TqInt Count,  bool& needsCast, bool CheckOnly )
 {
 	// Check if both arguments to the operator match type.
 	CqParseNode * pOperandA = m_pChild;
@@ -623,7 +641,7 @@ TqInt	CqParseNodeOp::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast,
 /// CqParseNodeRelOp::TypeCheck
 /// Do a type check based on suitable types requested, and add a cast in required.
 
-TqInt	CqParseNodeRelOp::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast, TqBool CheckOnly )
+TqInt	CqParseNodeRelOp::TypeCheck( TqInt* pTypes, TqInt Count,  bool& needsCast, bool CheckOnly )
 {
 	TqInt RelType;
 	RelType = CqParseNodeOp::TypeCheck( pAllTypes(), Type_Last - 1, needsCast, CheckOnly );
@@ -639,7 +657,7 @@ TqInt	CqParseNodeRelOp::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCas
 			return ( Type_Float );
 		else
 		{
-			needsCast = TqTrue;
+			needsCast = true;
 			if(!CheckOnly)
 			{
 				CqParseNodeCast* pCast = new CqParseNodeCast( NewType );
@@ -668,7 +686,7 @@ TqInt	CqParseNodeRelOp::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCas
 /// CqParseNodeUnaryOp::TypeCheck
 /// Do a type check based on suitable types requested, and add a cast in required.
 
-TqInt	CqParseNodeUnaryOp::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast, TqBool CheckOnly )
+TqInt	CqParseNodeUnaryOp::TypeCheck( TqInt* pTypes, TqInt Count,  bool& needsCast, bool CheckOnly )
 {
 	// Check if the operand can be cast to a requested type.
 	if ( m_pChild != 0 )
@@ -682,7 +700,7 @@ TqInt	CqParseNodeUnaryOp::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsC
 /// CqParseNodeOpDot::TypeCheck
 /// Do a type check based on suitable types requested, and add a cast in required.
 
-TqInt	CqParseNodeMathOpDot::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast, TqBool CheckOnly )
+TqInt	CqParseNodeMathOpDot::TypeCheck( TqInt* pTypes, TqInt Count,  bool& needsCast, bool CheckOnly )
 {
 	static TqInt aArgTypes[ 3 ] = {Type_Point, Type_Vector, Type_Normal};
 
@@ -700,7 +718,7 @@ TqInt	CqParseNodeMathOpDot::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& need
 	TqInt RetType, index;
 	if ( ( RetType = FindCast( Type_Float, pTypes, Count, index ) ) != Type_Nil )
 	{
-		TqBool fValid = TqFalse;
+		bool fValid = false;
 		TqUint i;
 		for ( i = 0; i < sizeof( aArgTypes ) / sizeof( aArgTypes[ 0 ] ); i++ )
 		{
@@ -714,7 +732,7 @@ TqInt	CqParseNodeMathOpDot::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& need
 						pOperandA->TypeCheck( &aArgTypes[ i ], 1, needsCast, CheckOnly );
 						pOperandB->TypeCheck( &aArgTypes[ i ], 1, needsCast, CheckOnly );
 					}
-					fValid = TqTrue;
+					fValid = true;
 					break;
 				}
 			}
@@ -723,7 +741,7 @@ TqInt	CqParseNodeMathOpDot::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& need
 		{
 			if ( RetType != Type_Float)
 			{
-				needsCast = TqTrue;
+				needsCast = true;
 				if(!CheckOnly)
 				{
 					CqParseNodeCast * pCast = new CqParseNodeCast( RetType );
@@ -753,7 +771,7 @@ TqInt	CqParseNodeMathOpDot::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& need
 /// CqParseNodeConst::TypeCheck
 /// Perform a typecheck, and add cast if required.
 
-TqInt CqParseNodeConst::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast, TqBool CheckOnly )
+TqInt CqParseNodeConst::TypeCheck( TqInt* pTypes, TqInt Count,  bool& needsCast, bool CheckOnly )
 {
 	// Check if the variable type matches any of the requested ones.
 	TqInt	MyType = ResType();
@@ -767,7 +785,7 @@ TqInt CqParseNodeConst::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCas
 	// If we got here, we are not of the correct type, so find a suitable cast.
 	TqInt index;
 	TqInt NewType = FindCast( MyType, pTypes, Count, index );
-	needsCast = TqTrue;
+	needsCast = true;
 	if ( !CheckOnly )
 	{
 		CqParseNodeCast * pCast = new CqParseNodeCast( NewType );
@@ -794,7 +812,7 @@ TqInt CqParseNodeConst::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCas
 /// CqParseNodeCast::TypeCheck
 /// Do a type check based on suitable types requested, and add a cast in required.
 
-TqInt	CqParseNodeCast::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast, TqBool CheckOnly )
+TqInt	CqParseNodeCast::TypeCheck( TqInt* pTypes, TqInt Count,  bool& needsCast, bool CheckOnly )
 {
 	// Perform a typecheck on the cast expression
 	CqParseNode * pExpr = m_pChild;
@@ -825,7 +843,7 @@ TqInt	CqParseNodeCast::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast
 	}
 	else
 	{
-		needsCast = TqTrue;
+		needsCast = true;
 		if ( !CheckOnly )
 		{
 			CqParseNodeCast * pCast = new CqParseNodeCast( NewType );
@@ -841,7 +859,7 @@ TqInt	CqParseNodeCast::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast
 /// CqParseNodeTriple::TypeCheck
 /// Do a type check based on suitable types requested, and add a cast in required.
 
-TqInt	CqParseNodeTriple::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast, TqBool CheckOnly )
+TqInt	CqParseNodeTriple::TypeCheck( TqInt* pTypes, TqInt Count,  bool& needsCast, bool CheckOnly )
 {
 	static TqInt ExprType = Type_Float;
 	// Perform a typecheck on the three float expressions
@@ -859,7 +877,7 @@ TqInt	CqParseNodeTriple::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCa
 			return ( Type_Triple );
 
 	// If we got here, we are not of the correct type, so find a suitable cast.
-	needsCast = TqTrue;
+	needsCast = true;
 	TqInt index;
 	TqInt NewType = FindCast( Type_Triple, pTypes, Count, index );
 	if(!CheckOnly)
@@ -888,15 +906,17 @@ TqInt	CqParseNodeTriple::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCa
 /// CqParseNodeHexTuple::TypeCheck
 /// Do a type check based on suitable types requested, and add a cast in required.
 
-TqInt	CqParseNodeHexTuple::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast, TqBool CheckOnly )
+TqInt	CqParseNodeHexTuple::TypeCheck( TqInt* pTypes, TqInt Count,  bool& needsCast, bool CheckOnly )
 {
 	static TqInt ExprType = Type_Float;
 	// Perform a typecheck on the three float expressions
 	CqParseNode* pExpr = m_pChild;
 	while ( pExpr != 0 )
 	{
+		// Have to get this here, in case the type check inserts a cast above the current expression.
+		CqParseNode* pnextChild = pExpr->pNext();
 		pExpr->TypeCheck( &ExprType, 1, needsCast, CheckOnly );
-		pExpr = pExpr->pNext();
+		pExpr = pnextChild;
 	}
 	// Check if expecting a sixteentuple, if not add a cast
 	TqInt i;
@@ -905,7 +925,7 @@ TqInt	CqParseNodeHexTuple::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needs
 			return ( Type_HexTuple );
 
 	// If we got here, we are not of the correct type, so find a suitable cast.
-	needsCast = TqTrue;
+	needsCast = true;
 	TqInt index;
 	TqInt NewType = FindCast( Type_Matrix, pTypes, Count, index );
 	if(!CheckOnly)
@@ -933,7 +953,7 @@ TqInt	CqParseNodeHexTuple::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needs
 ///---------------------------------------------------------------------
 /// CqParseNodeCommFunction::TypeCheck
 
-TqInt CqParseNodeCommFunction::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast, TqBool CheckOnly )
+TqInt CqParseNodeCommFunction::TypeCheck( TqInt* pTypes, TqInt Count,  bool& needsCast, bool CheckOnly )
 {
 	// Check if the variable type matches any of the requested ones.
 	TqInt	MyType = ResType();
@@ -944,7 +964,7 @@ TqInt CqParseNodeCommFunction::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& n
 			return ( MyType );
 	}
 	// If we got here, we are not of the correct type, so find a suitable cast.
-	needsCast = TqTrue;
+	needsCast = true;
 	TqInt index;
 	TqInt NewType = FindCast( MyType, pTypes, Count, index );
 	if(!CheckOnly)
@@ -973,7 +993,7 @@ TqInt CqParseNodeCommFunction::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& n
 /// CqParseNodeQCond::TypeCheck
 /// Type check the return from a conditional statement and cast the true/false expressions if necessary.
 
-TqInt	CqParseNodeQCond::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast, TqBool CheckOnly )
+TqInt	CqParseNodeQCond::TypeCheck( TqInt* pTypes, TqInt Count,  bool& needsCast, bool CheckOnly )
 {
 	// Ensure that the conditional expression is type checked.
 	CqParseNode * pCondExp = m_pChild;
@@ -1024,5 +1044,5 @@ TqInt	CqParseNodeQCond::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCas
 }
 
 
-END_NAMESPACE( Aqsis )
+} // namespace Aqsis
 //---------------------------------------------------------------------
